@@ -1,4 +1,4 @@
-/* Copyright (c) 2014 The Chromium OS Authors. All rights reserved.
+/* Copyright 2014 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -30,7 +30,7 @@ static void usart_rx_interrupt_handler(struct usart_config const *config)
 		uint8_t byte = STM32_USART_RDR(base);
 
 		if (!queue_add_unit(config->producer.queue, &byte))
-			atomic_add(&config->state->rx_dropped, 1);
+			atomic_add((atomic_t *)&(config->state->rx_dropped), 1);
 	}
 }
 
