@@ -11,12 +11,20 @@
 #include "fusb302.h"
 #include "task.h"
 #include "hooks.h"
-#include "tcpm.h"
+#include "tcpm/tcpci.h"
+#include "tcpm/tcpm.h"
 #include "timer.h"
+#include "usb_charge.h"
 #include "usb_pd.h"
 #include "usb_pd_tcpc.h"
+#include "usb_pd_tcpm.h"
 #include "util.h"
 
+#if defined(CONFIG_USB_PD_DUAL_ROLE_AUTO_TOGGLE) || \
+	defined(CONFIG_USB_PD_DISCHARGE_TCPC)
+#error "Unsupported config options of fusb302 PD driver"
+#endif
+
 #define PACKET_IS_GOOD_CRC(head) (PD_HEADER_TYPE(head) == PD_CTRL_GOOD_CRC && \
 				 PD_HEADER_CNT(head) == 0)
 
@@ -28,7 +36,9 @@ static struct fusb302_chip_state {
 	int rx_enable;
 	uint8_t mdac_vnc;
 	uint8_t mdac_rd;
-} state[CONFIG_USB_PD_PORT_COUNT];
+} state[CONFIG_USB_PD_PORT_MAX_COUNT];
+
+static struct mutex measure_lock;
 
 /*
  * Bring the FUSB302 out of reset after Hard Reset signaling. This will
@@ -90,11 +100,11 @@ static int convert_bc_lvl(int port, int bc_lvl)
 			ret = TYPEC_CC_VOLT_RD;
 	} else {
 		if (bc_lvl == 0x1)
-			ret = TYPEC_CC_VOLT_SNK_DEF;
+			ret = TYPEC_CC_VOLT_RP_DEF;
 		else if (bc_lvl == 0x2)
-			ret = TYPEC_CC_VOLT_SNK_1_5;
+			ret = TYPEC_CC_VOLT_RP_1_5;
 		else if (bc_lvl == 0x3)
-			ret = TYPEC_CC_VOLT_SNK_3_0;
+			ret = TYPEC_CC_VOLT_RP_3_0;
 	}
 
 	return ret;
@@ -106,6 +116,8 @@ static int measure_cc_pin_source(int port, int cc_measure)
 	int reg;
 	int cc_lvl;
 
+	mutex_lock(&measure_lock);
+
 	/* Read status register */
 	tcpc_read(port, TCPC_REG_SWITCHES0, &reg);
 	/* Save current value */
@@ -153,11 +165,15 @@ static int measure_cc_pin_source(int port, int cc_measure)
 	/* Restore SWITCHES0 register to its value prior */
 	tcpc_write(port, TCPC_REG_SWITCHES0, switches0_reg);
 
+	mutex_unlock(&measure_lock);
+
 	return cc_lvl;
 }
 
 /* Determine cc pin state for source when in manual detect mode */
-static void detect_cc_pin_source_manual(int port, int *cc1_lvl, int *cc2_lvl)
+static void detect_cc_pin_source_manual(int port,
+	enum tcpc_cc_voltage_status *cc1_lvl,
+	enum tcpc_cc_voltage_status *cc2_lvl)
 {
 	int cc1_measure = TCPC_REG_SWITCHES0_MEAS_CC1;
 	int cc2_measure = TCPC_REG_SWITCHES0_MEAS_CC2;
@@ -177,7 +193,8 @@ static void detect_cc_pin_source_manual(int port, int *cc1_lvl, int *cc2_lvl)
 }
 
 /* Determine cc pin state for sink */
-static void detect_cc_pin_sink(int port, int *cc1, int *cc2)
+static void detect_cc_pin_sink(int port, enum tcpc_cc_voltage_status *cc1,
+	enum tcpc_cc_voltage_status *cc2)
 {
 	int reg;
 	int orig_meas_cc1;
@@ -185,6 +202,8 @@ static void detect_cc_pin_sink(int port, int *cc1, int *cc2)
 	int bc_lvl_cc1;
 	int bc_lvl_cc2;
 
+	mutex_lock(&measure_lock);
+
 	/*
 	 * Measure CC1 first.
 	 */
@@ -255,6 +274,8 @@ static void detect_cc_pin_sink(int port, int *cc1, int *cc2)
 		reg &= ~TCPC_REG_SWITCHES0_MEAS_CC2;
 
 	tcpc_write(port, TCPC_REG_SWITCHES0, reg);
+
+	mutex_unlock(&measure_lock);
 }
 
 /* Parse header bytes for the size of packet */
@@ -321,9 +342,7 @@ static int fusb302_send_message(int port, uint16_t header, const uint32_t *data,
 	buf[buf_pos++] = FUSB302_TKN_TXON;
 
 	/* burst write for speed! */
-	tcpc_lock(port, 1);
-	rv = tcpc_xfer(port, buf, buf_pos, 0, 0, I2C_XFER_SINGLE);
-	tcpc_lock(port, 0);
+	rv = tcpc_xfer(port, buf, buf_pos, 0, 0);
 
 	return rv;
 }
@@ -334,6 +353,9 @@ static int fusb302_tcpm_select_rp_value(int port, int rp)
 	int rv;
 	uint8_t vnc, rd;
 
+	/* Keep track of current RP value */
+	tcpci_set_cached_rp(port, rp);
+
 	rv = tcpc_read(port, TCPC_REG_CONTROL0, &reg);
 	if (rv)
 		return rv;
@@ -382,8 +404,7 @@ static int fusb302_tcpm_init(int port)
 	/* Turn on retries and set number of retries */
 	tcpc_read(port, TCPC_REG_CONTROL3, &reg);
 	reg |= TCPC_REG_CONTROL3_AUTO_RETRY;
-	reg |= (PD_RETRY_COUNT & 0x3) <<
-		TCPC_REG_CONTROL3_N_RETRIES_POS;
+	reg |= (CONFIG_PD_RETRY_COUNT & 0x3) << TCPC_REG_CONTROL3_N_RETRIES_POS;
 	tcpc_write(port, TCPC_REG_CONTROL3, reg);
 
 	/* Create interrupt masks */
@@ -394,6 +415,12 @@ static int fusb302_tcpm_init(int port)
 	reg &= ~TCPC_REG_MASK_COLLISION;
 	/* misc alert */
 	reg &= ~TCPC_REG_MASK_ALERT;
+#ifdef CONFIG_USB_PD_VBUS_DETECT_TCPC
+	/* TODO(crbug.com/791109): Clean up VBUS notification. */
+
+	/* VBUS threshold crossed (~4.0V) */
+	reg &= ~TCPC_REG_MASK_VBUSOK;
+#endif
 	tcpc_write(port, TCPC_REG_MASK, reg);
 
 	reg = 0xFF;
@@ -421,14 +448,31 @@ static int fusb302_tcpm_init(int port)
 	tcpm_set_polarity(port, 0);
 	tcpm_set_vconn(port, 0);
 
-	/* Turn on the power! */
 	/* TODO: Reduce power consumption */
 	tcpc_write(port, TCPC_REG_POWER, TCPC_REG_POWER_PWR_ALL);
 
+#if defined(CONFIG_USB_PD_VBUS_DETECT_TCPC) && defined(CONFIG_USB_CHARGER)
+	/* Wait for the reference voltage to stablize */
+	usleep(250);
+	/*
+	 * Initialize VBUS supplier when VBUS is already present before
+	 * init (e.g. Cold reboot with charger plugged).
+	 */
+	tcpc_read(port, TCPC_REG_STATUS0, &reg);
+	if (reg & TCPC_REG_STATUS0_VBUSOK)
+		usb_charger_vbus_change(port, 1);
+#endif
+
 	return 0;
 }
 
-static int fusb302_tcpm_get_cc(int port, int *cc1, int *cc2)
+static int fusb302_tcpm_release(int port)
+{
+	return EC_ERROR_UNIMPLEMENTED;
+}
+
+static int fusb302_tcpm_get_cc(int port, enum tcpc_cc_voltage_status *cc1,
+	enum tcpc_cc_voltage_status *cc2)
 {
 	if (state[port].pulling_up) {
 		/* Source mode? */
@@ -513,7 +557,7 @@ static int fusb302_tcpm_set_cc(int port, int pull)
 	return 0;
 }
 
-static int fusb302_tcpm_set_polarity(int port, int polarity)
+static int fusb302_tcpm_set_polarity(int port, enum tcpc_cc_polarity polarity)
 {
 	/* Port polarity : 0 => CC1 is CC line, 1 => CC2 is CC line */
 	int reg;
@@ -526,7 +570,7 @@ static int fusb302_tcpm_set_polarity(int port, int polarity)
 
 	if (state[port].vconn_enabled) {
 		/* set VCONN switch to be non-CC line */
-		if (polarity)
+		if (polarity_rm_dts(polarity))
 			reg |= TCPC_REG_SWITCHES0_VCONN_CC1;
 		else
 			reg |= TCPC_REG_SWITCHES0_VCONN_CC2;
@@ -537,7 +581,7 @@ static int fusb302_tcpm_set_polarity(int port, int polarity)
 	reg &= ~TCPC_REG_SWITCHES0_MEAS_CC2;
 
 	/* set rx polarity */
-	if (polarity)
+	if (polarity_rm_dts(polarity))
 		reg |= TCPC_REG_SWITCHES0_MEAS_CC2;
 	else
 		reg |= TCPC_REG_SWITCHES0_MEAS_CC1;
@@ -551,7 +595,7 @@ static int fusb302_tcpm_set_polarity(int port, int polarity)
 	reg &= ~TCPC_REG_SWITCHES1_TXCC2_EN;
 
 	/* set tx polarity */
-	if (polarity)
+	if (polarity_rm_dts(polarity))
 		reg |= TCPC_REG_SWITCHES1_TXCC2_EN;
 	else
 		reg |= TCPC_REG_SWITCHES1_TXCC1_EN;
@@ -564,6 +608,24 @@ static int fusb302_tcpm_set_polarity(int port, int polarity)
 	return 0;
 }
 
+__maybe_unused static int fusb302_tcpm_decode_sop_prime_enable(int port,
+								bool enable)
+{
+	int reg;
+
+	if (tcpc_read(port, TCPC_REG_CONTROL1, &reg))
+		return EC_ERROR_UNKNOWN;
+
+	if (enable)
+		reg |= (TCPC_REG_CONTROL1_ENSOP1 |
+				TCPC_REG_CONTROL1_ENSOP2);
+	else
+		reg &= ~(TCPC_REG_CONTROL1_ENSOP1 |
+			TCPC_REG_CONTROL1_ENSOP2);
+
+	return tcpc_write(port, TCPC_REG_CONTROL1, reg);
+}
+
 static int fusb302_tcpm_set_vconn(int port, int enable)
 {
 	/*
@@ -582,6 +644,14 @@ static int fusb302_tcpm_set_vconn(int port, int enable)
 	if (enable) {
 		/* set to saved polarity */
 		tcpm_set_polarity(port, state[port].cc_polarity);
+
+		if (IS_ENABLED(CONFIG_USB_PD_DECODE_SOP)) {
+			if (state[port].rx_enable) {
+				if (fusb302_tcpm_decode_sop_prime_enable(port,
+									true))
+					return EC_ERROR_UNKNOWN;
+			}
+		}
 	} else {
 
 		tcpc_read(port, TCPC_REG_SWITCHES0, &reg);
@@ -591,6 +661,14 @@ static int fusb302_tcpm_set_vconn(int port, int enable)
 		reg &= ~TCPC_REG_SWITCHES0_VCONN_CC2;
 
 		tcpc_write(port, TCPC_REG_SWITCHES0, reg);
+
+		if (IS_ENABLED(CONFIG_USB_PD_DECODE_SOP)) {
+			if (state[port].rx_enable) {
+				if (fusb302_tcpm_decode_sop_prime_enable(port,
+									false))
+					return EC_ERROR_UNKNOWN;
+			}
+		}
 	}
 
 	return 0;
@@ -663,6 +741,20 @@ static int fusb302_tcpm_set_rx_enable(int port, int enable)
 				   reg & ~TCPC_REG_MASK_BC_LVL);
 	}
 
+#ifdef CONFIG_USB_PD_DECODE_SOP
+	/*
+	 * Only the VCONN Source is allowed to communicate
+	 * with the Cable Plugs.
+	 */
+	if (state[port].vconn_enabled) {
+		if (tcpc_read(port, TCPC_REG_CONTROL1, &reg))
+			return EC_ERROR_UNKNOWN;
+
+		reg |= (TCPC_REG_CONTROL1_ENSOP1 | TCPC_REG_CONTROL1_ENSOP2);
+		tcpc_write(port, TCPC_REG_CONTROL1, reg);
+	}
+#endif
+
 	fusb302_auto_goodcrc_enable(port, enable);
 
 	return 0;
@@ -677,7 +769,7 @@ static int fusb302_rx_fifo_is_empty(int port)
 	       (reg & TCPC_REG_STATUS1_RX_EMPTY);
 }
 
-static int fusb302_tcpm_get_message(int port, uint32_t *payload, int *head)
+static int fusb302_tcpm_get_message_raw(int port, uint32_t *payload, int *head)
 {
 	/*
 	 * This is the buffer that will get the burst-read data
@@ -690,10 +782,6 @@ static int fusb302_tcpm_get_message(int port, uint32_t *payload, int *head)
 	uint8_t buf[32];
 	int rv, len;
 
-	/* If our FIFO is empty then we have no packet */
-	if (fusb302_rx_fifo_is_empty(port))
-		return EC_ERROR_UNKNOWN;
-
 	/* Read until we have a non-GoodCRC packet or an empty FIFO */
 	do {
 		buf[0] = TCPC_REG_FIFOS;
@@ -703,7 +791,7 @@ static int fusb302_tcpm_get_message(int port, uint32_t *payload, int *head)
 		 * PART 1 OF BURST READ: Write in register address.
 		 * Issue a START, no STOP.
 		 */
-		rv = tcpc_xfer(port, buf, 1, 0, 0, I2C_XFER_START);
+		rv = tcpc_xfer_unlocked(port, buf, 1, 0, 0, I2C_XFER_START);
 
 		/*
 		 * PART 2 OF BURST READ: Read up to the header.
@@ -712,7 +800,7 @@ static int fusb302_tcpm_get_message(int port, uint32_t *payload, int *head)
 		 * and determine how many more bytes we need to read.
 		 * TODO: Check token to ensure valid packet.
 		 */
-		rv |= tcpc_xfer(port, 0, 0, buf, 3, I2C_XFER_START);
+		rv |= tcpc_xfer_unlocked(port, 0, 0, buf, 3, I2C_XFER_START);
 
 		/* Grab the header */
 		*head = (buf[1] & 0xFF);
@@ -726,7 +814,7 @@ static int fusb302_tcpm_get_message(int port, uint32_t *payload, int *head)
 		 * No START, but do issue a STOP at the end.
 		 * add 4 to len to read CRC out
 		 */
-		rv |= tcpc_xfer(port, 0, 0, buf, len+4, I2C_XFER_STOP);
+		rv |= tcpc_xfer_unlocked(port, 0, 0, buf, len+4, I2C_XFER_STOP);
 
 		tcpc_lock(port, 0);
 	} while (!rv && PACKET_IS_GOOD_CRC(*head) &&
@@ -740,17 +828,24 @@ static int fusb302_tcpm_get_message(int port, uint32_t *payload, int *head)
 			memcpy(payload, buf, len);
 	}
 
-	/*
-	 * If our FIFO is non-empty then we may have a packet, we may get
-	 * fewer interrupts than packets due to interrupt latency.
-	 */
-	if (!fusb302_rx_fifo_is_empty(port))
-		task_set_event(PD_PORT_TO_TASK_ID(port), PD_EVENT_RX, 0);
+#ifdef CONFIG_USB_PD_DECODE_SOP
+	{
+		int reg;
+
+		if (tcpc_read(port, TCPC_REG_STATUS1, &reg))
+			return EC_ERROR_UNKNOWN;
+
+		if (reg & TCPC_REG_STATUS1_RXSOP1)
+			*head |= PD_HEADER_SOP(TCPCI_MSG_SOP_PRIME);
+		else if (reg & TCPC_REG_STATUS1_RXSOP2)
+			*head |= PD_HEADER_SOP(TCPCI_MSG_SOP_PRIME_PRIME);
+	}
+#endif
 
 	return rv;
 }
 
-static int fusb302_tcpm_transmit(int port, enum tcpm_transmit_type type,
+static int fusb302_tcpm_transmit(int port, enum tcpci_msg_type type,
 				 uint16_t header, const uint32_t *data)
 {
 	/*
@@ -776,7 +871,7 @@ static int fusb302_tcpm_transmit(int port, enum tcpm_transmit_type type,
 	fusb302_flush_tx_fifo(port);
 
 	switch (type) {
-	case TCPC_TX_SOP:
+	case TCPCI_MSG_SOP:
 
 		/* put register address first for of burst tcpc write */
 		buf[buf_pos++] = TCPC_REG_FIFOS;
@@ -788,14 +883,38 @@ static int fusb302_tcpm_transmit(int port, enum tcpm_transmit_type type,
 		buf[buf_pos++] = FUSB302_TKN_SYNC2;
 
 		return fusb302_send_message(port, header, data, buf, buf_pos);
-	case TCPC_TX_HARD_RESET:
+	case TCPCI_MSG_SOP_PRIME:
+
+		/* put register address first for of burst tcpc write */
+		buf[buf_pos++] = TCPC_REG_FIFOS;
+
+		/* Write the SOP' Ordered Set into TX FIFO */
+		buf[buf_pos++] = FUSB302_TKN_SYNC1;
+		buf[buf_pos++] = FUSB302_TKN_SYNC1;
+		buf[buf_pos++] = FUSB302_TKN_SYNC3;
+		buf[buf_pos++] = FUSB302_TKN_SYNC3;
+
+		return fusb302_send_message(port, header, data, buf, buf_pos);
+	case TCPCI_MSG_SOP_PRIME_PRIME:
+
+		/* put register address first for of burst tcpc write */
+		buf[buf_pos++] = TCPC_REG_FIFOS;
+
+		/* Write the SOP'' Ordered Set into TX FIFO */
+		buf[buf_pos++] = FUSB302_TKN_SYNC1;
+		buf[buf_pos++] = FUSB302_TKN_SYNC3;
+		buf[buf_pos++] = FUSB302_TKN_SYNC1;
+		buf[buf_pos++] = FUSB302_TKN_SYNC3;
+
+		return fusb302_send_message(port, header, data, buf, buf_pos);
+	case TCPCI_MSG_TX_HARD_RESET:
 		/* Simply hit the SEND_HARD_RESET bit */
 		tcpc_read(port, TCPC_REG_CONTROL3, &reg);
 		reg |= TCPC_REG_CONTROL3_SEND_HARDRESET;
 		tcpc_write(port, TCPC_REG_CONTROL3, reg);
 
 		break;
-	case TCPC_TX_BIST_MODE_2:
+	case TCPCI_MSG_TX_BIST_MODE_2:
 		/* Hit the BIST_MODE2 bit and start TX */
 		tcpc_read(port, TCPC_REG_CONTROL1, &reg);
 		reg |= TCPC_REG_CONTROL1_BIST_MODE2;
@@ -821,14 +940,17 @@ static int fusb302_tcpm_transmit(int port, enum tcpm_transmit_type type,
 }
 
 #ifdef CONFIG_USB_PD_VBUS_DETECT_TCPC
-static int fusb302_tcpm_get_vbus_level(int port)
+static bool fusb302_tcpm_check_vbus_level(int port, enum vbus_level level)
 {
 	int reg;
 
 	/* Read status register */
 	tcpc_read(port, TCPC_REG_STATUS0, &reg);
 
-	return (reg & TCPC_REG_STATUS0_VBUSOK) ? 1 : 0;
+	if (level == VBUS_PRESENT)
+		return (reg & TCPC_REG_STATUS0_VBUSOK) ? 1 : 0;
+	else
+		return (reg & TCPC_REG_STATUS0_VBUSOK) ? 0 : 1;
 }
 #endif
 
@@ -854,7 +976,7 @@ void fusb302_tcpc_alert(int port)
 
 	if (interrupt & TCPC_REG_INTERRUPT_BC_LVL) {
 		/* CC Status change */
-		task_set_event(PD_PORT_TO_TASK_ID(port), PD_EVENT_CC, 0);
+		task_set_event(PD_PORT_TO_TASK_ID(port), PD_EVENT_CC);
 	}
 
 	if (interrupt & TCPC_REG_INTERRUPT_COLLISION) {
@@ -862,11 +984,24 @@ void fusb302_tcpc_alert(int port)
 		pd_transmit_complete(port, TCPC_TX_COMPLETE_FAILED);
 	}
 
+#ifdef CONFIG_USB_PD_VBUS_DETECT_TCPC
+	if (interrupt & TCPC_REG_INTERRUPT_VBUSOK) {
+		/* VBUS crossed threshold */
+#ifdef CONFIG_USB_CHARGER
+		usb_charger_vbus_change(port,
+					fusb302_tcpm_check_vbus_level(port,
+							VBUS_PRESENT));
+#else
+		if (!fusb302_tcpm_check_vbus_level(port, VBUS_PRESENT))
+			pd_vbus_low(port);
+#endif
+		task_wake(PD_PORT_TO_TASK_ID(port));
+		hook_notify(HOOK_AC_CHANGE);
+	}
+#endif
+
 	/* GoodCRC was received, our FIFO is now non-empty */
 	if (interrupta & TCPC_REG_INTERRUPTA_TX_SUCCESS) {
-		task_set_event(PD_PORT_TO_TASK_ID(port),
-				PD_EVENT_RX, 0);
-
 		pd_transmit_complete(port, TCPC_TX_COMPLETE_SUCCESS);
 	}
 
@@ -889,18 +1024,17 @@ void fusb302_tcpc_alert(int port)
 
 		/* bring FUSB302 out of reset */
 		fusb302_pd_reset(port);
-
-		pd_execute_hard_reset(port);
-
-		task_wake(PD_PORT_TO_TASK_ID(port));
+		task_set_event(PD_PORT_TO_TASK_ID(port),
+			       PD_EVENT_RX_HARD_RESET);
 	}
 
 	if (interruptb & TCPC_REG_INTERRUPTB_GCRCSENT) {
 		/* Packet received and GoodCRC sent */
 		/* (this interrupt fires after the GoodCRC finishes) */
 		if (state[port].rx_enable) {
-			task_set_event(PD_PORT_TO_TASK_ID(port),
-					PD_EVENT_RX, 0);
+			/* Pull all RX messages from TCPC into EC memory */
+			while (!fusb302_rx_fifo_is_empty(port))
+				tcpm_enqueue_message(port);
 		} else {
 			/* flush rx fifo if rx isn't enabled */
 			fusb302_flush_rx_fifo(port);
@@ -924,19 +1058,148 @@ void tcpm_set_bist_test_data(int port)
 	tcpc_write(port, TCPC_REG_CONTROL3, reg);
 }
 
+#ifdef CONFIG_USB_PD_TCPC_LOW_POWER
+static int fusb302_set_toggle_mode(int port, int mode)
+{
+	int reg, rv;
+
+	rv = i2c_read8(tcpc_config[port].i2c_info.port,
+		tcpc_config[port].i2c_info.addr_flags,
+		TCPC_REG_CONTROL2, &reg);
+	if (rv)
+		return rv;
+
+	reg &= ~TCPC_REG_CONTROL2_MODE_MASK;
+	reg |= mode << TCPC_REG_CONTROL2_MODE_POS;
+	return i2c_write8(tcpc_config[port].i2c_info.port,
+		tcpc_config[port].i2c_info.addr_flags,
+		TCPC_REG_CONTROL2, reg);
+}
+
+static int fusb302_tcpm_enter_low_power_mode(int port)
+{
+	int reg, rv, mode = TCPC_REG_CONTROL2_MODE_DRP;
+
+	/**
+	 * vendor's suggested LPM flow:
+	 * - enable low power mode and set up other things
+	 * - sleep 250 us
+	 * - start toggling
+	 */
+	rv = i2c_write8(tcpc_config[port].i2c_info.port,
+			  tcpc_config[port].i2c_info.addr_flags,
+			  TCPC_REG_POWER, TCPC_REG_POWER_PWR_LOW);
+	if (rv)
+		return rv;
+
+	switch (pd_get_dual_role(port)) {
+	case PD_DRP_TOGGLE_ON:
+		mode = TCPC_REG_CONTROL2_MODE_DRP;
+		break;
+	case PD_DRP_TOGGLE_OFF:
+		mode = TCPC_REG_CONTROL2_MODE_UFP;
+		break;
+	case PD_DRP_FREEZE:
+		mode = pd_get_power_role(port) == PD_ROLE_SINK ?
+			TCPC_REG_CONTROL2_MODE_UFP :
+			TCPC_REG_CONTROL2_MODE_DFP;
+		break;
+	case PD_DRP_FORCE_SINK:
+		mode = TCPC_REG_CONTROL2_MODE_UFP;
+		break;
+	case PD_DRP_FORCE_SOURCE:
+		mode = TCPC_REG_CONTROL2_MODE_DFP;
+		break;
+	}
+	rv = fusb302_set_toggle_mode(port, mode);
+	if (rv)
+		return rv;
+
+	usleep(250);
+
+	rv = i2c_read8(tcpc_config[port].i2c_info.port,
+		tcpc_config[port].i2c_info.addr_flags,
+		TCPC_REG_CONTROL2, &reg);
+	if (rv)
+		return rv;
+	reg |= TCPC_REG_CONTROL2_TOGGLE;
+	return i2c_write8(tcpc_config[port].i2c_info.port,
+		tcpc_config[port].i2c_info.addr_flags,
+		TCPC_REG_CONTROL2, reg);
+}
+#endif
+
+/*
+ * Compare VBUS voltage with given mdac reference voltage.
+ * returns non-zero if VBUS voltage >= (mdac + 1) * 420 mV
+ */
+static int fusb302_compare_mdac(int port, int mdac)
+{
+	int orig_reg, status0;
+
+	mutex_lock(&measure_lock);
+
+	/* backup REG_MEASURE */
+	tcpc_read(port, TCPC_REG_MEASURE, &orig_reg);
+	/* set reg_measure bit 0~5 to mdac, and bit6 to 1(measure vbus) */
+	tcpc_write(port, TCPC_REG_MEASURE,
+		(mdac & TCPC_REG_MEASURE_MDAC_MASK) | TCPC_REG_MEASURE_VBUS);
+
+	/* Wait on measurement */
+	usleep(350);
+
+	/*
+	 * Read status register, if STATUS0_COMP=1 then vbus is higher than
+	 * (mdac + 1) * 0.42V
+	 */
+	tcpc_read(port, TCPC_REG_STATUS0, &status0);
+	/* write back original value */
+	tcpc_write(port, TCPC_REG_MEASURE, orig_reg);
+
+	mutex_unlock(&measure_lock);
+
+	return status0 & TCPC_REG_STATUS0_COMP;
+}
+
+int tcpc_get_vbus_voltage(int port)
+{
+	int mdac = 0, i;
+
+	/*
+	 * Implement by comparing VBUS with MDAC reference voltage, and binary
+	 * search the value of MDAC.
+	 *
+	 * MDAC register has 6 bits, so we can simply search 1 bit per
+	 * iteration, from MSB to LSB.
+	 */
+	for (i = 5; i >= 0; i--) {
+		if (fusb302_compare_mdac(port, mdac | BIT(i)))
+			mdac |= BIT(i);
+	}
+
+	return (mdac + 1) * 420;
+}
+
 const struct tcpm_drv fusb302_tcpm_drv = {
 	.init			= &fusb302_tcpm_init,
+	.release		= &fusb302_tcpm_release,
 	.get_cc			= &fusb302_tcpm_get_cc,
 #ifdef CONFIG_USB_PD_VBUS_DETECT_TCPC
-	.get_vbus_level		= &fusb302_tcpm_get_vbus_level,
+	.check_vbus_level	= &fusb302_tcpm_check_vbus_level,
 #endif
 	.select_rp_value	= &fusb302_tcpm_select_rp_value,
 	.set_cc			= &fusb302_tcpm_set_cc,
 	.set_polarity		= &fusb302_tcpm_set_polarity,
+#ifdef CONFIG_USB_PD_DECODE_SOP
+	.sop_prime_enable	= &fusb302_tcpm_decode_sop_prime_enable,
+#endif
 	.set_vconn		= &fusb302_tcpm_set_vconn,
 	.set_msg_header		= &fusb302_tcpm_set_msg_header,
 	.set_rx_enable		= &fusb302_tcpm_set_rx_enable,
-	.get_message		= &fusb302_tcpm_get_message,
+	.get_message_raw	= &fusb302_tcpm_get_message_raw,
 	.transmit		= &fusb302_tcpm_transmit,
 	.tcpc_alert		= &fusb302_tcpc_alert,
+#ifdef CONFIG_USB_PD_TCPC_LOW_POWER
+	.enter_low_power_mode	= &fusb302_tcpm_enter_low_power_mode,
+#endif
 };
