@@ -1,4 +1,4 @@
-/* Copyright (c) 2014 The Chromium OS Authors. All rights reserved.
+/* Copyright 2014 The ChromiumOS Authors
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
@@ -16,6 +16,67 @@
 #include "2sysincludes.h"
 #include "2tpm_bootmode.h"
 
+vb2_error_t vb2api_inject_kernel_subkey(
+	struct vb2_context *ctx,
+	const uint8_t *kernel_packed_key_data,
+	uint32_t kernel_packed_key_data_size)
+{
+	struct vb2_shared_data *sd;
+	enum vb2_boot_mode *boot_mode;
+	struct vb2_workbuf wb;
+	struct vb2_packed_key *kernel_packed_key;
+	uint32_t kernel_packed_key_size;
+	void *dst_packed_key;
+
+	sd = vb2_get_sd(ctx);
+	vb2_workbuf_from_ctx(ctx, &wb);
+
+	/* Fully initialize the context and shared data. */
+	sd->flags = 0;
+	/* Not in recovery. */
+	sd->recovery_reason = 0;
+	/* FW not used. */
+	sd->last_fw_slot = VB2_FW_SLOT_A;
+	sd->last_fw_result = VB2_FW_RESULT_UNKNOWN;
+	sd->fw_slot = VB2_FW_SLOT_A;
+	sd->fw_version = 0;
+	sd->fw_version_secdata = 0;
+	/* Clear status field. */
+	sd->status = 0;
+	/* Invalid offset indicating GBB data is not available. */
+	sd->gbb_offset = 0;
+	sd->kernel_version = 0;
+	sd->kernel_version_secdata = 0;
+	ctx->flags = 0;
+	boot_mode = (enum vb2_boot_mode *)&ctx->boot_mode;
+	*boot_mode = VB2_BOOT_MODE_NORMAL;
+
+	/* Make sure passed buffer is big enough for the packed key. */
+	kernel_packed_key = (struct vb2_packed_key *)kernel_packed_key_data;
+	VB2_TRY(vb2_verify_packed_key_inside(kernel_packed_key_data,
+					     kernel_packed_key_data_size,
+					     kernel_packed_key));
+
+	/* Allocate space in the workbuf in which to copy the key. */
+	kernel_packed_key_size =
+		kernel_packed_key->key_offset + kernel_packed_key->key_size;
+	dst_packed_key = vb2_workbuf_alloc(&wb, kernel_packed_key_size);
+	if (!dst_packed_key)
+		return VB2_ERROR_WORKBUF_SMALL;
+
+	/* Copy the packed key data into the workbuf. */
+	memcpy(dst_packed_key, kernel_packed_key_data, kernel_packed_key_size);
+
+	/* Set the location of the kernel key data in the context. */
+	sd->kernel_key_offset = vb2_offset_of(sd, dst_packed_key);
+	sd->kernel_key_size = kernel_packed_key_size;
+
+	vb2_set_workbuf_used(ctx,
+			     sd->kernel_key_offset + kernel_packed_key_size);
+
+	return VB2_SUCCESS;
+}
+
 vb2_error_t vb2api_fw_phase1(struct vb2_context *ctx)
 {
 	vb2_error_t rv;
@@ -244,14 +305,10 @@ vb2_error_t vb2api_init_hash(struct vb2_context *ctx, uint32_t tag)
 
 	/*
 	 * Unpack the firmware data key to see which hashing algorithm we
-	 * should use.
-	 *
-	 * TODO: really, the firmware body should be hashed, and not signed,
-	 * because the signature we're checking is already signed as part of
-	 * the firmware preamble.  But until we can change the signing scripts,
-	 * we're stuck with a signature here instead of a hash.
+	 * should use. Zero body data size means, that signature contains
+	 * metadata hash, so vb2api_get_metadata_hash() should be used instead.
 	 */
-	if (!sd->data_key_size)
+	if (!sd->data_key_size || !pre->body_signature.data_size)
 		return VB2_ERROR_API_INIT_HASH_DATA_KEY;
 
 	VB2_TRY(vb2_unpack_key_buffer(&key,
@@ -308,8 +365,9 @@ vb2_error_t vb2api_check_hash_get_digest(struct vb2_context *ctx,
 		return VB2_ERROR_API_CHECK_HASH_TAG;
 
 	/*
-	 * The body signature is currently a *signature* of the body data, not
-	 * just its hash.  So we need to verify the signature.
+	 * In case of verifying a whole memory region the body signature
+	 * is a *signature* of the body data, not just its hash.
+	 * So we need to verify the signature.
 	 */
 
 	/* Unpack the data key */
@@ -372,3 +430,33 @@ union vb2_fw_boot_info vb2api_get_fw_boot_info(struct vb2_context *ctx)
 
 	return info;
 }
+
+vb2_error_t vb2api_get_metadata_hash(struct vb2_context *ctx,
+				     struct vb2_hash **hash_ptr_out)
+{
+	struct vb2_shared_data *sd = vb2_get_sd(ctx);
+	struct vb2_workbuf wb;
+	struct vb2_fw_preamble *pre;
+
+	vb2_workbuf_from_ctx(ctx, &wb);
+
+	if (!sd->preamble_size)
+		return VB2_ERROR_API_CHECK_HASH_PREAMBLE;
+	pre = vb2_member_of(sd, sd->preamble_offset);
+
+	/* Zero size of body signature indicates, that signature holds
+	   vb2_hash inside. */
+	if (pre->body_signature.data_size)
+		return VB2_ERROR_API_INIT_HASH_DATA_KEY;
+
+	struct vb2_hash *hash =
+		(struct vb2_hash *)vb2_signature_data(&pre->body_signature);
+	const uint32_t hsize = vb2_digest_size(hash->algo);
+	if (!hsize || pre->body_signature.sig_size <
+			      offsetof(struct vb2_hash, raw) + hsize)
+		return VB2_ERROR_API_CHECK_HASH_SIG_SIZE;
+
+	*hash_ptr_out = hash;
+
+	return VB2_SUCCESS;
+}
