@@ -1,4 +1,4 @@
-/* Copyright (c) 2014 The Chromium OS Authors. All rights reserved.
+/* Copyright 2014 The ChromiumOS Authors
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
@@ -43,10 +43,14 @@ uint32_t vb2api_get_firmware_size(struct vb2_context *ctx)
 {
 	struct vb2_shared_data *sd = vb2_get_sd(ctx);
 	if (!sd->preamble_size)
-		return 0;
+		VB2_DIE("Firmware preamble size is zero\n");
 
 	const struct vb2_fw_preamble *pre = (const struct vb2_fw_preamble *)
 		vb2_member_of(sd, sd->preamble_offset);
+
+	if (!pre->body_signature.data_size)
+		VB2_DIE("Firmware body data size in signature is zero\n");
+
 	return pre->body_signature.data_size;
 }
 
@@ -78,17 +82,31 @@ vb2_error_t vb2_read_gbb_header(struct vb2_context *ctx,
 	return VB2_SUCCESS;
 }
 
-test_mockable
-void vb2api_fail(struct vb2_context *ctx, uint8_t reason, uint8_t subcode)
+static void fail_impl(struct vb2_context *ctx,
+		      uint8_t reason, uint8_t subcode, bool previous_boot)
 {
 	struct vb2_shared_data *sd = vb2_get_sd(ctx);
+	uint32_t last_fw_slot, last_fw_result, fw_slot;
 
 	/* If NV data hasn't been initialized, initialize it now */
 	if (!(sd->status & VB2_SD_STATUS_NV_INIT))
 		vb2_nv_init(ctx);
 
+	/*
+	 * Donot overwrite any existing failure with a new failure reported
+	 * through vb2api_previous_boot_fail(). Existing failure might have
+	 * been set through vb2api_fail() in the previous boot and the new
+	 * failure can stand.
+	 */
+	if (previous_boot &&
+	    vb2_nv_get(ctx, VB2_NV_FW_RESULT) == VB2_FW_RESULT_FAILURE)
+		return;
+
 	/* See if we were far enough in the boot process to choose a slot */
-	if (sd->status & VB2_SD_STATUS_CHOSE_SLOT) {
+	if (previous_boot || (sd->status & VB2_SD_STATUS_CHOSE_SLOT)) {
+		last_fw_slot = vb2_nv_get(ctx, VB2_NV_FW_PREV_TRIED);
+		last_fw_result = vb2_nv_get(ctx, VB2_NV_FW_PREV_RESULT);
+		fw_slot = vb2_nv_get(ctx, VB2_NV_FW_TRIED);
 
 		/* Boot failed */
 		vb2_nv_set(ctx, VB2_NV_FW_RESULT, VB2_FW_RESULT_FAILURE);
@@ -96,20 +114,22 @@ void vb2api_fail(struct vb2_context *ctx, uint8_t reason, uint8_t subcode)
 		/* Use up remaining tries */
 		vb2_nv_set(ctx, VB2_NV_TRY_COUNT, 0);
 
-		/*
-		 * Try the other slot next time.  We'll alternate
-		 * between slots, which may help if one or both slots is
-		 * flaky.
-		 */
-		vb2_nv_set(ctx, VB2_NV_TRY_NEXT, 1 - sd->fw_slot);
+		if (!(ctx->flags & VB2_CONTEXT_SLOT_A_ONLY)) {
+			/*
+			 * Try the other slot next time.  We'll alternate
+			 * between slots, which may help if one or both slots
+			 * is flaky.
+			 */
+			vb2_nv_set(ctx, VB2_NV_TRY_NEXT, 1 - fw_slot);
 
-		/*
-		 * If we didn't try the other slot last boot, or we tried it
-		 * and it didn't fail, try it next boot.
-		 */
-		if (sd->last_fw_slot != 1 - sd->fw_slot ||
-		    sd->last_fw_result != VB2_FW_RESULT_FAILURE)
-			return;
+			/*
+			 * If we didn't try the other slot last boot, or we
+			 * tried it and it didn't fail, try it next boot.
+			 */
+			if (last_fw_slot != 1 - fw_slot ||
+			    last_fw_result != VB2_FW_RESULT_FAILURE)
+				return;
+		}
 	}
 
 	/*
@@ -128,6 +148,24 @@ void vb2api_fail(struct vb2_context *ctx, uint8_t reason, uint8_t subcode)
 	}
 }
 
+test_mockable
+void vb2api_fail(struct vb2_context *ctx, uint8_t reason, uint8_t subcode)
+{
+	fail_impl(ctx, reason, subcode, false);
+}
+
+test_mockable
+void vb2api_previous_boot_fail(struct vb2_context *ctx,
+			       uint8_t reason, uint8_t subcode)
+{
+	struct vb2_shared_data *sd = vb2_get_sd(ctx);
+
+	VB2_ASSERT(!(sd->status & VB2_SD_STATUS_NV_INIT) &&
+		   !(sd->status & VB2_SD_STATUS_CHOSE_SLOT));
+
+	fail_impl(ctx, reason, subcode, true);
+}
+
 void vb2_check_recovery(struct vb2_context *ctx)
 {
 	struct vb2_shared_data *sd = vb2_get_sd(ctx);
@@ -169,6 +207,7 @@ void vb2_check_recovery(struct vb2_context *ctx)
 	sd->status |= VB2_SD_STATUS_RECOVERY_DECIDED;
 }
 
+test_mockable
 vb2_error_t vb2_fw_init_gbb(struct vb2_context *ctx)
 {
 	struct vb2_shared_data *sd = vb2_get_sd(ctx);
@@ -195,6 +234,7 @@ vb2_error_t vb2_fw_init_gbb(struct vb2_context *ctx)
 	return VB2_SUCCESS;
 }
 
+test_mockable
 vb2_error_t vb2_check_dev_switch(struct vb2_context *ctx)
 {
 	struct vb2_shared_data *sd = vb2_get_sd(ctx);
@@ -295,6 +335,7 @@ vb2_error_t vb2_check_dev_switch(struct vb2_context *ctx)
 	return VB2_SUCCESS;
 }
 
+test_mockable
 vb2_error_t vb2_check_tpm_clear(struct vb2_context *ctx)
 {
 	vb2_error_t rv;
@@ -323,6 +364,7 @@ vb2_error_t vb2_check_tpm_clear(struct vb2_context *ctx)
 	return VB2_SUCCESS;
 }
 
+test_mockable
 vb2_error_t vb2_select_fw_slot(struct vb2_context *ctx)
 {
 	struct vb2_shared_data *sd = vb2_get_sd(ctx);
@@ -339,6 +381,10 @@ vb2_error_t vb2_select_fw_slot(struct vb2_context *ctx)
 	/* Clear result, since we don't know what will happen this boot */
 	vb2_nv_set(ctx, VB2_NV_FW_RESULT, VB2_FW_RESULT_UNKNOWN);
 
+	/* If there is only one slot, next try should always be slot A */
+	if (ctx->flags & VB2_CONTEXT_SLOT_A_ONLY)
+		vb2_nv_set(ctx, VB2_NV_TRY_NEXT, 0);
+
 	/* Get slot to try */
 	sd->fw_slot = vb2_nv_get(ctx, VB2_NV_TRY_NEXT);
 
@@ -348,6 +394,12 @@ vb2_error_t vb2_select_fw_slot(struct vb2_context *ctx)
 	if (sd->last_fw_result == VB2_FW_RESULT_TRYING &&
 	    sd->last_fw_slot == sd->fw_slot &&
 	    tries == 0) {
+		/*
+		 * If there is only RW A slot available, we have no other slot
+		 * to fall back to.
+		 */
+		if (ctx->flags & VB2_CONTEXT_SLOT_A_ONLY)
+			return VB2_ERROR_API_NEXT_SLOT_UNAVAILABLE;
 		/*
 		 * We used up our last try on the previous boot, so fall back
 		 * to the other slot this boot.
@@ -416,7 +468,7 @@ void vb2api_request_diagnostics(struct vb2_context *ctx) {
 	VB2_DEBUG("Diagnostics requested\n");
 }
 
-void vb2_clear_recovery(struct vb2_context *ctx)
+void vb2api_clear_recovery(struct vb2_context *ctx)
 {
 	struct vb2_shared_data *sd = vb2_get_sd(ctx);
 	uint32_t reason = vb2_nv_get(ctx, VB2_NV_RECOVERY_REQUEST);
@@ -754,10 +806,16 @@ void vb2_set_boot_mode(struct vb2_context *ctx)
 
 bool vb2api_hwcrypto_allowed(struct vb2_context *ctx)
 {
+	struct vb2_shared_data *sd = vb2_get_sd(ctx);
+
 	/* disable hwcrypto in recovery mode */
 	if (ctx->flags & VB2_CONTEXT_RECOVERY_MODE)
 		return 0;
 
+	/* disable hwcrypto if secdata isn't initialized */
+	if (!(sd->status & VB2_SD_STATUS_SECDATA_KERNEL_INIT))
+		return 0;
+
 	/* enable hwcrypto only if RW firmware set the flag */
 	return vb2_secdata_kernel_get(ctx, VB2_SECDATA_KERNEL_FLAGS) &
 		VB2_SECDATA_KERNEL_FLAG_HWCRYPTO_ALLOWED;
