@@ -1,4 +1,4 @@
-/* Copyright (c) 2013 The Chromium OS Authors. All rights reserved.
+/* Copyright 2013 The ChromiumOS Authors
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
@@ -150,19 +150,37 @@ enum vb2_pcr_digest {
  *	Verify the hash of each section of code/data you need to boot the RW
  *	firmware.  For each section:
  *
- *		Call vb2_init_hash() to see if the hash exists.
+ *	1) Normal verification:
  *
- *		Load the data for the section.  Call vb2_extend_hash() on the
+ *		Call vb2api_init_hash() to see if the hash exists.
+ *
+ *		Load the data for the section.  Call vb2api_extend_hash() on the
  *		data as you load it.  You can load it all at once and make one
  *		call, or load and hash-extend a block at a time.
  *
- *		Call vb2_check_hash() to see if the hash is valid.
+ *		Call vb2api_check_hash() to see if the hash is valid.
  *
  *			If it is valid, you may use the data and/or execute
  *			code from that section.
  *
  *			If the hash was invalid, you must reboot.
  *
+ *	2) Verification with CBFS integration:
+ *
+ *		Call vb2api_get_metadata_hash() to get hash of CBFS metadata.
+ *
+ *		Initialize CBFS using stored hash as correct metadata hash.
+ *
+ *			If CBFS initialization fails because of metadata hash
+ *			mismatch, you must reboot.
+ *
+ *			If CBFS initialization succeeds, you may use the data
+ *			and/or execute code from that section.
+ *			IMPORTANT: Be aware, that to have full section
+ *			verification, the CBFS_VERIFICATION has to be enabled.
+ *			Initialization of CBFS volume only checks hash of files
+ *			metadata, not their contents!
+ *
  * At this point, firmware verification is done, and vb2_context contains the
  * kernel key needed to verify the kernel.  That context should be preserved
  * and passed on to kernel selection.  The kernel selection process may be
@@ -385,6 +403,42 @@ vb2_error_t vb2api_secdata_fwmp_check(struct vb2_context *ctx, uint8_t *size);
  */
 void vb2api_fail(struct vb2_context *ctx, uint8_t reason, uint8_t subcode);
 
+/**
+ * Report firmware failure from previous boot to vboot.
+ *
+ * This function can only be called before vb2api_fw_phase1 (nvdata is
+ * initialized). Otherwise an assert is raised. This function is required to be
+ * called in the following environment:
+ * - Context has to be initialized using vb2api_init
+ * - NV data has to be read into context
+ * - Secdata may or may not have been read
+ * - vb2api_fw_phase1 must not have been called.
+ *
+ * If the other slot is not known bad then try the other firmware slot.
+ * If both the slots are known bad, then request recovery.
+ *
+ * @param reason	Recovery reason
+ * @param subcode	Recovery subcode
+ */
+void vb2api_previous_boot_fail(struct vb2_context *ctx,
+			       uint8_t reason, uint8_t subcode);
+
+/**
+ * Entry point for setting up a context that can only load and verify a kernel.
+ *
+ * The only allowed usage is to call vb2api_init, then this entry point,
+ * then vb2api_load_kernel.
+ *
+ * @param ctx				Vboot context
+ * @param kernel_packed_key_data	Packed public key for kernel
+ *					verification
+ * @param kernel_packed_key_data_size	Size in bytes of kernel_packed_key_data
+ * @return VB2_SUCCESS, or error code on error.
+ */
+vb2_error_t vb2api_inject_kernel_subkey(struct vb2_context *ctx,
+					const uint8_t *kernel_packed_key_data,
+					uint32_t kernel_packed_key_data_size);
+
 /**
  * Firmware selection, phase 1.
  *
@@ -426,6 +480,9 @@ vb2_error_t vb2api_fw_phase3(struct vb2_context *ctx);
 
 /**
  * Initialize hashing data for the specified tag.
+ * This function is not legal when running from a coreboot image that has
+ * CONFIG_VBOOT_CBFS_INTEGRATION=y set. In that case, vb2api_get_metadata_hash()
+ * must be used instead.
  *
  * @param ctx		Vboot context
  * @param tag		Tag to start hashing (enum vb2_hash_tag)
@@ -467,6 +524,23 @@ vb2_error_t vb2api_check_hash_get_digest(struct vb2_context *ctx,
 					 void *digest_out,
 					 uint32_t digest_out_size);
 
+/**
+ * Get pointer to metadata hash from body signature in preamble.
+ * Body signature data size has to be zero to indicate that it contains
+ * metadata hash. This is only legal to call after vb2api_fw_phase3() has
+ * returned successfully, and will return with error otherwise.
+ * This function is only legal to call from coreboot with
+ * CONFIG_VBOOT_CBFS_INTEGRATION=y. `futility sign` will automatically detect
+ * the presence of that option in an image and prepare the correct kind
+ * of signature.
+ *
+ * @param ctx			Vboot context
+ * @param hash_ptr_out		pointer to output hash to
+ * @return VB2_SUCCESS, or error code on error.
+ */
+vb2_error_t vb2api_get_metadata_hash(struct vb2_context *ctx,
+				     struct vb2_hash **hash_ptr_out);
+
 /**
  * Get a PCR digest
  *
@@ -526,8 +600,8 @@ struct vb2_kernel_params {
 	vb2ex_disk_handle_t disk_handle;
 	/* Partition number on disk to boot (1...M). */
 	uint32_t partition_number;
-	/* Address of bootloader image in RAM. */
-	uint64_t bootloader_address;
+	/* Offset of bootloader image from `kernel_buffer` address. */
+	uint64_t bootloader_offset;
 	/* Size of bootloader image in bytes. */
 	uint32_t bootloader_size;
 	/* UniquePartitionGuid for boot partition. */
@@ -710,6 +784,8 @@ vb2_gbb_flags_t vb2api_gbb_get_flags(struct vb2_context *ctx);
 /**
  * Get the size of the signed firmware body. This is only legal to call after
  * vb2api_fw_phase3() has returned successfully, and will return 0 otherwise.
+ * It will also return 0 when body signature contains metadata hash instead
+ * of body hash.
  *
  * @param ctx		Vboot context
  *
@@ -1236,13 +1312,11 @@ vb2_error_t vb2ex_ec_update_image(enum vb2_firmware_selection select);
 
 /**
  * Lock the EC code to prevent updates until the EC is rebooted.
- * Subsequent calls to vb2ex_ec_update_image() with the same region this
- * boot will fail.
+ * Subsequent calls to vb2ex_ec_update_image() during boot will fail.
  *
- * @param select	Image to get expected hash for (RO or RW).
  * @return VB2_SUCCESS, or error code on error.
  */
-vb2_error_t vb2ex_ec_protect(enum vb2_firmware_selection select);
+vb2_error_t vb2ex_ec_protect(void);
 
 /**
  * Perform EC post-verification / updating / jumping actions.
@@ -1290,12 +1364,8 @@ char *vb2api_get_debug_info(struct vb2_context *ctx);
  */
 uint32_t vb2ex_mtime(void);
 
-/**
- * Delay for at least the specified number of milliseconds.
- *
- * @param msec			Duration in milliseconds.
- */
-void vb2ex_msleep(uint32_t msec);
+/*****************************************************************************/
+/* Firmware slot information. */
 
 union vb2_fw_boot_info {
 	uint8_t raw[4];
@@ -1323,4 +1393,26 @@ union vb2_fw_boot_info {
  */
 union vb2_fw_boot_info vb2api_get_fw_boot_info(struct vb2_context *ctx);
 
+/**
+ * Clear recovery request appropriately.
+ *
+ * To avoid the recovery request "sticking" and the user being in a permanent
+ * recovery loop, the recovery request must be cleared and committed to nvdata.
+ * Note that this should be done at some point after we are certain the system
+ * does not require any reboots for non-vboot-related reasons (e.g. FSP
+ * initialization), and before triggering a reboot to exit a transient recovery
+ * mode (e.g. memory retraining request).
+ *
+ * In BROKEN cases, the recovery reason will be stowed away as subcode, to be
+ * retrieved after the user reboots in manual recovery.  In manual recovery,
+ * subcode will be left alone to keep available for subsequent manual recovery
+ * requests, or for accessing from userspace on the next boot.
+ *
+ * This function modifies nvdata in vb2_context, but the caller is still
+ * expected to call vb2_commit_data.
+ *
+ * @param ctx		Vboot context
+ */
+void vb2api_clear_recovery(struct vb2_context *ctx);
+
 #endif  /* VBOOT_REFERENCE_2API_H_ */
