@@ -1,5 +1,4 @@
-/*
- * Copyright 2021 The Chromium OS Authors. All rights reserved.
+/* Copyright 2021 The ChromiumOS Authors
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -18,6 +17,7 @@
 #include "gsc_ro.h"
 #include "host_key21.h"
 #include "host_keyblock.h"
+#include "host_misc.h"
 #include "host_signature.h"
 
 /*
@@ -54,18 +54,20 @@
 /* Command line options processing support. */
 enum no_short_opts {
 	OPT_OUTFILE = 1000,
+	OPT_RO_GSCVD_FILE = 1001,
 };
 
 static const struct option long_opts[] = {
 	/* name       hasarg *flag  val */
-	{"outfile",       1, NULL, OPT_OUTFILE},
-	{"ranges",        1, NULL, 'R'},
 	{"add_gbb",       0, NULL, 'G'},
 	{"board_id",      1, NULL, 'b'},
-	{"root_pub_key",  1, NULL, 'r'},
-	{"keyblock",      1, NULL, 'k'},
-	{"platform_priv", 1, NULL, 'p'},
 	{"help",          0, NULL, 'h'},
+	{"keyblock",      1, NULL, 'k'},
+	{"outfile",       1, NULL, OPT_OUTFILE},
+	{"platform_priv", 1, NULL, 'p'},
+	{"ranges",        1, NULL, 'R'},
+	{"gscvd_out",     1, NULL, OPT_RO_GSCVD_FILE},
+	{"root_pub_key",  1, NULL, 'r'},
 	{}
 };
 
@@ -74,20 +76,18 @@ static const char *short_opts = "R:Gb:hk:p:r:";
 static const char usage[] =
 	"\n"
 	"This utility creates an RO verification space in the Chrome OS AP\n"
-	"firmware image or allows to validate a previously prepared image\n"
-	"containing the RO verification space.\n\n"
-	"Usage: " MYNAME " gscvd PARAMS <AP FIRMWARE FILE> [<root key hash>]\n"
-	"\n\nCreation of RO Verification space:\n\n"
+	"firmware image, allows to validate a previously prepared image\n"
+	"containing the RO verification space, and prints out the hash of the\n"
+	"payload of the root public key.\n\n"
+	"Create a new GSCVD from scratch:\n"
+	"  "MYNAME" gscvd -R <ranges> PARAMS <firmware image>\n\n"
+	"Re-sign an existing GSCVD with new keys, preserving ranges:\n"
+	"  "MYNAME" gscvd PARAMS <firmware image>\n\n"
+	"Validate an existing GSCVD with given root key hash:\n"
+	"  "MYNAME" gscvd <firmware image> [<root key hash in hex>]\n\n"
+	"Print the hash of a public root key:\n"
+	"  "MYNAME" gscvd -r <root key .vpubk file>\n\n"
 	"Required PARAMS:\n"
-	"  -R|--ranges        STRING        Comma separated colon delimited\n"
-	"                                     hex tuples <offset>:<size>, the\n"
-	"                                     areas of the RO covered by the\n"
-	"                                     signature\n"
-	"  -G|--add_gbb                     Add the `GBB` FMAP section to the\n"
-	"                                     ranges covered by the signature.\n"
-	"                                     This option takes special care\n"
-	"                                     to exclude the HWID (and its\n"
-	"                                     digest) from this range.\n"
 	"  -b|--board_id  <string|hex>      The Board ID of the board for\n"
 	"                                     which the image is signed.\n"
 	"                                     Can be passed as a 4-letter\n"
@@ -102,14 +102,22 @@ static const char usage[] =
 	"                                     format, used for signing RO\n"
 	"                                     verification data\n"
 	"Optional PARAMS:\n"
+	"  -G|--add_gbb                     Add the `GBB` FMAP section to the\n"
+	"                                     ranges covered by the signature.\n"
+	"                                     This option takes special care\n"
+	"                                     to exclude the HWID (and its\n"
+	"                                     digest) from this range.\n"
+	"  -R|--ranges        STRING        Comma separated colon delimited\n"
+	"                                     hex tuples <offset>:<size>, the\n"
+	"                                     areas of the RO covered by the\n"
+	"                                     signature, if omitted the\n"
+	"                                     ranges are expected to be\n"
+	"                                     present in the GSCVD section\n"
+	"                                     of the input file\n"
 	"  [--outfile]        OUTFILE       Output firmware image containing\n"
 	"                                     RO verification information\n"
-	"\n\n"
-	"Validation of RO Verification space:\n\n"
-	"   The only required parameter is <AP FIRMWARE FILE>, if optional\n"
-	"   <root key hash> is given, it is compared to the hash\n"
-	"   of the root key found in <AP_FIRMWARE_FILE>.\n"
-	"\n\n"
+	"  [--gscvd_out]      GSCVD_FILE    A binary blob containing just the\n"
+	"                                     unpadded RO_GSCVD section\n"
 	"  -h|--help                        Print this message\n\n";
 
 /* Structure helping to keep track of the file mapped into memory. */
@@ -118,12 +126,16 @@ struct file_buf {
 	uint8_t *data;
 	int fd;
 	FmapAreaHeader *ro_gscvd;
+	/* Cached GBB information. */
+	const FmapAreaHeader *gbb_area;
+	uint32_t gbb_maxlen;
 };
 
 /*
  * Max number of RO ranges to cover. 32 is more than enough, this must be kept
- * in sync with APRO_MAX_NUM_RANGES declaration in
- * common/ap_ro_integrity_check.c in the Cr50 tree.
+ * in sync with
+ * - AP_RO_MAX_NUM_RANGES in cr50/common/ap_ro_integrity_check.c
+ * - MAX_RO_RANGES in ti50/common/capsules/src/ap_ro_verification/gscvd.rs
  */
 #define MAX_RANGES 32
 
@@ -151,6 +163,8 @@ struct gscvd_ro_ranges {
 static int load_ap_firmware(const char *file_name, struct file_buf *file,
 			int mode)
 {
+	memset(file, 0, sizeof(*file));
+
 	if (futil_open_and_map_file(file_name, &file->fd, mode, &file->data,
 				    &file->len))
 		return 1;
@@ -166,6 +180,43 @@ static int load_ap_firmware(const char *file_name, struct file_buf *file,
 		return 1;
 	}
 
+	/*
+	 * Try finding FMAP gbb area and validating the GBB. It's not a
+	 * failure if GBB is not found, it might not be required after all.
+	 */
+	FmapAreaHeader *area;
+	while (fmap_find_by_name(file->data, file->len, NULL, "GBB", &area)) {
+		struct vb2_gbb_header *gbb;
+		uint32_t maxlen;
+
+		gbb = (void *)(file->data + area->area_offset);
+
+		if (!futil_valid_gbb_header(gbb, area->area_size, &maxlen)) {
+			ERROR("GBB is invalid.\n");
+			break;
+		}
+
+		/*
+		 * This implementation relies on the fact that no meaningful
+		 * fields come after the `hwid_digest` field in the header. If
+		 * we ever make new GBB versions that add more fields, the
+		 * code below and in add_gbb() needs to be adapted. Older
+		 * versions than 1.2 or GBBs with a bmpblk are not expected
+		 * with GSCVD images.
+		 */
+		if (gbb->major_version != 1 || gbb->minor_version != 2 ||
+		    gbb->bmpfv_size != 0) {
+			ERROR("Unsupported GBB version.\n");
+			break;
+		}
+
+
+		file->gbb_area = area;
+		file->gbb_maxlen = maxlen;
+
+		break;
+	}
+
 	return 0;
 }
 
@@ -362,34 +413,13 @@ static int parse_ranges(const char *input, struct gscvd_ro_ranges *output)
  */
 static int add_gbb(struct gscvd_ro_ranges *ranges, const struct file_buf *file)
 {
-	FmapAreaHeader *area;
-
-	if (!fmap_find_by_name(file->data, file->len, NULL, "GBB", &area)) {
+	if (!file->gbb_area) {
 		ERROR("Could not find a GBB area in the FMAP.\n");
 		return 1;
 	}
 
-	struct vb2_gbb_header *gbb = (void *)file->data + area->area_offset;
-	uint32_t maxlen;
-
-	if (!futil_valid_gbb_header(gbb, area->area_size, &maxlen)) {
-		ERROR("GBB is invalid.\n");
-		return 1;
-	}
-
-	/*
-	 * This implementation relies on the fact that no meaningful fields come
-	 * after the `hwid_digest` field in the header. If we ever make new GBB
-	 * versions that add more fields, the code below needs to be adapted.
-	 * Older versions than 1.2 or GBBs with a bmpblk are not expected with
-	 * GSCVD images.
-	 */
-	if (gbb->major_version != 1 || gbb->minor_version != 2 ||
-	    gbb->bmpfv_size != 0) {
-		ERROR("Unsupported GBB version.\n");
-		return 1;
-	}
-
+	const struct vb2_gbb_header *gbb = (void *)(file->data +
+					      file->gbb_area->area_offset);
 	uint32_t lower_key_offset = VB2_MIN(gbb->rootkey_offset,
 					    gbb->recovery_key_offset);
 	if (gbb->hwid_offset > lower_key_offset) {
@@ -402,19 +432,85 @@ static int add_gbb(struct gscvd_ro_ranges *ranges, const struct file_buf *file)
 		return 1;
 	}
 
-	ranges->ranges[ranges->range_count].offset = area->area_offset;
+	ranges->ranges[ranges->range_count].offset =
+		file->gbb_area->area_offset;
 	ranges->ranges[ranges->range_count].size =
-				offsetof(struct vb2_gbb_header, hwid_digest);
+		offsetof(struct vb2_gbb_header, hwid_digest);
 	ranges->range_count++;
 
-	ranges->ranges[ranges->range_count].offset = area->area_offset +
-						     lower_key_offset;
-	ranges->ranges[ranges->range_count].size = maxlen - lower_key_offset;
+	ranges->ranges[ranges->range_count].offset =
+		file->gbb_area->area_offset + lower_key_offset;
+	ranges->ranges[ranges->range_count].size =
+		file->gbb_maxlen - lower_key_offset;
 	ranges->range_count++;
 
 	return 0;
 }
 
+/**
+ * Extend AP RO hash digest with data from an address range.
+ *
+ * If the flags_offset value is non zero and happens to fall into the passed
+ * in range, do not read values from flash in the flags_offset..+flags_size
+ * range, instead feed zeros to the hashing function.
+ *
+ * NOTE that flags are expected to fully fit into the range, cases of overlap
+ * are not supported.
+ *
+ * @param ap_firmware_file  pointer to the AP firmware file layout descriptor
+ * @param dc  pointer to the hash calculating context
+ * @param offset  offset of the beginning of the range in AP SPI flash
+ * @param size  size of the range
+ * @param flags_offset  if nonzero - offset of the GBB flags field in
+ *			AP SPI flash
+ *
+ * @return VB2_SUCCESS or digest extension error, if any.
+ */
+static vb2_error_t extend_digest(const struct file_buf *ap_firmware_file,
+				 struct vb2_digest_context *dc,
+				 uint32_t offset,
+				 uint32_t size,
+				 uint32_t flags_offset)
+{
+	/* Define it as array to simplify calling vb2_digest_extend() below. */
+	const uint8_t flags[sizeof(vb2_gbb_flags_t)] = {0};
+
+	VB2_DEBUG("%s: %#x..+%#x\n", __func__, offset, size);
+
+	if (flags_offset &&
+	    (flags_offset >= offset) &&
+	    (flags_offset < (offset + size))) {
+		uint32_t flags_size;
+		vb2_error_t rv;
+
+		/*
+		 * This range includes GBB flags, which need to be zeroized.
+		 *
+		 * First get the hash of up to the flags.
+		 */
+		rv = vb2_digest_extend(dc, ap_firmware_file->data + offset,
+				       flags_offset - offset);
+		if (rv != VB2_SUCCESS)
+			return rv;
+
+		size -= flags_offset - offset;
+		offset = flags_offset;
+
+		/* Now hash the flag space, maybe partially. */
+		flags_size = VB2_MIN(size, sizeof(flags));
+		rv = vb2_digest_extend(dc, flags, flags_size);
+		if (rv != VB2_SUCCESS)
+			return rv;
+
+		/* Update size and offset to cover the rest of the range. */
+		size -= flags_size;
+
+		offset += flags_size;
+	}
+
+	return vb2_digest_extend(dc,ap_firmware_file->data + offset, size);
+}
+
 /**
  * Calculate hash of the RO ranges.
  *
@@ -425,16 +521,23 @@ static int add_gbb(struct gscvd_ro_ranges *ranges, const struct file_buf *file)
  * @param digest  memory to copy the calculated hash to
  * @param digest_ size requested size of the digest, padded with zeros if the
  *	          SHA digest size is smaller than digest_size
+ * @param override_gbb_flags  if true, replace GBB flags value with zero
  *
  * @return zero on success, -1 on failure.
  */
 static int calculate_ranges_digest(const struct file_buf *ap_firmware_file,
 				   const struct gscvd_ro_ranges *ranges,
 				   enum vb2_hash_algorithm hash_alg,
-				   void *digest, size_t digest_size)
+				   void *digest, size_t digest_size,
+				   bool override_gbb_flags)
 {
 	struct vb2_digest_context dc;
 	size_t i;
+	uint32_t flags_offset = 0;
+
+	if (override_gbb_flags && ap_firmware_file->gbb_area)
+		flags_offset = offsetof(struct vb2_gbb_header, flags) +
+			ap_firmware_file->gbb_area->area_offset;
 
 	/* Calculate the ranges digest. */
 	if (vb2_digest_init(&dc, false, hash_alg, 0) != VB2_SUCCESS) {
@@ -443,10 +546,10 @@ static int calculate_ranges_digest(const struct file_buf *ap_firmware_file,
 	}
 
 	for (i = 0; i < ranges->range_count; i++) {
-		if (vb2_digest_extend(&dc,
-				      ap_firmware_file->data +
-					      ranges->ranges[i].offset,
-				      ranges->ranges[i].size) != VB2_SUCCESS) {
+		if (extend_digest(ap_firmware_file, &dc,
+				  ranges->ranges[i].offset,
+				  ranges->ranges[i].size,
+				  flags_offset) != VB2_SUCCESS) {
 			ERROR("Failed to extend digest!\n");
 			return -1;
 		}
@@ -517,7 +620,8 @@ struct gsc_verification_data *create_gvd(struct file_buf *ap_firmware_file,
 
 	if (calculate_ranges_digest(ap_firmware_file, ranges, gvd->hash_alg,
 				    gvd->ranges_digest,
-				    sizeof(gvd->ranges_digest))) {
+				    sizeof(gvd->ranges_digest),
+				    true)) {
 		free(gvd);
 		return NULL;
 	}
@@ -568,17 +672,19 @@ struct gsc_verification_data *create_gvd(struct file_buf *ap_firmware_file,
  *
  * All trust chain components have been verified, AP RO sections digest
  * calculated, and GVD signature created; put it all together in the dedicated
- * FMAP area.
+ * FMAP area and save in a binary blob if requested.
  *
  * @param ap_firmware_file  pointer to the AP firmware file layout descriptor
  * @param gvd  pointer to the GVD header
  * @param keyblock  pointer to the keyblock container
- *
+ * @param gscvd_file_name  if not NULL the name of the file to save the
+ *			RO_GSCVD section in.
  * @return zero on success, -1 on failure
  */
 static int fill_gvd_area(struct file_buf *ap_firmware_file,
 			 struct gsc_verification_data *gvd,
-			 struct vb2_keyblock *keyblock)
+			 struct vb2_keyblock *keyblock,
+			 const char *gscvd_file_name)
 {
 	size_t total;
 	uint8_t *cursor;
@@ -602,6 +708,11 @@ static int fill_gvd_area(struct file_buf *ap_firmware_file,
 	/* Keyblock, size includes everything. */
 	memcpy(cursor, keyblock, keyblock->keyblock_size);
 
+	if (gscvd_file_name) {
+		if (vb2_write_file(gscvd_file_name, cursor - gvd->size,
+				   total) != VB2_SUCCESS)
+			return -1;
+	}
 	return 0;
 }
 
@@ -844,6 +955,69 @@ static int validate_gvd_signature(struct gsc_verification_data *gvd,
 	return rv;
 }
 
+/*
+ * Try retrieving GVD ranges from the passed in AP firmware file.
+ *
+ * The passed in ranges structure is set to the set of ranges retrieved from
+ * the firmware file, if any.
+ */
+static void try_retrieving_ranges_from_the_image(const char *file_name,
+						 struct gscvd_ro_ranges *ranges,
+						 struct file_buf
+						 *ap_firmware_file)
+{
+	struct gsc_verification_data *gvd;
+	size_t i;
+
+	ranges->range_count = 0;
+
+	/* Look for ranges in GVD and copy them if found. */
+	gvd = (struct gsc_verification_data
+	       *)(ap_firmware_file->data +
+		  ap_firmware_file->ro_gscvd->area_offset);
+
+	if (validate_gvd(gvd, ap_firmware_file))
+		return;
+
+	if (copy_ranges(ap_firmware_file, gvd, ranges))
+		return;
+
+	if (!ranges->range_count) {
+		printf("No ranges found in the input file\n");
+	} else {
+		printf("Will re-sign the following %zd ranges:\n",
+		       ranges->range_count);
+		for (i = 0; i < ranges->range_count; i++) {
+			printf("%08x:%08x\n",
+			       ranges->ranges[i].offset,
+			       ranges->ranges[i].size);
+		}
+	}
+}
+
+/*
+ * Calculate ranges digest and compare it with the value stored in gvd, with
+ * or without ignoring GBB flags, as requested by the caller.
+ *
+ * @return zero on success, nonzero on failure.
+ */
+static int validate_digest(struct file_buf *ap_firmware_file,
+			   const struct gscvd_ro_ranges *ranges,
+			   const struct gsc_verification_data *gvd,
+			   bool override_gbb_flags)
+{
+	uint8_t digest[sizeof(gvd->ranges_digest)];
+
+	if (calculate_ranges_digest(ap_firmware_file, ranges,
+				    gvd->hash_alg, digest,
+				    sizeof(digest),
+				    override_gbb_flags)) {
+		return 1;
+	}
+
+	return memcmp(digest, gvd->ranges_digest, sizeof(digest));
+}
+
 /*
  * Validate GVD of the passed in AP firmware file and possibly the root key hash
  *
@@ -852,7 +1026,7 @@ static int validate_gvd_signature(struct gsc_verification_data *gvd,
  * hash of the root public key included in the RO_GSCVD area of the AP
  * firmware file.
  *
- * @return zero on success, -1 on failure.
+ * @return zero on success, nonzero on failure.
  */
 static int validate_gscvd(int argc, char *argv[])
 {
@@ -861,7 +1035,6 @@ static int validate_gscvd(int argc, char *argv[])
 	struct gscvd_ro_ranges ranges;
 	struct gsc_verification_data *gvd;
 	const char *file_name;
-	uint8_t digest[sizeof(gvd->ranges_digest)];
 	struct vb2_hash root_key_digest = { .algo = VB2_HASH_SHA256 };
 
 	/* Guaranteed to be available. */
@@ -891,14 +1064,15 @@ static int validate_gscvd(int argc, char *argv[])
 		if (copy_ranges(&ap_firmware_file, gvd, &ranges))
 			break;
 
-		if (calculate_ranges_digest(&ap_firmware_file, &ranges,
-					    gvd->hash_alg, digest,
-					    sizeof(digest)))
-			break;
-
-		if (memcmp(digest, gvd->ranges_digest, sizeof(digest))) {
-			ERROR("Ranges digest mismatch\n");
-			break;
+		/* First try validating without ignoring GBB flags. */
+		if (validate_digest(&ap_firmware_file, &ranges, gvd, false)) {
+			/* It failed, maybe GBB flags are not cleared yet. */
+			if (validate_digest(&ap_firmware_file, &ranges,
+					    gvd, true)) {
+				ERROR("Ranges digest mismatch\n");
+				break;
+			}
+			WARN("Ranges digest matches with zeroed GBB flags\n");
 		}
 
 		/* Find the keyblock. */
@@ -980,6 +1154,7 @@ static int do_gscvd(int argc, char *argv[])
 	struct gsc_verification_data *gvd = NULL;
 	struct file_buf ap_firmware_file = { .fd = -1 };
 	uint32_t board_id = UINT32_MAX;
+	char *ro_gscvd_file = NULL;
 	int rv = 0;
 
 	ranges.range_count = 0;
@@ -990,6 +1165,9 @@ static int do_gscvd(int argc, char *argv[])
 		case OPT_OUTFILE:
 			outfile = optarg;
 			break;
+		case OPT_RO_GSCVD_FILE:
+			ro_gscvd_file = optarg;
+			break;
 		case 'R':
 			if (parse_ranges(optarg, &ranges)) {
 				ERROR("Could not parse ranges\n");
@@ -1065,13 +1243,13 @@ static int do_gscvd(int argc, char *argv[])
 		}
 	}
 
-	if ((optind == 1) && (argc > 1))
+	if ((optind == 1) && (argc > 1)) {
+		if (ro_gscvd_file) {
+			ERROR("Unexpected --gscvd_out in command line\n");
+			goto usage_out;
+		}
 		/* This must be a validation request. */
 		return validate_gscvd(argc - 1, argv + 1);
-
-	if (optind != (argc - 1)) {
-		ERROR("Misformatted command line\n");
-		goto usage_out;
 	}
 
 	if (errorcount) /* Error message(s) should have been printed by now. */
@@ -1080,8 +1258,26 @@ static int do_gscvd(int argc, char *argv[])
 	if (!root_pubk) {
 		ERROR("Missing --root_pub_key argument\n");
 		goto usage_out;
+	} else if (argc == 3) {
+		if (ro_gscvd_file) {
+			ERROR("Unexpected --gscvd_out in command line\n");
+			goto usage_out;
+		}
+		/*
+		 * This is a request to print out the hash of the root pub key
+		 * payload.
+		 */
+		dump_pubk_hash(root_pubk);
+		return 0;
 	}
 
+	if (optind != (argc - 1)) {
+		ERROR("Misformatted command line\n");
+		goto usage_out;
+	}
+
+	infile = argv[optind];
+
 	if (!kblock) {
 		ERROR("Missing --keyblock argument\n");
 		goto usage_out;
@@ -1097,15 +1293,9 @@ static int do_gscvd(int argc, char *argv[])
 		goto usage_out;
 	}
 
-	if (!ranges.range_count && !do_gbb) {
-		ERROR("Missing --ranges argument\n");
-		goto usage_out;
-	}
-
-	infile = argv[optind];
-
 	if (outfile) {
-		futil_copy_file_or_die(infile, outfile);
+		if (futil_copy_file(infile, outfile) < 0)
+			exit(1);
 		work_file = outfile;
 	} else {
 		work_file = infile;
@@ -1123,6 +1313,16 @@ static int do_gscvd(int argc, char *argv[])
 		if (load_ap_firmware(work_file, &ap_firmware_file, FILE_RW))
 			break;
 
+		if (!ranges.range_count)
+			try_retrieving_ranges_from_the_image(infile,
+							     &ranges,
+							     &ap_firmware_file);
+
+		if (!ranges.range_count && !do_gbb) {
+			ERROR("Missing --ranges argument and no ranges in the input file\n");
+			break;
+		}
+
 		if (do_gbb && add_gbb(&ranges, &ap_firmware_file))
 			break;
 
@@ -1134,11 +1334,10 @@ static int do_gscvd(int argc, char *argv[])
 		if (!gvd)
 			break;
 
-		if (fill_gvd_area(&ap_firmware_file, gvd, kblock))
+		if (fill_gvd_area(&ap_firmware_file, gvd,
+				  kblock, ro_gscvd_file))
 			break;
 
-		dump_pubk_hash(root_pubk);
-
 		rv = 0;
 	} while (false);
 
