@@ -1,4 +1,4 @@
-/* Copyright 2018 The Chromium OS Authors. All rights reserved.
+/* Copyright 2018 The ChromiumOS Authors
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
@@ -21,12 +21,16 @@ struct quirks_record {
 	const char * const quirks;
 };
 
+/*
+ * The 'match by firmware name' is now deprecated. Please do not add any
+ * new records below. We now support reading quirks from CBFS, which is
+ * easier and more reliable. To do that, create a text file 'updater_quirks'
+ * and install to the CBFS.
+ *
+ * Examples: CL:*3365287, CL:*3351831, CL:*4441527
+ */
 static const struct quirks_record quirks_records[] = {
-	{ .match = "Google_Whirlwind.", .quirks = "enlarge_image" },
-	{ .match = "Google_Arkham.", .quirks = "enlarge_image" },
-	{ .match = "Google_Storm.", .quirks = "enlarge_image" },
-	{ .match = "Google_Gale.", .quirks = "enlarge_image" },
-
+	/* reference design: 'glados' */
 	{ .match = "Google_Chell.", .quirks = "unlock_me_for_update" },
 	{ .match = "Google_Lars.", .quirks = "unlock_me_for_update" },
 	{ .match = "Google_Sentry.", .quirks = "unlock_me_for_update" },
@@ -37,6 +41,7 @@ static const struct quirks_record quirks_records[] = {
 	{ .match = "Google_Eve.",
 	  .quirks = "unlock_me_for_update,eve_smm_store" },
 
+	/* reference design: sarien/drallion (wilco) */
 	{ .match = "Google_Sarien.",
 	  .quirks = "unlock_wilco_me_for_update" },
 	{ .match = "Google_Arcada.",
@@ -48,18 +53,12 @@ static const struct quirks_record quirks_records[] = {
 	{ .match = "Google_Scarlet.", .quirks = "min_platform_version=1" },
 	{ .match = "Google_Trogdor.", .quirks = "min_platform_version=2" },
 
-        /* Legacy white label units. */
-        { .match = "Google_Enguarde.", .quirks = "allow_empty_wltag" },
-        { .match = "Google_Expresso.", .quirks = "allow_empty_wltag" },
-        { .match = "Google_Hana.", .quirks = "allow_empty_wltag" },
-        { .match = "Google_Veyron_Jaq.", .quirks = "allow_empty_wltag" },
-        { .match = "Google_Veyron_Jerry.", .quirks = "allow_empty_wltag" },
-        { .match = "Google_Veyron_Mighty.", .quirks = "allow_empty_wltag" },
-        { .match = "Google_Reks.", .quirks = "allow_empty_wltag" },
-        { .match = "Google_Relm.", .quirks = "allow_empty_wltag" },
-        { .match = "Google_Wizpig.", .quirks = "allow_empty_wltag" },
+        /* Legacy custom label units. */
+	/* reference design: oak */
+	{ .match = "Google_Hana.", .quirks = "allow_empty_custom_label_tag" },
 
-        { .match = "Google_Phaser.", .quirks = "override_signature_id" },
+	/* reference design: octopus */
+	{ .match = "Google_Phaser.", .quirks = "override_signature_id" },
 };
 
 /* Preserves meta data and reload image contents from given file path. */
@@ -77,8 +76,14 @@ static int is_ec_software_sync_enabled(struct updater_config *cfg)
 {
 	const struct vb2_gbb_header *gbb;
 
+	int vdat_flags = dut_get_property_int("vdat_flags", cfg);
+	if (vdat_flags < 0) {
+		WARN("Failed to identify DUT vdat_flags.\n");
+		return 0;
+	}
+
 	/* Check if current system has disabled software sync or no support. */
-	if (!(VbGetSystemPropertyInt("vdat_flags") & VBSD_EC_SOFTWARE_SYNC)) {
+	if (!(vdat_flags & VBSD_EC_SOFTWARE_SYNC)) {
 		INFO("EC Software Sync is not available.\n");
 		return 0;
 	}
@@ -116,10 +121,10 @@ static int ec_ro_software_sync(struct updater_config *cfg)
 		ERROR("EC image has invalid section '%s'.\n", "EC_RO");
 		return 1;
 	}
-	ec_ro_path = cbfs_extract_file(tmp_path, FMAP_RO_SECTION, "ecro",
+	ec_ro_path = cbfs_extract_file(tmp_path, FMAP_RO_CBFS, "ecro",
 				       &cfg->tempfiles);
 	if (!ec_ro_path ||
-	    !cbfs_file_exists(tmp_path, FMAP_RO_SECTION, "ecro.hash")) {
+	    !cbfs_file_exists(tmp_path, FMAP_RO_CBFS, "ecro.hash")) {
 		INFO("No valid EC RO for software sync in AP firmware.\n");
 		return 1;
 	}
@@ -143,18 +148,18 @@ static int ec_ro_software_sync(struct updater_config *cfg)
 		      "update by EC RO software sync.\n");
 		return 1;
 	}
-	VbSetSystemPropertyInt("try_ro_sync", 1);
+	dut_set_property_int("try_ro_sync", 1, cfg);
 	return 0;
 }
 
 /*
  * Returns True if EC is running in RW.
  */
-static int is_ec_in_rw(void)
+static int is_ec_in_rw(struct updater_config *cfg)
 {
 	char buf[VB_MAX_STRING_PROPERTY];
-	return (VbGetSystemPropertyString("ecfw_act", buf, sizeof(buf)) &&
-		strcasecmp(buf, "RW") == 0);
+	return (dut_get_property_string("ecfw_act", buf, sizeof(buf), cfg) == 0
+		&& strcasecmp(buf, "RW") == 0);
 }
 
 /*
@@ -200,30 +205,7 @@ static int quirk_enlarge_image(struct updater_config *cfg)
  */
 static int quirk_unlock_me_for_update(struct updater_config *cfg)
 {
-	struct firmware_section section;
-	struct firmware_image *image_to = &cfg->image;
-	const int flash_master_offset = 128;
-	const uint8_t flash_master[] = {
-		0x00, 0xff, 0xff, 0xff, 0x00, 0xff, 0xff, 0xff, 0x00, 0xff,
-		0xff, 0xff
-	};
-
-	find_firmware_section(&section, image_to, FMAP_SI_DESC);
-	if (section.size < flash_master_offset + ARRAY_SIZE(flash_master))
-		return 0;
-	if (memcmp(section.data + flash_master_offset, flash_master,
-		   ARRAY_SIZE(flash_master)) == 0) {
-		VB2_DEBUG("Target ME not locked.\n");
-		return 0;
-	}
-	/*
-	 * b/35568719: We should only update with unlocked ME and let
-	 * board-postinst lock it.
-	 */
-	INFO("Changed Flash Master Values to unlocked.\n");
-	memcpy(section.data + flash_master_offset, flash_master,
-	       ARRAY_SIZE(flash_master));
-	return 0;
+	return unlock_flash_master(&cfg->image);
 }
 
 /*
@@ -231,6 +213,10 @@ static int quirk_unlock_me_for_update(struct updater_config *cfg)
  * so the system has a chance to make sure SI_ME won't be corrupted on next boot
  * before locking the Flash Master values in SI_DESC.
  * Returns 0 on success, otherwise failure.
+ *
+ * TODO(b/270275115): Reuse the common unlock_flash_master() function and apply
+ * the Wilco-specific changes on top (or drop them if the common settings work
+ * for Wilco too).
  */
 static int quirk_unlock_wilco_me_for_update(struct updater_config *cfg)
 {
@@ -263,7 +249,7 @@ static int quirk_unlock_wilco_me_for_update(struct updater_config *cfg)
 static int quirk_min_platform_version(struct updater_config *cfg)
 {
 	int min_version = get_config_quirk(QUIRK_MIN_PLATFORM_VERSION, cfg);
-	int platform_version = get_system_property(SYS_PROP_PLATFORM_VER, cfg);
+	int platform_version = dut_get_property(DUT_PROP_PLATFORM_VER, cfg);
 
 	VB2_DEBUG("Minimum required version=%d, current platform version=%d\n",
 		  min_version, platform_version);
@@ -331,6 +317,11 @@ static int quirk_eve_smm_store(struct updater_config *cfg)
  * only RO and expect EC software sync to update RW later, or perform EC RO
  * software sync.
  *
+ * Note: EC RO software sync was not fully tested and may cause problems
+ *       (b/218612817, b/187789991).
+ *       RO-update (without extra sysjump) needs support from flashrom and is
+ *       currently disabled.
+ *
  * Returns:
  *  EC_RECOVERY_FULL to indicate a full recovery is needed.
  *  EC_RECOVERY_RO to indicate partial update (WP_RO) is needed.
@@ -345,29 +336,16 @@ static int quirk_ec_partial_recovery(struct updater_config *cfg)
 	 */
 	const char *ec_ro = "WP_RO";
 	struct firmware_image *ec_image = &cfg->ec_image;
-
 	int do_partial = get_config_quirk(QUIRK_EC_PARTIAL_RECOVERY, cfg);
-	if (do_partial == -1) {
-		char arch[VB_MAX_STRING_PROPERTY];
-		/*
-		 * Don't do partial update if can't decide arch (usually implies
-		 * running outside DUT).
-		 */
-		do_partial = 0;
-		if (VbGetSystemPropertyString("arch", arch, sizeof(arch)) > 0) {
-			/* By default disabled for x86, otherwise enabled. */
-			do_partial = !!strcmp(arch, "x86");
-		}
-	}
 
 	if (!do_partial) {
-		return EC_RECOVERY_FULL;
+		/* Need full update. */
 	} else if (!firmware_section_exists(ec_image, ec_ro)) {
 		INFO("EC image does not have section '%s'.\n", ec_ro);
 		/* Need full update. */
 	} else if (!is_ec_software_sync_enabled(cfg)) {
 		/* Message already printed, need full update. */
-	} else if (is_ec_in_rw()) {
+	} else if (is_ec_in_rw(cfg)) {
 		WARN("EC Software Sync detected, will only update EC RO. "
 		     "The contents in EC RW will be updated after reboot.\n");
 		return EC_RECOVERY_RO;
@@ -386,7 +364,7 @@ static int quirk_ec_partial_recovery(struct updater_config *cfg)
  *
  * Updating ME region while SoC is in S0 state is an unsupported use-case. On
  * recent platforms, we are seeing issues more frequently because of this use-
- * case. For the firmware updates performed using firmware update archive,
+ * case. For the firmware updates performed for autoupdate firmware updates,
  * preserve the ME region so that it gets updated in the successive boot.
  *
  * Returns:
@@ -395,25 +373,76 @@ static int quirk_ec_partial_recovery(struct updater_config *cfg)
  */
 static int quirk_preserve_me(struct updater_config *cfg)
 {
-	/* For a factory update donot preserve ME. */
-	if (cfg->factory_update) {
-	       WARN("Factory update. Not preserving ME.\n");
-	       return 0;
-	}
-
 	/*
-	 * For a non-archive update, the post boot script that updates ME
-	 * does not have access to the firmware image after the boot. Hence
-	 * donot preserve ME.
+	 * Only preserve the ME if performing an autoupdate-mode firmware
+	 * update. Recovery, factory and any other update modes cannot leave the
+	 * ME as is. Otherwise, a recovery firmware update cannot be relied upon
+	 * to update the ME to a valid version for WP-disabled devices.
 	 */
-	if (!cfg->archive) {
-		WARN("Update using a non-archive image. Not preserving ME.\n");
+	if (cfg->try_update == TRY_UPDATE_OFF) {
+		INFO("No auto-update requested. Not preserving ME.\n");
 		return 0;
 	}
+	INFO("Auto-update requested. Preserving ME.\n");
+
+	/*
+	 * b/213706510: subratabanik@ confirmed CSE may modify itself while we
+	 * are doing system update, and currently the 'preserve' is done by
+	 * flashing the same (e.g., "previously read") contents to skip erasing
+	 * and writing; so we have to use the diff image to prevent contents
+	 * being changed when writing.
+	 */
+	cfg->use_diff_image = 1;
 
 	return 1;
 }
 
+static int quirk_clear_mrc_data(struct updater_config *cfg)
+{
+	struct firmware_section section;
+	struct firmware_image *image = &cfg->image_current;
+	int i, count = 0;
+	int flash_now = 0;
+
+	/*
+	 * Devices with multiple MRC caches (RECOVERY, RW, RW_VAR) will have the
+	 * UNIFIED_MRC_CACHE; and devices with single RW cache will only have
+	 * RW_MRC_CACHE (for example MediaTek devices).
+	 */
+	const char * const mrc_names[] = {
+		"UNIFIED_MRC_CACHE",
+		"RW_MRC_CACHE",
+	};
+
+	if (is_write_protection_enabled(cfg) || cfg->try_update)
+		flash_now = 1;
+
+	for (i = 0; i < ARRAY_SIZE(mrc_names); i++) {
+		const char *name = mrc_names[i];
+
+		find_firmware_section(&section, image, name);
+		if (!section.size)
+			continue;
+
+		WARN("Wiping memory training data: %s\n", name);
+		memset(section.data, 0xff, section.size);
+		if (flash_now) {
+			const char *write_names[] = {name};
+			write_system_firmware(cfg, image, write_names,
+					      ARRAY_SIZE(write_names));
+		}
+		count++;
+		break;
+	}
+
+	if (count)
+		WARN("Next boot will take a few mins for memory training.\n");
+	else
+		ERROR("No known memory training data in the firmware image.\n");
+
+	return 0;
+}
+
 /*
  * Disable checking platform compatibility.
  */
@@ -424,6 +453,16 @@ static int quirk_no_check_platform(struct updater_config *cfg)
 	return 0;
 }
 
+/*
+ * Disable verifying contents after flashing.
+ */
+static int quirk_no_verify(struct updater_config *cfg)
+{
+	WARN("Disabled verifying flashed contents. You are on your own.\n");
+	cfg->do_verify = 0;
+	return 0;
+}
+
 /*
  * Registers known quirks to a updater_config object.
  */
@@ -460,9 +499,9 @@ void updater_register_quirks(struct updater_config *cfg)
 		       "dedicated FMAP section.";
 	quirks->apply = quirk_eve_smm_store;
 
-	quirks = &cfg->quirks[QUIRK_ALLOW_EMPTY_WLTAG];
-	quirks->name = "allow_empty_wltag";
-	quirks->help = "chromium/906962; allow devices without white label "
+	quirks = &cfg->quirks[QUIRK_ALLOW_EMPTY_CUSTOM_LABEL_TAG];
+	quirks->name = "allow_empty_custom_label_tag";
+	quirks->help = "chromium/906962; allow devices without custom label "
 		       "tags set to use default keys.";
 	quirks->apply = NULL;  /* Simple config. */
 
@@ -470,7 +509,6 @@ void updater_register_quirks(struct updater_config *cfg)
 	quirks->name = "ec_partial_recovery";
 	quirks->help = "chromium/1024401; recover EC by partial RO update.";
 	quirks->apply = quirk_ec_partial_recovery;
-	quirks->value = -1;  /* Decide at runtime. */
 
 	quirks = &cfg->quirks[QUIRK_OVERRIDE_SIGNATURE_ID];
 	quirks->name = "override_signature_id";
@@ -488,6 +526,21 @@ void updater_register_quirks(struct updater_config *cfg)
 	quirks->name = "no_check_platform";
 	quirks->help = "Do not check platform name.";
 	quirks->apply = quirk_no_check_platform;
+
+	quirks = &cfg->quirks[QUIRK_NO_VERIFY];
+	quirks->name = "no_verify";
+	quirks->help = "Do not verify when flashing.";
+	quirks->apply = quirk_no_verify;
+
+	quirks = &cfg->quirks[QUIRK_EXTRA_RETRIES];
+	quirks->name = "extra_retries";
+	quirks->help = "Extra retries when writing to system firmware.";
+	quirks->apply = NULL;  /* Simple config. */
+
+	quirks = &cfg->quirks[QUIRK_CLEAR_MRC_DATA];
+	quirks->name = "clear_mrc_data";
+	quirks->help = "b/255617349: Clear memory training data (MRC).";
+	quirks->apply = quirk_clear_mrc_data;
 }
 
 /*
