@@ -1,4 +1,4 @@
-/* Copyright 2019 The Chromium OS Authors. All rights reserved.
+/* Copyright 2019 The ChromiumOS Authors
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
@@ -16,16 +16,12 @@
 #endif
 
 #include "2common.h"
-#include "crossystem.h"
 #include "host_misc.h"
 #include "util_misc.h"
 #include "updater.h"
 
 #define COMMAND_BUFFER_SIZE 256
 
-/* System environment values. */
-static const char * const STR_REV = "rev";
-
 /*
  * Strips a string (usually from shell execution output) by removing all the
  * trailing characters in pattern. If pattern is NULL, match by space type
@@ -162,6 +158,10 @@ static int load_firmware_version(struct firmware_image *image,
 	return 0;
 }
 
+/*
+ * Fills in the other fields of image using image->data.
+ * Returns IMAGE_LOAD_SUCCESS or IMAGE_PARSE_FAILURE.
+ */
 static int parse_firmware_image(struct firmware_image *image)
 {
 	int ret = IMAGE_LOAD_SUCCESS;
@@ -357,6 +357,16 @@ const struct vb2_gbb_header *find_gbb(const struct firmware_image *image)
 	return gbb_header;
 }
 
+/*
+ * Returns true if the write protection is enabled on current system.
+ */
+int is_write_protection_enabled(struct updater_config *cfg)
+{
+	/* Assume HW/SW WP are enabled if -1 error code is returned */
+	return dut_get_property(DUT_PROP_WP_HW, cfg) &&
+	       dut_get_property(DUT_PROP_WP_SW, cfg);
+}
+
 /*
  * Executes a command on current host and returns stripped command output.
  * If the command has failed (exit code is not zero), returns an empty string.
@@ -392,64 +402,15 @@ char *host_shell(const char *command)
 	return strdup(buf);
 }
 
-
-/* An helper function to return "mainfw_act" system property.  */
-static int host_get_mainfw_act(void)
+void prepare_servo_control(const char *control_name, bool on)
 {
-	char buf[VB_MAX_STRING_PROPERTY];
+	char *cmd;
+	if (!control_name)
+		return;
 
-	if (!VbGetSystemPropertyString("mainfw_act", buf, sizeof(buf)))
-		return SLOT_UNKNOWN;
-
-	if (strcmp(buf, FWACT_A) == 0)
-		return SLOT_A;
-	else if (strcmp(buf, FWACT_B) == 0)
-		return SLOT_B;
-
-	return SLOT_UNKNOWN;
-}
-
-/* A helper function to return the "tpm_fwver" system property. */
-static int host_get_tpm_fwver(void)
-{
-	return VbGetSystemPropertyInt("tpm_fwver");
-}
-
-/* A helper function to return the "hardware write protection" status. */
-static int host_get_wp_hw(void)
-{
-	/* wpsw refers to write protection 'switch', not 'software'. */
-	return VbGetSystemPropertyInt("wpsw_cur") ? WP_ENABLED : WP_DISABLED;
-}
-
-/* A helper function to return "fw_vboot2" system property. */
-static int host_get_fw_vboot2(void)
-{
-	return VbGetSystemPropertyInt("fw_vboot2");
-}
-
-/* A help function to get $(mosys platform version). */
-static int host_get_platform_version(void)
-{
-	char *result = host_shell("mosys platform version");
-	long rev = -1;
-
-	/* Result should be 'revN' */
-	if (strncmp(result, STR_REV, strlen(STR_REV)) == 0)
-		rev = strtol(result + strlen(STR_REV), NULL, 0);
-
-	/* we should never have negative or extremely large versions,
-	 * but clamp just to be sure
-	 */
-	if (rev < 0)
-		rev = 0;
-	if (rev > INT_MAX)
-		rev = INT_MAX;
-
-	VB2_DEBUG("Raw data = [%s], parsed version is %ld\n", result, rev);
-
-	free(result);
-	return rev;
+	ASPRINTF(&cmd, "dut-control %s:%s", control_name, on ? "on" : "off");
+	free(host_shell(cmd));
+	free(cmd);
 }
 
 /*
@@ -465,11 +426,13 @@ char *host_detect_servo(const char **prepare_ctrl_name)
 	char *ret = NULL;
 	char *servo_serial = NULL;
 
+	static const char * const raiden_debug_spi = "raiden_debug_spi";
 	static const char * const cpu_fw_spi = "cpu_fw_spi";
 	static const char * const ccd_cpu_fw_spi = "ccd_cpu_fw_spi";
 
 	/* By default, no control is needed. */
 	*prepare_ctrl_name = NULL;
+	VB2_DEBUG("servo_type: %s\n", servo_type);
 
 	/* Get serial name if servo port is provided. */
 	if ((servo_port && *servo_port) || (servo_name && *servo_name)) {
@@ -490,26 +453,32 @@ char *host_detect_servo(const char **prepare_ctrl_name)
 		VB2_DEBUG("Servo SN=%s (serial cmd: %s)\n", servo_serial, cmd);
 	}
 
+	/* servo_type names: chromite/lib/firmware/servo_lib.py */
 	if (!*servo_type) {
 		ERROR("Failed to get servo type. Check servod.\n");
 	} else if (servo_serial && !*servo_serial) {
 		ERROR("Failed to get serial at servo port %s.\n", servo_port);
+	} else if (strcmp(servo_type, "servo_v2") == 0) {
+		VB2_DEBUG("Selected Servo V2.\n");
+		programmer = "ft2232_spi:type=google-servo-v2";
+		*prepare_ctrl_name = cpu_fw_spi;
 	} else if (strstr(servo_type, "servo_micro")) {
 		VB2_DEBUG("Selected Servo Micro.\n");
-		programmer = "raiden_debug_spi";
+		programmer = raiden_debug_spi;
 		*prepare_ctrl_name = cpu_fw_spi;
 	} else if (strstr(servo_type, "c2d2")) {
 		VB2_DEBUG("Selected C2D2.\n");
-		programmer = "raiden_debug_spi";
+		programmer = raiden_debug_spi;
 		*prepare_ctrl_name = cpu_fw_spi;
 	} else if (strstr(servo_type, "ccd_cr50") ||
-		   strstr(servo_type, "ccd_gsc")) {
+		   strstr(servo_type, "ccd_gsc") ||
+		   strstr(servo_type, "ccd_ti50")) {
 		VB2_DEBUG("Selected CCD.\n");
 		programmer = "raiden_debug_spi:target=AP,custom_rst=true";
 		*prepare_ctrl_name = ccd_cpu_fw_spi;
 	} else {
-		VB2_DEBUG("Selected Servo V2.\n");
-		programmer = "ft2232_spi:type=google-servo-v2";
+		WARN("Unknown servo: %s\nAssuming debug header.\n", servo_type);
+		programmer = raiden_debug_spi;
 		*prepare_ctrl_name = cpu_fw_spi;
 	}
 
@@ -548,228 +517,19 @@ static int is_the_same_programmer(const struct firmware_image *image1,
 	return strcmp(image1->programmer, image2->programmer) == 0;
 }
 
-enum flash_command {
-	FLASH_READ = 0,
-	FLASH_WRITE,
-};
-
-/* Converts the flashrom_params to an equivalent flashrom command. */
-static char *get_flashrom_command(enum flash_command flash_cmd,
-				  struct flashrom_params *params,
-				  const char *image_name,
-				  const char *contents_name)
-{
-	int i, len = 0;
-	char *partial = NULL;
-	char *cmd = NULL;
-
-	if (!image_name)
-		image_name = "<IMAGE>";
-	if (!contents_name)
-		contents_name = "<OLD-IMAGE>";
-
-	for (i = 0; params->regions && params->regions[i]; i++)
-		len += strlen(params->regions[i]) + strlen(" -i ");
-
-	if (len) {
-		partial = (char *)malloc(len + 1);
-		if (!partial) {
-			ERROR("Failed to allocate a string buffer.\n");
-			return NULL;
-		}
-
-		partial[0] = '\0';
-		for (i = 0; params->regions[i]; i++) {
-			strcat(partial, " -i ");
-			strcat(partial, params->regions[i]);
-		}
-		assert(strlen(partial) == len);
-	}
-
-	switch (flash_cmd) {
-	case FLASH_READ:
-		ASPRINTF(&cmd, "flashrom -r %s -p %s%s%s",
-			 image_name,
-			 params->image->programmer,
-			 params->verbose > 1 ? " -V" : "",
-			 partial ? partial : "");
-		break;
-
-	case FLASH_WRITE:
-		ASPRINTF(&cmd, "flashrom -w %s -p %s%s%s%s%s%s",
-			 image_name,
-			 params->image->programmer,
-			 params->flash_contents ? " --flash-contents " : "",
-			 params->flash_contents ? contents_name : "",
-			 params->noverify ? " --noverify" : "",
-			 params->verbose > 1 ? " -V" : "",
-			 partial ? partial : "");
-		break;
-
-	default:
-		ERROR("Unknown command: %d.\n", flash_cmd);
-		break;
-	}
-	free(partial);
-	return cmd;
-}
-
-/*
- * Emulates writing a firmware image to the system.
- * Returns 0 if success, non-zero if error.
- */
-static int emulate_write_firmware(const char *filename,
-				  const struct firmware_image *image,
-				  const char * const sections[])
-{
-	int i, errorcnt = 0;
-	struct firmware_image to_image = {0};
-
-	INFO("Writing from %s to %s (emu=%s).\n",
-	     image->file_name, image->programmer, filename);
-
-	if (load_firmware_image(&to_image, filename, NULL)) {
-		ERROR("Cannot load image from %s.\n", filename);
-		return -1;
-	}
-
-	if (image->size != to_image.size) {
-		ERROR("Image size is different (%s:%d != %s:%d)\n",
-		      image->file_name, image->size, to_image.file_name,
-		      to_image.size);
-		errorcnt++;
-		goto exit;
-	}
-
-	if (!sections) {
-		VB2_DEBUG(" - write the whole image.\n");
-		memmove(to_image.data, image->data, image->size);
-	}
-	for (i = 0; sections && sections[i]; i++) {
-		VB2_DEBUG(" - write the section: %s.\n", sections[i]);
-		if (preserve_firmware_section(image, &to_image, sections[i])) {
-			ERROR("Failed to write the section: %s\n", sections[i]);
-			errorcnt++;
-			/*
-			 * Exit the loop, but still write the file to reflect
-			 * the partial changes - same as real flashrom behavior.
-			 */
-			break;
-		}
-	}
-
-	if (vb2_write_file(filename, to_image.data, to_image.size)) {
-		ERROR("Failed writing to file: %s\n", filename);
-		errorcnt++;
-		goto exit;
-	}
-
-exit:
-	free_firmware_image(&to_image);
-	return errorcnt;
-}
-
-static int external_flashrom(enum flash_command flash_cmd,
-			     struct flashrom_params *params,
-			     struct tempfile *tempfiles)
-{
-	int r;
-	char *cmd;
-	const char *image_name = NULL, *contents_name = NULL;
-
-	switch (flash_cmd) {
-	case FLASH_READ:
-		image_name = create_temp_file(tempfiles);
-		break;
-
-	case FLASH_WRITE:
-		image_name = get_firmware_image_temp_file(
-				params->image, tempfiles);
-		if (params->flash_contents)
-			contents_name = get_firmware_image_temp_file(
-					params->flash_contents, tempfiles);
-		break;
-
-	default:
-		ERROR("Unknown command: %d\n", flash_cmd);
-		return -1;
-	}
-
-	cmd = get_flashrom_command(flash_cmd, params, image_name,
-				   contents_name);
-	if (!cmd)
-		return -1;
-
-	VB2_DEBUG(cmd);
-	r = system(cmd);
-	free(cmd);
-	if (r)
-		return r;
-
-	switch (flash_cmd) {
-	case FLASH_READ:
-		r = load_firmware_image(params->image, image_name, NULL);
-		break;
-	default:
-		break;
-	}
-
-	return r;
-}
-
-static int read_flash(struct flashrom_params *params,
-		      struct updater_config *cfg)
-{
-	if (get_config_quirk(QUIRK_EXTERNAL_FLASHROM, cfg))
-		return external_flashrom(FLASH_READ, params, &cfg->tempfiles);
-
-	return flashrom_read_image(params->image, NULL, params->verbose);
-}
-
-static int write_flash(struct flashrom_params *params,
-		       struct updater_config *cfg)
-{
-	int r;
-
-	if (get_config_quirk(QUIRK_EXTERNAL_FLASHROM, cfg))
-		return external_flashrom(FLASH_WRITE, params, &cfg->tempfiles);
-
-	r = flashrom_write_image(params->image,
-				 params->regions,
-				 params->flash_contents,
-				 !params->noverify,
-				 params->verbose);
-	/*
-	 * Force a newline to flush stdout in case if
-	 * flashrom_write_image left some messages in the buffer.
-	 */
-	fprintf(stdout, "\n");
-	return r;
-}
-
-/*
- * Loads the active system firmware image (usually from SPI flash chip).
- * Returns 0 if success, non-zero if error.
- */
 int load_system_firmware(struct updater_config *cfg,
 			 struct firmware_image *image)
 {
 	int r, i;
-	char *cmd;
 	const int tries = 1 + get_config_quirk(QUIRK_EXTRA_RETRIES, cfg);
-	struct flashrom_params params = {0};
 
-	params.image = image;
-	params.verbose = cfg->verbosity + 1; /* libflashrom verbose 1 = WARN. */
+	int verbose = cfg->verbosity + 1; /* libflashrom verbose 1 = WARN. */
 
-	cmd = get_flashrom_command(FLASH_READ, &params, NULL, NULL);
-	INFO("%s\n", cmd);
-	free(cmd);
-
-	for (i = 1, r = -1; i <= tries && r != 0; i++, params.verbose++) {
+	for (i = 1, r = -1; i <= tries && r != 0; i++, verbose++) {
 		if (i > 1)
 			WARN("Retry reading firmware (%d/%d)...\n", i, tries);
-		r = read_flash(&params, cfg);
+		INFO("Reading SPI Flash..\n");
+		r = flashrom_read_image(image, NULL, 0, verbose);
 	}
 	if (!r)
 		r = parse_firmware_image(image);
@@ -784,59 +544,36 @@ int load_system_firmware(struct updater_config *cfg,
  */
 int write_system_firmware(struct updater_config *cfg,
 			  const struct firmware_image *image,
-			  const char * const sections[])
+			  const char * const regions[],
+				const size_t regions_len)
 {
 	int r = 0, i;
-	char *cmd;
 	const int tries = 1 + get_config_quirk(QUIRK_EXTRA_RETRIES, cfg);
-	struct flashrom_params params = {0};
 	struct firmware_image *flash_contents = NULL;
 
-	if (cfg->emulation)
-		return emulate_write_firmware(cfg->emulation, image, sections);
-
 	if (cfg->use_diff_image && cfg->image_current.data &&
 	    is_the_same_programmer(&cfg->image_current, image))
 		flash_contents = &cfg->image_current;
 
-	params.image = (struct firmware_image *)image;
-	params.flash_contents = flash_contents;
-	params.regions = sections;
-	params.noverify = !cfg->do_verify;
-	params.noverify_all = true;
-	params.verbose = cfg->verbosity + 1; /* libflashrom verbose 1 = WARN. */
+	int verbose = cfg->verbosity + 1; /* libflashrom verbose 1 = WARN. */
 
-	cmd = get_flashrom_command(FLASH_WRITE, &params, NULL, NULL);
-	INFO("%s\n", cmd);
-	free(cmd);
-
-	for (i = 1, r = -1; i <= tries && r != 0; i++, params.verbose++) {
+	for (i = 1, r = -1; i <= tries && r != 0; i++, verbose++) {
 		if (i > 1)
 			WARN("Retry writing firmware (%d/%d)...\n", i, tries);
-		r = write_flash(&params, cfg);
+		INFO("Writing SPI Flash..\n");
+		r = flashrom_write_image(image, regions, regions_len,
+					 flash_contents, cfg->do_verify,
+					 verbose);
+		/*
+		 * Force a newline to flush stdout in case if
+		 * flashrom_write_image left some messages in the buffer.
+		 */
+		fprintf(stdout, "\n");
+
 	}
 	return r;
 }
 
-/* Helper function to return host software write protection status. */
-static int host_get_wp_sw(void)
-{
-	return flashrom_get_wp(PROG_HOST, -1);
-}
-
-/* Helper function to configure all properties. */
-void init_system_properties(struct system_property *props, int num)
-{
-	memset(props, 0, num * sizeof(*props));
-	assert(num >= SYS_PROP_MAX);
-	props[SYS_PROP_MAINFW_ACT].getter = host_get_mainfw_act;
-	props[SYS_PROP_TPM_FWVER].getter = host_get_tpm_fwver;
-	props[SYS_PROP_FW_VBOOT2].getter = host_get_fw_vboot2;
-	props[SYS_PROP_PLATFORM_VER].getter = host_get_platform_version;
-	props[SYS_PROP_WP_HW].getter = host_get_wp_hw;
-	props[SYS_PROP_WP_SW].getter = host_get_wp_sw;
-}
-
 /*
  * Helper function to create a new temporary file.
  * All files created will be removed remove_all_temp_files().
@@ -920,3 +657,96 @@ const char *get_firmware_rootkey_hash(const struct firmware_image *image)
 
 	return packed_key_sha1_string(rootkey);
 }
+
+/*
+ * Overwrite the given offset of a section in the firmware image with the
+ * given values.
+ * Returns 0 on success, otherwise failure.
+ */
+static int overwrite_section(struct firmware_image *image,
+			     const char *fmap_section, size_t offset,
+			     size_t size, const uint8_t *new_values)
+{
+	struct firmware_section section;
+
+	find_firmware_section(&section, image, fmap_section);
+	if (section.size < offset + size) {
+		ERROR("Section smaller than given offset + size\n");
+		return -1;
+	}
+
+	if (memcmp(section.data + offset, new_values, size) == 0) {
+		VB2_DEBUG("Section already contains given values.\n");
+		return 0;
+	}
+
+	memcpy(section.data + offset, new_values, size);
+	return 0;
+}
+
+/*
+ * Unlock the Flash Master values in SI_DESC.
+ * Returns 0 on success, otherwise failure.
+ *
+ * TODO(b/270275115): Replace with a call to ifdtool.
+ */
+int unlock_flash_master(struct firmware_image *image)
+{
+	const int flash_master_offset = 0x80;
+	const uint8_t flash_master[] = {
+		0x00, 0xff, 0xff, 0xff, 0x00, 0xff, 0xff, 0xff, 0x00, 0xff,
+		0xff, 0xff
+	};
+
+	if (overwrite_section(image, FMAP_SI_DESC, flash_master_offset,
+			      ARRAY_SIZE(flash_master), flash_master)) {
+		ERROR("Failed unlocking Flash Master values\n");
+		return -1;
+	}
+
+	INFO("Changed Flash Master Values to unlocked.\n");
+	return 0;
+}
+
+/*
+ * Disable GPR0 (Global Protected Range). When enabled, it provides
+ * write-protection to part of the SI_ME region, specifically CSE_RO and
+ * part of CSE_DATA, so it must be disabled to allow updating SI_ME.
+ * Returns 0 on success, otherwise failure.
+ *
+ * TODO(b/270275115): Replace with a call to ifdtool.
+ */
+static int disable_gpr0(struct firmware_image *image)
+{
+	const int gpr0_offset = 0x154;
+	const uint8_t gpr0_value_disabled[] = { 0x00, 0x00, 0x00, 0x00 };
+
+	if (overwrite_section(image, FMAP_SI_DESC, gpr0_offset,
+			      ARRAY_SIZE(gpr0_value_disabled),
+			      gpr0_value_disabled)) {
+		ERROR("Failed disabling GPR0.\n");
+		return -1;
+	}
+
+	INFO("Disabled GPR0.\n");
+	return 0;
+}
+
+/*
+ * Unlock the Intel ME by:
+ * - Unlocking the FLMSTR values in the descriptor.
+ * - Disabling GPR0 in the descriptor.
+ * This allows the SI_DESC and SI_ME regions to be updated.
+ * Returns 0 on success, otherwise failure.
+ */
+int unlock_me(struct firmware_image *image)
+{
+	if (unlock_flash_master(image))
+		return -1;
+
+	if (disable_gpr0(image))
+		return -1;
+
+	INFO("Unlocked Intel ME.\n");
+	return 0;
+}
