@@ -1,4 +1,4 @@
-/* Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+/* Copyright 2012 The ChromiumOS Authors
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -83,6 +83,10 @@
 #define GPIO_BASE_PATH "/sys/class/gpio"
 #define GPIO_EXPORT_PATH GPIO_BASE_PATH "/export"
 
+/* Base for SMBIOS information files */
+#define SMBIOS_BASE_PATH "/sys/class/dmi/id"
+#define SMBIOS_PRODUCT_VERSION_PATH SMBIOS_BASE_PATH "/product_version"
+
 /* Filename for NVRAM file */
 #define NVRAM_PATH "/dev/nvram"
 
@@ -733,6 +737,9 @@ static const struct GpioChipset chipsets_supported[] = {
 	/* INTC105x are for Alderlake */
 	{ "INTC1055:00", FindGpioChipOffsetByLabel },
 	{ "INTC1056:00", FindGpioChipOffsetByLabel },
+	{ "INTC1057:00", FindGpioChipOffsetByLabel },
+	/* INTC108x are for Meteor Lake */
+	{ "INTC1083:00", FindGpioChipOffsetByLabel },
 	/* INT3453 are for GLK */
 	{ "INT3453:00", FindGpioChipOffsetByLabel },
 	{ "INT3453:01", FindGpioChipOffsetByLabel },
@@ -831,6 +838,25 @@ static int ReadGpio(unsigned signal_type)
 	return (value == active_high ? 1 : 0);
 }
 
+static int GetBoardId(void)
+{
+	/*
+	 * Can't use vb2_read_file here, as it expects to be able to
+	 * seek to the end of the file to tell the size, and the sysfs
+	 * SMBIOS implementation will seek to offset 4096.
+	 */
+	int board_id = -1;
+	FILE *f = fopen(SMBIOS_PRODUCT_VERSION_PATH, "r");
+
+	if (!f)
+		return -1;
+
+	if (fscanf(f, "rev%d\n", &board_id) != 1)
+		board_id = -1;
+
+	fclose(f);
+	return board_id;
+}
 
 int VbGetArchPropertyInt(const char* name)
 {
@@ -894,6 +920,9 @@ int VbGetArchPropertyInt(const char* name)
 			value = (int)fwupdate_value;
 	}
 
+	if (!strcasecmp(name, "board_id"))
+		return GetBoardId();
+
 	return value;
 }
 
