@@ -1,12 +1,14 @@
-/* Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+/* Copyright 2012 The ChromiumOS Authors
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
 
 #include <ctype.h>
+#include <fcntl.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/file.h>
 #include <unistd.h>
 
 #include "2api.h"
@@ -22,14 +24,12 @@
 #include "subprocess.h"
 #include "vboot_struct.h"
 
+/* Filename for crossystem lock */
+#define CROSSYSTEM_LOCK_PATH "/run/lock/crossystem.lock"
+
 /* Filename for kernel command line */
 #define KERNEL_CMDLINE_PATH "/proc/cmdline"
 
-/* Filename for the mount-encrypted key */
-/* TODO(b/174807059): Remove this after we land driver-level TPM simulator on
- * all VM boards */
-#define MOUNT_ENCRYPTED_KEY_PATH "/mnt/stateful_partition/encrypted.key"
-
 /* Filename for the TPM simulator NV data */
 #define TPM_SIMULATOR_NVCHIP_PATH \
 	"/mnt/stateful_partition/unencrypted/tpm2-simulator/NVChip"
@@ -38,7 +38,8 @@
 typedef enum VdatStringField {
 	VDAT_STRING_DEPRECATED_TIMERS = 0,  /* Timer values */
 	VDAT_STRING_LOAD_FIRMWARE_DEBUG,  /* LoadFirmware() debug info */
-	VDAT_STRING_DEPRECATED_LOAD_KERNEL_DEBUG,  /* LoadKernel() debug info */
+	VDAT_STRING_DEPRECATED_LOAD_KERNEL_DEBUG,  /* vb2api_load_kernel()
+						      debug info */
 	VDAT_STRING_MAINFW_ACT  /* Active main firmware */
 } VdatStringField;
 
@@ -86,12 +87,38 @@ static const char *default_boot[] = {"disk", "usb", "altfw"};
 int FwidStartsWith(const char *start)
 {
 	char fwid[VB_MAX_STRING_PROPERTY];
-	if (!VbGetSystemPropertyString("fwid", fwid, sizeof(fwid)))
+	if (VbGetSystemPropertyString("fwid", fwid, sizeof(fwid)) != 0)
 		return 0;
 
 	return 0 == strncmp(fwid, start, strlen(start));
 }
 
+/* Acquire the lock for crossystem SetSystemProperty call. */
+static int AcquireCrossystemLock(void)
+{
+	int lock_fd;
+
+	lock_fd = open(CROSSYSTEM_LOCK_PATH, O_RDWR | O_CREAT, 0600);
+	if (lock_fd < 0)
+		return -1;
+
+	if (flock(lock_fd, LOCK_EX) < 0)
+		return -1;
+
+	return lock_fd;
+}
+
+/* Release the lock for crossystem SetSystemProperty call. */
+static int ReleaseCrossystemLock(int lock_fd)
+{
+	if (flock(lock_fd, F_UNLCK) < 0)
+		return -1;
+
+	close(lock_fd);
+
+	return 0;
+}
+
 static struct vb2_context *get_fake_context(void)
 {
 	static uint8_t fake_workbuf[sizeof(struct vb2_shared_data) + 16]
@@ -235,8 +262,8 @@ static int VbGetCrosDebug(void)
 	return 0;
 }
 
-static char *GetVdatLoadFirmwareDebug(char *dest, int size,
-				      const VbSharedDataHeader *sh)
+static int GetVdatLoadFirmwareDebug(char *dest, int size,
+				    const VbSharedDataHeader *sh)
 {
 	snprintf(dest, size,
 		 "Check A result=%d\n"
@@ -249,16 +276,16 @@ static char *GetVdatLoadFirmwareDebug(char *dest, int size,
 		 sh->firmware_index,
 		 sh->fw_version_tpm_start,
 		 sh->fw_version_lowest);
-	return dest;
+	return 0;
 }
 
-static char *GetVdatString(char *dest, int size, VdatStringField field)
+static int GetVdatString(char *dest, int size, VdatStringField field)
 {
 	VbSharedDataHeader *sh = VbSharedDataRead();
-	char *value = dest;
+	int value = 0;
 
 	if (!sh)
-		return NULL;
+		return -1;
 
 	switch (field) {
 		case VDAT_STRING_LOAD_FIRMWARE_DEBUG:
@@ -277,12 +304,12 @@ static char *GetVdatString(char *dest, int size, VdatStringField field)
 					StrCopy(dest, "recovery", size);
 					break;
 				default:
-					value = NULL;
+					value = -1;
 			}
 			break;
 
 		default:
-			value = NULL;
+			value = -1;
 			break;
 	}
 
@@ -381,12 +408,9 @@ int VbGetSystemPropertyInt(const char *name)
 	} else if (!strcasecmp(name,"disable_dev_request")) {
 		value = vb2_get_nv_storage(VB2_NV_DISABLE_DEV_REQUEST);
 	} else if (!strcasecmp(name,"clear_tpm_owner_request")) {
-		if (TPM2_SIMULATOR && VTPM_PROXY)
+		if (TPM2_SIMULATOR)
 			/* Check TPM simulator NVChip status */
 			value = access(TPM_SIMULATOR_NVCHIP_PATH, F_OK) != 0;
-		else if (TPM2_SIMULATOR)
-			/* Check mount-encrypted key status */
-			value = access(MOUNT_ENCRYPTED_KEY_PATH, F_OK) != 0;
 		else
 			value = vb2_get_nv_storage(
 				VB2_NV_CLEAR_TPM_OWNER_REQUEST);
@@ -449,6 +473,9 @@ int VbGetSystemPropertyInt(const char *name)
 	} else if (!strcasecmp(name, "wpsw_cur")) {
 		/* Use "write-protect at boot" as a fallback value. */
 		value = GetVdatInt(VDAT_INT_HW_WPSW_BOOT);
+		fprintf(stderr,
+			"Fallback to WPSW_BOOT (%d), which may be invalid\n",
+			value);
 	} else if (!strcasecmp(name,"vdat_flags")) {
 		value = GetVdatInt(VDAT_INT_FLAGS);
 	} else if (!strcasecmp(name,"tpm_fwver")) {
@@ -471,11 +498,13 @@ int VbGetSystemPropertyInt(const char *name)
 		 * HWID is present, it is a baremetal Chrome OS machine. Other
 		 * cases are errors. */
 		char hwid[VB_MAX_STRING_PROPERTY];
-		if (!VbGetSystemPropertyString("hwid", hwid, sizeof(hwid))) {
+		if (VbGetSystemPropertyString("hwid", hwid,
+					      sizeof(hwid)) != 0) {
 			char fwtype_buf[VB_MAX_STRING_PROPERTY];
-			const char *fwtype = VbGetSystemPropertyString(
+			int fwtype_ret = VbGetSystemPropertyString(
 				"mainfw_type", fwtype_buf, sizeof(fwtype_buf));
-			if (fwtype && !strcasecmp(fwtype, "nonchrome")) {
+			if (fwtype_ret == 0 &&
+			    !strcasecmp(fwtype_buf, "nonchrome")) {
 				value = 1;
 			}
 		} else {
@@ -488,9 +517,13 @@ int VbGetSystemPropertyInt(const char *name)
 	return value;
 }
 
-const char *VbGetSystemPropertyString(const char *name, char *dest,
-				      size_t size)
+int VbGetSystemPropertyString(const char *name, char *dest, size_t size)
 {
+	if (dest == NULL || size == 0)
+	{
+		fprintf(stderr, "invalid dest buffer\n");
+		return -1;
+	}
 	/* Check for HWID override via cros_config */
 	if (!strcasecmp(name, "hwid")) {
 		char *hwid_override;
@@ -499,61 +532,77 @@ const char *VbGetSystemPropertyString(const char *name, char *dest,
 					       &hwid_override) == VB2_SUCCESS) {
 			StrCopy(dest, hwid_override, size);
 			free(hwid_override);
-			return dest;
+			return 0;
 		}
 	}
 
 	/* Check architecture-dependent properties */
 	if (VbGetArchPropertyString(name, dest, size))
-		return dest;
+		return 0;
 
 	if (!strcasecmp(name,"kernkey_vfy")) {
 		switch(GetVdatInt(VDAT_INT_KERNEL_KEY_VERIFIED)) {
 			case 0:
-				return "hash";
+				StrCopy(dest, "hash", size);
+				return 0;
 			case 1:
-				return "sig";
+				StrCopy(dest, "sig", size);
+				return 0;
 			default:
-				return NULL;
+				return -1;
 		}
 	} else if (!strcasecmp(name, "mainfw_act")) {
 		return GetVdatString(dest, size, VDAT_STRING_MAINFW_ACT);
 	} else if (!strcasecmp(name, "vdat_lfdebug")) {
 		return GetVdatString(dest, size,
-				     VDAT_STRING_LOAD_FIRMWARE_DEBUG);
+				VDAT_STRING_LOAD_FIRMWARE_DEBUG);
 	} else if (!strcasecmp(name, "fw_try_next")) {
-		return vb2_get_nv_storage(VB2_NV_TRY_NEXT) ? "B" : "A";
+		StrCopy(dest,
+			vb2_get_nv_storage(VB2_NV_TRY_NEXT) ? "B" : "A",
+			size);
+		return 0;
 	} else if (!strcasecmp(name, "fw_tried")) {
-		return vb2_get_nv_storage(VB2_NV_FW_TRIED) ? "B" : "A";
+		StrCopy(dest,
+			vb2_get_nv_storage(VB2_NV_FW_TRIED) ? "B" : "A",
+			size);
+		return 0;
 	} else if (!strcasecmp(name, "fw_result")) {
 		int v = vb2_get_nv_storage(VB2_NV_FW_RESULT);
 		if (v < ARRAY_SIZE(fw_results))
-			return fw_results[v];
+			StrCopy(dest, fw_results[v], size);
 		else
-			return "unknown";
+			StrCopy(dest, "unknown", size);
+		return 0;
 	} else if (!strcasecmp(name, "fw_prev_tried")) {
-		return vb2_get_nv_storage(VB2_NV_FW_PREV_TRIED) ? "B" : "A";
+		StrCopy(dest,
+			vb2_get_nv_storage(VB2_NV_FW_PREV_TRIED) ? "B" : "A",
+			size);
+		return 0;
 	} else if (!strcasecmp(name, "fw_prev_result")) {
 		int v = vb2_get_nv_storage(VB2_NV_FW_PREV_RESULT);
 		if (v < ARRAY_SIZE(fw_results))
-			return fw_results[v];
+			StrCopy(dest, fw_results[v], size);
 		else
-			return "unknown";
+			StrCopy(dest, "unknown", size);
+		return 0;
 	} else if (!strcasecmp(name,"dev_default_boot")) {
 		int v = vb2_get_nv_storage(VB2_NV_DEV_DEFAULT_BOOT);
 		if (v < ARRAY_SIZE(default_boot))
-			return default_boot[v];
+			StrCopy(dest, default_boot[v], size);
 		else
-			return "unknown";
+			StrCopy(dest, "unknown", size);
+		return 0;
 	} else if (!strcasecmp(name, "minios_priority")) {
-		return vb2_get_nv_storage(VB2_NV_MINIOS_PRIORITY) ? "B" : "A";
+		StrCopy(dest,
+			vb2_get_nv_storage(VB2_NV_MINIOS_PRIORITY) ?"B" : "A",
+			size);
+		return 0;
 	}
 
-	return NULL;
+	return -1;
 }
 
-
-int VbSetSystemPropertyInt(const char *name, int value)
+static int VbSetSystemPropertyIntInternal(const char *name, int value)
 {
 	/* Check architecture-dependent properties first */
 
@@ -581,13 +630,10 @@ int VbSetSystemPropertyInt(const char *name, int value)
 			 * on simulator */
 			if (value == 0)
 				return -1;
-			const char *tpm_path =
-				VTPM_PROXY ? TPM_SIMULATOR_NVCHIP_PATH
-					   : MOUNT_ENCRYPTED_KEY_PATH;
 			/* Check TPM simulator data status */
-			if (!access(tpm_path, F_OK)) {
+			if (!access(TPM_SIMULATOR_NVCHIP_PATH, F_OK)) {
 				/* Remove the TPM2.0 simulator data */
-				return remove(tpm_path);
+				return remove(TPM_SIMULATOR_NVCHIP_PATH);
 			} else {
 				/* Return success when the file is already
 				 * removed */
@@ -673,7 +719,25 @@ int VbSetSystemPropertyInt(const char *name, int value)
 	return -1;
 }
 
-int VbSetSystemPropertyString(const char* name, const char* value)
+int VbSetSystemPropertyInt(const char *name, int value)
+{
+	int result = -1;
+	int lock_fd;
+
+	lock_fd = AcquireCrossystemLock();
+	if (lock_fd < 0)
+		return -1;
+
+	result = VbSetSystemPropertyIntInternal(name, value);
+
+	if (ReleaseCrossystemLock(lock_fd) < 0)
+		return -1;
+
+	return result;
+}
+
+static int VbSetSystemPropertyStringInternal(const char *name,
+					     const char *value)
 {
 	/* Chain to architecture-dependent properties */
 	if (0 == VbSetArchPropertyString(name, value))
@@ -725,6 +789,23 @@ int VbSetSystemPropertyString(const char* name, const char* value)
 	return -1;
 }
 
+int VbSetSystemPropertyString(const char *name, const char *value)
+{
+	int result = -1;
+	int lock_fd;
+
+	lock_fd = AcquireCrossystemLock();
+	if (lock_fd < 0)
+		return -1;
+
+	result = VbSetSystemPropertyStringInternal(name, value);
+
+	if (ReleaseCrossystemLock(lock_fd) < 0)
+		return -1;
+
+	return result;
+}
+
 /**
  * Get index of the last valid VBNV entry in an EEPROM.
  *
@@ -768,21 +849,21 @@ int vb2_read_nv_storage_flashrom(struct vb2_context *ctx)
 {
 	int index;
 	int vbnv_size = vb2_nv_get_size(ctx);
-	uint8_t *flash_buf;
-	uint32_t flash_size;
 
-	if (flashrom_read(FLASHROM_PROGRAMMER_INTERNAL_AP, VBNV_FMAP_REGION,
-			  &flash_buf, &flash_size))
+	struct firmware_image image = {
+		.programmer = FLASHROM_PROGRAMMER_INTERNAL_AP,
+	};
+	if (flashrom_read(&image, VBNV_FMAP_REGION))
 		return -1;
 
-	index = vb2_nv_index(flash_buf, flash_size, vbnv_size);
+	index = vb2_nv_index(image.data, image.size, vbnv_size);
 	if (index < 0) {
-		free(flash_buf);
+		free(image.data);
 		return -1;
 	}
 
-	memcpy(ctx->nvdata, &flash_buf[index * vbnv_size], vbnv_size);
-	free(flash_buf);
+	memcpy(ctx->nvdata, &image.data[index * vbnv_size], vbnv_size);
+	free(image.data);
 	return 0;
 }
 
@@ -792,34 +873,33 @@ int vb2_write_nv_storage_flashrom(struct vb2_context *ctx)
 	int current_index;
 	int next_index;
 	int vbnv_size = vb2_nv_get_size(ctx);
-	uint8_t *flash_buf;
-	uint32_t flash_size;
 
-	if (flashrom_read(FLASHROM_PROGRAMMER_INTERNAL_AP, VBNV_FMAP_REGION,
-			  &flash_buf, &flash_size))
+	struct firmware_image image = {
+		.programmer = FLASHROM_PROGRAMMER_INTERNAL_AP,
+	};
+	if (flashrom_read(&image, VBNV_FMAP_REGION))
 		return -1;
 
-	current_index = vb2_nv_index(flash_buf, flash_size, vbnv_size);
+	current_index = vb2_nv_index(image.data, image.size, vbnv_size);
 	if (current_index < 0) {
 		rv = -1;
 		goto exit;
 	}
 
 	next_index = current_index + 1;
-	if (next_index * vbnv_size == flash_size) {
+	if (next_index * vbnv_size == image.size) {
 		/* VBNV is full.  Erase and write at beginning. */
-		memset(flash_buf, 0xff, flash_size);
+		memset(image.data, 0xff, image.size);
 		next_index = 0;
 	}
 
-	memcpy(&flash_buf[next_index * vbnv_size], ctx->nvdata, vbnv_size);
-	if (flashrom_write(FLASHROM_PROGRAMMER_INTERNAL_AP, VBNV_FMAP_REGION,
-			   flash_buf, flash_size)) {
+	memcpy(&image.data[next_index * vbnv_size], ctx->nvdata, vbnv_size);
+	if (flashrom_write(&image, VBNV_FMAP_REGION)) {
 		rv = -1;
 		goto exit;
 	}
 
  exit:
-	free(flash_buf);
+	free(image.data);
 	return rv;
 }
