@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+# Copyright 2012 The ChromiumOS Authors
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 #
@@ -238,8 +238,8 @@ main() {
       local old_cwd=$(pwd)
       cd "${expanded_firmware_dir}"
 
-      ${FUTILITY} sign --type rwsig --prikey "${ec_efs_prvkey}" "${IMAGE_EC}" ||
-        die "Failed to sign EC image"
+      ${FUTILITY} sign --type rwsig --prikey "${ec_efs_prvkey}" \
+        --ecrw_out "${rw_bin}" "${IMAGE_EC}" || die "Failed to sign EC image"
       # Above command produces EC_RW.bin. Compute its hash.
       openssl dgst -sha256 -binary "${rw_bin}" > "${rw_hash}"
 
