@@ -1,6 +1,6 @@
 #!/bin/bash -u
 #
-# Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+# Copyright 2012 The ChromiumOS Authors
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 #
@@ -23,7 +23,7 @@ SIGNPRIVATE="${DATA_DIR}/data_0.vbprivk"
 SIGNPUBLIC="${DATA_DIR}/root_0.vbpubk"
 CONFIG="${DATA_DIR}/dummy_config.txt"
 
-# Create some big and little files for the kernel and bootloader
+# Create some big and little files for the kernel and optional bootloader
 BIG="${TMPDIR}/big.bin"
 dd if=/dev/urandom bs=32768 count=1 of="${BIG}" 2>/dev/null
 SMALL="${TMPDIR}/small.bin"
@@ -52,7 +52,7 @@ while [ "$k" -lt "${#KERN_VALS[*]}" ]; do
         --arch arm \
         --config "${CONFIG}" \
         "${KERN_VALS[$k]}" \
-        "${BOOT_VALS[$k]}" >/dev/null
+        "${BOOT_VALS[$b]}" >/dev/null
       then
         echo -e "${COL_RED}FAILED${COL_STOP}"
         : $(( errs++ ))
