@@ -83,6 +83,8 @@ enum {
 	CB_TAG_BOARD_CONFIG		= 0x0040,
 	CB_TAG_ACPI_CNVS		= 0x0041,
 	CB_TAG_TYPE_C_INFO		= 0x0042,
+	CB_TAG_ACPI_RSDP                = 0x0043,
+	CB_TAG_PCIE			= 0x0044,
 	CB_TAG_CMOS_OPTION_TABLE	= 0x00c8,
 	CB_TAG_OPTION			= 0x00c9,
 	CB_TAG_OPTION_ENUM		= 0x00ca,
@@ -90,10 +92,7 @@ enum {
 	CB_TAG_OPTION_CHECKSUM		= 0x00cc,
 };
 
-struct cbuint64 {
-	u32 lo;
-	u32 hi;
-};
+typedef __aligned(4) uint64_t cb_uint64_t;
 
 struct cb_header {
 	u8 signature[4];
@@ -110,8 +109,8 @@ struct cb_record {
 };
 
 struct cb_memory_range {
-	struct cbuint64 start;
-	struct cbuint64 size;
+	cb_uint64_t start;
+	cb_uint64_t size;
 	u32 type;
 };
 
@@ -126,7 +125,7 @@ struct cb_memory_range {
 struct cb_memory {
 	u32 tag;
 	u32 size;
-	struct cb_memory_range map[0];
+	struct cb_memory_range map[];
 };
 
 struct cb_hwrpb {
@@ -140,7 +139,7 @@ struct cb_mainboard {
 	u32 size;
 	u8 vendor_idx;
 	u8 part_number_idx;
-	u8 strings[0];
+	u8 strings[];
 };
 
 enum type_c_orientation {
@@ -167,13 +166,13 @@ struct type_c_port_info {
 
 struct type_c_info {
 	u32 port_count;
-	struct type_c_port_info port_info[0];
+	struct type_c_port_info port_info[];
 };
 
 struct cb_string {
 	u32 tag;
 	u32 size;
-	u8 string[0];
+	u8 string[];
 };
 
 struct cb_serial {
@@ -192,14 +191,6 @@ struct cb_serial {
 	 * decisions as to which dividers to select and their values
 	 * to eventually arrive at the desired console baud-rate. */
 	u32 input_hertz;
-
-	/* UART PCI address: bus, device, function
-	 * 1 << 31 - Valid bit, PCI UART in use
-	 * Bus << 20
-	 * Device << 15
-	 * Function << 12
-	 */
-	u32 uart_pci_addr;
 };
 
 struct cb_console {
@@ -264,20 +255,26 @@ struct cb_gpios {
 	u32 size;
 
 	u32 count;
-	struct cb_gpio gpios[0];
+	struct cb_gpio gpios[];
+};
+
+struct cb_pcie {
+	uint32_t tag;
+	uint32_t size;
+	cb_uint64_t ctrl_base;	/* Base address of PCIe controller */
 };
 
 struct lb_range {
 	uint32_t tag;
 	uint32_t size;
-	uint64_t range_start;
+	cb_uint64_t range_start;
 	uint32_t range_size;
 };
 
 struct cb_cbmem_tab {
 	uint32_t tag;
 	uint32_t size;
-	uint64_t cbmem_tab;
+	cb_uint64_t cbmem_tab;
 };
 
 struct cb_x86_rom_mtrr {
@@ -308,17 +305,27 @@ struct cb_spi_flash {
 	 * space and host address space. This determines the number of entries in mmap_table.
 	 */
 	uint32_t mmap_count;
-	struct flash_mmap_window mmap_table[0];
+	struct flash_mmap_window mmap_table[];
 };
 
 struct cb_boot_media_params {
 	uint32_t tag;
 	uint32_t size;
 	/* offsets are relative to start of boot media */
-	uint64_t fmap_offset;
-	uint64_t cbfs_offset;
-	uint64_t cbfs_size;
-	uint64_t boot_media_size;
+	cb_uint64_t fmap_offset;
+	cb_uint64_t cbfs_offset;
+	cb_uint64_t cbfs_size;
+	cb_uint64_t boot_media_size;
+};
+
+
+struct cb_cbmem_entry {
+	uint32_t tag;
+	uint32_t size;
+
+	cb_uint64_t address;
+	uint32_t entry_size;
+	uint32_t id;
 };
 
 struct cb_tsc_info {
@@ -337,7 +344,7 @@ struct cb_macs {
 	uint32_t tag;
 	uint32_t size;
 	uint32_t count;
-	struct mac_address mac_addrs[0];
+	struct mac_address mac_addrs[];
 };
 
 struct cb_mmc_info {
@@ -358,7 +365,7 @@ struct cb_board_config {
 	uint32_t tag;
 	uint32_t size;
 
-	struct cbuint64 fw_config;
+	cb_uint64_t fw_config;
 	uint32_t board_id;
 	uint32_t ram_code;
 	uint32_t sku_id;
@@ -412,12 +419,17 @@ struct	cb_cmos_checksum {
 	u32 type;
 };
 
-/* Helpful inlines */
+/*
+ * Handoff the ACPI RSDP
+ */
+struct cb_acpi_rsdp {
+	uint32_t tag;
+	uint32_t size;
+	cb_uint64_t rsdp_pointer; /* Address of the ACPI RSDP */
+};
 
-static inline u64 cb_unpack64(struct cbuint64 val)
-{
-	return (((u64) val.hi) << 32) | val.lo;
-}
+
+/* Helpful inlines */
 
 static inline u16 cb_checksum(const void *ptr, unsigned len)
 {
@@ -443,6 +455,4 @@ static inline const char *cb_mb_part_string(const struct cb_mainboard *cbm)
 	(void *)(((u8 *) (_rec)) + sizeof(*(_rec)) \
 		+ (sizeof((_rec)->map[0]) * (_idx)))
 
-/* Helper functions */
-uintptr_t get_cbmem_addr(const void *cbmem_tab_entry);
 #endif
