@@ -488,7 +488,6 @@ void x86_exception(struct eregs *info)
 #else /* !CONFIG_GDB_STUB */
 
 	int logical_processor = 0;
-	u32 apic_id = CONFIG(SMP) ? lapicid() : 0;
 
 	if (info->vector == DEBUG_VECTOR) {
 		if (breakpoint_dispatch_handler(info) == 0)
@@ -513,7 +512,7 @@ void x86_exception(struct eregs *info)
 		"r10: %016llx r11: %016llx\n"
 		"r12: %016llx r13: %016llx\n"
 		"r14: %016llx r15: %016llx\n",
-		logical_processor, apic_id,
+		logical_processor, early_lapicid(),
 		info->vector, info->cs, info->rip,
 		info->error_code, info->rflags, read_cr2(),
 		info->rax, info->rbx, info->rcx, info->rdx,
@@ -530,7 +529,7 @@ void x86_exception(struct eregs *info)
 		"Code: %d eflags: %08x cr2: %08x\n"
 		"eax: %08x ebx: %08x ecx: %08x edx: %08x\n"
 		"edi: %08x esi: %08x ebp: %08x esp: %08x\n",
-		logical_processor, apic_id,
+		logical_processor, early_lapicid(),
 		info->vector, info->cs, info->eip,
 		info->error_code, info->eflags, read_cr2(),
 		info->eax, info->ebx, info->ecx, info->edx,
