@@ -316,7 +316,12 @@ struct elog_event_mem_cache_update {
 #define   ELOG_CROS_DIAG_TYPE_STORAGE_TEST_EXTENDED 0x03
 #define   ELOG_CROS_DIAG_TYPE_MEMORY_QUICK          0x04
 #define   ELOG_CROS_DIAG_TYPE_MEMORY_FULL           0x05
-/* Result codes for elog_event_cros_diag_log in ELOG_CROS_DIAGNOSTICS_LOGS */
+/*
+ * Result codes for elog_event_cros_diag_log in ELOG_CROS_DIAGNOSTICS_LOGS
+ *
+ * These values are persisted to logs. Entries should not be renumbered and
+ * numeric values should never be reused.
+ */
 #define   ELOG_CROS_DIAG_RESULT_PASSED              0x01
 #define   ELOG_CROS_DIAG_RESULT_ERROR               0x02
 #define   ELOG_CROS_DIAG_RESULT_FAILED              0x03
@@ -355,7 +360,14 @@ struct elog_event_extended_event {
  * Firmware boot related information retrieved from vboot and store as
  * per `union vb2_fw_boot_info` data structure.
  */
-#define ELOG_TYPE_FW_VBOOT_INFO        0xb7
+#define ELOG_TYPE_FW_VBOOT_INFO           0xb7
+
+/*
+ * Events related to early Sign Of Life
+ */
+#define ELOG_TYPE_FW_EARLY_SOL            0xb8
+#define  ELOG_FW_EARLY_SOL_CSE_SYNC        0x0
+#define  ELOG_FW_EARLY_SOL_MRC             0x1
 
 /* Only the 7-LSB are used for size */
 #define ELOG_MAX_EVENT_SIZE                    0x7F
