@@ -77,7 +77,7 @@ enum {
 	LB_TAG_MAC_ADDRS		= 0x0033,
 	LB_TAG_VBOOT_WORKBUF		= 0x0034,
 	LB_TAG_MMC_INFO			= 0x0035,
-	LB_TAG_TCPA_LOG			= 0x0036,
+	LB_TAG_TPM_CB_LOG		= 0x0036,
 	LB_TAG_FMAP			= 0x0037,
 	LB_TAG_PLATFORM_BLOB_VERSION	= 0x0038,
 	LB_TAG_SMMSTOREV2		= 0x0039,
@@ -85,6 +85,8 @@ enum {
 	LB_TAG_BOARD_CONFIG		= 0x0040,
 	LB_TAG_ACPI_CNVS		= 0x0041,
 	LB_TAG_TYPE_C_INFO		= 0x0042,
+	LB_TAG_ACPI_RSDP		= 0x0043,
+	LB_TAG_PCIE			= 0x0044,
 	/* The following options are CMOS-related */
 	LB_TAG_CMOS_OPTION_TABLE	= 0x00c8,
 	LB_TAG_OPTION			= 0x00c9,
@@ -93,38 +95,19 @@ enum {
 	LB_TAG_OPTION_CHECKSUM		= 0x00cc,
 };
 
+/* All table entry base addresses and sizes must be 4-byte aligned. */
+#define LB_ENTRY_ALIGN 4
+
 /* Since coreboot is usually compiled 32bit, gcc will align 64bit
  * types to 32bit boundaries. If the coreboot table is dumped on a
  * 64bit system, a uint64_t would be aligned to 64bit boundaries,
  * breaking the table format.
  *
- * lb_uint64 will keep 64bit coreboot table values aligned to 32bit
- * to ensure compatibility. They can be accessed with the two functions
- * below: unpack_lb64() and pack_lb64()
- *
- * See also: util/lbtdump/lbtdump.c
+ * lb_uint64_t will keep 64bit coreboot table values aligned to 32bit
+ * to ensure compatibility.
  */
 
-struct lb_uint64 {
-	uint32_t lo;
-	uint32_t hi;
-};
-
-static inline uint64_t unpack_lb64(struct lb_uint64 value)
-{
-	uint64_t result;
-	result = value.hi;
-	result = (result << 32) + value.lo;
-	return result;
-}
-
-static inline struct lb_uint64 pack_lb64(uint64_t value)
-{
-	struct lb_uint64 result;
-	result.lo = (value >> 0) & 0xffffffff;
-	result.hi = (value >> 32) & 0xffffffff;
-	return result;
-}
+typedef __aligned(LB_ENTRY_ALIGN) uint64_t lb_uint64_t;
 
 struct lb_header {
 	uint8_t  signature[4]; /* LBIO */
@@ -147,8 +130,8 @@ struct lb_record {
 };
 
 struct lb_memory_range {
-	struct lb_uint64 start;
-	struct lb_uint64 size;
+	lb_uint64_t start;
+	lb_uint64_t size;
 	uint32_t type;
 #define LB_MEM_RAM		 1	/* Memory anyone can use */
 #define LB_MEM_RESERVED		 2	/* Don't use this memory region */
@@ -157,18 +140,27 @@ struct lb_memory_range {
 #define LB_MEM_UNUSABLE		 5	/* Unusable address space */
 #define LB_MEM_VENDOR_RSVD	 6	/* Vendor Reserved */
 #define LB_MEM_TABLE		16    /* Ram configuration tables are kept in */
+#define LB_MEM_SOFT_RESERVED	0xefffffff /* Specific purpose memory */
 };
 
 struct lb_memory {
 	uint32_t tag;
 	uint32_t size;
-	struct lb_memory_range map[0];
+	struct lb_memory_range map[];
 };
 
+struct lb_pcie {
+	uint32_t tag;
+	uint32_t size;
+	lb_uint64_t ctrl_base;		/* Base address of PCIe controller */
+};
+_Static_assert(_Alignof(struct lb_pcie) == 4,
+	       "lb_uint64_t alignment doesn't work as expected for struct lb_pcie!");
+
 struct lb_hwrpb {
 	uint32_t tag;
 	uint32_t size;
-	uint64_t hwrpb;
+	lb_uint64_t hwrpb;
 };
 
 struct lb_mainboard {
@@ -176,13 +168,13 @@ struct lb_mainboard {
 	uint32_t size;
 	uint8_t  vendor_idx;
 	uint8_t  part_number_idx;
-	uint8_t  strings[0];
+	uint8_t  strings[];
 };
 
 struct lb_string {
 	uint32_t tag;
 	uint32_t size;
-	uint8_t  string[0];
+	uint8_t  string[];
 };
 
 struct lb_timestamp {
@@ -209,20 +201,13 @@ struct lb_serial {
 	 * decisions as to which dividers to select and their values
 	 * to eventually arrive at the desired console baud-rate. */
 	uint32_t input_hertz;
-
-	/* UART PCI address: bus, device, function
-	 * 1 << 31 - Valid bit, PCI UART in use
-	 * Bus << 20
-	 * Device << 15
-	 * Function << 12
-	 */
-	uint32_t uart_pci_addr;
 };
 
 struct lb_console {
 	uint32_t tag;
 	uint32_t size;
 	uint16_t type;
+	uint8_t pad[2];
 };
 
 #define LB_TAG_CONSOLE_SERIAL8250	0
@@ -236,7 +221,7 @@ struct lb_console {
 struct lb_forward {
 	uint32_t tag;
 	uint32_t size;
-	uint64_t forward;
+	lb_uint64_t forward;
 };
 
 /**
@@ -294,7 +279,7 @@ struct lb_framebuffer {
 	uint32_t tag;
 	uint32_t size;
 
-	uint64_t physical_address;
+	lb_uint64_t physical_address;
 	uint32_t x_resolution;
 	uint32_t y_resolution;
 	uint32_t bytes_per_line;
@@ -308,6 +293,7 @@ struct lb_framebuffer {
 	uint8_t reserved_mask_pos;
 	uint8_t reserved_mask_size;
 	uint8_t orientation;
+	uint8_t pad[2];
 };
 
 struct lb_gpio {
@@ -325,14 +311,14 @@ struct lb_gpios {
 	uint32_t size;
 
 	uint32_t count;
-	struct lb_gpio gpios[0];
+	struct lb_gpio gpios[];
 };
 
 struct lb_range {
 	uint32_t tag;
 	uint32_t size;
 
-	uint64_t range_start;
+	lb_uint64_t range_start;
 	uint32_t range_size;
 };
 
@@ -342,7 +328,7 @@ struct lb_cbmem_ref {
 	uint32_t tag;
 	uint32_t size;
 
-	uint64_t cbmem_addr;
+	lb_uint64_t cbmem_addr;
 };
 
 struct lb_x86_rom_mtrr {
@@ -371,17 +357,17 @@ struct lb_spi_flash {
 	 */
 
 	uint32_t mmap_count;
-	struct flash_mmap_window mmap_table[0];
+	struct flash_mmap_window mmap_table[];
 };
 
 struct lb_boot_media_params {
 	uint32_t tag;
 	uint32_t size;
 	/* offsets are relative to start of boot media */
-	uint64_t fmap_offset;
-	uint64_t cbfs_offset;
-	uint64_t cbfs_size;
-	uint64_t boot_media_size;
+	lb_uint64_t fmap_offset;
+	lb_uint64_t cbfs_offset;
+	lb_uint64_t cbfs_size;
+	lb_uint64_t boot_media_size;
 };
 
 /*
@@ -391,7 +377,7 @@ struct lb_cbmem_entry {
 	uint32_t tag;
 	uint32_t size;
 
-	uint64_t address;
+	lb_uint64_t address;
 	uint32_t entry_size;
 	uint32_t id;
 };
@@ -446,21 +432,21 @@ struct type_c_port_info {
 
 struct type_c_info {
 	uint32_t port_count;
-	struct type_c_port_info port_info[0];
+	struct type_c_port_info port_info[];
 };
 
 struct lb_macs {
 	uint32_t tag;
 	uint32_t size;
 	uint32_t count;
-	struct mac_address mac_addrs[0];
+	struct mac_address mac_addrs[];
 };
 
 struct lb_board_config {
 	uint32_t tag;
 	uint32_t size;
 
-	struct lb_uint64 fw_config;
+	lb_uint64_t fw_config;
 	uint32_t board_id;
 	uint32_t ram_code;
 	uint32_t sku_id;
@@ -551,7 +537,7 @@ struct lb_smmstorev2 {
 	uint8_t unused[3];		/* Set to zero */
 };
 
-enum lb_tmp_ppi_tpm_version {
+enum lb_tpm_ppi_tpm_version {
 	LB_TPM_VERSION_UNSPEC = 0,
 	LB_TPM_VERSION_TPM_VERSION_1_2,
 	LB_TPM_VERSION_TPM_VERSION_2,
@@ -573,5 +559,17 @@ struct lb_tpm_physical_presence {
 	uint32_t ppi_address;	/* Address of ACPI PPI communication buffer */
 	uint8_t tpm_version;	/* 1: TPM1.2, 2: TPM2.0 */
 	uint8_t ppi_version;	/* BCD encoded */
-} __packed;
+	uint8_t pad[2];
+};
+
+
+/*
+ * Handoff the ACPI RSDP
+ */
+struct lb_acpi_rsdp {
+	uint32_t tag;
+	uint32_t size;
+	lb_uint64_t rsdp_pointer; /* Address of the ACPI RSDP */
+};
+
 #endif
