@@ -16,7 +16,7 @@ struct timestamp_table {
 	uint16_t	max_entries;
 	uint16_t	tick_freq_mhz;
 	uint32_t	num_entries;
-	struct timestamp_entry entries[0]; /* Variable number of entries */
+	struct timestamp_entry entries[]; /* Variable number of entries */
 } __packed;
 
 enum timestamp_id {
@@ -143,6 +143,7 @@ enum timestamp_id {
 
 	/* 990+ reserved for vendorcode extensions (990-999: Intel ME continued) */
 	TS_ME_ROM_START = 990,
+	TS_ISSE_DMU_LOAD_END = 991,
 
 	/* 1000+ reserved for payloads */
 
@@ -332,6 +333,7 @@ static const struct timestamp_id_to_name {
 
 	/* Intel ME continued */
 	TS_NAME_DEF(TS_ME_ROM_START, 0, "CSME ROM started execution"),
+	TS_NAME_DEF(TS_ISSE_DMU_LOAD_END, 0, "Die Management Unit (DMU) load completed"),
 
 	/* Depthcharge entry timestamp */
 	TS_NAME_DEF(TS_DC_START, 0, "depthcharge start"),
