@@ -1,12 +1,12 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
-#include <console/console.h>
 #include <acpi/acpi.h>
 #include <acpi/acpigen.h>
-#include <arch/cpu.h>
-#include <cpu/x86/msr.h>
+#include <console/console.h>
+#include <cpu/cpu.h>
 #include <cpu/intel/speedstep.h>
 #include <cpu/intel/turbo.h>
+#include <cpu/x86/msr.h>
 #include <device/device.h>
 #include <stdint.h>
 
@@ -95,18 +95,9 @@ static int get_logical_cores_per_package(void)
 	return msr.lo & 0xffff;
 }
 
-static void generate_C_state_entries(void)
+static void generate_C_state_entries(const struct device *dev)
 {
-	struct device *lapic;
-	struct cpu_intel_model_206ax_config *conf = NULL;
-
-	/* Find the SpeedStep CPU in the device tree using magic APIC ID */
-	lapic = dev_find_lapic(SPEEDSTEP_APIC_MAGIC);
-	if (!lapic)
-		return;
-	conf = lapic->chip_info;
-	if (!conf)
-		return;
+	struct cpu_intel_model_206ax_config *conf = dev->chip_info;
 
 	const int acpi_cstates[3] = { conf->acpi_c1, conf->acpi_c2, conf->acpi_c3 };
 
@@ -303,36 +294,35 @@ static void generate_P_state_entries(int core, int cores_per_package)
 	acpigen_pop_len();
 }
 
+static void generate_cpu_entry(const struct device *device, int cpu, int core, int cores_per_package)
+{
+	/* Generate Scope(\_SB) { Device(CPUx */
+	acpigen_write_processor_device(cpu * cores_per_package + core);
+
+	/* Generate P-state tables */
+	generate_P_state_entries(cpu, cores_per_package);
+
+	/* Generate C-state tables */
+	generate_C_state_entries(device);
+
+	/* Generate T-state tables */
+	generate_T_state_entries(cpu, cores_per_package);
+
+	acpigen_write_processor_device_end();
+}
+
 void generate_cpu_entries(const struct device *device)
 {
-	int coreID, cpuID;
 	int totalcores = dev_count_cpu();
 	int cores_per_package = get_logical_cores_per_package();
-	int numcpus = totalcores/cores_per_package;
+	int numcpus = totalcores / cores_per_package;
 
 	printk(BIOS_DEBUG, "Found %d CPU(s) with %d core(s) each.\n",
 	       numcpus, cores_per_package);
 
-	for (cpuID = 1; cpuID <= numcpus; cpuID++) {
-		for (coreID = 1; coreID <= cores_per_package; coreID++) {
-			/* Generate processor \_SB.CPUx */
-			acpigen_write_processor(
-				(cpuID-1)*cores_per_package+coreID-1, 0, 0);
-
-			/* Generate P-state tables */
-			generate_P_state_entries(
-				cpuID-1, cores_per_package);
-
-			/* Generate C-state tables */
-			generate_C_state_entries();
-
-			/* Generate T-state tables */
-			generate_T_state_entries(
-				cpuID-1, cores_per_package);
-
-			acpigen_pop_len();
-		}
-	}
+	for (int cpu_id = 0; cpu_id < numcpus; cpu_id++)
+		for (int core_id = 0; core_id < cores_per_package; core_id++)
+			generate_cpu_entry(device, cpu_id, core_id, cores_per_package);
 
 	/* PPKG is usually used for thermal management
 	   of the first and only package. */
