@@ -13,6 +13,7 @@
 #include <cpu/x86/msr.h>
 #include <cpu/x86/mtrr.h>
 #include <cpu/x86/smm.h>
+#include <cpu/x86/topology.h>
 #include <cpu/x86/mp.h>
 #include <delay.h>
 #include <device/device.h>
@@ -24,6 +25,9 @@
 #include <thread.h>
 #include <types.h>
 
+/* Generated header */
+#include <ramstage/cpu/x86/smm_start32_offset.h>
+
 #include <security/intel/stm/SmmStm.h>
 
 struct mp_callback {
@@ -107,7 +111,7 @@ struct saved_msr {
 /* The sipi vector rmodule is included in the ramstage using 'objdump -B'. */
 extern char _binary_sipi_vector_start[];
 
-/* The SIPI vector is loaded at the SMM_DEFAULT_BASE. The reason is at the
+/* The SIPI vector is loaded at the SMM_DEFAULT_BASE. The reason is that the
  * memory range is already reserved so the OS cannot use it. That region is
  * free to use for AP bringup before SMM is initialized. */
 static const uintptr_t sipi_vector_location = SMM_DEFAULT_BASE;
@@ -121,9 +125,6 @@ struct mp_flight_plan {
 static int global_num_aps;
 static struct mp_flight_plan mp_info;
 
-/* Keep track of device structure for each CPU. */
-static struct device *cpus_dev[CONFIG_MAX_CPUS];
-
 static inline void barrier_wait(atomic_t *b)
 {
 	while (atomic_read(b) == 0)
@@ -151,6 +152,7 @@ static enum cb_err wait_for_aps(atomic_t *val, int target, int total_delay,
 	}
 
 	/* APs ready before timeout */
+	printk(BIOS_SPEW, "APs are ready after %dus\n", delayed);
 	return CB_SUCCESS;
 }
 
@@ -174,29 +176,42 @@ static void park_this_cpu(void *unused)
 	stop_this_cpu();
 }
 
+static struct bus *g_cpu_bus;
+
 /* By the time APs call ap_init() caching has been setup, and microcode has
  * been loaded. */
-static void asmlinkage ap_init(void)
+static asmlinkage void ap_init(unsigned int index)
 {
-	struct cpu_info *info = cpu_info();
-
 	/* Ensure the local APIC is enabled */
 	enable_lapic();
 	setup_lapic_interrupts();
 
-	info->cpu = cpus_dev[info->index];
+	struct device *dev;
+	int i = 0;
+	for (dev = g_cpu_bus->children; dev; dev = dev->sibling)
+		if (i++ == index)
+			break;
 
-	cpu_add_map_entry(info->index);
+	if (!dev) {
+		printk(BIOS_ERR, "Could not find allocated device for index %u\n", index);
+		return;
+	}
+
+	set_cpu_info(index, dev);
 
 	/* Fix up APIC id with reality. */
-	info->cpu->path.apic.apic_id = lapicid();
+	dev->path.apic.apic_id = lapicid();
+	dev->path.apic.initial_lapicid = initial_lapicid();
+	dev->enabled = 1;
+
+	set_cpu_topology_from_leaf_b(dev);
 
 	if (cpu_is_intel())
-		printk(BIOS_INFO, "AP: slot %zu apic_id %x, MCU rev: 0x%08x\n", info->index,
-		       info->cpu->path.apic.apic_id, get_current_microcode_rev());
+		printk(BIOS_INFO, "AP: slot %u apic_id %x, MCU rev: 0x%08x\n", index,
+		       dev->path.apic.apic_id, get_current_microcode_rev());
 	else
-		printk(BIOS_INFO, "AP: slot %zu apic_id %x\n", info->index,
-		       info->cpu->path.apic.apic_id);
+		printk(BIOS_INFO, "AP: slot %u apic_id %x\n", index,
+		       dev->path.apic.apic_id);
 
 	/* Walk the flight plan */
 	ap_do_flight_plan();
@@ -205,6 +220,8 @@ static void asmlinkage ap_init(void)
 	park_this_cpu(NULL);
 }
 
+static __aligned(16) uint8_t ap_stack[CONFIG_AP_STACK_SIZE * CONFIG_MAX_CPUS];
+
 static void setup_default_sipi_vector_params(struct sipi_params *sp)
 {
 	sp->gdt = (uintptr_t)&gdt;
@@ -212,8 +229,8 @@ static void setup_default_sipi_vector_params(struct sipi_params *sp)
 	sp->idt_ptr = (uintptr_t)&idtarg;
 	sp->per_cpu_segment_descriptors = (uintptr_t)&per_cpu_segment_descriptors;
 	sp->per_cpu_segment_selector = per_cpu_segment_selector;
-	sp->stack_size = CONFIG_STACK_SIZE;
-	sp->stack_top = ALIGN_DOWN((uintptr_t)&_estack, CONFIG_STACK_SIZE);
+	sp->stack_size = CONFIG_AP_STACK_SIZE;
+	sp->stack_top = (uintptr_t)ap_stack + ARRAY_SIZE(ap_stack);
 }
 
 static const unsigned int fixed_mtrrs[NUM_FIXED_MTRRS] = {
@@ -347,6 +364,13 @@ static atomic_t *load_sipi_vector(struct mp_params *mp_params)
 	ap_count = &sp->ap_count;
 	atomic_set(ap_count, 0);
 
+	/* Make sure SIPI data hits RAM so the APs that come up will see the
+	   startup code even if the caches are disabled. */
+	if (clflush_supported())
+		clflush_region((uintptr_t)mod_loc, module_size);
+	else
+		wbinvd();
+
 	return ap_count;
 }
 
@@ -365,25 +389,16 @@ static int allocate_cpu_devices(struct bus *cpu_bus, struct mp_params *p)
 
 	info = cpu_info();
 	for (i = 1; i < max_cpus; i++) {
-		struct device_path cpu_path;
-		struct device *new;
-
-		/* Build the CPU device path */
-		cpu_path.type = DEVICE_PATH_APIC;
-
 		/* Assuming linear APIC space allocation. AP will set its own
 		   APIC id in the ap_init() path above. */
-		cpu_path.apic.apic_id = info->cpu->path.apic.apic_id + i;
-
-		/* Allocate the new CPU device structure */
-		new = alloc_find_dev(cpu_bus, &cpu_path);
+		struct device *new = add_cpu_device(cpu_bus, info->cpu->path.apic.apic_id + i, 1);
 		if (new == NULL) {
 			printk(BIOS_CRIT, "Could not allocate CPU device\n");
 			max_cpus--;
 			continue;
 		}
 		new->name = processor_name;
-		cpus_dev[i] = new;
+		new->enabled = 0; /* Runtime will enable it */
 	}
 
 	return max_cpus;
@@ -430,7 +445,7 @@ static enum cb_err send_sipi_to_aps(int ap_count, atomic_t *num_aps, int sipi_ve
 
 static enum cb_err start_aps(struct bus *cpu_bus, int ap_count, atomic_t *num_aps)
 {
-	int sipi_vector;
+	int sipi_vector, total_delay;
 	/* Max location is 4KiB below 1MiB */
 	const int max_vector_loc = ((1 << 20) - (1 << 12)) >> 12;
 
@@ -468,7 +483,7 @@ static enum cb_err start_aps(struct bus *cpu_bus, int ap_count, atomic_t *num_ap
 		if (send_sipi_to_aps(ap_count, num_aps, sipi_vector) != CB_SUCCESS)
 			return CB_ERR;
 
-		/* Wait for CPUs to check in up to 200 us. */
+		/* Wait for CPUs to check in. */
 		wait_for_aps(num_aps, ap_count, 200 /* us */, 15 /* us */);
 	}
 
@@ -477,7 +492,8 @@ static enum cb_err start_aps(struct bus *cpu_bus, int ap_count, atomic_t *num_ap
 		return CB_ERR;
 
 	/* Wait for CPUs to check in. */
-	if (wait_for_aps(num_aps, ap_count, 100000 /* 100 ms */, 50 /* us */) != CB_SUCCESS) {
+	total_delay = 50000 * ap_count; /* 50 ms per AP */
+	if (wait_for_aps(num_aps, ap_count, total_delay, 50 /* us */) != CB_SUCCESS) {
 		printk(BIOS_ERR, "Not all APs checked in: %d/%d.\n",
 		       atomic_read(num_aps), ap_count);
 		return CB_ERR;
@@ -527,9 +543,8 @@ static enum cb_err bsp_do_flight_plan(struct mp_params *mp_params)
 	return ret;
 }
 
-static void init_bsp(struct bus *cpu_bus)
+static enum cb_err init_bsp(struct bus *cpu_bus)
 {
-	struct device_path cpu_path;
 	struct cpu_info *info;
 
 	/* Print processor name */
@@ -540,20 +555,25 @@ static void init_bsp(struct bus *cpu_bus)
 	enable_lapic();
 	setup_lapic_interrupts();
 
-	/* Set the device path of the boot CPU. */
-	cpu_path.type = DEVICE_PATH_APIC;
-	cpu_path.apic.apic_id = lapicid();
+	struct device *bsp = add_cpu_device(cpu_bus, lapicid(), 1);
+	if (bsp == NULL) {
+		printk(BIOS_CRIT, "Failed to find or allocate BSP struct device\n");
+		return CB_ERR;
+	}
+	bsp->path.apic.initial_lapicid = initial_lapicid();
+	set_cpu_topology_from_leaf_b(bsp);
 
 	/* Find the device structure for the boot CPU. */
+	set_cpu_info(0, bsp);
 	info = cpu_info();
-	info->cpu = alloc_find_dev(cpu_bus, &cpu_path);
+	info->cpu = bsp;
 	info->cpu->name = processor_name;
 
-	if (info->index != 0)
+	if (info->index != 0) {
 		printk(BIOS_CRIT, "BSP index(%zd) != 0!\n", info->index);
-
-	/* Track BSP in cpu_map structures. */
-	cpu_add_map_entry(info->index);
+		return CB_ERR;
+	}
+	return CB_SUCCESS;
 }
 
 /*
@@ -578,7 +598,12 @@ static enum cb_err mp_init(struct bus *cpu_bus, struct mp_params *p)
 	int num_cpus;
 	atomic_t *ap_count;
 
-	init_bsp(cpu_bus);
+	g_cpu_bus = cpu_bus;
+
+	if (init_bsp(cpu_bus) != CB_SUCCESS) {
+		printk(BIOS_CRIT, "Setting up BSP failed\n");
+		return CB_ERR;
+	}
 
 	if (p == NULL || p->flight_plan == NULL || p->num_records < 1) {
 		printk(BIOS_CRIT, "Invalid MP parameters\n");
@@ -608,10 +633,6 @@ static enum cb_err mp_init(struct bus *cpu_bus, struct mp_params *p)
 	if (ap_count == NULL)
 		return CB_ERR;
 
-	/* Make sure SIPI data hits RAM so the APs that come up will see
-	 * the startup code even if the caches are disabled.  */
-	wbinvd();
-
 	/* Start the APs providing number of APs and the cpus_entered field. */
 	global_num_aps = p->num_cpus - 1;
 	if (start_aps(cpu_bus, global_num_aps, ap_count) != CB_SUCCESS) {
@@ -625,14 +646,6 @@ static enum cb_err mp_init(struct bus *cpu_bus, struct mp_params *p)
 	return bsp_do_flight_plan(p);
 }
 
-/* Calls cpu_initialize(info->index) which calls the coreboot CPU drivers. */
-static void mp_initialize_cpu(void)
-{
-	/* Call back into driver infrastructure for the AP initialization.   */
-	struct cpu_info *info = cpu_info();
-	cpu_initialize(info->index);
-}
-
 void smm_initiate_relocation_parallel(void)
 {
 	if (lapic_busy()) {
@@ -671,7 +684,6 @@ struct mp_state {
 	uintptr_t perm_smbase;
 	size_t perm_smsize;
 	size_t smm_save_state_size;
-	uintptr_t reloc_start32_offset;
 	bool do_smm;
 } mp_state;
 
@@ -696,7 +708,7 @@ static void smm_enable(void)
  * means that ENV_SMM is 0, but we are actually executing in the environment
  * setup by the smm_stub.
  */
-static void asmlinkage smm_do_relocation(void *arg)
+static asmlinkage void smm_do_relocation(void *arg)
 {
 	const struct smm_module_params *p;
 	int cpu;
@@ -737,19 +749,10 @@ static void asmlinkage smm_do_relocation(void *arg)
 		stm_setup(mseg, p->cpu,
 				perm_smbase,
 				mp_state.perm_smbase,
-				mp_state.reloc_start32_offset);
+				SMM_START32_OFFSET);
 	}
 }
 
-static void adjust_smm_apic_id_map(struct smm_loader_params *smm_params)
-{
-	int i;
-	struct smm_stub_params *stub_params = smm_params->stub_params;
-
-	for (i = 0; i < CONFIG_MAX_CPUS; i++)
-		stub_params->apic_id_to_cpu[i] = cpu_get_apic_id(i);
-}
-
 static enum cb_err install_relocation_handler(int num_cpus, size_t save_state_size)
 {
 	if (CONFIG(X86_SMM_SKIP_RELOCATION_HANDLER))
@@ -766,9 +769,6 @@ static enum cb_err install_relocation_handler(int num_cpus, size_t save_state_si
 		printk(BIOS_ERR, "%s: smm setup failed\n", __func__);
 		return CB_ERR;
 	}
-	adjust_smm_apic_id_map(&smm_params);
-
-	mp_state.reloc_start32_offset = smm_params.stub_params->start32_offset;
 
 	return CB_SUCCESS;
 }
@@ -794,8 +794,6 @@ static enum cb_err install_permanent_handler(int num_cpus, uintptr_t smbase,
 	if (smm_load_module(smbase, smsize, &smm_params))
 		return CB_ERR;
 
-	adjust_smm_apic_id_map(&smm_params);
-
 	return CB_SUCCESS;
 }
 
@@ -1075,7 +1073,7 @@ static struct mp_flight_record mp_steps[] = {
 	/* Perform SMM relocation. */
 	MP_FR_NOBLOCK_APS(trigger_smm_relocation, trigger_smm_relocation),
 	/* Initialize each CPU through the driver framework. */
-	MP_FR_BLOCK_APS(mp_initialize_cpu, mp_initialize_cpu),
+	MP_FR_BLOCK_APS(cpu_initialize, cpu_initialize),
 	/* Wait for APs to finish then optionally start looking for work. */
 	MP_FR_BLOCK_APS(ap_wait_for_instruction, NULL),
 };
