@@ -1,11 +1,14 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <arch/io.h>
+#include <commonlib/bsd/compiler.h>
+#include <commonlib/region.h>
 #include <console/cbmem_console.h>
 #include <console/console.h>
-#include <commonlib/region.h>
+#include <cpu/cpu.h>
 #include <cpu/x86/smm.h>
 #include <rmodule.h>
+#include <types.h>
 
 #if CONFIG(SPI_FLASH_SMM)
 #include <spi-generic.h>
@@ -49,6 +52,13 @@ static void smi_release_lock(void)
 	);
 }
 
+#if CONFIG(RUNTIME_CONFIGURABLE_SMM_LOGLEVEL)
+int get_console_loglevel(void)
+{
+	return smm_runtime.smm_log_level;
+}
+#endif
+
 void smm_get_cbmemc_buffer(void **buffer_out, size_t *size_out)
 {
 	*buffer_out = smm_runtime.cbmemc;
@@ -62,23 +72,12 @@ void io_trap_handler(int smif)
 	 */
 	printk(BIOS_DEBUG, "SMI function trap 0x%x: ", smif);
 
-	if (southbridge_io_trap_handler(smif))
-		return;
-
 	if (mainboard_io_trap_handler(smif))
 		return;
 
 	printk(BIOS_DEBUG, "Unknown function\n");
 }
 
-/**
- * @brief Set the EOS bit
- */
-static void smi_set_eos(void)
-{
-	southbridge_smi_set_eos();
-}
-
 static u32 pci_orig;
 
 /**
@@ -193,9 +192,16 @@ asmlinkage void smm_handler_start(void *arg)
 	smi_release_lock();
 
 	/* De-assert SMI# signal to allow another SMI */
-	smi_set_eos();
+	southbridge_smi_set_eos();
 }
 
+#if CONFIG(SMM_PCI_RESOURCE_STORE)
+const volatile struct smm_pci_resource_info *smm_get_pci_resource_store(void)
+{
+	return &smm_runtime.pci_resources[0];
+}
+#endif
+
 RMODULE_ENTRY(smm_handler_start);
 
 /* Provide a default implementation for all weak handlers so that relocation
