@@ -60,24 +60,14 @@ struct rom_header *pci_rom_probe(const struct device *dev)
 
 	/* If the ROM is in flash, then don't check the PCI device for it. */
 	if (CONFIG(CHECK_REV_IN_OPROM_NAME)) {
-		rom_header = cbfs_boot_map_optionrom_revision(dev->vendor, dev->device, rev);
 		map_oprom_vendev_rev(&mapped_vendev, &mapped_rev);
+		rom_header = cbfs_boot_map_optionrom_revision(mapped_vendev >> 16,
+							      mapped_vendev & 0xffff,
+							      mapped_rev);
 	} else {
-		rom_header = cbfs_boot_map_optionrom(dev->vendor, dev->device);
 		mapped_vendev = map_oprom_vendev(vendev);
-	}
-
-	if (!rom_header) {
-		if (CONFIG(CHECK_REV_IN_OPROM_NAME) &&
-				(vendev != mapped_vendev || rev != mapped_rev)) {
-			rom_header = cbfs_boot_map_optionrom_revision(
-					mapped_vendev >> 16,
-					mapped_vendev & 0xffff, mapped_rev);
-		} else if (vendev != mapped_vendev) {
-			rom_header = cbfs_boot_map_optionrom(
-					mapped_vendev >> 16,
-					mapped_vendev & 0xffff);
-		}
+		rom_header = cbfs_boot_map_optionrom(mapped_vendev >> 16,
+						     mapped_vendev & 0xffff);
 	}
 
 	if (rom_header) {
@@ -159,10 +149,10 @@ struct rom_header *pci_rom_load(struct device *dev,
 
 	do {
 		/* Get next image. */
-		rom_header = (struct rom_header *)((void *) rom_header
+		rom_header = (struct rom_header *)((void *)rom_header
 							    + image_size);
 
-		rom_data = (struct pci_data *)((void *) rom_header
+		rom_data = (struct pci_data *)((void *)rom_header
 				+ le32_to_cpu(rom_header->data));
 
 		image_size = le32_to_cpu(rom_data->ilen) * 512;
@@ -190,7 +180,7 @@ struct rom_header *pci_rom_load(struct device *dev,
 			memcpy((void *)PCI_VGA_RAM_IMAGE_START, rom_header,
 			       rom_size);
 		}
-		return (struct rom_header *) (PCI_VGA_RAM_IMAGE_START);
+		return (struct rom_header *)(PCI_VGA_RAM_IMAGE_START);
 	}
 
 	printk(BIOS_DEBUG, "Copying non-VGA ROM image from %p to %p, 0x%x bytes\n",
@@ -198,7 +188,7 @@ struct rom_header *pci_rom_load(struct device *dev,
 
 	memcpy(pci_ram_image_start, rom_header, rom_size);
 	pci_ram_image_start += rom_size;
-	return (struct rom_header *) (pci_ram_image_start-rom_size);
+	return (struct rom_header *)(pci_ram_image_start-rom_size);
 }
 
 /* ACPI */
