@@ -12,7 +12,7 @@
 #define MAX_LINK_FREQ_ENTRIES	4
 #define MAX_CLK_CONFIGS		2
 #define MAX_GPIO_CONFIGS	4
-#define MAX_PWR_OPS		5
+#define MAX_PWR_OPS		6
 #define MAX_GUARDED_RESOURCES	10
 #define IMGCLKOUT_0		0
 #define IMGCLKOUT_1		1
@@ -257,6 +257,17 @@ struct drivers_intel_mipi_camera_config {
 	bool has_power_resource;
 	/* Perform low power probe */
 	bool low_power_probe;
+	/*
+	 * This will create a _DSC method in ACPI which returns an integer, to tell the kernel
+	 * the highest allowed D state for a device during probe
+	 * Number   State   Description
+	 * 0	    D0	    Device fully powered on
+	 * 1	    D1
+	 * 2	    D2
+	 * 3	    D3hot
+	 * 4	    D3cold  Off
+	 */
+	uint8_t max_dstate_for_probe;
 };
 
 #endif
