@@ -28,6 +28,7 @@
 #define TPM_ACCESS_REG    (TPM_LOCALITY_0_SPI_BASE + 0)
 #define TPM_STS_REG       (TPM_LOCALITY_0_SPI_BASE + 0x18)
 #define TPM_DATA_FIFO_REG (TPM_LOCALITY_0_SPI_BASE + 0x24)
+#define TPM_INTF_ID_REG   (TPM_LOCALITY_0_SPI_BASE + 0x30)
 #define TPM_DID_VID_REG   (TPM_LOCALITY_0_SPI_BASE + 0xf00)
 #define TPM_RID_REG       (TPM_LOCALITY_0_SPI_BASE + 0xf04)
 #define TPM_FW_VER	  (TPM_LOCALITY_0_SPI_BASE + 0xf90)
@@ -60,23 +61,6 @@ void tpm2_get_info(struct tpm2_info *info)
 	*info = tpm_info;
 }
 
-__weak int tis_plat_irq_status(void)
-{
-	static int warning_displayed;
-
-	if (!CONFIG(TPM_GOOGLE))
-		dead_code();
-
-	if (!warning_displayed) {
-		printk(BIOS_WARNING, "%s() not implemented, wasting 10ms to wait on"
-		       " Cr50!\n", __func__);
-		warning_displayed = 1;
-	}
-	mdelay(10);
-
-	return 1;
-}
-
 /*
  * Each TPM2 SPI transaction starts the same: CS is asserted, the 4 byte
  * header is sent to the TPM, the master waits til TPM is ready to continue.
@@ -412,7 +396,7 @@ static const uint32_t supported_did_vids[] = {
 
 int tpm2_init(struct spi_slave *spi_if)
 {
-	uint32_t did_vid, status;
+	uint32_t did_vid, status, intf_id;
 	uint8_t cmd;
 	int retries;
 
@@ -420,7 +404,7 @@ int tpm2_init(struct spi_slave *spi_if)
 
 	/* Clear any pending IRQs. */
 	if (CONFIG(TPM_GOOGLE))
-		tis_plat_irq_status();
+		cr50_plat_irq_status();
 
 	/*
 	 * 150 ms should be enough to synchronize with the TPM even under the
@@ -454,6 +438,20 @@ int tpm2_init(struct spi_slave *spi_if)
 
 	printk(BIOS_INFO, " done!\n");
 
+	/* Google TPMs haven't always been 100% accurate in reflecting the spec (particularly
+	 * on older versions) and are always TPM 2.0. */
+	if (!CONFIG(TPM_GOOGLE)) {
+		if (tpm2_read_reg(TPM_INTF_ID_REG, &intf_id, sizeof(intf_id)) != CB_SUCCESS) {
+			printk(BIOS_ERR, "\n%s: Failed to read interface ID register\n",
+			       __func__);
+			return -1;
+		}
+		if ((be32toh(intf_id) & 0xF) == 0xF) {
+			printk(BIOS_DEBUG, "\n%s: Not a TPM2 device\n", __func__);
+			return -1;
+		}
+	}
+
 	// FIXME: Move this to tpm_setup()
 	if (tpm_first_access_this_boot())
 		/*
