@@ -6,6 +6,7 @@
 #include <ec/acpi/ec.h>
 #include <option.h>
 #include <pc80/keyboard.h>
+#include <halt.h>
 
 #include "ec.h"
 #include "ecdefs.h"
@@ -26,6 +27,62 @@ static uint8_t get_ec_value_from_option(const char *name,
 	return lut[index];
 }
 
+static void ec_mirror_with_count(void)
+{
+	unsigned int cmos_mirror_flag_counter = get_uint_option("mirror_flag_counter", UINT_MAX);
+
+	if (cmos_mirror_flag_counter == UINT_MAX)
+		return;
+
+	printk(BIOS_DEBUG, "ITE: mirror_flag_counter = %u\n", cmos_mirror_flag_counter);
+
+	/* Avoid boot loops by only trying a state change once */
+	if (cmos_mirror_flag_counter < MIRROR_ATTEMPTS) {
+		cmos_mirror_flag_counter++;
+		set_uint_option("mirror_flag_counter", cmos_mirror_flag_counter);
+		printk(BIOS_DEBUG, "ITE: Mirror attempt %u/%u.\n", cmos_mirror_flag_counter,
+			MIRROR_ATTEMPTS);
+
+		/* Write the EC mirror flag */
+		ec_write(ECRAM_MIRROR_FLAG, MIRROR_ENABLED);
+
+		/* Check what has been written */
+		if (ec_read(ECRAM_MIRROR_FLAG) == MIRROR_ENABLED)
+			poweroff();
+	} else {
+		/*
+		 * If the mirror flags fails after 1 attempt, it will
+		 * likely need a cold boot, or recovering.
+		 */
+		printk(BIOS_ERR, "ITE: Failed to mirror the EC in %u attempts!\n",
+			MIRROR_ATTEMPTS);
+	}
+}
+
+
+void ec_mirror_flag(void)
+{
+	/*
+	 * For the mirror flag to work, the status of the EC pin must be known
+	 * at all times, which means external power. This can be either a DC
+	 * charger, or PD with CCG6. PD with an ANX7447 requires configuration
+	 * from the EC, so the update will interrupt this.
+	 *
+	 * This means we can unconditionally apply the mirror flag to devices
+	 * that have CCG6, present on devices with TBT, but have a manual
+	 * flag for devices without it.
+	 */
+	uint16_t ec_version = ec_get_version();
+
+	if (CONFIG(EC_STARLABS_MIRROR_SUPPORT) &&
+		(CONFIG(DRIVERS_INTEL_USB4_RETIMER) || get_uint_option("mirror_flag", 0)) &&
+		(ec_version != CONFIG_EC_STARLABS_MIRROR_VERSION)) {
+		printk(BIOS_ERR, "ITE: EC version 0x%x doesn't match coreboot version 0x%x.\n",
+			ec_version, CONFIG_EC_STARLABS_MIRROR_VERSION);
+		ec_mirror_with_count();
+	}
+}
+
 static uint16_t ec_get_chip_id(unsigned int port)
 {
 	return (pnp_read_index(port, ITE_CHIPID1) << 8) |
@@ -57,6 +114,8 @@ static void merlin_init(struct device *dev)
 		return;
 	}
 
+	ec_mirror_flag();
+
 	pc_keyboard_init(NO_AUX_DEVICE);
 
 	/*
@@ -111,7 +170,7 @@ static void merlin_init(struct device *dev)
 
 	ec_write(ECRAM_FN_CTRL_REVERSE,
 		get_ec_value_from_option("fn_ctrl_swap",
-					 1,
+					 0,
 					 fn_ctrl_swap,
 					 ARRAY_SIZE(fn_ctrl_swap)));
 
