@@ -51,6 +51,7 @@ enum acpi_device_sleep_states {
 	ACPI_DEVICE_SLEEP_D3		= 3,
 	ACPI_DEVICE_SLEEP_D3_HOT	= ACPI_DEVICE_SLEEP_D3,
 	ACPI_DEVICE_SLEEP_D3_COLD	= 4,
+	ACPI_DEVICE_SLEEP_NONE		= 5,
 };
 
 #define RSDP_SIG		"RSD PTR "  /* RSDT pointer signature */
@@ -67,13 +68,16 @@ enum acpi_device_sleep_states {
 /* List of ACPI HID that use the coreboot ACPI ID */
 enum coreboot_acpi_ids {
 	COREBOOT_ACPI_ID_CBTABLE	= 0x0000, /* BOOT0000 */
+	COREBOOT_ACPI_ID_IGD_GMBUS_ARB  = 0x0001, /* BOOT0001 */
+	COREBOOT_ACPI_ID_IGD_GMBUS_LINK = 0x0002, /* BOOT0002 */
 	COREBOOT_ACPI_ID_MAX		= 0xFFFF, /* BOOTFFFF */
 };
 
 enum acpi_tables {
 	/* Tables defined by ACPI and used by coreboot */
-	BERT, DBG2, DMAR, DSDT, EINJ, FACS, FADT, HEST, HMAT, HPET, IVRS, MADT,
-	MCFG, RSDP, RSDT, SLIT, SRAT, SSDT, TCPA, TPM2, XSDT, ECDT, LPIT,
+	BERT, CEDT, DBG2, DMAR, DSDT, EINJ, FACS, FADT, HEST, HMAT, HPET, IVRS,
+	MADT, MCFG, RSDP, RSDT, SLIT, SRAT, SSDT, TCPA, TPM2, XSDT, ECDT, LPIT,
+	SPCR, GTDT,
 	/* Additional proprietary tables used by coreboot */
 	VFCT, NHLT, SPMI, CRAT
 };
@@ -221,6 +225,60 @@ typedef struct acpi_mcfg_mmconfig {
 	u8 reserved[4];
 } __packed acpi_mcfg_mmconfig_t;
 
+/*
+ * CEDT (CXL Early Discovery Table)
+ * CXL spec 2.0 section 9.14.1
+ */
+typedef struct acpi_cedt {
+	acpi_header_t header;
+	/* Followed by CEDT structures[n] */
+} __packed acpi_cedt_t;
+
+#define ACPI_CEDT_STRUCTURE_CHBS 0
+#define ACPI_CEDT_STRUCTURE_CFMWS 1
+
+#define ACPI_CEDT_CHBS_CXL_VER_1_1 0x00
+#define ACPI_CEDT_CHBS_CXL_VER_2_0 0x01
+
+/* CHBS: CXL Host Bridge Structure */
+typedef struct acpi_cedt_chbs {
+	u8 type; /* Always 0, other values reserved */
+	u8 resv1;
+	u16 length; /* Length in bytes (32) */
+	u32 uid;    /* CXL Host Bridge Unique ID */
+	u32 cxl_ver;
+	u32 resv2;
+	/*
+	 * For CXL 1.1, the base is Downstream Port Root Complex Resource Block;
+	 * For CXL 2.0, the base is CXL Host Bridge Component Registers.
+	 */
+	u64 base;
+	u64 len;
+} __packed acpi_cedt_chbs_t;
+
+#define ACPI_CEDT_CFMWS_RESTRICTION_TYPE_2_MEM (1 << 0)
+#define ACPI_CEDT_CFMWS_RESTRICTION_TYPE_3_MEM (1 << 1)
+#define ACPI_CEDT_CFMWS_RESTRICTION_VOLATIL    (1 << 2)
+#define ACPI_CEDT_CFMWS_RESTRICTION_PERSISTENT (1 << 3)
+#define ACPI_CEDT_CFMWS_RESTRICTION_FIXED      (1 << 4)
+
+/* CFMWS: CXL Fixed Memory Window Structure */
+typedef struct acpi_cedt_cfmws {
+	u8 type; /* Type (0) */
+	u8 resv1;
+	u16 length; /* Length in bytes (32) */
+	u32 resv2;
+	u64 base_hpa;		  /* Base of the HPA range, 256MB aligned */
+	u64 window_size;	  /* Number of bytes this window represents */
+	u8 eniw;		  /* Encoded Number of Interleave Ways */
+	u8 interleave_arithmetic; /* Standard Modulo arithmetic (0) */
+	u16 resv3;
+	u32 hbig; /* Host Bridge Interleave Granularity */
+	u16 restriction;
+	u16 qtg_id;
+	u32 interleave_target[]; /* Interleave Target List */
+} __packed acpi_cedt_cfmws_t;
+
 /*
  * HMAT (Heterogeneous Memory Attribute Table)
  * ACPI spec 6.4 section 5.2.27
@@ -298,6 +356,18 @@ typedef struct acpi_srat {
 #define ACPI_SRAT_STRUCTURE_MEM   1
 #define ACPI_SRAT_STRUCTURE_GIA   5
 
+/* SRAT: Processor x2APIC Structure */
+typedef struct acpi_srat_x2apic {
+	u8 type;			/* Type (0) */
+	u8 length;			/* Length in bytes (16) */
+	u16 reserved;			/* Reserved - Must be zero */
+	u32 proximity_domain;		/* Proximity domain */
+	u32 x2apic_id;			/* x2APIC ID */
+	u32 flags;			/* Enable bit 0 = 1, other bits reserved to 0 */
+	u32 clock_domain;		/* _CDM Clock Domain */
+	u32 reserved1;			/* Reserved */
+} __packed acpi_srat_x2apic_t;
+
 /* SRAT: Processor Local APIC/SAPIC Affinity Structure */
 typedef struct acpi_srat_lapic {
 	u8 type;			/* Type (0) */
@@ -436,7 +506,7 @@ typedef struct acpi_ivrs_ivhd {
 	uint16_t pci_segment_group;
 	uint16_t iommu_info;
 	uint32_t iommu_feature_info;
-	uint8_t entry[0];
+	uint8_t entry[];
 } __packed acpi_ivrs_ivhd_t;
 
 /* IVRS (I/O Virtualization Reporting Structure) Type 10h */
@@ -479,7 +549,7 @@ typedef struct acpi_ivrs_ivhd_11 {
 	uint32_t efr_reg_image_low;
 	uint32_t efr_reg_image_high;
 	uint32_t reserved[2];
-	uint8_t entry[0];
+	uint8_t entry[];
 } __packed acpi_ivrs_ivhd11_t;
 
 enum dev_scope_type {
@@ -499,7 +569,7 @@ typedef struct dev_scope {
 	struct {
 		u8 dev;
 		u8 fn;
-	} __packed path[0];
+	} __packed path[];
 } __packed dev_scope_t;
 
 enum dmar_type {
@@ -581,7 +651,7 @@ typedef struct acpi_dmar {
 	u8 host_address_width;
 	u8 flags;
 	u8 reserved[10];
-	dmar_entry_t structure[0];
+	dmar_entry_t structure[];
 } __packed acpi_dmar_t;
 
 /* MADT: APIC Structure Types */
@@ -615,6 +685,8 @@ typedef struct acpi_madt_lapic {
 	u32 flags;			/* Local APIC flags */
 } __packed acpi_madt_lapic_t;
 
+#define ACPI_MADT_MAX_LAPIC_ID		0xfe
+
 /* MADT: Local APIC NMI Structure */
 typedef struct acpi_madt_lapic_nmi {
 	u8 type;			/* Type (4) */
@@ -624,7 +696,8 @@ typedef struct acpi_madt_lapic_nmi {
 	u8 lint;			/* Local APIC LINT# */
 } __packed acpi_madt_lapic_nmi_t;
 
-#define ACPI_MADT_LAPIC_NMI_ALL_PROCESSORS 0xff
+#define ACPI_MADT_LAPIC_NMI_ALL_PROCESSORS	0xff
+#define ACPI_MADT_LX2APIC_NMI_ALL_PROCESSORS	((u32)-1)
 
 /* MADT: I/O APIC Structure */
 typedef struct acpi_madt_ioapic {
@@ -675,6 +748,93 @@ typedef struct acpi_madt_lx2apic_nmi {
 	u8 reserved[3];
 } __packed acpi_madt_lx2apic_nmi_t;
 
+/* MADT: GIC CPU Interface (GICC) Structure 6.5 */
+struct gicc_flags {
+	uint32_t enabled : 1;
+	/* 0 - Level-triggered | 1 - Edge-Triggered */
+	uint32_t performance_interrupt_mode : 1;
+	uint32_t vgic_maintenance_interrupt_mode : 1;
+	uint32_t online_capable : 1;
+	uint32_t reserved : 28;
+};
+_Static_assert(sizeof(struct gicc_flags) == sizeof(uint32_t), "Wrong gicc_flags size\n");
+
+typedef struct acpi_gicc {
+	uint8_t type;
+	uint8_t length;
+	uint16_t reserved;
+	uint32_t cpu_interface_number;
+	uint32_t acpi_processor_uid;
+	struct gicc_flags flags;
+	uint32_t parking_protocol_version;
+	uint32_t performance_interrupt_gsiv;
+	uint64_t parked_address;
+	uint64_t physical_base_address; /* GIC v1/v2 or GIC v3/v4 in v2 compat mode */
+	uint64_t gicv;
+	uint64_t gich;
+	uint32_t vgic_maintenance_interrupt;
+	uint64_t gicr_base_address; /* Only GIC v3 and above */
+	uint64_t mpidr;
+	uint8_t processor_power_efficiency_class;
+	uint8_t reserved1;
+	uint16_t spe_overflow_interrupt;
+	uint16_t trbe_interrupt;
+} __packed acpi_madt_gicc_t;
+_Static_assert(sizeof(acpi_madt_gicc_t) == 82, "Wrong acpi_madt_gicc_t size\n");
+
+/* MADT: GIC Distributor (GICD) Structure */
+typedef struct acpi_gicd {
+	uint8_t type;
+	uint8_t length;
+	uint16_t reserved1;
+	uint32_t gic_id;
+	uint64_t physical_base_address;
+	uint32_t system_vector_base;
+	uint8_t gic_version;
+	uint8_t reserved2[3];
+} __packed acpi_madt_gicd_t;
+_Static_assert(sizeof(acpi_madt_gicd_t) == 24, "Wrong acpi_madt_gicd_t size\n");
+
+/* MADT: GIC MSI Frame Structure */
+struct gic_msi_flags {
+	uint32_t spi_count_select : 1;
+	uint32_t reserved : 31;
+};
+_Static_assert(sizeof(struct gic_msi_flags) == sizeof(uint32_t), "Wrong gic_msi_flags size\n");
+
+typedef struct acpi_gic_msi {
+	uint8_t type;
+	uint8_t length;
+	uint16_t reserved;
+	uint32_t gic_msi_frame_id;
+	uint64_t physical_base_address;
+	struct gic_msi_flags flags;
+	uint16_t spi_count;
+	uint16_t spi_base;
+} __packed acpi_gic_msi_t;
+_Static_assert(sizeof(acpi_gic_msi_t) == 24, "Wrong acpi_gic_msi_t size\n");
+
+/* MADT: GIC Redistributor (GICR) Structure */
+typedef struct acpi_girr {
+	uint8_t type;
+	uint8_t length;
+	uint16_t reserved;
+	uint64_t discovery_range_base_address;
+	uint32_t discovery_range_length;
+} __packed acpi_madt_gicr_t;
+_Static_assert(sizeof(acpi_madt_gicr_t) == 16, "Wrong acpi_madt_gicr_t size\n");
+
+/* MADT: GIC Interrupt Translation Service (ITS) Structure */
+typedef struct acpi_gic_its {
+	uint8_t type;
+	uint8_t length;
+	uint16_t reserved;
+	uint32_t gic_its_id;
+	uint64_t physical_base_address;
+	uint32_t reserved2;
+} __packed acpi_madt_gic_its_t;
+_Static_assert(sizeof(acpi_madt_gic_its_t) == 20, "Wrong MADT acpi_madt_gic_its_t size\n");
+
 #define ACPI_DBG2_PORT_SERIAL			0x8000
 #define  ACPI_DBG2_PORT_SERIAL_16550		0x0000
 #define  ACPI_DBG2_PORT_SERIAL_16550_DBGP	0x0001
@@ -1173,7 +1333,7 @@ typedef struct acpi_einj_trigger_table {
 	u32 revision;
 	u32 table_size;
 	u32 entry_count;
-	acpi_einj_action_table_t trigger_action[1];
+	acpi_einj_action_table_t trigger_action[];
 } __packed acpi_einj_trigger_table_t;
 
 typedef struct set_error_type {
@@ -1228,6 +1388,138 @@ typedef struct acpi_einj {
 	acpi_einj_action_table_t action_table[ACTION_COUNT];
 } __packed acpi_einj_t;
 
+/* SPCR (Serial Port Console Redirection Table) */
+typedef struct acpi_spcr {
+	acpi_header_t header;
+	uint8_t interface_type;
+	uint8_t reserved[3];
+	acpi_addr_t base_address;
+	uint8_t interrupt_type;
+	uint8_t irq;
+	uint32_t global_system_interrupt;
+	uint8_t configured_baudrate;
+	uint8_t parity;
+	uint8_t stop_bits;
+	uint8_t flow_control;
+	uint8_t terminal_type;
+	uint8_t language;
+	uint16_t pci_did;
+	uint16_t pci_vid;
+	uint8_t pci_bus;
+	uint8_t pci_dev;
+	uint8_t pci_fun;
+	uint32_t pci_flags;
+	uint8_t pci_segment;
+	uint32_t uart_clock;
+	uint32_t precise_baud_rate;
+	uint16_t namespace_string_length;
+	uint16_t namespace_string_offset;
+	char namespacestring[];
+} __packed acpi_spcr_t;
+_Static_assert(sizeof(acpi_spcr_t) == 88, "acpi_spcr_t must have an 88 byte size\n");
+
+#define PC_AT_COMPATIBLE_INTERRUPT (1 << 0)
+#define IO_APIC_COMPATIBLE_INTERRUPT (1 << 1)
+#define IO_SAPIC_COMPATIBLE_INTERRUPT (1 << 2)
+#define ARMH_GIC_COMPATIBLE_INTERRUPT (1 << 3)
+#define RISCV_PLIC_COMPATIBLE_INTERRUPT (1 << 4)
+
+/* GTDT - Generic Timer Description Table (ACPI 5.1) Version 2 */
+typedef struct acpi_table_gtdt {
+	acpi_header_t header;	/* Common ACPI table header */
+	u64 counter_block_addresss;
+	u32 reserved;
+	u32 secure_el1_interrupt;
+	u32 secure_el1_flags;
+	u32 non_secure_el1_interrupt;
+	u32 non_secure_el1_flags;
+	u32 virtual_timer_interrupt;
+	u32 virtual_timer_flags;
+	u32 non_secure_el2_interrupt;
+	u32 non_secure_el2_flags;
+	u64 counter_read_block_address;
+	u32 platform_timer_count;
+	u32 platform_timer_offset;
+} __packed acpi_gtdt_t;
+
+/* Flag Definitions: Timer Block Physical Timers and Virtual timers */
+
+#define ACPI_GTDT_INTERRUPT_MODE        (1)
+#define ACPI_GTDT_INTERRUPT_POLARITY    (1<<1)
+#define ACPI_GTDT_ALWAYS_ON             (1<<2)
+
+struct acpi_gtdt_el2 {
+	u32 virtual_el2_timer_gsiv;
+	u32 virtual_el2_timer_flags;
+};
+
+/* Common GTDT subtable header */
+
+struct acpi_gtdt_header {
+	u8 type;
+	u16 length;
+} __packed;
+
+/* Values for GTDT subtable type above */
+
+enum acpi_gtdt_type {
+	ACPI_GTDT_TYPE_TIMER_BLOCK = 0,
+	ACPI_GTDT_TYPE_WATCHDOG = 1,
+	ACPI_GTDT_TYPE_RESERVED = 2	/* 2 and greater are reserved */
+};
+
+/* GTDT Subtables, correspond to Type in struct acpi_gtdt_header */
+
+/* 0: Generic Timer Block */
+
+struct acpi_gtdt_timer_block {
+	struct acpi_gtdt_header header;
+	u8 reserved;
+	u64 block_address;
+	u32 timer_count;
+	u32 timer_offset;
+} __packed;
+
+/* Timer Sub-Structure, one per timer */
+
+struct acpi_gtdt_timer_entry {
+	u8 frame_number;
+	u8 reserved[3];
+	u64 base_address;
+	u64 el0_base_address;
+	u32 timer_interrupt;
+	u32 timer_flags;
+	u32 virtual_timer_interrupt;
+	u32 virtual_timer_flags;
+	u32 common_flags;
+} __packed;
+
+/* Flag Definitions: timer_flags and virtual_timer_flags above */
+
+#define ACPI_GTDT_GT_IRQ_MODE               (1)
+#define ACPI_GTDT_GT_IRQ_POLARITY           (1<<1)
+
+/* Flag Definitions: common_flags above */
+
+#define ACPI_GTDT_GT_IS_SECURE_TIMER        (1)
+#define ACPI_GTDT_GT_ALWAYS_ON              (1<<1)
+
+/* 1: SBSA Generic Watchdog Structure */
+
+struct acpi_gtdt_watchdog {
+	struct acpi_gtdt_header header;
+	u8 reserved;
+	u64 refresh_frame_address;
+	u64 control_frame_address;
+	u32 timer_interrupt;
+	u32 timer_flags;
+} __packed;
+
+/* Flag Definitions: timer_flags above */
+
+#define ACPI_GTDT_WATCHDOG_IRQ_MODE         (1)
+#define ACPI_GTDT_WATCHDOG_IRQ_POLARITY     (1<<1)
+#define ACPI_GTDT_WATCHDOG_SECURE           (1<<2)
 
 uintptr_t get_coreboot_rsdp(void);
 void acpi_create_einj(acpi_einj_t *einj, uintptr_t addr, u8 actions);
@@ -1236,20 +1528,17 @@ unsigned long fw_cfg_acpi_tables(unsigned long start);
 
 /* These are implemented by the target port or north/southbridge. */
 void preload_acpi_dsdt(void);
-unsigned long write_acpi_tables(unsigned long addr);
+unsigned long write_acpi_tables(const unsigned long addr);
 unsigned long acpi_fill_madt(unsigned long current);
-unsigned long acpi_fill_ivrs_ioapic(acpi_ivrs_t *ivrs, unsigned long current);
-void acpi_create_ssdt_generator(acpi_header_t *ssdt, const char *oem_table_id);
-void acpi_write_bert(acpi_bert_t *bert, uintptr_t region, size_t length);
-void acpi_create_fadt(acpi_fadt_t *fadt, acpi_facs_t *facs, void *dsdt);
-
-void soc_lpi_get_constraints(void *unused);
+unsigned long acpi_arch_fill_madt(acpi_madt_t *madt, unsigned long current);
 
 void acpi_fill_fadt(acpi_fadt_t *fadt);
 void arch_fill_fadt(acpi_fadt_t *fadt);
 void soc_fill_fadt(acpi_fadt_t *fadt);
 void mainboard_fill_fadt(acpi_fadt_t *fadt);
 
+void fill_fadt_extended_pm_io(acpi_fadt_t *fadt);
+
 void acpi_fill_gnvs(void);
 void acpi_fill_cnvs(void);
 
@@ -1260,19 +1549,24 @@ u8 acpi_checksum(u8 *table, u32 length);
 
 void acpi_add_table(acpi_rsdp_t *rsdp, void *table);
 
-int acpi_create_madt_lapic(acpi_madt_lapic_t *lapic, u8 cpu, u8 apic);
-int acpi_create_madt_ioapic(acpi_madt_ioapic_t *ioapic, u8 id, u32 addr,
-			    u32 gsi_base);
-int acpi_create_madt_irqoverride(acpi_madt_irqoverride_t *irqoverride,
-				 u8 bus, u8 source, u32 gsirq, u16 flags);
-int acpi_create_madt_lapic_nmi(acpi_madt_lapic_nmi_t *lapic_nmi, u8 cpu,
-			       u16 flags, u8 lint);
-void acpi_create_madt(acpi_madt_t *madt);
-unsigned long acpi_create_madt_lapics(unsigned long current);
-int acpi_create_madt_lx2apic(acpi_madt_lx2apic_t *lapic, u32 cpu, u32 apic);
-int acpi_create_madt_lx2apic_nmi(acpi_madt_lx2apic_nmi_t *lapic_nmi, u32 cpu,
-				 u16 flags, u8 lint);
+/* Create CXL Early Discovery Table */
+void acpi_create_cedt(acpi_cedt_t *cedt,
+	unsigned long (*acpi_fill_cedt)(unsigned long current));
+/* Create a CXL Host Bridge Structure for CEDT */
+int acpi_create_cedt_chbs(acpi_cedt_chbs_t *chbs, u32 uid, u32 cxl_ver, u64 base);
+/* Create a CXL Fixed Memory Window Structure for CEDT */
+int acpi_create_cedt_cfmws(acpi_cedt_cfmws_t *cfmws, u64 base_hpa, u64 window_size,
+	u8 eniw, u32 hbig, u16 restriction, u16 qtg_id, const u32 *interleave_target);
+
+
+int acpi_create_madt_ioapic_from_hw(acpi_madt_ioapic_t *ioapic, u32 addr);
+
+unsigned long acpi_create_madt_one_lapic(unsigned long current, u32 cpu, u32 apic);
+
+unsigned long acpi_create_madt_lapic_nmis(unsigned long current);
+
 int acpi_create_srat_lapic(acpi_srat_lapic_t *lapic, u8 node, u8 apic);
+int acpi_create_srat_x2apic(acpi_srat_x2apic_t *x2apic, u32 node, u32 apic);
 int acpi_create_srat_mem(acpi_srat_mem_t *mem, u8 node, u32 basek, u32 sizek,
 			 u32 flags);
 /*
@@ -1281,8 +1575,6 @@ int acpi_create_srat_mem(acpi_srat_mem_t *mem, u8 node, u32 basek, u32 sizek,
  */
 int acpi_create_srat_gia_pci(acpi_srat_gia_t *gia, u32 proximity_domain,
 		u16 seg, u8 bus, u8 dev, u8 func, u32 flags);
-int acpi_create_mcfg_mmconfig(acpi_mcfg_mmconfig_t *mmconfig, u32 base,
-			      u16 seg_nr, u8 start, u8 end);
 unsigned long acpi_create_srat_lapics(unsigned long current);
 void acpi_create_srat(acpi_srat_t *srat,
 		      unsigned long (*acpi_fill_srat)(unsigned long current));
@@ -1323,22 +1615,12 @@ void acpi_create_crat(struct acpi_crat_header *crat,
 		      unsigned long (*acpi_fill_crat)(struct acpi_crat_header *crat_struct,
 		      unsigned long current));
 
-void acpi_create_hpet(acpi_hpet_t *hpet);
 unsigned long acpi_write_hpet(const struct device *device, unsigned long start,
 			      acpi_rsdp_t *rsdp);
 
 /* cpu/intel/speedstep/acpi.c */
 void generate_cpu_entries(const struct device *device);
 
-void acpi_create_mcfg(acpi_mcfg_t *mcfg);
-
-void acpi_create_facs(acpi_facs_t *facs);
-
-void acpi_create_dbg2(acpi_dbg2_header_t *dbg2_header,
-		      int port_type, int port_subtype,
-		      acpi_addr_t *address, uint32_t address_size,
-		      const char *device_path);
-
 unsigned long acpi_write_dbg2_pci_uart(acpi_rsdp_t *rsdp, unsigned long current,
 				const struct device *dev, uint8_t access_size);
 void acpi_create_dmar(acpi_dmar_t *dmar, enum dmar_flags flags,
@@ -1366,6 +1648,8 @@ unsigned long acpi_create_dmar_ds_pci(unsigned long current,
 unsigned long acpi_create_dmar_ds_ioapic(unsigned long current,
 					      u8 enumeration_id,
 					      u8 bus, u8 dev, u8 fn);
+unsigned long acpi_create_dmar_ds_ioapic_from_hw(unsigned long current,
+						 u32 addr, u8 bus, u8 dev, u8 fn);
 unsigned long acpi_create_dmar_ds_msi_hpet(unsigned long current,
 						u8 enumeration_id,
 						u8 bus, u8 dev, u8 fn);
@@ -1375,12 +1659,18 @@ void acpi_write_hest(acpi_hest_t *hest,
 unsigned long acpi_create_hest_error_source(acpi_hest_t *hest,
 	acpi_hest_esd_t *esd, u16 type, void *data, u16 len);
 
-void acpi_create_lpit(acpi_lpit_t *lpit);
 unsigned long acpi_create_lpi_desc_ncst(acpi_lpi_desc_ncst_t *lpi_desc, uint16_t uid);
 
 /* chipsets that select ACPI_BERT must implement this function */
 enum cb_err acpi_soc_get_bert_region(void **region, size_t *length);
 
+void acpi_soc_fill_gtdt(acpi_gtdt_t *gtdt);
+unsigned long acpi_soc_gtdt_add_timers(uint32_t *count, unsigned long current);
+unsigned long acpi_gtdt_add_timer_block(unsigned long current, const uint64_t address,
+					   struct acpi_gtdt_timer_entry *timers, size_t number);
+unsigned long acpi_gtdt_add_watchdog(unsigned long current, uint64_t refresh_frame,
+				     uint64_t control_frame, uint32_t gsiv, uint32_t flags);
+
 /* For ACPI S3 support. */
 void __noreturn acpi_resume(void *wake_vec);
 void mainboard_suspend_resume(void);
