@@ -22,6 +22,7 @@ enum device_path_type {
 	DEVICE_PATH_USB,
 	DEVICE_PATH_MMIO,
 	DEVICE_PATH_GPIO,
+	DEVICE_PATH_MDIO,
 
 	/*
 	 * When adding path types to this table, please also update the
@@ -46,6 +47,7 @@ enum device_path_type {
 		"DEVICE_PATH_USB",		\
 		"DEVICE_PATH_MMIO",		\
 		"DEVICE_PATH_GPIO",		\
+		"DEVICE_PATH_MDIO",		\
 }
 
 struct domain_path {
@@ -71,11 +73,13 @@ struct spi_path {
 };
 
 struct apic_path {
+	unsigned int initial_lapicid;
 	unsigned int apic_id;
 	unsigned int package_id;
 	unsigned int node_id;
 	unsigned int core_id;
 	unsigned int thread_id;
+	unsigned char core_type;
 };
 
 struct ioapic_path {
@@ -112,6 +116,10 @@ struct gpio_path {
 	unsigned int id;
 };
 
+struct mdio_path {
+	unsigned int addr;
+};
+
 struct device_path {
 	enum device_path_type type;
 	union {
@@ -129,6 +137,7 @@ struct device_path {
 		struct usb_path		usb;
 		struct mmio_path	mmio;
 		struct gpio_path	gpio;
+		struct mdio_path	mdio;
 	};
 };
 
