@@ -422,6 +422,8 @@
 #define  PCI_EXP_DEVSTA_AUXPD	0x10	/* AUX Power Detected */
 #define  PCI_EXP_DEVSTA_TRPND	0x20	/* Transactions Pending */
 #define PCI_EXP_LNKCAP		12	/* Link Capabilities */
+#define  PCI_EXP_LNKCAP_MLS	0x000f	/* Maximum Link Speed */
+#define  PCI_EXP_LNKCAP_MLW	0x03f0	/* Maximum Link Width */
 #define  PCI_EXP_LNKCAP_ASPMS	0xc00	/* ASPM Support */
 #define  PCI_EXP_LNKCAP_L0SEL	0x7000	/* L0s Exit Latency */
 #define  PCI_EXP_LNKCAP_L1EL	0x38000	/* L1 Exit Latency */
@@ -429,13 +431,14 @@
 #define  PCI_EXP_LNKCAP_PORT	0xff000000 /* Port Number */
 #define PCI_EXP_LNKCTL		16	/* Link Control */
 #define  PCI_EXP_LNKCTL_RL	0x20	/* Retrain Link */
-#define  PCI_EXP_LNKCTL_CCC	0x40	/* Common Clock COnfiguration */
+#define  PCI_EXP_LNKCTL_CCC	0x40	/* Common Clock Configuration */
 #define  PCI_EXP_EN_CLK_PM	0x100	/* Enable Clock Power Management */
 #define PCI_EXP_LNKSTA		18	/* Link Status */
 #define  PCI_EXP_LNKSTA_LT	0x800	/* Link Training */
 #define  PCI_EXP_LNKSTA_SLC	0x1000	/* Slot Clock Configuration */
 #define PCI_EXP_SLTCAP		20	/* Slot Capabilities */
 #define  PCI_EXP_SLTCAP_HPC	0x0040	/* Hot-Plug Capable */
+#define  PCI_EXP_SLTCAP_PSN	0xfff80000 /* Physical Slot Number */
 #define PCI_EXP_SLTCTL		24	/* Slot Control */
 #define PCI_EXP_SLTSTA		26	/* Slot Status */
 #define PCI_EXP_RTCTL		28	/* Root Control */
@@ -465,9 +468,11 @@
 /* Extended Capability lists*/
 #define PCIE_EXT_CAP_OFFSET		0x100
 #define  PCIE_EXT_CAP_AER_ID		 0x0001
+#define  PCIE_EXT_CAP_ID_ATS		 0x000F
 #define  PCIE_EXT_CAP_L1SS_ID		 0x001E
 #define  PCIE_EXT_CAP_LTR_ID		 0x0018
 #define  PCIE_EXT_CAP_RESIZABLE_BAR	 0x0015
+#define  PCIE_EXT_CAP_RCECEA_ID	0x0007
 
 /* Secondary PCI Express Extended Capability Structure */
 #define PCI_EXP_SEC_CAP_ID	0x19
@@ -478,6 +483,7 @@
 #define PCI_ERR_UNCOR_STATUS	4	/* Uncorrectable Error Status */
 #define  PCI_ERR_UNC_TRAIN	0x00000001	/* Training */
 #define  PCI_ERR_UNC_DLP	0x00000010	/* Data Link Protocol */
+#define  PCI_ERR_SURPRISE_DN	0x00000020	/* Surprise Down */
 #define  PCI_ERR_UNC_POISON_TLP	0x00001000	/* Poisoned TLP */
 #define  PCI_ERR_UNC_FCP	0x00002000	/* Flow Control Protocol */
 #define  PCI_ERR_UNC_COMP_TIME	0x00004000	/* Completion Timeout */
@@ -487,6 +493,8 @@
 #define  PCI_ERR_UNC_MALF_TLP	0x00040000	/* Malformed TLP */
 #define  PCI_ERR_UNC_ECRC	0x00080000	/* ECRC Error Status */
 #define  PCI_ERR_UNC_UNSUP	0x00100000	/* Unsupported Request */
+#define  PCI_ERR_UNC_INTL_ERR	0x00400000	/* Uncorrectable Internal Error */
+#define  PCI_ERR_UNC_POISON_BLK	0x04000000	/* Poisoned TLP Egress Blocked */
 #define PCI_ERR_UNCOR_MASK	8	/* Uncorrectable Error Mask */
 	/* Same bits as above */
 #define PCI_ERR_UNCOR_SEVER	12	/* Uncorrectable Error Severity */
@@ -497,6 +505,9 @@
 #define  PCI_ERR_COR_BAD_DLLP	0x00000080	/* Bad DLLP Status */
 #define  PCI_ERR_COR_REP_ROLL	0x00000100	/* REPLAY_NUM Rollover */
 #define  PCI_ERR_COR_REP_TIMER	0x00001000	/* Replay Timer Timeout */
+#define  PCI_ERR_COR_ANF	0x00002000	/* Advisory Non-Fatal Error */
+#define  PCI_ERR_COR_INTL_ERR	0x00004000	/* Corrected Internal Error */
+#define  PCI_ERR_COR_LOG_OVER	0x00008000	/* Header Log Overflow Error */
 #define PCI_ERR_COR_MASK	20	/* Correctable Error Mask */
 	/* Same bits as above */
 #define PCI_ERR_CAP		24	/* Advanced Error Capabilities */
@@ -507,10 +518,23 @@
 #define  PCI_ERR_CAP_ECRC_CHKE	0x00000100	/* ECRC Check Enable */
 #define PCI_ERR_HEADER_LOG	28	/* Header Log Register (16 bytes) */
 #define PCI_ERR_ROOT_COMMAND	44	/* Root Error Command */
+#define  PCI_EXP_ROOT_CERE	0x0001		/* Correctable Error Reporting Enable */
+#define  PCI_EXP_ROOT_NFERE	0x0002		/* Non-Fatal Error Reporting Enable */
+#define  PCI_EXP_ROOT_FERE	0x0004		/* Fatal Error Reporting Enable */
 #define PCI_ERR_ROOT_STATUS	48
 #define PCI_ERR_ROOT_COR_SRC	52
 #define PCI_ERR_ROOT_SRC	54
 
+/* DPC Capability Structure */
+#define PCIE_DPC_CAP_ID			0x1d
+#define PCIE_DPC_CTL			0x6
+#define  DPC_TRIGGER_EN			(1 << 1)
+#define  DPC_INT_EN			(1 << 3)
+#define  DPC_ERR_COR_EN			(1 << 4)
+#define  DPC_POISON_TLP			(1 << 5)
+#define PCIE_DPC_RP_PIO_MASK		0x10
+#define PCIE_DPC_SEV			0x14
+
 /* Virtual Channel */
 #define PCI_VC_PORT_REG1	4
 #define PCI_VC_PORT_REG2	8
@@ -546,6 +570,10 @@
 #define  PCI_REBAR_CTRL_SIZE_MASK	 0xffff0000
 #define  PCI_REBAR_CTRL_SIZE_SHIFT	 16
 
+/* Root Complex Event Collector Endpoint Association */
+#define PCI_RCECEA_BITMAP 4
+#define PCI_RCECEA_BUSNUM 8
+
 /*
  * The PCI interface treats multi-function devices as independent
  * devices.  The slot/function address of each device is encoded
@@ -562,4 +590,8 @@
 #define PCI_DEV2DEVFN(sdev)		(((sdev)>>12) & 0xff)
 #define PCI_DEV2SEGBUS(sdev)	(((sdev)>>20) & 0xfff)
 
+/* Fields from within the device's class value. */
+#define PCI_CLASS_GET_DEVICE(c) (c >> 8)
+#define PCI_CLASS_GET_PROG(c) (c & 0xff)
+
 #endif /* PCI_DEF_H */
