@@ -12,6 +12,7 @@
 #include <list.h>
 #include <metadata_hash.h>
 #include <security/tpm/tspi/crtm.h>
+#include <security/vboot/vboot_common.h>
 #include <security/vboot/misc.h>
 #include <stdlib.h>
 #include <string.h>
@@ -57,7 +58,10 @@ enum cb_err _cbfs_boot_lookup(const char *name, bool force_ro,
 			   RO CBFS would have been caught when building the mcache in cbfs_get
 			   boot_device(). (Note that TOCTOU_SAFETY implies !NO_CBFS_MCACHE.) */
 			assert(cbd == vboot_get_cbfs_boot_device());
-			die("TODO: set metadata_hash to RW metadata hash here.\n");
+			if (!CONFIG(VBOOT)
+			    || vb2api_get_metadata_hash(vboot_get_context(), &metadata_hash)
+				       != VB2_SUCCESS)
+				die("Failed to get RW metadata hash");
 		}
 		err = cbfs_lookup(&cbd->rdev, name, mdata, &data_offset, metadata_hash);
 	}
@@ -138,7 +142,7 @@ static inline bool cbfs_lzma_enabled(void)
 		return false;
 	if (ENV_ROMSTAGE && CONFIG(POSTCAR_STAGE))
 		return false;
-	if ((ENV_ROMSTAGE || ENV_POSTCAR) && !CONFIG(COMPRESS_RAMSTAGE))
+	if ((ENV_ROMSTAGE || ENV_POSTCAR) && !CONFIG(COMPRESS_RAMSTAGE_LZMA))
 		return false;
 	if (ENV_SMM)
 		return false;
@@ -160,8 +164,14 @@ static bool cbfs_file_hash_mismatch(const void *buffer, size_t size,
 			ERROR("'%s' does not have a file hash!\n", mdata->h.filename);
 			return true;
 		}
-		if (vb2_hash_verify(vboot_hwcrypto_allowed(), buffer, size, hash)) {
+
+		vb2_error_t rv = vb2_hash_verify(vboot_hwcrypto_allowed(), buffer, size, hash);
+		if (rv != VB2_SUCCESS) {
 			ERROR("'%s' file hash mismatch!\n", mdata->h.filename);
+			if (CONFIG(VBOOT_CBFS_INTEGRATION) && !vboot_recovery_mode_enabled()
+			    && vboot_logic_executed())
+				vboot_fail_and_reboot(vboot_get_context(), VB2_RECOVERY_FW_BODY,
+						      rv);
 			return true;
 		}
 	}
@@ -180,7 +190,7 @@ static bool cbfs_file_hash_mismatch(const void *buffer, size_t size,
 
 		if (!hash ||
 		    tspi_cbfs_measurement(mdata->h.filename, be32toh(mdata->h.type), hash))
-			ERROR("failed to measure '%s' into TCPA log\n", mdata->h.filename);
+			ERROR("failed to measure '%s' into TPM log\n", mdata->h.filename);
 			/* We intentionally continue to boot on measurement errors. */
 	}
 
@@ -198,6 +208,13 @@ static size_t cbfs_load_and_decompress(const struct region_device *rdev, void *b
 	DEBUG("Decompressing %zu bytes from '%s' to %p with algo %d\n",
 	      in_size, mdata->h.filename, buffer, compression);
 
+	if (CONFIG(CBFS_VERIFICATION) && !CONFIG(CBFS_ALLOW_UNVERIFIED_DECOMPRESSION) &&
+	    skip_verification && compression != CBFS_COMPRESS_NONE) {
+		ERROR("Refusing to decompress unverified file '%s' with algo %d\n",
+		      mdata->h.filename, compression);
+		return 0;
+	}
+
 	switch (compression) {
 	case CBFS_COMPRESS_NONE:
 		if (buffer_size < in_size)
