@@ -256,8 +256,7 @@ struct device_tree *fdt_unflatten(const void *blob)
 		return NULL;
 	}
 	if (version > FDT_SUPPORTED_VERSION)
-		printk(BIOS_DEBUG,
-		       "NOTE: FDT version %u too new, should add support!\n",
+		printk(BIOS_NOTICE, "FDT version %u too new, should add support!\n",
 		       version);
 
 	uint32_t struct_offset = be32toh(header->structure_offset);
@@ -554,7 +553,7 @@ struct device_tree_node *dt_find_node(struct device_tree_node *parent,
 		if (!create)
 			return NULL;
 
-		found = malloc(sizeof(*found));
+		found = calloc(1, sizeof(*found));
 		if (!found)
 			return NULL;
 		found->name = strdup(*path);
@@ -1501,7 +1500,7 @@ int dt_apply_overlay(struct device_tree *tree, struct device_tree *overlay)
 	uint32_t phandle_base = tree->max_phandle;
 	uint32_t new_max = dt_adjust_all_phandles(overlay->root, phandle_base);
 	if (!new_max) {
-		printk(BIOS_DEBUG, "ERROR: invalid phandles in overlay\n");
+		printk(BIOS_ERR, "invalid phandles in overlay\n");
 		return -1;
 	}
 	tree->max_phandle = new_max;
@@ -1512,7 +1511,7 @@ int dt_apply_overlay(struct device_tree *tree, struct device_tree *overlay)
 					"/__local_fixups__", NULL, NULL, 0);
 	if (local_fixups && dt_fixup_locals(overlay->root, local_fixups,
 					    phandle_base) < 0) {
-		printk(BIOS_DEBUG, "ERROR: invalid local fixups in overlay\n");
+		printk(BIOS_ERR, "invalid local fixups in overlay\n");
 		return -1;
 	}
 
@@ -1536,8 +1535,7 @@ int dt_apply_overlay(struct device_tree *tree, struct device_tree *overlay)
 		"/__symbols__", NULL, NULL, 0);
 	if (fixups && dt_fixup_all_externals(tree, symbols, overlay,
 					     fixups, overlay_symbols) < 0) {
-		printk(BIOS_DEBUG,
-		       "ERROR: cannot match external fixups from overlay\n");
+		printk(BIOS_ERR, "cannot match external fixups from overlay\n");
 		return -1;
 	}
 
@@ -1546,7 +1544,7 @@ int dt_apply_overlay(struct device_tree *tree, struct device_tree *overlay)
 	struct device_tree_node *fragment;
 	list_for_each(fragment, overlay->root->children, list_node)
 		if (dt_import_fragment(tree, fragment, overlay_symbols) < 0) {
-			printk(BIOS_DEBUG, "ERROR: bad DT fragment '%s'\n",
+			printk(BIOS_ERR, "bad DT fragment '%s'\n",
 			       fragment->name);
 			return -1;
 		}
