@@ -103,7 +103,7 @@ static const struct pad_config gpio_table[] = {
 	/* C0  : SMBCLK ==> EN_PP3300_TCHSCR */
 	PAD_CFG_GPO(GPP_C0, 1, DEEP),
 	/* C1  : SMBDATA ==> USI_RST_L */
-	PAD_CFG_GPO(GPP_C1, 0, DEEP),
+	PAD_CFG_GPO(GPP_C1, 1, DEEP),
 	/* C2  : SMBALERT# ==> GPP_C2_STRAP */
 	PAD_NC(GPP_C2, NONE),
 	/* C3 : SML0CLK ==> EN_UCAM_PWR */
@@ -240,7 +240,7 @@ static const struct pad_config gpio_table[] = {
 	/* F13 : GSXDOUT ==> GSPI_PCH_DI_FPMCU_DO */
 	PAD_CFG_NF_LOCK(GPP_F13, NONE, NF4, LOCK_CONFIG),
 	/* F14 : GSXDIN ==> TCHPAD_INT_ODL */
-	PAD_CFG_GPI_IRQ_WAKE_LOCK(GPP_F14, NONE, LEVEL, INVERT, LOCK_CONFIG),
+	PAD_CFG_GPI_IRQ_WAKE(GPP_F14, NONE, PWROK, LEVEL, INVERT),
 	/* F15 : GSXSRESET# ==> FPMCU_INT_L */
 	PAD_CFG_GPI_IRQ_WAKE_LOCK(GPP_F15, NONE, LEVEL, INVERT, LOCK_CONFIG),
 	/* F16 : GSXCLK ==> GSPI_PCH_CS_FPMCU_R_L */
@@ -380,6 +380,27 @@ static const struct pad_config gpio_table[] = {
 	PAD_NC(GPP_VGPIO_19, NONE),
 	PAD_NC(GPP_VGPIO_20, NONE),
 	PAD_NC(GPP_VGPIO_21, NONE),
+
+	/* Put option BT_I2S and SoC SSP2 pins in NC mode, ensure that
+	 * all the relevant VGPIOs were able to be overritten via
+	 *  override_gpio_table of each variant of gpio.c
+	 */
+	/* BT_I2S_BCLK */
+	PAD_NC(GPP_VGPIO_30, NONE),
+	/* BT_I2S_SYNC */
+	PAD_NC(GPP_VGPIO_31, NONE),
+	/* BT_I2S_SDO */
+	PAD_NC(GPP_VGPIO_32, NONE),
+	/* BT_I2S_SDI */
+	PAD_NC(GPP_VGPIO_33, NONE),
+	/* SSP2_SCLK */
+	PAD_NC(GPP_VGPIO_34, NONE),
+	/* SSP2_SFRM */
+	PAD_NC(GPP_VGPIO_35, NONE),
+	/* SSP2_TXD */
+	PAD_NC(GPP_VGPIO_36, NONE),
+	/* SSP2_RXD */
+	PAD_NC(GPP_VGPIO_37, NONE),
 };
 
 /* Early pad configuration in bootblock */
