@@ -14,14 +14,21 @@
 #include <boardid.h>
 
 /* Pad configuration in ramstage */
-static const struct pad_config gpio_table_id0[] = {
-	/* GPP_A00 : GPP_A00 ==> ESPI_SOC_IO0_R configured on reset, do not touch */
-	/* GPP_A01 : GPP_A01 ==> ESPI_SOC_IO1_R configured on reset, do not touch */
-	/* GPP_A02 : GPP_A02 ==> ESPI_SOC_IO2_R configured on reset, do not touch */
-	/* GPP_A03 : GPP_A03 ==> ESPI_SOC_IO3_R configured on reset, do not touch */
-	/* GPP_A04 : GPP_A04 ==> ESPI_SOC_CS0_L configured on reset, do not touch */
-	/* GPP_A05 : GPP_A05 ==> ESPI_SOC_CLK_R configured on reset, do not touch */
-	/* GPP_A06 : GPP_A06 ==> ESPI_SOC_RESET_L configured on reset, do not touch */
+static const struct pad_config gpio_table[] = {
+	/* GPP_A00 : GPP_A00 ==> ESPI_SOC_IO0_R */
+	PAD_CFG_NF_IOSSTATE(GPP_A00, UP_20K, DEEP, NF1, IGNORE),
+	/* GPP_A01 : GPP_A01 ==> ESPI_SOC_IO1_R */
+	PAD_CFG_NF_IOSSTATE(GPP_A01, UP_20K, DEEP, NF1, IGNORE),
+	/* GPP_A02 : GPP_A02 ==> ESPI_SOC_IO2_R */
+	PAD_CFG_NF_IOSSTATE(GPP_A02, UP_20K, DEEP, NF1, IGNORE),
+	/* GPP_A03 : GPP_A03 ==> ESPI_SOC_IO3_R */
+	PAD_CFG_NF_IOSSTATE(GPP_A03, UP_20K, DEEP, NF1, IGNORE),
+	/* GPP_A04 : GPP_A04 ==> ESPI_SOC_CS0_L */
+	PAD_CFG_NF_IOSSTATE(GPP_A04, UP_20K, DEEP, NF1, IGNORE),
+	/* GPP_A05 : GPP_A05 ==> ESPI_SOC_CLK_R */
+	PAD_CFG_NF_IOSSTATE(GPP_A05, UP_20K, DEEP, NF1, IGNORE),
+	/* GPP_A06 : GPP_A06 ==> ESPI_SOC_RESET_L */
+	PAD_CFG_NF_IOSSTATE(GPP_A06, UP_20K, DEEP, NF1, IGNORE),
 	/* GPP_A11 : [] ==> EN_UCAM_SENR_PWR */
 	PAD_CFG_GPO(GPP_A11, 0, DEEP),
 	/* GPP_A12 : [] ==> EN_UCAM_PWR */
@@ -29,10 +36,11 @@ static const struct pad_config gpio_table_id0[] = {
 	/* GPP_A13 : [] ==> SD_PE_LS_PRSNT_L */
 	PAD_CFG_GPI_LOCK(GPP_A13, NONE, LOCK_CONFIG),
 	/* GPP_A14 : [] ==> WWAN_RF_DISABLE_ODL */
-	PAD_CFG_GPO(GPP_A14, 1, DEEP),
+	PAD_NC_LOCK(GPP_A14, NONE, LOCK_CONFIG),
 	/* GPP_A15 : [] ==> WWAN_RST_L */
 	PAD_CFG_GPO(GPP_A15, 1, DEEP),
-	/* GPP_A16 : GPP_A16 ==> ESPI_SOC_ALERT_L configured on reset, do not touch */
+	/* GPP_A16 : GPP_A16 ==> ESPI_SOC_ALERT_L */
+	PAD_CFG_NF_IOSSTATE(GPP_A16, UP_20K, DEEP, NF1, IGNORE),
 	/* GPP_A17 : [] ==> EC_SOC_INT_ODL */
 	PAD_CFG_GPI_APIC_LOCK(GPP_A17, NONE, LEVEL, INVERT, LOCK_CONFIG),
 
@@ -94,11 +102,11 @@ static const struct pad_config gpio_table_id0[] = {
 	/* GPP_B23 : [] ==> WWAN_CONFIG0 */
 	PAD_CFG_GPI_LOCK(GPP_B23, NONE, LOCK_CONFIG),
 
-	/* GPP_C00 : [] ==> EN_PP3300_TCHSCR */
+	/* GPP_C00 : [] ==> EN_TCHSCR_PWR */
 	PAD_CFG_GPO(GPP_C00, 0, DEEP),
-	/* GPP_C01 : [] ==> USI_RST_L */
+	/* GPP_C01 : [] ==>  SOC_TCHSCR_RST_R_L */
 	PAD_CFG_GPO(GPP_C01, 0, DEEP),
-	/* GPP_C02 : GPP_C02_STRAP ==> Component NC */
+	/* GPP_C02 : SOC_TCHSCR_SPI_INT_STRAP ==> Component NC */
 	PAD_NC(GPP_C02, NONE),
 	/* GPP_C03 : [] ==> EN_WCAM_SENR_PWR */
 	PAD_CFG_GPO_LOCK(GPP_C03, 0, LOCK_CONFIG),
@@ -106,23 +114,8 @@ static const struct pad_config gpio_table_id0[] = {
 	PAD_CFG_GPO_LOCK(GPP_C04, 0, LOCK_CONFIG),
 	/* GPP_C05 : [] ==> WWAN_PERST_L_STRAP */
 	PAD_CFG_GPO(GPP_C05, 1, PLTRST),
-	/* GPP_C06 : [] ==> USI_REPORT_EN */
-	/*
-	 * FIXME: Remove this code after resolving b/247029304.
-	 *
-	 * ELAN6918 Power Sequencing seems not perfectly matching
-	 * with the previous platforms and setting GPP_C06 to high prior
-	 * to the power sequencing is actually makes it work.
-	 *
-	 * Ideally Power Sequencing should be as below for ELAN6918 (in ACPI)
-	 * `POWER enabled -> RESET deasserted -> Report EN enabled`
-	 *
-	 * But below sequence is only working currently:
-	 * `Report EN enabled (ramstage) -> POWER enabled (ACPI) -> RESET deasserted (ACPI)`
-	 */
-	PAD_CFG_GPO(GPP_C06, 1, DEEP),
-	/* GPP_C07 : [] ==> USI_INT */
-	PAD_CFG_GPI_APIC(GPP_C07, NONE, PLTRST, LEVEL, NONE),
+	/* GPP_C06 : [] ==> SOC_TCHSCR_RPT_EN */
+	PAD_CFG_GPO(GPP_C06, 0, DEEP),
 	/* GPP_C08 : [] ==> SOCHOT_ODL */
 	PAD_CFG_NF(GPP_C08, NONE, DEEP, NF2),
 	/* GPP_C09 : net NC is not present in the given design */
@@ -135,8 +128,8 @@ static const struct pad_config gpio_table_id0[] = {
 	PAD_CFG_NF(GPP_C12, NONE, DEEP, NF1),
 	/* GPP_C13 : [] ==> SSD_CLKREQ_ODL */
 	PAD_CFG_NF(GPP_C13, NONE, DEEP, NF1),
-	/* GPP_C15 : [] ==> GPP_C15_STRAP */
-	PAD_NC(GPP_C15, NONE),
+	/* GPP_C15 : [] ==> WWAN_DPR_SAR_ODL */
+	PAD_CFG_GPO(GPP_C15, 1, DEEP),
 	/* GPP_C16 : [] ==> USB_C0_LSX_TX */
 	PAD_CFG_NF(GPP_C16, NONE, DEEP, NF1),
 	/* GPP_C17 : [] ==> USB_C0_LSX_RX */
@@ -168,8 +161,8 @@ static const struct pad_config gpio_table_id0[] = {
 	PAD_NC(GPP_D05, NONE),
 	/* GPP_D06 : net NC.  Test pad.*/
 	PAD_NC(GPP_D06, NONE),
-	/* GPP_D07 : net NC.  Test pad. */
-	PAD_NC(GPP_D07, NONE),
+	/* GPP_D07 : [] ==> FPMCU_UWB_MUX_SEL */
+	PAD_CFG_GPO_LOCK(GPP_D07, 1, LOCK_CONFIG),
 	/* GPP_D08 : net NC.  Test pad. */
 	PAD_NC(GPP_D08, NONE),
 	/* GPP_D09 : [] ==> I2S_MCLK_R */
@@ -192,8 +185,8 @@ static const struct pad_config gpio_table_id0[] = {
 	PAD_CFG_NF(GPP_D17, NONE, DEEP, NF2),
 	/* GPP_D18 : net NC is not present in the given design */
 	PAD_NC(GPP_D18, NONE),
-	/* GPP_D19 : net NC is not present in the given design */
-	PAD_NC(GPP_D19, NONE),
+	/* GPP_D19 : [] ==> EC_SOC_REC_SWITCH_ODL */
+	PAD_CFG_GPI_LOCK(GPP_D19, NONE, LOCK_CONFIG),
 	/* GPP_D20 : net NC is not present in the given design */
 	PAD_NC(GPP_D20, NONE),
 	/* GPP_D21 : [] ==> WLAN_CLKREQ_ODLl */
@@ -211,8 +204,8 @@ static const struct pad_config gpio_table_id0[] = {
 	PAD_CFG_GPI_LOCK(GPP_E02, NONE, LOCK_CONFIG),
 	/* GPP_E03 : [] ==> GSC_SOC_INT_ODL */
 	PAD_CFG_GPI_APIC_LOCK(GPP_E03, NONE, LEVEL, INVERT, LOCK_CONFIG),
-	/* GPP_E04 : [] ==> HPS_INT_L */
-	PAD_CFG_GPI_IRQ_WAKE(GPP_E04, NONE, PLTRST, LEVEL, NONE),
+	/* GPP_E04 : [] ==> SOC_PEN_DETECT */
+	PAD_CFG_GPI_IRQ_WAKE(GPP_E04, NONE, PLTRST, LEVEL, INVERT),
 	/* GPP_E05 : [] ==> USB_A0_RT_RST_ODL */
 	PAD_CFG_GPO(GPP_E05, 1, DEEP),
 	/* GPP_E06 : GPP_E06_STRAP ==> Component NC */
@@ -237,65 +230,65 @@ static const struct pad_config gpio_table_id0[] = {
 	PAD_NC(GPP_E15, NONE),
 	/* GPP_E16 : net NC.  Test pad. */
 	PAD_NC(GPP_E16, NONE),
-	/* GPP_E17 : [] ==> EN_HPS_PWR */
-	PAD_CFG_GPO(GPP_E17, 1, DEEP),
-	/* GPP_E22 : net EN_PP3300_WLAN is not present in the given design */
-	PAD_NC(GPP_E22, NONE),
+	/* GPP_E17 : net NC is not present in the given design */
+	PAD_NC(GPP_E17, NONE),
+	/* GPP_E22 : [] ==> EN_PP3300_WLAN */
+	PAD_CFG_GPO(GPP_E22, 1, DEEP),
 
 	/* GPP_F00 :  [] ==> CNV_BRI_DT_R */
-	PAD_CFG_NF(GPP_F00, NONE, DEEP, NF1),
+	PAD_CFG_NF_IOSTANDBY_IGNORE(GPP_F00, NONE, DEEP, NF1),
 	/* GPP_F01 :  [] ==> CNV_BRI_RSP */
-	PAD_CFG_NF(GPP_F01, UP_20K, DEEP, NF1),
+	PAD_CFG_NF_IOSTANDBY_IGNORE(GPP_F01, UP_20K, DEEP, NF1),
 	/* GPP_F02 :  [] ==> CNV_RGI_DT_Rl */
-	PAD_CFG_NF(GPP_F02, NONE, DEEP, NF1),
+	PAD_CFG_NF_IOSTANDBY_IGNORE(GPP_F02, NONE, DEEP, NF1),
 	/* GPP_F03 :  [] ==> CNV_RGI_RSP */
-	PAD_CFG_NF(GPP_F03, UP_20K, DEEP, NF1),
+	PAD_CFG_NF_IOSTANDBY_IGNORE(GPP_F03, UP_20K, DEEP, NF1),
 	/* GPP_F04 :  [] ==> CNV_RF_RST_L */
-	PAD_CFG_NF(GPP_F04, NONE, DEEP, NF1),
+	PAD_CFG_NF_IOSTANDBY_IGNORE(GPP_F04, NONE, DEEP, NF1),
 	/* GPP_F05 :  [] ==> CNV_CLKREQ */
-	PAD_CFG_NF(GPP_F05, NONE, DEEP, NF3),
+	PAD_CFG_NF_IOSTANDBY_IGNORE(GPP_F05, NONE, DEEP, NF3),
 	/* GPP_F06 :  [] ==> WWAN_WLAN_COEX3 */
 	PAD_CFG_NF(GPP_F06, NONE, DEEP, NF1),
 	/* GPP_F07 :  [] ==> UCAM_MCLK_R */
 	PAD_CFG_GPO(GPP_F07, 0, DEEP),
 	/* GPP_F08 : [] ==> WLAN_PERST_L */
 	PAD_CFG_GPO(GPP_F08, 1, DEEP),
-	/* GPP_F09 : [] ==> EN_PP3300_WLAN */
-	PAD_CFG_GPO(GPP_F09, 1, DEEP),
+	/* GPP_F09 : [] ==> WLAN_PE_WAKE_ODL */
+	PAD_CFG_GPI_IRQ_WAKE(GPP_F09, NONE, PLTRST, LEVEL, INVERT),
 	/* GPP_F10 : [] ==> WWAN_PCIE_WAKE_ODL */
 	PAD_CFG_GPI_IRQ_WAKE(GPP_F10, NONE, PLTRST, LEVEL, INVERT),
 	/* GPP_F11 : GSP1_SOC_CLK_R */
 	PAD_CFG_NF(GPP_F11, NONE, DEEP, NF5),
 	/* GPP_F12 : GSPI1_SOC_DO_FPMCU_DI_R */
 	PAD_CFG_NF(GPP_F12, NONE, DEEP, NF5),
-	/* GPP_F13 : GSPI1_SOC_DI_FPMCU_DO_LS */
+	/* GPP_F13 : GSPI1_SOC_DI_FPMCU_DO_LS_R */
 	PAD_CFG_NF(GPP_F13, NONE, DEEP, NF5),
-	/* GPP_F14 : GSPI_SOC_DO_TCHSCR_DI */
+	/* GPP_F14 : GSPI0_SOC_DO_TCHSCR_DI */
 	PAD_CFG_NF(GPP_F14, NONE, DEEP, NF8),
-	/* GPP_F15 : [] ==> GSPI_SOC_DI_TCHSCR_DO */
+	/* GPP_F15 : [] ==> GSPI0_SOC_DI_TCHSCR_DO */
 	PAD_CFG_NF(GPP_F15, NONE, DEEP, NF8),
-	/* GPP_F16 : [] ==> GSPI_SOC_TCHSCR_CLK */
+	/* GPP_F16 : [] ==> GSPI0_SOC_TCHSCR_CLK */
 	PAD_CFG_NF(GPP_F16, NONE, DEEP, NF8),
 	/* GPP_F17 : [] ==> GSPI1_SOC_CS_L */
 	PAD_CFG_NF(GPP_F17, NONE, DEEP, NF5),
-	/* GPP_F18 : [] ==> GSPI_SOC_TCHSCR_CS_L */
+	/* GPP_F18 : [] ==> GSPI0_SOC_TCHSCR_CS_L */
 	PAD_CFG_NF(GPP_F18, NONE, DEEP, NF8),
 	/* GPP_F19 : [] ==> GPP_F19_STRAP */
 	PAD_NC(GPP_F19, NONE),
 	/* GPP_F20 : [] ==> GPP_F20_STRAP */
 	PAD_NC(GPP_F20, NONE),
-	/* GPP_F21 : [] ==> GPP_F21_STRAP */
+	/* GPP_F21 : [] ==> SPI_SOC_CS_UWB_L_STRAP */
 	PAD_NC(GPP_F21, NONE),
 	/* GPP_F22 : net NC is not present in the given design */
 	PAD_NC(GPP_F22, NONE),
 	/* GPP_F23 : net NC is not present in the given design */
 	PAD_NC(GPP_F23, NONE),
 
-	/* GPP_H00 : GPP_H00_STRAP ==> Component NC */
+	/* GPP_H00 : SPI_SOC_CLK_UWB_STRAP_R ==> Component NC */
 	PAD_NC(GPP_H00, NONE),
-	/* GPP_H01 : GPP_H01_STRAP ==> Component NC */
+	/* GPP_H01 : SPI_SOC_DO_UWB_DI_STRAP_R ==> Component NC */
 	PAD_NC(GPP_H01, NONE),
-	/* GPP_H02 : GPP_H02_STRAP ==> Component NC */
+	/* GPP_H02 : SPI_SOC_DI_UWB_DO_STRAP ==> Component NC */
 	PAD_NC(GPP_H02, NONE),
 	/* GPP_H04 : [] ==> WWAN_WLAN_COEX1 */
 	PAD_CFG_NF(GPP_H04, NONE, DEEP, NF2),
@@ -317,8 +310,8 @@ static const struct pad_config gpio_table_id0[] = {
 	PAD_CFG_NF(GPP_H13, NONE, DEEP, NF1),
 	/* GPP_H14 : [] ==> SLP_S0_GATE_R */
 	PAD_CFG_GPO(GPP_H14, 1, PLTRST),
-	/* GPP_H15 : net NC is not present in the given design */
-	PAD_NC(GPP_H15, NONE),
+	/* GPP_H15 : [] ==> EN_DMIC_SOC_DATA */
+	PAD_CFG_GPO(GPP_H15, 0, PLTRST),
 	/* GPP_H16 : [] ==> DDIB_HDMI_CTRLCLK*/
 	PAD_CFG_NF(GPP_H16, NONE, DEEP, NF1),
 	/* GPP_H17 : [] ==> DDIB_HDMI_CTRLDATA */
@@ -332,13 +325,13 @@ static const struct pad_config gpio_table_id0[] = {
 	/* GPP_H22 : [] ==>  SOC_I2C_TCHSCR_SCL */
 	PAD_CFG_NF(GPP_H22, NONE, DEEP, NF1),
 
-	/* GPP_S00 :  [] ==> SDW_HP_CLK  */
+	/* GPP_S00 :  [] ==> SDW_HP_CLK_WLAN_PCM_CLK  */
 	PAD_CFG_NF(GPP_S00, NONE, DEEP, NF1),
-	/* GPP_S01 :  [] ==> SDW_HP_DATA */
+	/* GPP_S01 :  [] ==> SDW_HP_DATA_WLAN_PCM_SYNC */
 	PAD_CFG_NF(GPP_S01, NONE, DEEP, NF1),
-	/* GPP_S02 : [] ==> DMIC_SOC_CLK0_DB_RC */
+	/* GPP_S02 : [] ==> DMIC_SOC_CLK0_WLAN_PCM_OUT */
 	PAD_CFG_NF(GPP_S02, NONE, DEEP, NF3),
-	/* GPP_S03 : [] ==> DMIC_SOC_DATA0_DB_R */
+	/* GPP_S03 : [] ==> DMIC_SOC_DATA0_WLAN_PCM_IN */
 	PAD_CFG_NF(GPP_S03, NONE, DEEP, NF3),
 	/* GPP_S04 : [] ==> SDW_SPKR_CLK */
 	PAD_CFG_NF(GPP_S04, NONE, DEEP, NF1),
@@ -382,7 +375,7 @@ static const struct pad_config gpio_table_id0[] = {
 };
 
 /* Early pad configuration in bootblock */
-static const struct pad_config early_gpio_table_id0[] = {
+static const struct pad_config early_gpio_table[] = {
 	/* GPP_B17 : [] ==> EN_WWAN_PWR */
 	PAD_CFG_GPO(GPP_B17, 1, DEEP),
 	/* GPP_B18 : [] ==> SOC_I2C_TPM_SDA */
@@ -416,78 +409,41 @@ static const struct pad_config early_gpio_table_id0[] = {
 	PAD_CFG_GPI_GPIO_DRIVER_LOCK(GPP_H10, NONE, LOCK_CONFIG),
 };
 
-/* Default/Minimal early pad configuration if we can't find board_id */
-static const struct pad_config default_early_gpio_table[] = {
-	/* GPP_B18 : [] ==> SOC_I2C_TPM_SDA */
-	PAD_CFG_NF(GPP_B18, NONE, DEEP, NF2),
-	/* GPP_B19 : [] ==> SOC_I2C_TPM_SCL */
-	PAD_CFG_NF(GPP_B19, NONE, DEEP, NF2),
-	/* GPP_E03 : [] ==> GSC_SOC_INT_ODL */
-	PAD_CFG_GPI_APIC(GPP_E03, NONE, PLTRST, LEVEL, INVERT),
-
-	/* GPP_H08 : [] ==> UART_DBG_TX_SOC_RX_R */
-	PAD_CFG_NF(GPP_H08, NONE, DEEP, NF1),
-	/* GPP_H09 : [] ==> UART_SOC_TX_DBG_RX_R */
-	PAD_CFG_NF(GPP_H09, NONE, DEEP, NF1),
-};
-
-static const struct pad_config romstage_gpio_table_id0[] = {
-	/* A20  : [] ==> SSD_PERST_L */
+static const struct pad_config romstage_gpio_table[] = {
+	/* GPP_B11 : [] ==> EN_FP_PWR */
+	PAD_CFG_GPO(GPP_B11, 0, DEEP),
+	/* A20 : [] ==> SSD_PERST_L */
 	PAD_CFG_GPO(GPP_A20, 0, DEEP),
+	/* GPP_C23 : [] ==> FP_RST_ODL */
+	PAD_CFG_GPO(GPP_C23, 0, DEEP),
 	/* GPP_E07 : [] ==> WWAN_FCPO_L */
 	PAD_CFG_GPO(GPP_E07, 1, DEEP),
+	/* GPP_D02 : [] ==> SD_PERST_L */
+	PAD_CFG_GPO(GPP_D02, 1, DEEP),
 };
 
 const struct pad_config *variant_gpio_table(size_t *num)
 {
-	const uint32_t id = board_id();
-	switch (id) {
-	case 0:
-		*num = ARRAY_SIZE(gpio_table_id0);
-		return gpio_table_id0;
-
-	case BOARD_ID_UNKNOWN:
-	default:
-		printk(BIOS_ERR, "board_id() not found.  Unable to load gpio table.\n");
-		*num = 0;
-		return NULL;
-	}
+	*num = ARRAY_SIZE(gpio_table);
+	return gpio_table;
 }
 
 const struct pad_config *variant_early_gpio_table(size_t *num)
 {
-	const uint32_t id = board_id();
-	switch (id) {
-	case 0:
-		*num = ARRAY_SIZE(early_gpio_table_id0);
-		return early_gpio_table_id0;
-
-	case BOARD_ID_UNKNOWN:
-	default:
-		printk(BIOS_ERR, "board_id() not found.  Loading default early gpio table.\n");
-		*num = ARRAY_SIZE(default_early_gpio_table);
-		return default_early_gpio_table;
-	}
+	*num = ARRAY_SIZE(early_gpio_table);
+	return early_gpio_table;
 }
 
 /* Create the stub for romstage gpio, typically use for power sequence */
 const struct pad_config *variant_romstage_gpio_table(size_t *num)
 {
-	const uint32_t id = board_id();
-	switch (id) {
-	case 0:
-		*num = ARRAY_SIZE(romstage_gpio_table_id0);
-		return romstage_gpio_table_id0;
-
-	case BOARD_ID_UNKNOWN:
-	default:
-		printk(BIOS_ERR,
-		       "board_id() not found.  Unable to load romstage gpio table.\n");
-		*num = 0;
-		return NULL;
-	}
+	*num = ARRAY_SIZE(romstage_gpio_table);
+	return romstage_gpio_table;
 }
 
-static const struct cros_gpio cros_gpios[] = {};
+static const struct cros_gpio cros_gpios[] = {
+	CROS_GPIO_REC_AL(CROS_GPIO_VIRTUAL, CROS_GPIO_DEVICE_NAME),
+	CROS_GPIO_WP_AH(GPIO_PCH_WP, CROS_GPIO_DEVICE_NAME),
+};
 
-DECLARE_WEAK_CROS_GPIOS(cros_gpios);
+DECLARE_CROS_GPIOS(cros_gpios);
