@@ -4,32 +4,11 @@
 #include <cpu/x86/smm.h>
 #include <ec/google/chromeec/smm.h>
 #include <soc/iomap.h>
-#include <soc/nvs.h>
 #include <soc/pm.h>
 #include <intelblocks/smihandler.h>
 #include "ec.h"
 #include "gpio.h"
 
-int mainboard_io_trap_handler(int smif)
-{
-	switch (smif) {
-	case 0x99:
-		printk(BIOS_DEBUG, "Sample\n");
-		gnvs->smif = 0;
-		break;
-	default:
-		return 0;
-	}
-
-	/* On success, the IO Trap Handler returns 0
-	 * On failure, the IO Trap Handler returns a value != 0
-	 *
-	 * For now, we force the return value to 0 and log all traps to
-	 * see what's going on.
-	 */
-	return 1;
-}
-
 void mainboard_smi_gpi_handler(const struct gpi_status *sts)
 {
 	if (gpi_status_get(sts, EC_SMI_GPI))
