@@ -6,13 +6,13 @@
 #include <device/device.h>
 #include <device/pci.h>
 #include <cpu/cpu.h>
+#include <cpu/x86/smm.h>
 
 #include "e7505.h"
 
 static void mch_domain_read_resources(struct device *dev)
 {
 	int idx;
-	unsigned long tolmk;
 	uint64_t tom, remapbase, remaplimit;
 	struct device *mc_dev;
 
@@ -22,9 +22,6 @@ static void mch_domain_read_resources(struct device *dev)
 	if (!mc_dev)
 		die("Could not find MCH device\n");
 
-	tolmk = pci_read_config16(mc_dev, TOLM) >> 11;
-	tolmk <<= 17;
-
 	tom = pci_read_config8(mc_dev, DRB_ROW_7);
 	tom <<= 26;
 
@@ -39,9 +36,15 @@ static void mch_domain_read_resources(struct device *dev)
 
 	/* Report the memory regions */
 	idx = 10;
-	ram_resource_kb(dev, idx++, 0, tolmk);
-	mmio_resource_kb(dev, idx++, 0xa0000 / KiB, (0xc0000 - 0xa0000) / KiB);
+	ram_range(dev, idx++, 0, 0xa0000);
+	mmio_from_to(dev, idx++, 0xa0000, 0xc0000);
+	ram_from_to(dev, idx++, 0xc0000, 1 * MiB);
 
+	uintptr_t tseg_base;
+	size_t tseg_size;
+	smm_region(&tseg_base, &tseg_size);
+	ram_from_to(dev, idx++, 1 * MiB, tseg_base);
+	mmio_range(dev, idx++, tseg_base, tseg_size);
 
 	ASSERT(tom == remapbase);
 	upper_ram_end(dev, idx++, remaplimit);
@@ -52,36 +55,20 @@ static void mch_domain_set_resources(struct device *dev)
 	assign_resources(dev->link_list);
 }
 
-static struct device_operations pci_domain_ops = {
+struct device_operations e7505_pci_domain_ops = {
 	.read_resources   = mch_domain_read_resources,
 	.set_resources    = mch_domain_set_resources,
 	.scan_bus         = pci_domain_scan_bus,
 	.ops_pci          = &pci_dev_ops_pci,
 };
 
-static void cpu_bus_init(struct device *dev)
-{
-	initialize_cpus(dev->link_list);
-}
 
-static struct device_operations cpu_bus_ops = {
+struct device_operations e7505_cpu_bus_ops = {
 	.read_resources   = noop_read_resources,
 	.set_resources    = noop_set_resources,
-	.init             = cpu_bus_init,
+	.init             = mp_cpu_bus_init,
 };
 
-static void enable_dev(struct device *dev)
-{
-	/* Set the operations if it is a special bus type */
-	if (dev->path.type == DEVICE_PATH_DOMAIN) {
-		dev->ops = &pci_domain_ops;
-	}
-	else if (dev->path.type == DEVICE_PATH_CPU_CLUSTER) {
-		dev->ops = &cpu_bus_ops;
-	}
-}
-
 struct chip_operations northbridge_intel_e7505_ops = {
 	CHIP_NAME("Intel E7505 Northbridge")
-	.enable_dev = enable_dev,
 };
