@@ -7,7 +7,6 @@
 #include <cpu/intel/haswell/haswell.h>
 #include <device/device.h>
 #include <device/pci.h>
-#include <device/pci_def.h>
 #include <device/pci_ids.h>
 #include <device/pci_ops.h>
 #include <boot/tables.h>
@@ -34,7 +33,7 @@ static const char *northbridge_acpi_name(const struct device *dev)
 	return NULL;
 }
 
-static struct device_operations pci_domain_ops = {
+struct device_operations haswell_pci_domain_ops = {
 	.read_resources    = pci_domain_read_resources,
 	.set_resources     = pci_domain_set_resources,
 	.scan_bus          = pci_domain_scan_bus,
@@ -97,7 +96,6 @@ static void mc_add_fixed_mmio_resources(struct device *dev)
 	for (i = 0; i < ARRAY_SIZE(mc_fixed_resources); i++) {
 		u32 base;
 		u32 size;
-		struct resource *resource;
 		unsigned int index;
 
 		size = mc_fixed_resources[i].size;
@@ -105,12 +103,7 @@ static void mc_add_fixed_mmio_resources(struct device *dev)
 		if (!mc_fixed_resources[i].get_resource(dev, index, &base, &size))
 			continue;
 
-		resource = new_resource(dev, mc_fixed_resources[i].index);
-		resource->base = base;
-		resource->size = size;
-		resource->flags = IORESOURCE_MEM | IORESOURCE_FIXED | IORESOURCE_STORED |
-				  IORESOURCE_RESERVE | IORESOURCE_ASSIGNED;
-
+		mmio_range(dev, mc_fixed_resources[i].index, base, size);
 		printk(BIOS_DEBUG, "%s: Adding %s @ %x 0x%08lx-0x%08lx.\n",
 		       __func__, mc_fixed_resources[i].description, index,
 		       (unsigned long)base, (unsigned long)(base + size - 1));
@@ -237,8 +230,7 @@ static void mc_report_map_entries(struct device *dev, uint64_t *values)
 
 static void mc_add_dram_resources(struct device *dev, int *resource_cnt)
 {
-	unsigned long base_k, size_k, index;
-	struct resource *resource;
+	int index;
 	uint64_t mc_values[NUM_MAP_ENTRIES];
 
 	/* Read in the MAP registers and report their values */
@@ -284,43 +276,29 @@ static void mc_add_dram_resources(struct device *dev, int *resource_cnt)
 	 */
 	index = *resource_cnt;
 
-	/* 0 - > 0xa0000 */
-	base_k = 0;
-	size_k = (0xa0000 >> 10) - base_k;
-	ram_resource_kb(dev, index++, base_k, size_k);
-
-	/* 0xc0000 -> TSEG - DPR */
-	base_k = 0xc0000 >> 10;
-	size_k = (unsigned long)(mc_values[TSEG_REG] >> 10) - base_k;
-	size_k -= dpr.size * MiB / KiB;
-	ram_resource_kb(dev, index++, base_k, size_k);
-
-	/* TSEG - DPR -> BGSM */
-	resource = new_resource(dev, index++);
-	resource->base = mc_values[TSEG_REG] - dpr.size * MiB;
-	resource->size = mc_values[BGSM_REG] - (mc_values[TSEG_REG] - dpr.size * MiB);
-	resource->flags = IORESOURCE_MEM | IORESOURCE_FIXED | IORESOURCE_STORED |
-			  IORESOURCE_RESERVE | IORESOURCE_ASSIGNED | IORESOURCE_CACHEABLE;
-
-	/* BGSM -> TOLUD. If the IGD is disabled, BGSM can equal TOLUD. */
-	if (mc_values[BGSM_REG] != mc_values[TOLUD_REG]) {
-		resource = new_resource(dev, index++);
-		resource->base = mc_values[BGSM_REG];
-		resource->size = mc_values[TOLUD_REG] - mc_values[BGSM_REG];
-		resource->flags = IORESOURCE_MEM | IORESOURCE_FIXED | IORESOURCE_STORED |
-				  IORESOURCE_RESERVE | IORESOURCE_ASSIGNED;
-	}
-
-	/* 4GiB -> TOUUD */
-	upper_ram_end(dev, index++, mc_values[TOUUD_REG]);
-
-	/* Reserve everything between A segment and 1MB:
-	 *
+	/*
+	 * 0 - > 0xa0000: RAM
 	 * 0xa0000 - 0xbffff: Legacy VGA
 	 * 0xc0000 - 0xfffff: RAM
 	 */
-	mmio_resource_kb(dev, index++, (0xa0000 >> 10), (0xc0000 - 0xa0000) >> 10);
-	reserved_ram_resource_kb(dev, index++, (0xc0000 >> 10), (0x100000 - 0xc0000) >> 10);
+
+	ram_range(dev, index++, 0, 0xa0000);
+	mmio_from_to(dev, index++, 0xa0000, 0xc0000);
+	reserved_ram_from_to(dev, index++, 0xc0000, 1 * MiB);
+
+	/* 1MiB -> TSEG - DPR */
+	ram_from_to(dev, index++, 1 * MiB, mc_values[TSEG_REG] - dpr.size * MiB);
+
+	/* TSEG - DPR -> BGSM */
+	reserved_ram_from_to(dev, index++, mc_values[TSEG_REG] - dpr.size * MiB,
+			     mc_values[BGSM_REG]);
+
+	/* BGSM -> TOLUD. If the IGD is disabled, BGSM can equal TOLUD. */
+	if (mc_values[BGSM_REG] != mc_values[TOLUD_REG])
+		mmio_from_to(dev, index++, mc_values[BGSM_REG], mc_values[TOLUD_REG]);
+
+	/* 4GiB -> TOUUD */
+	upper_ram_end(dev, index++, mc_values[TOUUD_REG]);
 
 	*resource_cnt = index;
 }
@@ -338,8 +316,8 @@ static void mc_read_resources(struct device *dev)
 
 	/* Add VT-d MMIO resources, if capable */
 	if (vtd_capable) {
-		mmio_resource_kb(dev, index++, GFXVT_BASE_ADDRESS / KiB, GFXVT_BASE_SIZE / KiB);
-		mmio_resource_kb(dev, index++, VTVC0_BASE_ADDRESS / KiB, VTVC0_BASE_SIZE / KiB);
+		mmio_range(dev, index++, GFXVT_BASE_ADDRESS, GFXVT_BASE_SIZE);
+		mmio_range(dev, index++, VTVC0_BASE_ADDRESS, VTVC0_BASE_SIZE);
 	}
 
 	/* Calculate and add DRAM resources */
@@ -536,7 +514,6 @@ static struct device_operations mc_ops = {
 	.enable_resources	= pci_dev_enable_resources,
 	.init			= northbridge_init,
 	.final			= northbridge_final,
-	.acpi_fill_ssdt		= generate_cpu_entries,
 	.ops_pci		= &pci_dev_ops_pci,
 };
 
@@ -557,23 +534,13 @@ static const struct pci_driver mc_driver_hsw __pci_driver = {
 	.devices = mc_pci_device_ids,
 };
 
-static struct device_operations cpu_bus_ops = {
+struct device_operations haswell_cpu_bus_ops = {
 	.read_resources   = noop_read_resources,
 	.set_resources    = noop_set_resources,
 	.init             = mp_cpu_bus_init,
+	.acpi_fill_ssdt   = generate_cpu_entries,
 };
 
-static void enable_dev(struct device *dev)
-{
-	/* Set the operations if it is a special bus type. */
-	if (dev->path.type == DEVICE_PATH_DOMAIN) {
-		dev->ops = &pci_domain_ops;
-	} else if (dev->path.type == DEVICE_PATH_CPU_CLUSTER) {
-		dev->ops = &cpu_bus_ops;
-	}
-}
-
 struct chip_operations northbridge_intel_haswell_ops = {
 	CHIP_NAME("Intel Haswell integrated Northbridge")
-	.enable_dev = enable_dev,
 };
