@@ -22,7 +22,7 @@ static void report_cpu_info(void)
 	if (cpuidr.eax < 0x80000004) {
 		strcpy(cpu_string, "Platform info not available");
 	} else {
-		u32 *p = (u32*) cpu_string;
+		u32 *p = (u32 *)cpu_string;
 		for (i = 2; i <= 4; i++) {
 			cpuidr = cpuid(index + i);
 			*p++ = cpuidr.eax;
@@ -67,6 +67,11 @@ static struct {
 	{0x8c54, "C224"},
 	{0x8c56, "C226"},
 	{0x8c5c, "H81"},
+	{0x8cc1, "Mobile Engineering Sample (9 series)"},
+	{0x8cc2, "Desktop Engineering Sample (9 series)"},
+	{0x8cc3, "HM97"},
+	{0x8cc4, "Z97"},
+	{0x8cc6, "H97"},
 	{0x9c41, "LP Full Featured Engineering Sample"},
 	{0x9c43, "LP Premium"},
 	{0x9c45, "LP Mainstream"},
