@@ -168,7 +168,7 @@ static void sdram_initialize(struct pei_data *pei_data)
 			default:
 				printk(BIOS_ERR, "MRC returned %x.\n", rv);
 			}
-			die_with_post_code(POST_INVALID_VENDOR_BINARY,
+			die_with_post_code(POSTCODE_INVALID_VENDOR_BINARY,
 					   "Nonzero MRC return value.\n");
 		}
 	} else {
@@ -328,7 +328,7 @@ void perform_raminit(int s3resume)
 	struct mrc_var_data *mrc_var;
 
 	/* Prepare USB controller early in S3 resume */
-	if (!mainboard_should_reset_usb(s3resume))
+	if (s3resume)
 		enable_usb_bar();
 
 	memset(&pei_data, 0, sizeof(pei_data));
@@ -433,7 +433,7 @@ static void setup_sdram_meminfo(struct pei_data *pei_data)
 			dimm->mod_id =					/* bytes 117/118 */
 				(pei_data->spd_data[0][SPD_DIMM_MOD_ID2] << 8) |
 				(pei_data->spd_data[0][SPD_DIMM_MOD_ID1] & 0xFF);
-			dimm->mod_type = DDR3_SPD_SODIMM;
+			dimm->mod_type = SPD_DDR3_DIMM_TYPE_SO_DIMM;
 			dimm->bus_width = MEMORY_BUS_WIDTH_64;
 			dimm_cnt++;
 		}
@@ -457,7 +457,7 @@ static void setup_sdram_meminfo(struct pei_data *pei_data)
 			dimm->mod_id =					/* bytes 117/118 */
 				(pei_data->spd_data[0][SPD_DIMM_MOD_ID2] << 8) |
 				(pei_data->spd_data[0][SPD_DIMM_MOD_ID1] & 0xFF);
-			dimm->mod_type = DDR3_SPD_SODIMM;
+			dimm->mod_type = SPD_DDR3_DIMM_TYPE_SO_DIMM;
 			dimm->bus_width = MEMORY_BUS_WIDTH_64;
 			dimm_cnt++;
 		}
