@@ -3,8 +3,6 @@
 #ifndef TIS_H_
 #define TIS_H_
 
-#include <stddef.h>
-#include <stdint.h>
 #include <types.h>
 
 enum tis_access {
@@ -44,22 +42,12 @@ int tis_init(void);
 /*
  * tis_open()
  *
- * Requests access to locality 0 for the caller. After all commands have been
- * completed the caller is supposed to call tis_close().
+ * Requests access to locality 0 for the caller.
  *
  * Returns 0 on success, -1 on failure.
  */
 int tis_open(void);
 
-/*
- * tis_close()
- *
- * terminate the current session with the TPM by releasing the locked
- * locality. Returns 0 on success of -1 on failure (in case lock
- * removal did not succeed).
- */
-int tis_close(void);
-
 /*
  * tis_sendrecv()
  *
@@ -76,17 +64,6 @@ int tis_close(void);
 int tis_sendrecv(const u8 *sendbuf, size_t send_size, u8 *recvbuf,
 			size_t *recv_len);
 
-/* TODO: This is supposed to be used only for Google TPM.
-   Consider moving this to drivers/tpm/cr50.h. */
-/*
- * tis_plat_irq_status()
- *
- * Check tpm irq and clear it.
- *
- * Returns 1 when irq pending or 0 when not.
- */
-int tis_plat_irq_status(void);
-
 /*
  * tis_vendor_write()
  *
