@@ -2,15 +2,17 @@
 
 #include <console/console.h>
 #include <fmap.h>
+#include <bootstate.h>
 #include <cbfs.h>
+#include <symbols.h>
 #include "crtm.h"
 #include <string.h>
 
-static int tcpa_log_initialized;
-static inline int tcpa_log_available(void)
+static int tpm_log_initialized;
+static inline int tpm_log_available(void)
 {
 	if (ENV_BOOTBLOCK)
-		return tcpa_log_initialized;
+		return tpm_log_initialized;
 
 	return 1;
 }
@@ -33,10 +35,10 @@ static inline int tcpa_log_available(void)
  */
 static uint32_t tspi_init_crtm(void)
 {
-	/* Initialize TCPA PRERAM log. */
-	if (!tcpa_log_available()) {
-		tcpa_preram_log_clear();
-		tcpa_log_initialized = 1;
+	/* Initialize TPM PRERAM log. */
+	if (!tpm_log_available()) {
+		tpm_preram_log_clear();
+		tpm_log_initialized = 1;
 	} else {
 		printk(BIOS_WARNING, "TSPI: CRTM already initialized!\n");
 		return VB2_SUCCESS;
@@ -44,7 +46,7 @@ static uint32_t tspi_init_crtm(void)
 
 	struct region_device fmap;
 	if (fmap_locate_area_as_rdev("FMAP", &fmap) == 0) {
-		if (tpm_measure_region(&fmap, TPM_CRTM_PCR, "FMAP: FMAP")) {
+		if (tpm_measure_region(&fmap, CONFIG_PCR_SRTM, "FMAP: FMAP")) {
 			printk(BIOS_ERR,
 			       "TSPI: Couldn't measure FMAP into CRTM!\n");
 			return VB2_ERROR_UNKNOWN;
@@ -58,7 +60,7 @@ static uint32_t tspi_init_crtm(void)
 		struct region_device bootblock_fmap;
 		if (fmap_locate_area_as_rdev("BOOTBLOCK", &bootblock_fmap) == 0) {
 			if (tpm_measure_region(&bootblock_fmap,
-					TPM_CRTM_PCR,
+					CONFIG_PCR_SRTM,
 					"FMAP: BOOTBLOCK"))
 				return VB2_ERROR_UNKNOWN;
 		}
@@ -77,7 +79,7 @@ static uint32_t tspi_init_crtm(void)
 		/* Since none of the above conditions are met let the SOC code measure the
 		 * bootblock. This accomplishes for cases where the bootblock is treated
 		 * in a special way (e.g. part of IFWI or located in a different CBFS). */
-		if (tspi_soc_measure_bootblock(TPM_CRTM_PCR)) {
+		if (tspi_soc_measure_bootblock(CONFIG_PCR_SRTM)) {
 			printk(BIOS_INFO,
 			       "TSPI: Couldn't measure bootblock into CRTM on SoC level!\n");
 			return VB2_ERROR_UNKNOWN;
@@ -109,9 +111,9 @@ static bool is_runtime_data(const char *name)
 uint32_t tspi_cbfs_measurement(const char *name, uint32_t type, const struct vb2_hash *hash)
 {
 	uint32_t pcr_index;
-	char tcpa_metadata[TCPA_PCR_HASH_NAME];
+	char tpm_log_metadata[TPM_CB_LOG_PCR_HASH_NAME];
 
-	if (!tcpa_log_available()) {
+	if (!tpm_log_available()) {
 		if (tspi_init_crtm() != VB2_SUCCESS) {
 			printk(BIOS_WARNING,
 			       "Initializing CRTM failed!\n");
@@ -122,7 +124,7 @@ uint32_t tspi_cbfs_measurement(const char *name, uint32_t type, const struct vb2
 
 	switch (type) {
 	case CBFS_TYPE_MRC_CACHE:
-		pcr_index = TPM_RUNTIME_DATA_PCR;
+		pcr_index = CONFIG_PCR_RUNTIME_DATA;
 		break;
 	/*
 	 * mrc.bin is code executed on CPU, so it
@@ -132,55 +134,95 @@ uint32_t tspi_cbfs_measurement(const char *name, uint32_t type, const struct vb2
 	case CBFS_TYPE_STAGE:
 	case CBFS_TYPE_SELF:
 	case CBFS_TYPE_FIT_PAYLOAD:
-		pcr_index = TPM_CRTM_PCR;
+		pcr_index = CONFIG_PCR_SRTM;
 		break;
 	default:
 		if (is_runtime_data(name))
-			pcr_index = TPM_RUNTIME_DATA_PCR;
+			pcr_index = CONFIG_PCR_RUNTIME_DATA;
 		else
-			pcr_index = TPM_CRTM_PCR;
+			pcr_index = CONFIG_PCR_SRTM;
 		break;
 	}
 
-	snprintf(tcpa_metadata, TCPA_PCR_HASH_NAME, "CBFS: %s", name);
+	snprintf(tpm_log_metadata, TPM_CB_LOG_PCR_HASH_NAME, "CBFS: %s", name);
 
 	return tpm_extend_pcr(pcr_index, hash->algo, hash->raw, vb2_digest_size(hash->algo),
-			      tcpa_metadata);
+			      tpm_log_metadata);
+}
+
+void *tpm_log_init(void)
+{
+	static void *tclt;
+
+	/* We are dealing here with pre CBMEM environment.
+	 * If cbmem isn't available use CAR or SRAM */
+	if (!cbmem_possibly_online() &&
+		!CONFIG(VBOOT_RETURN_FROM_VERSTAGE))
+		return _tpm_log;
+	else if (ENV_CREATES_CBMEM
+		 && !CONFIG(VBOOT_RETURN_FROM_VERSTAGE)) {
+		tclt = tpm_log_cbmem_init();
+		if (!tclt)
+			return _tpm_log;
+	} else {
+		tclt = tpm_log_cbmem_init();
+	}
+
+	return tclt;
 }
 
 int tspi_measure_cache_to_pcr(void)
 {
 	int i;
-	struct tcpa_table *tclt = tcpa_log_init();
+	int pcr;
+	const char *event_name;
+	const uint8_t *digest_data;
+	enum vb2_hash_algorithm digest_algo;
 
 	/* This means the table is empty. */
-	if (!tcpa_log_available())
+	if (!tpm_log_available())
 		return VB2_SUCCESS;
 
-	if (!tclt) {
-		printk(BIOS_WARNING, "TCPA: Log non-existent!\n");
+	if (tpm_log_init() == NULL) {
+		printk(BIOS_WARNING, "TPM LOG: log non-existent!\n");
 		return VB2_ERROR_UNKNOWN;
 	}
 
-	printk(BIOS_DEBUG, "TPM: Write digests cached in TCPA log to PCR\n");
-	for (i = 0; i < tclt->num_entries; i++) {
-		struct tcpa_entry *tce = &tclt->entries[i];
-		if (tce) {
-			printk(BIOS_DEBUG, "TPM: Write digest for"
-			       " %s into PCR %d\n",
-			       tce->name, tce->pcr);
-			int result = tlcl_extend(tce->pcr,
-						 tce->digest,
-						 NULL);
-			if (result != TPM_SUCCESS) {
-				printk(BIOS_ERR, "TPM: Writing digest"
-				       " of %s into PCR failed with error"
-				       " %d\n",
-				       tce->name, result);
-				return VB2_ERROR_UNKNOWN;
-			}
+	printk(BIOS_DEBUG, "TPM: Write digests cached in TPM log to PCR\n");
+	i = 0;
+	while (!tpm_log_get(i++, &pcr, &digest_data, &digest_algo, &event_name)) {
+		printk(BIOS_DEBUG, "TPM: Write digest for %s into PCR %d\n", event_name, pcr);
+		int result = tlcl_extend(pcr, digest_data, digest_algo);
+		if (result != TPM_SUCCESS) {
+			printk(BIOS_ERR,
+			       "TPM: Writing digest of %s into PCR failed with error %d\n",
+				event_name, result);
+			return VB2_ERROR_UNKNOWN;
 		}
 	}
 
 	return VB2_SUCCESS;
 }
+
+#if !CONFIG(VBOOT_RETURN_FROM_VERSTAGE)
+static void recover_tpm_log(int is_recovery)
+{
+	const void *preram_log = _tpm_log;
+	void *ram_log = tpm_log_cbmem_init();
+
+	if (tpm_log_get_size(preram_log) > MAX_PRERAM_TPM_LOG_ENTRIES) {
+		printk(BIOS_WARNING, "TPM LOG: pre-RAM log is too full, possible corruption\n");
+		return;
+	}
+
+	if (ram_log == NULL) {
+		printk(BIOS_WARNING, "TPM LOG: CBMEM not available, something went wrong\n");
+		return;
+	}
+
+	tpm_log_copy_entries(_tpm_log, ram_log);
+}
+CBMEM_CREATION_HOOK(recover_tpm_log);
+#endif
+
+BOOT_STATE_INIT_ENTRY(BS_PAYLOAD_BOOT, BS_ON_ENTRY, tpm_log_dump, NULL);
