@@ -54,7 +54,7 @@ vb2_error_t vb2ex_read_resource(struct vb2_context *ctx,
 	return VB2_SUCCESS;
 }
 
-static int handle_digest_result(void *slot_hash, size_t slot_hash_sz)
+static vb2_error_t handle_digest_result(void *slot_hash, size_t slot_hash_sz)
 {
 	int is_resume;
 
@@ -63,14 +63,14 @@ static int handle_digest_result(void *slot_hash, size_t slot_hash_sz)
 	 * vboot_retrieve_hash(), if Chrome EC is not enabled then return.
 	 */
 	if (!CONFIG(EC_GOOGLE_CHROMEEC))
-		return 0;
+		return VB2_SUCCESS;
 
 	/*
 	 * Nothing to do since resuming on the platform doesn't require
 	 * vboot verification again.
 	 */
 	if (!CONFIG(RESUME_PATH_SAME_AS_BOOT))
-		return 0;
+		return VB2_SUCCESS;
 
 	/*
 	 * Assume that if vboot doesn't start in bootblock verified
@@ -78,7 +78,7 @@ static int handle_digest_result(void *slot_hash, size_t slot_hash_sz)
 	 * lives in RO CBFS.
 	 */
 	if (!CONFIG(VBOOT_STARTS_IN_BOOTBLOCK))
-		return 0;
+		return VB2_SUCCESS;
 
 	is_resume = platform_is_resuming();
 
@@ -86,18 +86,18 @@ static int handle_digest_result(void *slot_hash, size_t slot_hash_sz)
 		uint8_t saved_hash[VBOOT_MAX_HASH_SIZE];
 		const size_t saved_hash_sz = sizeof(saved_hash);
 
-		assert(slot_hash_sz == saved_hash_sz);
+		assert(slot_hash_sz <= saved_hash_sz);
 
 		printk(BIOS_DEBUG, "Platform is resuming.\n");
 
 		if (vboot_retrieve_hash(saved_hash, saved_hash_sz)) {
 			printk(BIOS_ERR, "Couldn't retrieve saved hash.\n");
-			return -1;
+			return VB2_ERROR_UNKNOWN;
 		}
 
 		if (memcmp(saved_hash, slot_hash, slot_hash_sz)) {
 			printk(BIOS_ERR, "Hash mismatch on resume.\n");
-			return -1;
+			return VB2_ERROR_UNKNOWN;
 		}
 	} else if (is_resume < 0)
 		printk(BIOS_ERR, "Unable to determine if platform resuming.\n");
@@ -111,10 +111,10 @@ static int handle_digest_result(void *slot_hash, size_t slot_hash_sz)
 		 * lead to a reboot loop. The consequence of this is that
 		 * we will most likely fail resuming because of EC issues or
 		 * the hash digest not matching. */
-		return 0;
+		return VB2_SUCCESS;
 	}
 
-	return 0;
+	return VB2_SUCCESS;
 }
 
 static vb2_error_t hash_body(struct vb2_context *ctx,
@@ -140,7 +140,7 @@ static vb2_error_t hash_body(struct vb2_context *ctx,
 	 * (This split won't make sense with memory-mapped media like on x86.)
 	 */
 	load_ts = timestamp_get();
-	timestamp_add(TS_START_HASH_BODY, load_ts);
+	timestamp_add(TS_HASH_BODY_START, load_ts);
 
 	remaining = region_device_sz(fw_body);
 	offset = 0;
@@ -169,26 +169,23 @@ static vb2_error_t hash_body(struct vb2_context *ctx,
 		offset += block_size;
 	}
 
-	timestamp_add(TS_DONE_LOADING, load_ts);
-	timestamp_add_now(TS_DONE_HASHING);
+	timestamp_add(TS_LOADING_END, load_ts);
+	timestamp_add_now(TS_HASHING_END);
 
 	/* Check the result (with RSA signature verification) */
 	rv = vb2api_check_hash_get_digest(ctx, hash_digest, hash_digest_sz);
 	if (rv)
 		return rv;
 
-	timestamp_add_now(TS_END_HASH_BODY);
+	timestamp_add_now(TS_HASH_BODY_END);
 
-	if (handle_digest_result(hash_digest, hash_digest_sz))
-		return VB2_ERROR_UNKNOWN;
-
-	return VB2_SUCCESS;
+	return handle_digest_result(hash_digest, hash_digest_sz);
 }
 
 static uint32_t extend_pcrs(struct vb2_context *ctx)
 {
-	return vboot_extend_pcr(ctx, 0, BOOT_MODE_PCR) ||
-		   vboot_extend_pcr(ctx, 1, HWID_DIGEST_PCR);
+	return vboot_extend_pcr(ctx, CONFIG_PCR_BOOT_MODE, BOOT_MODE_PCR) ||
+		   vboot_extend_pcr(ctx, CONFIG_PCR_HWID, HWID_DIGEST_PCR);
 }
 
 #define EC_EFS_BOOT_MODE_VERIFIED_RW	0x00
@@ -215,19 +212,19 @@ static void check_boot_mode(struct vb2_context *ctx)
 	rv = tlcl_cr50_get_boot_mode(&boot_mode);
 	switch (rv) {
 	case TPM_E_NO_SUCH_COMMAND:
-		printk(BIOS_WARNING, "Cr50 does not support GET_BOOT_MODE.\n");
+		printk(BIOS_WARNING, "GSC does not support GET_BOOT_MODE.\n");
 		/* Proceed to legacy boot model. */
 		return;
 	case TPM_SUCCESS:
 		break;
 	default:
 		printk(BIOS_ERR,
-		       "Communication error in getting Cr50 boot mode.\n");
-		vb2api_fail(ctx, VB2_RECOVERY_CR50_BOOT_MODE, rv);
+		       "Communication error in getting GSC boot mode.\n");
+		vb2api_fail(ctx, VB2_RECOVERY_GSC_BOOT_MODE, rv);
 		return;
 	}
 
-	printk(BIOS_INFO, "Cr50 says boot_mode is %s(0x%02x).\n",
+	printk(BIOS_INFO, "GSC says boot_mode is %s(0x%02x).\n",
 	       get_boot_mode_string(boot_mode), boot_mode);
 
 	if (boot_mode == EC_EFS_BOOT_MODE_UNTRUSTED_RO)
@@ -236,19 +233,13 @@ static void check_boot_mode(struct vb2_context *ctx)
 		ctx->flags |= VB2_CONTEXT_EC_TRUSTED;
 }
 
-/**
- * Verify and select the firmware in the RW image
- *
- * TODO: Avoid loading a stage twice (once in hash_body & again in load_stage).
- * when per-stage verification is ready.
- */
+/* Verify and select the firmware in the RW image */
 void verstage_main(void)
 {
 	struct vb2_context *ctx;
-	struct region_device fw_body;
 	vb2_error_t rv;
 
-	timestamp_add_now(TS_START_VBOOT);
+	timestamp_add_now(TS_VBOOT_START);
 
 	/* Lockdown SPI flash controller if required */
 	if (CONFIG(BOOTMEDIA_LOCK_IN_VERSTAGE))
@@ -258,7 +249,7 @@ void verstage_main(void)
 	ctx = vboot_get_context();
 
 	/* Initialize and read nvdata from non-volatile storage. */
-	vbnv_init(ctx->nvdata);
+	vbnv_init();
 
 	/* Set S3 resume flag if vboot should behave differently when selecting
 	 * which slot to boot.  This is only relevant to vboot if the platform
@@ -271,12 +262,12 @@ void verstage_main(void)
 	/* Read secdata from TPM. Initialize TPM if secdata not found. We don't
 	 * check the return value here because vb2api_fw_phase1 will catch
 	 * invalid secdata and tell us what to do (=reboot). */
-	timestamp_add_now(TS_START_TPMINIT);
+	timestamp_add_now(TS_TPMINIT_START);
 	if (vboot_setup_tpm(ctx) == TPM_SUCCESS) {
 		antirollback_read_space_firmware(ctx);
 		antirollback_read_space_kernel(ctx);
 	}
-	timestamp_add_now(TS_END_TPMINIT);
+	timestamp_add_now(TS_TPMINIT_END);
 
 	if (get_recovery_mode_switch()) {
 		ctx->flags |= VB2_CONTEXT_FORCE_RECOVERY_MODE;
@@ -303,9 +294,12 @@ void verstage_main(void)
 	 * If there is an communication error, a recovery reason will be set and
 	 * vb2api_fw_phase1 will route us to recovery mode.
 	 */
-	if (CONFIG(TPM_CR50))
+	if (CONFIG(TPM_GOOGLE))
 		check_boot_mode(ctx);
 
+	if (get_ec_is_trusted())
+		ctx->flags |= VB2_CONTEXT_EC_TRUSTED;
+
 	/* Do early init (set up secdata and NVRAM, load GBB) */
 	printk(BIOS_INFO, "Phase 1\n");
 	rv = vb2api_fw_phase1(ctx);
@@ -323,81 +317,71 @@ void verstage_main(void)
 			extend_pcrs(ctx); /* ignore failures */
 			goto verstage_main_exit;
 		}
-
-		printk(BIOS_INFO, "Reboot requested (%x)\n", rv);
-		vboot_save_data(ctx);
-		vboot_reboot();
+		vboot_save_and_reboot(ctx, rv);
 	}
 
 	/* Determine which firmware slot to boot (based on NVRAM) */
 	printk(BIOS_INFO, "Phase 2\n");
 	rv = vb2api_fw_phase2(ctx);
-	if (rv) {
-		printk(BIOS_INFO, "Reboot requested (%x)\n", rv);
-		vboot_save_data(ctx);
-		vboot_reboot();
-	}
+	if (rv)
+		vboot_save_and_reboot(ctx, rv);
 
 	/* Try that slot (verify its keyblock and preamble) */
 	printk(BIOS_INFO, "Phase 3\n");
-	timestamp_add_now(TS_START_VERIFY_SLOT);
+	timestamp_add_now(TS_VERIFY_SLOT_START);
 	rv = vb2api_fw_phase3(ctx);
-	timestamp_add_now(TS_END_VERIFY_SLOT);
-	if (rv) {
-		printk(BIOS_INFO, "Reboot requested (%x)\n", rv);
-		vboot_save_data(ctx);
-		vboot_reboot();
-	}
+	timestamp_add_now(TS_VERIFY_SLOT_END);
+	if (rv)
+		vboot_save_and_reboot(ctx, rv);
 
 	printk(BIOS_INFO, "Phase 4\n");
-	rv = vboot_locate_firmware(ctx, &fw_body);
-	if (rv)
-		die_with_post_code(POST_INVALID_ROM,
-			"Failed to read FMAP to locate firmware");
+	if (CONFIG(VBOOT_CBFS_INTEGRATION)) {
+		struct vb2_hash *metadata_hash;
+		rv = vb2api_get_metadata_hash(ctx, &metadata_hash);
+		if (rv == VB2_SUCCESS)
+			rv = handle_digest_result(metadata_hash->raw,
+						  vb2_digest_size(metadata_hash->algo));
+	} else {
+		struct region_device fw_body;
+		rv = vboot_locate_firmware(ctx, &fw_body);
+		if (rv)
+			die_with_post_code(POSTCODE_INVALID_ROM,
+					   "Failed to read FMAP to locate firmware");
 
-	rv = hash_body(ctx, &fw_body);
-	vboot_save_data(ctx);
-	if (rv) {
-		printk(BIOS_INFO, "Reboot requested (%x)\n", rv);
-		vboot_reboot();
+		rv = hash_body(ctx, &fw_body);
 	}
 
+	if (rv)
+		vboot_save_and_reboot(ctx, rv);
+	vboot_save_data(ctx);
+
 	/* Only extend PCRs once on boot. */
 	if (!(ctx->flags & VB2_CONTEXT_S3_RESUME)) {
-		timestamp_add_now(TS_START_TPMPCR);
+		timestamp_add_now(TS_TPMPCR_START);
 		rv = extend_pcrs(ctx);
 		if (rv) {
-			printk(BIOS_WARNING,
-			       "Failed to extend TPM PCRs (%#x)\n", rv);
-			vb2api_fail(ctx, VB2_RECOVERY_RO_TPM_U_ERROR, rv);
-			vboot_save_data(ctx);
-			vboot_reboot();
+			printk(BIOS_WARNING, "Failed to extend TPM PCRs (%#x)\n", rv);
+			vboot_fail_and_reboot(ctx, VB2_RECOVERY_RO_TPM_U_ERROR, rv);
 		}
-		timestamp_add_now(TS_END_TPMPCR);
+		timestamp_add_now(TS_TPMPCR_END);
 	}
 
 	/* Lock TPM */
 
-	timestamp_add_now(TS_START_TPMLOCK);
+	timestamp_add_now(TS_TPMLOCK_START);
 	rv = antirollback_lock_space_firmware();
 	if (rv) {
 		printk(BIOS_INFO, "Failed to lock TPM (%x)\n", rv);
-		vb2api_fail(ctx, VB2_RECOVERY_RO_TPM_L_ERROR, 0);
-		vboot_save_data(ctx);
-		vboot_reboot();
+		vboot_fail_and_reboot(ctx, VB2_RECOVERY_RO_TPM_L_ERROR, 0);
 	}
-	timestamp_add_now(TS_END_TPMLOCK);
+	timestamp_add_now(TS_TPMLOCK_END);
 
 	/* Lock rec hash space if available. */
 	if (CONFIG(VBOOT_HAS_REC_HASH_SPACE)) {
 		rv = antirollback_lock_space_mrc_hash(MRC_REC_HASH_NV_INDEX);
 		if (rv) {
-			printk(BIOS_INFO, "Failed to lock rec hash space(%x)\n",
-			       rv);
-			vb2api_fail(ctx, VB2_RECOVERY_RO_TPM_REC_HASH_L_ERROR,
-				    0);
-			vboot_save_data(ctx);
-			vboot_reboot();
+			printk(BIOS_INFO, "Failed to lock rec hash space(%x)\n", rv);
+			vboot_fail_and_reboot(ctx, VB2_RECOVERY_RO_TPM_REC_HASH_L_ERROR, 0);
 		}
 	}
 
@@ -405,5 +389,5 @@ void verstage_main(void)
 	       vboot_is_firmware_slot_a(ctx) ? 'A' : 'B');
 
  verstage_main_exit:
-	timestamp_add_now(TS_END_VBOOT);
+	timestamp_add_now(TS_VBOOT_END);
 }
