@@ -5,7 +5,6 @@
 #include <amdblocks/smm.h>
 #include <bootstate.h>
 #include <console/console.h>
-#include <cpu/amd/msr.h>
 #include <cpu/x86/mp.h>
 #include <cpu/x86/msr.h>
 #include <types.h>
@@ -21,7 +20,7 @@ static void soc_finalize(void *unused)
 			acpi_enable_sci();
 	}
 
-	post_code(POST_OS_BOOT);
+	post_code(POSTCODE_OS_BOOT);
 }
 
 BOOT_STATE_INIT_ENTRY(BS_OS_RESUME, BS_ON_ENTRY, soc_finalize, NULL);
