@@ -7,6 +7,12 @@
 #include <commonlib/helpers.h>
 #include <amdblocks/psp.h>
 
+#define CORE_2_PSP_MSG_38_OFFSET	0x10998 /* 4 byte */
+#define   CORE_2_PSP_MSG_38_FUSE_SPL		BIT(12)
+#define   CORE_2_PSP_MSG_38_SPL_FUSE_ERROR	BIT(13)
+#define   CORE_2_PSP_MSG_38_SPL_ENTRY_ERROR	BIT(14)
+#define   CORE_2_PSP_MSG_38_SPL_ENTRY_MISSING	BIT(15)
+
 /* x86 to PSP commands */
 #define MBOX_BIOS_CMD_SMM_INFO			0x02
 #define MBOX_BIOS_CMD_SX_INFO			0x03
