@@ -1,16 +1,21 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <assert.h>
+#include <bootsplash.h>
 #include <console/console.h>
 #include <cpu/intel/microcode.h>
+#include <delay.h>
 #include <device/device.h>
 #include <device/pci.h>
 #include <device/pci_ids.h>
+#include <device/pci_ops.h>
+#include <drivers/intel/gma/i915_reg.h>
 #include <fsp/api.h>
 #include <fsp/fsp_debug_event.h>
+#include <fsp/fsp_gop_blt.h>
 #include <fsp/ppi/mp_service_ppi.h>
 #include <fsp/util.h>
-#include <option.h>
+#include <gpio.h>
 #include <intelblocks/irq.h>
 #include <intelblocks/lpss.h>
 #include <intelblocks/mp_init.h>
@@ -19,8 +24,8 @@
 #include <intelpch/lockdown.h>
 #include <intelblocks/systemagent.h>
 #include <intelblocks/tcss.h>
+#include <option.h>
 #include <soc/cpu.h>
-#include <soc/gpio.h>
 #include <soc/intel/common/vbt.h>
 #include <soc/pci_devs.h>
 #include <soc/pcie.h>
@@ -46,6 +51,7 @@
 #define ICC_MAX_ID_ADL_M_MA	12000
 #define ICC_MAX_ID_ADL_N_MA	27000
 #define ICC_MAX_ADL_S		33000
+#define ICC_MAX_RPL_S		36000
 
 /*
  * ME End of Post configuration
@@ -429,7 +435,7 @@ static const SI_PCH_DEVICE_INTERRUPT_CONFIG *pci_irq_to_fsp(size_t *out_count)
 
 	/* Count PCH devices */
 	while (entry) {
-		if (PCI_SLOT(entry->devfn) >= MIN_PCH_SLOT)
+		if (is_pch_slot(entry->devfn))
 			++pch_total;
 		entry = entry->next;
 	}
@@ -438,7 +444,7 @@ static const SI_PCH_DEVICE_INTERRUPT_CONFIG *pci_irq_to_fsp(size_t *out_count)
 	config = calloc(pch_total, sizeof(*config));
 	entry = get_cached_pci_irqs();
 	while (entry) {
-		if (PCI_SLOT(entry->devfn) < MIN_PCH_SLOT) {
+		if (!is_pch_slot(entry->devfn)) {
 			entry = entry->next;
 			continue;
 		}
@@ -471,7 +477,9 @@ static const SI_PCH_DEVICE_INTERRUPT_CONFIG *pci_irq_to_fsp(size_t *out_count)
  */
 static int get_l1_substate_control(enum L1_substates_control ctl)
 {
-	if ((ctl > L1_SS_L1_2) || (ctl == L1_SS_FSP_DEFAULT))
+	if (CONFIG(SOC_INTEL_COMPLIANCE_TEST_MODE))
+		ctl = L1_SS_DISABLED;
+	else if ((ctl > L1_SS_L1_2) || (ctl == L1_SS_FSP_DEFAULT))
 		ctl = L1_SS_L1_2;
 	return ctl - 1;
 }
@@ -517,6 +525,8 @@ static uint16_t get_vccin_aux_imon_iccmax(void)
 	case PCI_DID_INTEL_RPL_P_ID_1:
 	case PCI_DID_INTEL_RPL_P_ID_2:
 	case PCI_DID_INTEL_RPL_P_ID_3:
+	case PCI_DID_INTEL_RPL_P_ID_4:
+	case PCI_DID_INTEL_RPL_P_ID_5:
 		tdp = get_cpu_tdp();
 		if (tdp == TDP_45W)
 			return ICC_MAX_TDP_45W;
@@ -535,7 +545,21 @@ static uint16_t get_vccin_aux_imon_iccmax(void)
 	case PCI_DID_INTEL_ADL_S_ID_10:
 	case PCI_DID_INTEL_ADL_S_ID_11:
 	case PCI_DID_INTEL_ADL_S_ID_12:
+	case PCI_DID_INTEL_RPL_HX_ID_1:
+	case PCI_DID_INTEL_RPL_HX_ID_2:
+	case PCI_DID_INTEL_RPL_HX_ID_3:
+	case PCI_DID_INTEL_RPL_HX_ID_4:
+	case PCI_DID_INTEL_RPL_HX_ID_5:
+	case PCI_DID_INTEL_RPL_HX_ID_6:
+	case PCI_DID_INTEL_RPL_HX_ID_7:
+	case PCI_DID_INTEL_RPL_HX_ID_8:
 		return ICC_MAX_ADL_S;
+	case PCI_DID_INTEL_RPL_S_ID_1:
+	case PCI_DID_INTEL_RPL_S_ID_2:
+	case PCI_DID_INTEL_RPL_S_ID_3:
+	case PCI_DID_INTEL_RPL_S_ID_4:
+	case PCI_DID_INTEL_RPL_S_ID_5:
+		return ICC_MAX_RPL_S;
 	default:
 		printk(BIOS_ERR, "Unknown MCH ID: 0x%4x, skipping VccInAuxImonIccMax config\n",
 			mch_id);
@@ -593,9 +617,9 @@ static void fill_fsps_cpu_params(FSP_S_CONFIG *s_cfg,
 	 * This would avoid APs from getting hijacked by FSP while coreboot
 	 * decides to set SkipMpInit UPD.
 	 */
-	s_cfg->CpuMpPpi = (uintptr_t) mp_fill_ppi_services_data();
+	s_cfg->CpuMpPpi = (uintptr_t)mp_fill_ppi_services_data();
 
-	if (CONFIG(USE_FSP_MP_INIT))
+	if (CONFIG(USE_FSP_FEATURE_PROGRAM_ON_APS))
 		/*
 		 * Fill `2nd microcode loading FSP UPD` if FSP is running CPU feature
 		 * programming.
@@ -646,7 +670,7 @@ static void fill_fsps_tcss_params(FSP_S_CONFIG *s_cfg,
 
 	/* D3Hot and D3Cold for TCSS */
 	s_cfg->D3HotEnable = !config->tcss_d3_hot_disable;
-	s_cfg->D3ColdEnable = !CONFIG(SOC_INTEL_ALDERLAKE_S3) && !config->tcss_d3_cold_disable;
+	s_cfg->D3ColdEnable = CONFIG(D3COLD_SUPPORT);
 
 	s_cfg->UsbTcPortEn = 0;
 	for (int i = 0; i < MAX_TYPE_C_PORTS; i++) {
@@ -800,11 +824,9 @@ static void fill_fsps_cnvi_params(FSP_S_CONFIG *s_cfg,
 		const struct soc_intel_alderlake_config *config)
 {
 	/* CNVi */
-#if CONFIG(SOC_INTEL_ALDERLAKE_PCH_P) || CONFIG(SOC_INTEL_ALDERLAKE_PCH_S)
-#if !CONFIG(SOC_INTEL_RAPTORLAKE)
-	/* This option is only available in public FSP headers of ADL-P and ADL-S */
+#if CONFIG(FSP_USE_REPO)
+	/* This option is only available in public FSP headers on FSP repo */
 	s_cfg->CnviWifiCore = is_devfn_enabled(PCH_DEVFN_CNVI_WIFI);
-#endif
 #endif
 	s_cfg->CnviMode = is_devfn_enabled(PCH_DEVFN_CNVI_WIFI);
 	s_cfg->CnviBtCore = config->cnvi_bt_core;
@@ -901,6 +923,41 @@ static void fill_fsps_pcie_params(FSP_S_CONFIG *s_cfg,
 		s_cfg->PcieRpDetectTimeoutMs[i] = rp_cfg->pcie_rp_detect_timeout_ms;
 	}
 	s_cfg->PcieComplianceTestMode = CONFIG(SOC_INTEL_COMPLIANCE_TEST_MODE);
+
+#if CONFIG(FSP_TYPE_IOT) && !CONFIG(SOC_INTEL_RAPTORLAKE)
+	/*
+	 * Intel requires that all enabled PCH PCIe ports have a CLK_REQ signal connected.
+	 * The CLK_REQ is used to wake the silicon when link entered L1 link-state. L1
+	 * link-state is also entered on PCI-PM D3, even with ASPM L1 disabled.
+	 * When no CLK_REQ signal is used, for example when it's using a free running
+	 * clock the Root port silicon will never wake from L1 link state.
+	 * This will trigger a MCE.
+	 *
+	 * Starting with FSP MR4 the UPD 'PchPcieClockGating' allows to work around
+	 * this issue by disabling ClockGating. Disabling ClockGating should be avoided
+	 * as the silicon draws more power when it is idle.
+	 */
+	for (int i = 0; i < CONFIG_MAX_PCH_ROOT_PORTS; i++) {
+		bool clk_req_missing = false;
+		if (!(enable_mask & BIT(i)))
+			continue;
+		const struct pcie_rp_config *rp_cfg = &config->pch_pcie_rp[i];
+		if (CONFIG(SOC_INTEL_COMPLIANCE_TEST_MODE)) {
+			clk_req_missing = true;
+		} else if (!rp_cfg->flags && rp_cfg->clk_src == 0 && rp_cfg->clk_req == 0) {
+			clk_req_missing = true;
+		} else if (rp_cfg->flags & PCIE_RP_CLK_REQ_UNUSED) {
+			clk_req_missing = true;
+		}
+		if (clk_req_missing) {
+			printk(BIOS_INFO, "PCH PCIe port %d has no CLK_REQ\n", i + 1);
+			printk(BIOS_INFO, "Disabling PCH PCIE ClockGating+PowerGating.\n");
+			s_cfg->PchPcieClockGating = false;
+			s_cfg->PchPciePowerGating = false;
+			break;
+		}
+	}
+#endif
 }
 
 static void fill_fsps_cpu_pcie_params(FSP_S_CONFIG *s_cfg,
@@ -935,6 +992,7 @@ static void fill_fsps_cpu_pcie_params(FSP_S_CONFIG *s_cfg,
 static void fill_fsps_misc_power_params(FSP_S_CONFIG *s_cfg,
 		const struct soc_intel_alderlake_config *config)
 {
+	u32 cpu_id = cpu_get_cpuid();
 	/* Skip setting D0I3 bit for all HECI devices */
 	s_cfg->DisableD0I3SettingForHeci = 1;
 	/*
@@ -1003,7 +1061,19 @@ static void fill_fsps_misc_power_params(FSP_S_CONFIG *s_cfg,
 
 	s_cfg->VrPowerDeliveryDesign = config->vr_power_delivery_design;
 
-	s_cfg->PkgCStateDemotion = !config->disable_package_c_state_demotion;
+	/* FIXME: Disable package C state demotion on Raptorlake as a W/A for S0ix issues */
+	if ((cpu_id == CPUID_RAPTORLAKE_J0) || (cpu_id == CPUID_RAPTORLAKE_Q0))
+		s_cfg->PkgCStateDemotion = 0;
+	else
+		s_cfg->PkgCStateDemotion = !config->disable_package_c_state_demotion;
+
+	if (cpu_id == CPUID_RAPTORLAKE_J0 || cpu_id == CPUID_RAPTORLAKE_Q0)
+		s_cfg->C1e = 0;
+	else
+		s_cfg->C1e = 1;
+#if CONFIG(SOC_INTEL_RAPTORLAKE) && !CONFIG(FSP_USE_REPO)
+	s_cfg->EnableHwpScalabilityTracking = config->enable_hwp_scalability_tracking;
+#endif
 }
 
 static void fill_fsps_irq_params(FSP_S_CONFIG *s_cfg,
@@ -1068,6 +1138,15 @@ static void fill_fsps_fivr_params(FSP_S_CONFIG *s_cfg,
 
 	s_cfg->PchFivrExtVnnRailIccMaximum =
 			config->ext_fivr_settings.vnn_icc_max_ma;
+
+#if CONFIG(SOC_INTEL_ALDERLAKE_PCH_N)
+	/* Enable the FIVR VCCST ICCMax Control for ADL-N.
+	 * TODO:Right now the UPD is update in partial headers for only ADL-N and when its
+	 * updated for ADL-P then we will remove the config since this needs to be enabled for
+	 * all the Alderlake platforms.
+	 */
+	s_cfg->PchFivrVccstIccMaxControl = 1;
+#endif
 }
 
 static void fill_fsps_fivr_rfi_params(FSP_S_CONFIG *s_cfg,
@@ -1197,6 +1276,46 @@ static void soc_silicon_init_params(FSP_S_CONFIG *s_cfg,
 		fill_fsps_params[i](s_cfg, config);
 }
 
+/*
+ * The Alder Lake PEIM graphics driver executed as part of the FSP does not wait
+ * for the panel power cycle to complete before it initializes communication
+ * with the display. It can result in AUX channel communication time out and
+ * PEIM graphics driver failing to bring up graphics.
+ *
+ * If we have performed some graphics operations in romstage, it is possible
+ * that a panel power cycle is still in progress. To prevent any issue with the
+ * PEIM graphics driver it is preferable to ensure that panel power cycle is
+ * complete.
+ *
+ * BUG:b:264526798
+ */
+static void wait_for_panel_power_cycle_done(const struct soc_intel_alderlake_config *config)
+{
+	const struct i915_gpu_panel_config *panel_cfg;
+	uint32_t bar0;
+	void *mmio;
+
+	if (!CONFIG(RUN_FSP_GOP))
+		return;
+
+	bar0 = pci_s_read_config32(SA_DEV_IGD, PCI_BASE_ADDRESS_0);
+	mmio = (void *)(bar0 & ~PCI_BASE_ADDRESS_MEM_ATTR_MASK);
+	if (!mmio)
+		return;
+
+	panel_cfg = &config->panel_cfg;
+	for (size_t i = 0;; i++) {
+		uint32_t status = read32(mmio + PCH_PP_STATUS);
+		if (!(status & PANEL_POWER_CYCLE_ACTIVE))
+			break;
+		if (i == panel_cfg->cycle_delay_ms) {
+			printk(BIOS_ERR, "Panel power cycle is still active.\n");
+			break;
+		}
+		mdelay(1);
+	}
+}
+
 /* UPD parameters to be initialized before SiliconInit */
 void platform_fsp_silicon_init_params_cb(FSPS_UPD *supd)
 {
@@ -1206,6 +1325,8 @@ void platform_fsp_silicon_init_params_cb(FSPS_UPD *supd)
 	config = config_of_soc();
 	soc_silicon_init_params(s_cfg, config);
 	mainboard_silicon_init_params(s_cfg);
+
+	wait_for_panel_power_cycle_done(config);
 }
 
 /*
@@ -1247,3 +1368,14 @@ __weak void mainboard_silicon_init_params(FSP_S_CONFIG *s_cfg)
 {
 	printk(BIOS_DEBUG, "WEAK: %s/%s called\n", __FILE__, __func__);
 }
+
+/* Handle FSP logo params */
+void soc_load_logo(FSPS_UPD *supd)
+{
+	fsp_convert_bmp_to_gop_blt(&supd->FspsConfig.LogoPtr,
+			 &supd->FspsConfig.LogoSize,
+			 &supd->FspsConfig.BltBufferAddress,
+			 &supd->FspsConfig.BltBufferSize,
+			 &supd->FspsConfig.LogoPixelHeight,
+			 &supd->FspsConfig.LogoPixelWidth);
+}
