@@ -163,5 +163,11 @@ struct tcss_port_config {
 	.ocpin            = (pin), \
 }
 
+/*
+ * Set bit corresponding to USB port in wake enable bitmap. Bit 0 corresponds
+ * to Port 1, Bit n corresponds to Port (n+1). This bitmap is later used to
+ * decide what ports need to set PORTSCN/PORTSCXUSB3 register bits.
+ */
+#define USB_PORT_WAKE_ENABLE(x)		(1 << ((x) - 1))
 
 #endif
