@@ -18,12 +18,13 @@
 #include <intelblocks/xdci.h>
 #include <fsp/api.h>
 #include <fsp/util.h>
+#include <gpio.h>
 #include <intelblocks/cpulib.h>
-#include <intelblocks/gpio.h>
 #include <intelblocks/itss.h>
 #include <intelblocks/pmclib.h>
 #include <intelblocks/systemagent.h>
 #include <option.h>
+#include <soc/ahci.h>
 #include <soc/cpu.h>
 #include <soc/heci.h>
 #include <soc/intel/common/vbt.h>
@@ -206,10 +207,16 @@ static struct device_operations pci_domain_ops = {
 	.acpi_fill_ssdt = ssdt_set_above_4g_pci,
 };
 
-static struct device_operations cpu_bus_ops = {
+struct device_operations apl_cpu_bus_ops = {
+	.read_resources = noop_read_resources,
+	.set_resources = noop_set_resources,
+	.init = mp_cpu_bus_init,
+	.acpi_fill_ssdt = generate_cpu_entries,
+};
+
+struct device_operations glk_cpu_bus_ops = {
 	.read_resources = noop_read_resources,
 	.set_resources = noop_set_resources,
-	.init = apollolake_init_cpus,
 	.acpi_fill_ssdt = generate_cpu_entries,
 };
 
@@ -218,8 +225,6 @@ static void enable_dev(struct device *dev)
 	/* Set the operations if it is a special bus type */
 	if (dev->path.type == DEVICE_PATH_DOMAIN)
 		dev->ops = &pci_domain_ops;
-	else if (dev->path.type == DEVICE_PATH_CPU_CLUSTER)
-		dev->ops = &cpu_bus_ops;
 	else if (dev->path.type == DEVICE_PATH_GPIO)
 		block_gpio_enable(dev);
 }
@@ -597,27 +602,27 @@ static void glk_fsp_silicon_init_params_cb(
 	 * improve boot performance, configure PmicPmcIpcCtrl for PMC to program
 	 * PMIC PCH_PWROK delay.
 	 */
-	silconfig->PmicPmcIpcCtrl = cfg->PmicPmcIpcCtrl;
+	silconfig->PmicPmcIpcCtrl = cfg->pmic_pmc_ipc_ctrl;
 
 	/*
 	 * Options to disable XHCI Link Compliance Mode.
 	 */
-	silconfig->DisableComplianceMode = cfg->DisableComplianceMode;
+	silconfig->DisableComplianceMode = cfg->disable_compliance_mode;
 
 	/*
 	 * Options to change USB3 ModPhy setting for Integrated Filter value.
 	 */
-	silconfig->ModPhyIfValue = cfg->ModPhyIfValue;
+	silconfig->ModPhyIfValue = cfg->mod_phy_if_value;
 
 	/*
 	 * Options to bump USB3 LDO voltage with 40mv.
 	 */
-	silconfig->ModPhyVoltageBump = cfg->ModPhyVoltageBump;
+	silconfig->ModPhyVoltageBump = cfg->mod_phy_voltage_bump;
 
 	/*
 	 * Options to adjust PMIC Vdd2 voltage.
 	 */
-	silconfig->PmicVdd2Voltage = cfg->PmicVdd2Voltage;
+	silconfig->PmicVdd2Voltage = cfg->pmic_vdd2_voltage;
 
 	/* FSP should let coreboot set subsystem IDs, which are read/write-once */
 	silconfig->SiSVID = 0;
@@ -684,7 +689,7 @@ void platform_fsp_silicon_init_params_cb(FSPS_UPD *silupd)
 	if (cfg->emmc_host_max_speed != 0)
 		silconfig->eMMCHostMaxSpeed = cfg->emmc_host_max_speed;
 
-	memcpy(silconfig->SataPortsHotPlug, cfg->SataPortsHotPlug,
+	memcpy(silconfig->SataPortsHotPlug, cfg->sata_ports_hot_plug,
 		sizeof(silconfig->SataPortsHotPlug));
 
 	silconfig->LPSS_S0ixEnable = cfg->lpss_s0ix_enable;
@@ -731,13 +736,16 @@ void platform_fsp_silicon_init_params_cb(FSPS_UPD *silupd)
 
 	/* SATA config */
 	if (is_devfn_enabled(PCH_DEVFN_SATA)) {
-		silconfig->SataSalpSupport = !(cfg->DisableSataSalpSupport);
-		memcpy(silconfig->SataPortsEnable, cfg->SataPortsEnable,
+		silconfig->SataSalpSupport = !(cfg->disable_sata_salp_support);
+		ahci_set_speed(cfg->sata_speed);
+		memcpy(silconfig->SataPortsEnable, cfg->sata_ports_enable,
 			sizeof(silconfig->SataPortsEnable));
+		memcpy(silconfig->SataPortsSolidStateDrive, cfg->sata_ports_ssd,
+			sizeof(silconfig->SataPortsSolidStateDrive));
 	}
 
 	/* Sata Power Optimisation */
-	silconfig->SataPwrOptEnable = !(cfg->SataPwrOptimizeDisable);
+	silconfig->SataPwrOptEnable = !(cfg->sata_pwr_optimize_disable);
 
 	/* 8254 Timer */
 	bool use_8254 = get_uint_option("legacy_8254_timer", CONFIG(USE_LEGACY_8254_TIMER));
