@@ -1,13 +1,13 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
-#include <assert.h>
 #include <acpi/acpigen.h>
+#include <assert.h>
 #include <console/console.h>
+#include <cpu/cpu.h>
 #include <cpu/intel/common/common.h>
 #include <cpu/intel/turbo.h>
 #include <cpu/x86/msr.h>
 #include <cpu/x86/mtrr.h>
-#include <arch/cpu.h>
 #include <intelblocks/cpulib.h>
 #include <intelblocks/fast_spi.h>
 #include <intelblocks/msr.h>
@@ -15,23 +15,6 @@
 #include <soc/soc_chip.h>
 #include <types.h>
 
-#define CPUID_EXTENDED_CPU_TOPOLOGY 0x0b
-#define LEVEL_TYPE_CORE 2
-#define LEVEL_TYPE_SMT 1
-
-#define CPUID_CPU_TOPOLOGY(x, val) \
-	(((val) >> CPUID_CPU_TOPOLOGY_##x##_SHIFT) & CPUID_CPU_TOPOLOGY_##x##_MASK)
-
-#define CPUID_CPU_TOPOLOGY_LEVEL_TYPE_SHIFT 0x8
-#define CPUID_CPU_TOPOLOGY_LEVEL_TYPE_MASK 0xff
-#define CPUID_CPU_TOPOLOGY_LEVEL(res) CPUID_CPU_TOPOLOGY(LEVEL_TYPE, (res).ecx)
-
-#define CPUID_CPU_TOPOLOGY_LEVEL_BITS_SHIFT 0x0
-#define CPUID_CPU_TOPOLOGY_LEVEL_BITS_MASK 0x1f
-#define CPUID_CPU_TOPOLOGY_THREAD_BITS(res) CPUID_CPU_TOPOLOGY(LEVEL_BITS, (res).eax)
-#define CPUID_CPU_TOPOLOGY_CORE_BITS(res, threadbits) \
-	((CPUID_CPU_TOPOLOGY(LEVEL_BITS, (res).eax)) - threadbits)
-
 #define CPUID_PROCESSOR_FREQUENCY		0X16
 #define CPUID_HYBRID_INFORMATION		0x1a
 
@@ -395,13 +378,59 @@ void cpu_lt_lock_memory(void)
 	msr_set(MSR_LT_CONTROL, LT_CONTROL_LOCK);
 }
 
+bool is_sgx_supported(void)
+{
+	struct cpuid_result cpuid_regs;
+	msr_t msr;
+
+	/* EBX[2] is feature capability */
+	cpuid_regs = cpuid_ext(CPUID_STRUCT_EXTENDED_FEATURE_FLAGS, 0x0);
+	msr = rdmsr(MTRR_CAP_MSR); /* Bit 12 is PRMRR enablement */
+	return ((cpuid_regs.ebx & SGX_SUPPORTED) && (msr.lo & MTRR_CAP_PRMRR));
+}
+
+static bool is_sgx_configured_and_supported(void)
+{
+	return CONFIG(SOC_INTEL_COMMON_BLOCK_SGX_ENABLE) && is_sgx_supported();
+}
+
+bool is_keylocker_supported(void)
+{
+	struct cpuid_result cpuid_regs;
+	msr_t msr;
+
+	/* ECX[23] is feature capability */
+	cpuid_regs = cpuid_ext(CPUID_STRUCT_EXTENDED_FEATURE_FLAGS, 0x0);
+	msr = rdmsr(MTRR_CAP_MSR); /* Bit 12 is PRMRR enablement */
+	return ((cpuid_regs.ecx & KEYLOCKER_SUPPORTED) && (msr.lo & MTRR_CAP_PRMRR));
+}
+
+static bool is_keylocker_configured_and_supported(void)
+{
+	return CONFIG(INTEL_KEYLOCKER) && is_keylocker_supported();
+}
+
+static bool check_prm_features_enabled(void)
+{
+	/*
+	 * Key Locker and SGX are the features that need PRM.
+	 * If either of them are enabled return true, otherwise false
+	 * */
+	return is_sgx_configured_and_supported() ||
+		is_keylocker_configured_and_supported();
+}
+
 int get_valid_prmrr_size(void)
 {
 	msr_t msr;
 	int i;
 	int valid_size;
 
-	if (!CONFIG(SOC_INTEL_COMMON_BLOCK_SGX_ENABLE))
+	/* If none of the features that need PRM are enabled then return 0 */
+	if (!check_prm_features_enabled())
+		return 0;
+
+	if (!CONFIG_SOC_INTEL_COMMON_BLOCK_PRMRR_SIZE)
 		return 0;
 
 	msr = rdmsr(MSR_PRMRR_VALID_CONFIG);
@@ -416,7 +445,7 @@ int get_valid_prmrr_size(void)
 	for (i = 8; i >= 0; i--) {
 		valid_size = msr.lo & (1 << i);
 
-		if (valid_size && valid_size <= CONFIG_SOC_INTEL_COMMON_BLOCK_SGX_PRMRR_SIZE)
+		if (valid_size && valid_size <= CONFIG_SOC_INTEL_COMMON_BLOCK_PRMRR_SIZE)
 			break;
 		else if (i == 0)
 			valid_size = 0;
@@ -424,7 +453,7 @@ int get_valid_prmrr_size(void)
 
 	if (!valid_size) {
 		printk(BIOS_WARNING, "Unsupported PRMRR size of %i MiB, check your config!\n",
-			CONFIG_SOC_INTEL_COMMON_BLOCK_SGX_PRMRR_SIZE);
+			CONFIG_SOC_INTEL_COMMON_BLOCK_PRMRR_SIZE);
 		return 0;
 	}
 
@@ -435,52 +464,6 @@ int get_valid_prmrr_size(void)
 	return valid_size;
 }
 
-/* Get number of bits for core ID and SMT ID */
-static void get_cpu_core_thread_bits(uint32_t *core_bits, uint32_t *thread_bits)
-{
-	struct cpuid_result cpuid_regs;
-	int level_num, cpu_id_op = 0;
-	const uint32_t cpuid_max_func = cpuid_get_max_func();
-
-	/* Assert if extended CPU topology not supported */
-	assert(cpuid_max_func >= CPUID_EXTENDED_CPU_TOPOLOGY);
-
-	cpu_id_op = CPUID_EXTENDED_CPU_TOPOLOGY;
-
-	*core_bits = level_num = 0;
-	cpuid_regs = cpuid_ext(cpu_id_op, level_num);
-
-	/* Sub-leaf index 0 enumerates SMT level, if not assert */
-	assert(CPUID_CPU_TOPOLOGY_LEVEL(cpuid_regs) == LEVEL_TYPE_SMT);
-
-	*thread_bits = CPUID_CPU_TOPOLOGY_THREAD_BITS(cpuid_regs);
-	do {
-		level_num++;
-		cpuid_regs = cpuid_ext(cpu_id_op, level_num);
-		if (CPUID_CPU_TOPOLOGY_LEVEL(cpuid_regs) == LEVEL_TYPE_CORE) {
-			*core_bits = CPUID_CPU_TOPOLOGY_CORE_BITS(cpuid_regs, *thread_bits);
-			break;
-		}
-	/* Stop when level type is invalid i.e 0 */
-	} while (CPUID_CPU_TOPOLOGY_LEVEL(cpuid_regs));
-}
-
-void get_cpu_topology_from_apicid(uint32_t apicid, uint8_t *package,
-		uint8_t *core, uint8_t *thread)
-{
-
-	uint32_t core_bits, thread_bits;
-
-	get_cpu_core_thread_bits(&core_bits, &thread_bits);
-
-	if (package)
-		*package = apicid >> (thread_bits + core_bits);
-	if (core)
-		*core = (apicid  >> thread_bits) & ((1 << core_bits) - 1);
-	if (thread)
-		*thread = apicid  & ((1 << thread_bits) - 1);
-}
-
 static void sync_core_prmrr(void)
 {
 	static msr_t msr_base, msr_mask;
@@ -506,8 +489,8 @@ bool is_tme_supported(void)
 {
 	struct cpuid_result cpuid_regs;
 
-	cpuid_regs = cpuid_ext(0x7, 0x0); /* ECX[13] is feature capability */
-
+	/* ECX[13] is feature capability */
+	cpuid_regs = cpuid_ext(CPUID_STRUCT_EXTENDED_FEATURE_FLAGS, 0x0);
 	return (cpuid_regs.ecx & TME_SUPPORTED);
 }
 
@@ -517,3 +500,18 @@ void set_tme_core_activate(void)
 
 	wrmsr(MSR_CORE_MKTME_ACTIVATION, msr);
 }
+
+/* Provide the max turbo frequency of the CPU */
+unsigned int smbios_cpu_get_max_speed_mhz(void)
+{
+	return cpu_get_max_turbo_ratio() * CONFIG_CPU_BCLK_MHZ;
+}
+
+void disable_three_strike_error(void)
+{
+	msr_t msr;
+
+	msr = rdmsr(MSR_PREFETCH_CTL);
+	msr.lo = msr.lo | DISABLE_CPU_ERROR;
+	wrmsr(MSR_PREFETCH_CTL, msr);
+}
