@@ -1,9 +1,11 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
-#include <console/console.h>
 #include <cbmem.h>
+#include <console/console.h>
 #include <delay.h>
+#include <device/mmio.h>
 #include <intelblocks/crashlog.h>
+#include <intelblocks/pmc_ipc.h>
 #include <string.h>
 
 int __weak cl_get_cpu_record_size(void)
@@ -21,6 +23,11 @@ u32 __weak cl_get_cpu_bar_addr(void)
 	return 0;
 }
 
+int __weak cl_get_ioe_record_size(void)
+{
+	return 0;
+}
+
 u32 __weak cl_get_cpu_tmp_bar(void)
 {
 	return 0;
@@ -41,6 +48,11 @@ bool __weak cl_pmc_sram_has_mmio_access(void)
 	return false;
 }
 
+bool __weak cl_ioe_sram_has_mmio_access(void)
+{
+	return false;
+}
+
 bool __weak cpu_crashlog_support(void)
 {
 	return false;
@@ -61,12 +73,19 @@ bool __weak cl_pmc_data_present(void)
 	return false;
 }
 
+bool __weak cl_ioe_data_present(void)
+{
+	return false;
+}
+
 __weak void reset_discovery_buffers(void) {}
 
 __weak void update_new_pmc_crashlog_size(u32 *pmc_crash_size) {}
 
 __weak void update_new_cpu_crashlog_size(u32 *cpu_crash_size) {}
 
+__weak void update_new_ioe_crashlog_size(u32 *ioe_crash_size) {}
+
 pmc_ipc_discovery_buf_t __weak cl_get_pmc_discovery_buf(void)
 {
 	pmc_ipc_discovery_buf_t discov_buf;
@@ -143,16 +162,18 @@ int pmc_cl_gen_descriptor_table(u32 desc_table_addr,
 	printk(BIOS_DEBUG, "CL PMC desc table: numb of regions is 0x%x at addr 0x%x\n",
 	       descriptor_table->numb_regions, desc_table_addr);
 	for (int i = 0; i < descriptor_table->numb_regions; i++) {
+		if (i >= ARRAY_SIZE(descriptor_table->regions)) {
+			printk(BIOS_ERR, "Maximum number of PMC crashLog descriptor table exceeded (%u/%zu)\n",
+			descriptor_table->numb_regions,
+			ARRAY_SIZE(descriptor_table->regions));
+			break;
+		}
 		desc_table_addr += 4;
 		descriptor_table->regions[i].data = read32((u32 *)(desc_table_addr));
 		total_data_size += descriptor_table->regions[i].bits.size * sizeof(u32);
 		printk(BIOS_DEBUG, "CL PMC desc table: region 0x%x has size 0x%x at offset 0x%x\n",
 			i, descriptor_table->regions[i].bits.size,
 			descriptor_table->regions[i].bits.offset);
-		if (i > 255) {
-			printk(BIOS_ERR, "More than 255 regions in PMC crashLog descriptor table");
-			break;
-		}
 	}
 	return total_data_size;
 }
@@ -267,6 +288,14 @@ bool cl_copy_data_from_sram(u32 src_bar,
 	u32 src_addr = src_bar + offset;
 
 	u32 data =  read32((u32 *)src_addr);
+
+	/* First 32bits of the record must not be 0xdeadbeef */
+	if (data == INVALID_CRASHLOG_RECORD) {
+		printk(BIOS_DEBUG, "Invalid data 0x%x at offset 0x%x from addr 0x%x\n",
+				data, offset, src_bar);
+		return false;
+	}
+
 	/* PMC: copy if 1st DWORD in buffer is not zero and its 31st bit is not set */
 	if (pmc_sram && !(data && !(data & BIT(31)))) {
 		printk(BIOS_DEBUG, "Invalid data 0x%x at offset 0x%x from addr 0x%x"
@@ -291,13 +320,13 @@ bool cl_copy_data_from_sram(u32 src_bar,
 	return true;
 }
 
-void cl_get_pmc_sram_data(void)
+void __weak cl_get_pmc_sram_data(void)
 {
 	u32 *dest = NULL;
 	u32 tmp_bar_addr = cl_get_cpu_tmp_bar();
 	u32 pmc_crashLog_size = cl_get_pmc_record_size();
 
-	if (!cl_pmc_sram_has_mmio_access())
+	if (!cl_pmc_sram_has_mmio_access() || !tmp_bar_addr)
 		return;
 
 	pmc_ipc_discovery_buf_t discovery_buf = cl_get_pmc_discovery_buf();
@@ -329,7 +358,7 @@ void cl_get_pmc_sram_data(void)
 	/* allocate mem for the record to be copied */
 	unsigned long pmc_cl_cbmem_addr;
 
-	pmc_cl_cbmem_addr = (unsigned long) cbmem_add(CBMEM_ID_PMC_CRASHLOG,
+	pmc_cl_cbmem_addr = (unsigned long)cbmem_add(CBMEM_ID_PMC_CRASHLOG,
 						pmc_crashLog_size);
 	if (!pmc_cl_cbmem_addr) {
 		printk(BIOS_ERR, "Unable to allocate CBMEM PMC crashLog entry.\n");
@@ -337,7 +366,7 @@ void cl_get_pmc_sram_data(void)
 	}
 
 	memset((void *)pmc_cl_cbmem_addr, 0, pmc_crashLog_size);
-	dest = (u32 *)(uintptr_t) pmc_cl_cbmem_addr;
+	dest = (u32 *)(uintptr_t)pmc_cl_cbmem_addr;
 	bool pmc_sram = true;
 	pmc_crashlog_desc_table_t descriptor_table =  cl_get_pmc_descriptor_table();
 	if (discovery_buf.bits.discov_mechanism == 1) {
@@ -400,16 +429,16 @@ void cl_get_cpu_sram_data(void)
 
 	/* allocate memory buffers for CPU crashog data to be copied */
 	unsigned long cpu_crashlog_cbmem_addr;
-	cpu_crashlog_cbmem_addr = (unsigned long) cbmem_add(CBMEM_ID_CPU_CRASHLOG,
+	cpu_crashlog_cbmem_addr = (unsigned long)cbmem_add(CBMEM_ID_CPU_CRASHLOG,
 								m_cpu_crashLog_size);
 	if (!cpu_crashlog_cbmem_addr) {
 		printk(BIOS_ERR, "Failed to add CPU main crashLog entries to CBMEM.\n");
 		return;
 	}
 
-	memset((void *) cpu_crashlog_cbmem_addr, 0, m_cpu_crashLog_size);
+	memset((void *)cpu_crashlog_cbmem_addr, 0, m_cpu_crashLog_size);
 	tmp_bar_addr = cl_get_cpu_bar_addr();
-	dest = (u32 *)(uintptr_t) cpu_crashlog_cbmem_addr;
+	dest = (u32 *)(uintptr_t)cpu_crashlog_cbmem_addr;
 	bool pmc_sram = false;
 
 	for (int i = 0 ; i < cpu_cl_disc_tab.header.fields.count ; i++) {
@@ -476,6 +505,10 @@ bool cl_fill_cpu_records(void *cl_record)
 
 	printk(BIOS_DEBUG, "CPU crash data collection.\n");
 	cl_src_addr = cbmem_find(CBMEM_ID_CPU_CRASHLOG);
+	if (!cl_src_addr) {
+		printk(BIOS_DEBUG, "CPU crash data, CBMEM not found\n");
+		return false;
+	}
 	memcpy(cl_record, cl_src_addr, m_cpu_crashLog_size);
 
 	return true;
@@ -494,7 +527,33 @@ bool cl_fill_pmc_records(void *cl_record)
 
 	printk(BIOS_DEBUG, "PMC crash data collection.\n");
 	cl_src_addr = cbmem_find(CBMEM_ID_PMC_CRASHLOG);
+	if (!cl_src_addr) {
+		printk(BIOS_DEBUG, "PMC crash data, CBMEM not found\n");
+		return false;
+	}
 	memcpy(cl_record, cl_src_addr, m_pmc_crashLog_size);
 
 	return true;
 }
+
+bool cl_fill_ioe_records(void *cl_record)
+{
+	void *cl_src_addr = NULL;
+
+	u32 m_ioe_crashLog_size = cl_get_ioe_record_size();
+
+	if (!cl_ioe_data_present() || m_ioe_crashLog_size == 0) {
+		printk(BIOS_DEBUG, "IOE crashLog not present, skipping.\n");
+		return false;
+	}
+
+	printk(BIOS_DEBUG, "IOE PMC crash data collection.\n");
+	cl_src_addr = cbmem_find(CBMEM_ID_IOE_CRASHLOG);
+	if (!cl_src_addr) {
+		printk(BIOS_DEBUG, "IOE crash data, CBMEM not found\n");
+		return false;
+	}
+	memcpy(cl_record, cl_src_addr, m_ioe_crashLog_size);
+
+	return true;
+}
