@@ -1,17 +1,19 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <acpi/acpi.h>
-#include <arch/cpu.h>
 #include <bootstate.h>
-#include <console/console.h>
 #include <cbfs.h>
 #include <commonlib/region.h>
+#include <console/console.h>
+#include <cpu/cpu.h>
+#include <crc_byte.h>
 #include <fmap.h>
+#include <intelbasecode/debug_feature.h>
 #include <intelblocks/cse.h>
 #include <intelblocks/cse_layout.h>
-#include <intelbasecode/debug_feature.h>
-#include <security/vboot/vboot_common.h>
+#include <intelblocks/spi.h>
 #include <security/vboot/misc.h>
+#include <security/vboot/vboot_common.h>
 #include <soc/intel/common/reset.h>
 #include <timestamp.h>
 
@@ -133,7 +135,31 @@ struct get_bp_info_rsp {
 
 static const char * const cse_regions[] = {"RO", "RW"};
 
-bool cse_get_boot_performance_data(struct cse_boot_perf_rsp *boot_perf_rsp)
+static struct cse_specific_info cse_info;
+
+void cse_log_ro_write_protection_info(bool mfg_mode)
+{
+	bool cse_ro_wp_en = is_spi_wp_cse_ro_en();
+
+	printk(BIOS_DEBUG, "ME: WP for RO is enabled        : %s\n",
+			cse_ro_wp_en ? "YES" : "NO");
+
+	if (cse_ro_wp_en) {
+		uint32_t base, limit;
+		spi_get_wp_cse_ro_range(&base, &limit);
+		printk(BIOS_DEBUG, "ME: RO write protection scope - Start=0x%X, End=0x%X\n",
+				base, limit);
+	}
+
+	/*
+	 * If manufacturing mode is disabled, but CSE RO is not write protected,
+	 * log error.
+	 */
+	if (!mfg_mode && !cse_ro_wp_en)
+		printk(BIOS_ERR, "ME: Write protection for CSE RO is not enabled\n");
+}
+
+enum cb_err cse_get_boot_performance_data(struct cse_boot_perf_rsp *boot_perf_rsp)
 {
 	struct cse_boot_perf_req {
 		struct mkhi_hdr hdr;
@@ -151,16 +177,16 @@ bool cse_get_boot_performance_data(struct cse_boot_perf_rsp *boot_perf_rsp)
 	if (heci_send_receive(&req, sizeof(req), boot_perf_rsp, &resp_size,
 									HECI_MKHI_ADDR)) {
 		printk(BIOS_ERR, "cse_lite: Could not get boot performance data\n");
-		return false;
+		return CB_ERR;
 	}
 
 	if (boot_perf_rsp->hdr.result) {
 		printk(BIOS_ERR, "cse_lite: Get boot performance data resp failed: %d\n",
 				boot_perf_rsp->hdr.result);
-		return false;
+		return CB_ERR;
 	}
 
-	return true;
+	return CB_SUCCESS;
 }
 
 
@@ -175,6 +201,88 @@ static const struct cse_bp_entry *cse_get_bp_entry(enum boot_partition_id bp,
 	return &cse_bp_info->bp_entries[bp];
 }
 
+static bool is_cbmem_cse_info_valid(const struct cse_specific_info *info)
+{
+	uint32_t crc = ~CRC(info, offsetof(struct cse_specific_info, crc), crc32_byte);
+
+	/*
+	 * Authenticate the CBMEM persistent data.
+	 *
+	 * The underlying assumption is that an event (i.e., CSE upgrade/downgrade) which
+	 * could change the values stored in this region has to also trigger the global
+	 * reset. Hence, CBMEM persistent data won't be available any time after such
+	 * event (global reset or cold reset) being initiated.
+	 *
+	 * During warm boot scenarios CBMEM contents remain persistent hence, we don't
+	 * want to override the existing data in CBMEM to avoid any additional boot latency.
+	 */
+	if (info->crc != crc)
+		return false;
+
+	return true;
+}
+
+static void cbmem_store_cse_info_crc(struct cse_specific_info *info)
+{
+	info->crc = ~CRC(info, offsetof(struct cse_specific_info, crc), crc32_byte);
+}
+
+/*
+ * Helper function that stores current CSE firmware version to CBMEM memory,
+ * except during recovery mode.
+ */
+static void cse_store_rw_fw_version(const struct cse_bp_entry *cse_bp)
+{
+	if (vboot_recovery_mode_enabled())
+		return;
+
+	if (CONFIG(SOC_INTEL_CSE_LITE_SYNC_IN_ROMSTAGE)) {
+		memcpy(&(cse_info.cse_fwp_version.cur_cse_fw_version), &(cse_bp->fw_ver),
+			 sizeof(struct fw_version));
+		return;
+	}
+
+	struct cse_specific_info *info = cbmem_add(CBMEM_ID_CSE_INFO, sizeof(*info));
+	if (!info)
+		return;
+
+	/* Avoid CBMEM update if CBMEM already has persistent data */
+	if (is_cbmem_cse_info_valid(info))
+		return;
+
+	memset(info, 0, sizeof(*info));
+
+	memcpy(&(info->cse_fwp_version.cur_cse_fw_version), &(cse_bp->fw_ver),
+		 sizeof(struct fw_version));
+
+	/* Update the CRC */
+	cbmem_store_cse_info_crc(info);
+}
+
+#if CONFIG(SOC_INTEL_CSE_LITE_SYNC_IN_ROMSTAGE)
+/* Function to copy PRERAM CSE specific info to pertinent CBMEM. */
+static void preram_cse_info_sync_to_cbmem(int is_recovery)
+{
+	if (vboot_recovery_mode_enabled() || !CONFIG(SOC_INTEL_STORE_CSE_FW_VERSION))
+		return;
+
+	struct cse_specific_info *info = cbmem_add(CBMEM_ID_CSE_INFO, sizeof(*info));
+	if (!info)
+		return;
+
+	/* Avoid sync into CBMEM if CBMEM already has persistent data */
+	if (is_cbmem_cse_info_valid(info))
+		return;
+
+	memcpy(info, &cse_info, sizeof(struct cse_specific_info));
+
+	/* Update the CRC */
+	cbmem_store_cse_info_crc(info);
+}
+
+CBMEM_CREATION_HOOK(preram_cse_info_sync_to_cbmem);
+#endif
+
 static void cse_print_boot_partition_info(const struct cse_bp_info *cse_bp_info)
 {
 	const struct cse_bp_entry *cse_bp;
@@ -200,6 +308,10 @@ static void cse_print_boot_partition_info(const struct cse_bp_info *cse_bp_info)
 			cse_bp->fw_ver.hotfix, cse_bp->fw_ver.build,
 			cse_bp->status, cse_bp->start_offset,
 			cse_bp->end_offset);
+
+	/* Store the CSE RW Firmware Version into CBMEM */
+	if (CONFIG(SOC_INTEL_STORE_CSE_FW_VERSION))
+		cse_store_rw_fw_version(cse_bp);
 }
 
 /*
@@ -226,7 +338,7 @@ static bool cse_is_bp_cmd_info_possible(void)
 	return false;
 }
 
-static bool cse_get_bp_info(struct get_bp_info_rsp *bp_info_rsp)
+static enum cb_err cse_get_bp_info(struct get_bp_info_rsp *bp_info_rsp)
 {
 	struct get_bp_info_req {
 		struct mkhi_hdr hdr;
@@ -241,7 +353,7 @@ static bool cse_get_bp_info(struct get_bp_info_rsp *bp_info_rsp)
 
 	if (!cse_is_bp_cmd_info_possible()) {
 		printk(BIOS_ERR, "cse_lite: CSE does not meet prerequisites\n");
-		return false;
+		return CB_ERR;
 	}
 
 	size_t resp_size = sizeof(struct get_bp_info_rsp);
@@ -249,18 +361,18 @@ static bool cse_get_bp_info(struct get_bp_info_rsp *bp_info_rsp)
 	if (heci_send_receive(&info_req, sizeof(info_req), bp_info_rsp, &resp_size,
 									HECI_MKHI_ADDR)) {
 		printk(BIOS_ERR, "cse_lite: Could not get partition info\n");
-		return false;
+		return CB_ERR;
 	}
 
 	if (bp_info_rsp->hdr.result) {
 		printk(BIOS_ERR, "cse_lite: Get partition info resp failed: %d\n",
 				bp_info_rsp->hdr.result);
-		return false;
+		return CB_ERR;
 	}
 
 	cse_print_boot_partition_info(&bp_info_rsp->bp_info);
 
-	return true;
+	return CB_SUCCESS;
 }
 /*
  * It sends HECI command to notify CSE about its next boot partition. When coreboot wants
@@ -269,7 +381,7 @@ static bool cse_get_bp_info(struct get_bp_info_rsp *bp_info_rsp)
  * This function must be used before EOP.
  * Returns false on failure and true on success.
  */
-static bool cse_set_next_boot_partition(enum boot_partition_id bp)
+static enum cb_err cse_set_next_boot_partition(enum boot_partition_id bp)
 {
 	struct set_boot_partition_info_req {
 		struct mkhi_hdr hdr;
@@ -286,14 +398,14 @@ static bool cse_set_next_boot_partition(enum boot_partition_id bp)
 
 	if (bp != RO && bp != RW) {
 		printk(BIOS_ERR, "cse_lite: Incorrect partition id(%d) is provided", bp);
-		return false;
+		return CB_ERR_ARG;
 	}
 
 	printk(BIOS_INFO, "cse_lite: Set Boot Partition Info Command (%s)\n", GET_BP_STR(bp));
 
 	if (!cse_is_bp_cmd_info_possible()) {
 		printk(BIOS_ERR, "cse_lite: CSE does not meet prerequisites\n");
-		return false;
+		return CB_ERR;
 	}
 
 	struct mkhi_hdr switch_resp;
@@ -301,18 +413,18 @@ static bool cse_set_next_boot_partition(enum boot_partition_id bp)
 
 	if (heci_send_receive(&switch_req, sizeof(switch_req), &switch_resp, &sw_resp_sz,
 									HECI_MKHI_ADDR))
-		return false;
+		return CB_ERR;
 
 	if (switch_resp.result) {
 		printk(BIOS_ERR, "cse_lite: Set Boot Partition Info Response Failed: %d\n",
 				switch_resp.result);
-		return false;
+		return CB_ERR;
 	}
 
-	return true;
+	return CB_SUCCESS;
 }
 
-static bool cse_data_clear_request(const struct cse_bp_info *cse_bp_info)
+static enum cb_err cse_data_clear_request(const struct cse_bp_info *cse_bp_info)
 {
 	struct data_clr_request {
 		struct mkhi_hdr hdr;
@@ -328,7 +440,7 @@ static bool cse_data_clear_request(const struct cse_bp_info *cse_bp_info)
 	if (!cse_is_hfs1_cws_normal() || !cse_is_hfs1_com_soft_temp_disable() ||
 			cse_get_current_bp(cse_bp_info) != RO) {
 		printk(BIOS_ERR, "cse_lite: CSE doesn't meet DATA CLEAR cmd prerequisites\n");
-		return false;
+		return CB_ERR;
 	}
 
 	printk(BIOS_DEBUG, "cse_lite: Sending DATA CLEAR HECI command\n");
@@ -338,16 +450,16 @@ static bool cse_data_clear_request(const struct cse_bp_info *cse_bp_info)
 
 	if (heci_send_receive(&data_clr_rq, sizeof(data_clr_rq), &data_clr_rsp,
 				&data_clr_rsp_sz, HECI_MKHI_ADDR)) {
-		return false;
+		return CB_ERR;
 	}
 
 	if (data_clr_rsp.result) {
 		printk(BIOS_ERR, "cse_lite: CSE DATA CLEAR command response failed: %d\n",
 				data_clr_rsp.result);
-		return false;
+		return CB_ERR;
 	}
 
-	return true;
+	return CB_SUCCESS;
 }
 
 __weak void cse_board_reset(void)
@@ -355,11 +467,16 @@ __weak void cse_board_reset(void)
 	/* Default weak implementation, does nothing. */
 }
 
-/* Set the CSE's next boot partition and issues system reset */
-static bool cse_set_and_boot_from_next_bp(enum boot_partition_id bp)
+__weak void cse_fw_update_misc_oper(void)
 {
-	if (!cse_set_next_boot_partition(bp))
-		return false;
+	/* Default weak implementation, does nothing. */
+}
+
+/* Set the CSE's next boot partition and issues system reset */
+static enum cb_err cse_set_and_boot_from_next_bp(enum boot_partition_id bp)
+{
+	if (cse_set_next_boot_partition(bp) != CB_SUCCESS)
+		return CB_ERR;
 
 	/* Allow the board to perform a reset for CSE RO<->RW jump */
 	cse_board_reset();
@@ -370,13 +487,13 @@ static bool cse_set_and_boot_from_next_bp(enum boot_partition_id bp)
 	die("cse_lite: Failed to reset the system\n");
 
 	/* Control never reaches here */
-	return false;
+	return CB_ERR;
 }
 
-static bool cse_boot_to_rw(const struct cse_bp_info *cse_bp_info)
+static enum cb_err cse_boot_to_rw(const struct cse_bp_info *cse_bp_info)
 {
 	if (cse_get_current_bp(cse_bp_info) == RW)
-		return true;
+		return CB_SUCCESS;
 
 	return cse_set_and_boot_from_next_bp(RW);
 }
@@ -394,7 +511,7 @@ static bool cse_is_rw_dp_valid(const struct cse_bp_info *cse_bp_info)
  * It returns true if RW partition doesn't indicate BP_STATUS_DATA_FAILURE
  * otherwise false if any operation fails.
  */
-static bool cse_fix_data_failure_err(const struct cse_bp_info *cse_bp_info)
+static enum cb_err cse_fix_data_failure_err(const struct cse_bp_info *cse_bp_info)
 {
 	/*
 	 * If RW partition status indicates BP_STATUS_DATA_FAILURE,
@@ -403,10 +520,10 @@ static bool cse_fix_data_failure_err(const struct cse_bp_info *cse_bp_info)
 	 *  - Issue GLOBAL RESET HECI command.
 	 */
 	if (cse_is_rw_dp_valid(cse_bp_info))
-		return true;
+		return CB_SUCCESS;
 
-	if (!cse_data_clear_request(cse_bp_info))
-		return false;
+	if (cse_data_clear_request(cse_bp_info) != CB_SUCCESS)
+		return CB_ERR;
 
 	return cse_boot_to_rw(cse_bp_info);
 }
@@ -454,23 +571,23 @@ static bool cse_is_rw_bp_status_valid(const struct cse_bp_info *cse_bp_info)
 	return true;
 }
 
-static bool cse_boot_to_ro(const struct cse_bp_info *cse_bp_info)
+static enum cb_err cse_boot_to_ro(const struct cse_bp_info *cse_bp_info)
 {
 	if (cse_get_current_bp(cse_bp_info) == RO)
-		return true;
+		return CB_SUCCESS;
 
 	return cse_set_and_boot_from_next_bp(RO);
 }
 
-static bool cse_get_rw_rdev(struct region_device *rdev)
+static enum cb_err cse_get_rw_rdev(struct region_device *rdev)
 {
 	if (fmap_locate_area_as_rdev_rw(CONFIG_SOC_INTEL_CSE_FMAP_NAME, rdev) < 0) {
 		printk(BIOS_ERR, "cse_lite: Failed to locate %s in FMAP\n",
 				CONFIG_SOC_INTEL_CSE_FMAP_NAME);
-		return false;
+		return CB_ERR;
 	}
 
-	return true;
+	return CB_SUCCESS;
 }
 
 static bool cse_is_rw_bp_sign_valid(const struct region_device *target_rdev)
@@ -485,7 +602,7 @@ static bool cse_is_rw_bp_sign_valid(const struct region_device *target_rdev)
 	return cse_bp_sign == CSE_RW_SIGNATURE;
 }
 
-static bool cse_get_target_rdev(const struct cse_bp_info *cse_bp_info,
+static enum cb_err cse_get_target_rdev(const struct cse_bp_info *cse_bp_info,
 		struct region_device *target_rdev)
 {
 	struct region_device cse_region_rdev;
@@ -493,19 +610,19 @@ static bool cse_get_target_rdev(const struct cse_bp_info *cse_bp_info,
 	uint32_t start_offset;
 	uint32_t end_offset;
 
-	if (!cse_get_rw_rdev(&cse_region_rdev))
-		return false;
+	if (cse_get_rw_rdev(&cse_region_rdev) != CB_SUCCESS)
+		return CB_ERR;
 
 	cse_get_bp_entry_range(cse_bp_info, RW, &start_offset, &end_offset);
 	size = end_offset + 1 - start_offset;
 
 	if (rdev_chain(target_rdev, &cse_region_rdev, start_offset, size))
-		return false;
+		return CB_ERR;
 
 	printk(BIOS_DEBUG, "cse_lite: CSE RW partition: offset = 0x%x, size = 0x%x\n",
-			(uint32_t)start_offset, (uint32_t) size);
+			(uint32_t)start_offset, (uint32_t)size);
 
-	return true;
+	return CB_SUCCESS;
 }
 
 static const char *cse_get_source_rdev_fmap(void)
@@ -562,24 +679,24 @@ static bool cse_verify_cbfs_rw_sha256(const uint8_t *expected_rw_blob_sha,
 	return true;
 }
 
-static bool cse_erase_rw_region(const struct region_device *target_rdev)
+static enum cb_err cse_erase_rw_region(const struct region_device *target_rdev)
 {
 	if (rdev_eraseat(target_rdev, 0, region_device_sz(target_rdev)) < 0) {
 		printk(BIOS_ERR, "cse_lite: CSE RW partition could not be erased\n");
-		return false;
+		return CB_ERR;
 	}
-	return true;
+	return CB_SUCCESS;
 }
 
-static bool cse_copy_rw(const struct region_device *target_rdev, const void *buf,
+static enum cb_err cse_copy_rw(const struct region_device *target_rdev, const void *buf,
 		size_t offset, size_t size)
 {
 	if (rdev_writeat(target_rdev, buf, offset, size) < 0) {
 		printk(BIOS_ERR, "cse_lite: Failed to update CSE firmware\n");
-		return false;
+		return CB_ERR;
 	}
 
-	return true;
+	return CB_SUCCESS;
 }
 
 enum cse_update_status {
@@ -602,31 +719,45 @@ static bool read_ver_field(const char *start, char **curr, size_t size, uint16_t
 	return true;
 }
 
+static enum cb_err get_cse_ver_from_cbfs(struct fw_version *cbfs_rw_version)
+{
+	char *version_str, *cbfs_ptr;
+	size_t size;
+
+	if (cbfs_rw_version == NULL)
+		return CB_ERR;
+
+	cbfs_ptr = cbfs_map(CONFIG_SOC_INTEL_CSE_RW_VERSION_CBFS_NAME, &size);
+	version_str = cbfs_ptr;
+	if (!version_str) {
+		printk(BIOS_ERR, "cse_lite: Failed to get %s\n",
+			  CONFIG_SOC_INTEL_CSE_RW_VERSION_CBFS_NAME);
+		return CB_ERR;
+	}
+
+	if (!read_ver_field(version_str, &cbfs_ptr, size, &cbfs_rw_version->major) ||
+	    !read_ver_field(version_str, &cbfs_ptr, size, &cbfs_rw_version->minor) ||
+		!read_ver_field(version_str, &cbfs_ptr, size, &cbfs_rw_version->hotfix) ||
+		!read_ver_field(version_str, &cbfs_ptr, size, &cbfs_rw_version->build)) {
+		cbfs_unmap(version_str);
+		return CB_ERR;
+	}
+
+	cbfs_unmap(version_str);
+	return CB_SUCCESS;
+}
+
 static enum cse_update_status cse_check_update_status(const struct cse_bp_info *cse_bp_info,
-						      struct region_device *target_rdev)
+							struct region_device *target_rdev)
 {
 	int ret;
 	struct fw_version cbfs_rw_version;
-	char *version_str, *ptr;
-	size_t size;
 
 	if (!cse_is_rw_bp_sign_valid(target_rdev))
 		return CSE_UPDATE_CORRUPTED;
 
-	ptr = version_str = cbfs_map(CONFIG_SOC_INTEL_CSE_RW_VERSION_CBFS_NAME, &size);
-	if (!version_str) {
-		printk(BIOS_ERR, "cse_lite: Failed to get %s\n",
-		       CONFIG_SOC_INTEL_CSE_RW_VERSION_CBFS_NAME);
+	if (get_cse_ver_from_cbfs(&cbfs_rw_version) == CB_ERR)
 		return CSE_UPDATE_METADATA_ERROR;
-	}
-
-	if (!read_ver_field(version_str, &ptr, size, &cbfs_rw_version.major) ||
-	    !read_ver_field(version_str, &ptr, size, &cbfs_rw_version.minor) ||
-	    !read_ver_field(version_str, &ptr, size, &cbfs_rw_version.hotfix) ||
-	    !read_ver_field(version_str, &ptr, size, &cbfs_rw_version.build)) {
-		cbfs_unmap(version_str);
-		return CSE_UPDATE_METADATA_ERROR;
-	}
 
 	printk(BIOS_DEBUG, "cse_lite: CSE CBFS RW version : %d.%d.%d.%d\n",
 			cbfs_rw_version.major,
@@ -634,8 +765,6 @@ static enum cse_update_status cse_check_update_status(const struct cse_bp_info *
 			cbfs_rw_version.hotfix,
 			cbfs_rw_version.build);
 
-	cbfs_unmap(version_str);
-
 	ret = cse_compare_sub_part_version(&cbfs_rw_version, cse_get_rw_version(cse_bp_info));
 	if (ret == 0)
 		return CSE_UPDATE_NOT_REQUIRED;
@@ -645,7 +774,7 @@ static enum cse_update_status cse_check_update_status(const struct cse_bp_info *
 		return CSE_UPDATE_UPGRADE;
 }
 
-static bool cse_write_rw_region(const struct region_device *target_rdev,
+static enum cb_err cse_write_rw_region(const struct region_device *target_rdev,
 		const void *cse_cbfs_rw, const size_t cse_cbfs_rw_sz)
 {
 	/* Points to CSE CBFS RW image after boot partition signature */
@@ -655,16 +784,16 @@ static bool cse_write_rw_region(const struct region_device *target_rdev,
 	uint32_t cse_cbfs_rw_wo_sign_sz = cse_cbfs_rw_sz - CSE_RW_SIGN_SIZE;
 
 	/* Update except CSE RW signature */
-	if (!cse_copy_rw(target_rdev, cse_cbfs_rw_wo_sign, CSE_RW_SIGN_SIZE,
-				cse_cbfs_rw_wo_sign_sz))
-		return false;
+	if (cse_copy_rw(target_rdev, cse_cbfs_rw_wo_sign, CSE_RW_SIGN_SIZE,
+				cse_cbfs_rw_wo_sign_sz) != CB_SUCCESS)
+		return CB_ERR;
 
 	/* Update CSE RW signature to indicate update is complete */
-	if (!cse_copy_rw(target_rdev, (void *)cse_cbfs_rw, 0, CSE_RW_SIGN_SIZE))
-		return false;
+	if (cse_copy_rw(target_rdev, (void *)cse_cbfs_rw, 0, CSE_RW_SIGN_SIZE) != CB_SUCCESS)
+		return CB_ERR;
 
 	printk(BIOS_INFO, "cse_lite: CSE RW Update Successful\n");
-	return true;
+	return CB_SUCCESS;
 }
 
 static bool is_cse_fw_update_enabled(void)
@@ -688,16 +817,16 @@ static enum csme_failure_reason cse_update_rw(const struct cse_bp_info *cse_bp_i
 		return CSE_LITE_SKU_LAYOUT_MISMATCH_ERROR;
 	}
 
-	if (!cse_erase_rw_region(target_rdev))
+	if (cse_erase_rw_region(target_rdev) != CB_SUCCESS)
 		return CSE_LITE_SKU_FW_UPDATE_ERROR;
 
-	if (!cse_write_rw_region(target_rdev, cse_cbfs_rw, cse_blob_sz))
+	if (cse_write_rw_region(target_rdev, cse_cbfs_rw, cse_blob_sz) != CB_SUCCESS)
 		return CSE_LITE_SKU_FW_UPDATE_ERROR;
 
 	return CSE_NO_ERROR;
 }
 
-static bool cse_prep_for_rw_update(const struct cse_bp_info *cse_bp_info,
+static enum cb_err cse_prep_for_rw_update(const struct cse_bp_info *cse_bp_info,
 				   enum cse_update_status status)
 {
 	/*
@@ -705,13 +834,13 @@ static bool cse_prep_for_rw_update(const struct cse_bp_info *cse_bp_info,
 	 * 1. Ensure CSE to boot from RO(BP1)
 	 * 2. Send HMRFPO_ENABLE command to CSE
 	 */
-	if (!cse_boot_to_ro(cse_bp_info))
-		return false;
+	if (cse_boot_to_ro(cse_bp_info) != CB_SUCCESS)
+		return CB_ERR;
 
 	if ((status == CSE_UPDATE_DOWNGRADE) || (status == CSE_UPDATE_CORRUPTED)) {
-		if (!cse_data_clear_request(cse_bp_info)) {
+		if (cse_data_clear_request(cse_bp_info) != CB_SUCCESS) {
 			printk(BIOS_ERR, "cse_lite: CSE data clear failed!\n");
-			return false;
+			return CB_ERR;
 		}
 	}
 
@@ -756,11 +885,12 @@ static enum csme_failure_reason cse_trigger_fw_update(const struct cse_bp_info *
 		goto error_exit;
 	}
 
-	if (!cse_prep_for_rw_update(cse_bp_info, status)) {
+	if (cse_prep_for_rw_update(cse_bp_info, status) != CB_SUCCESS) {
 		rv = CSE_COMMUNICATION_ERROR;
 		goto error_exit;
 	}
 
+	cse_fw_update_misc_oper();
 	rv = cse_update_rw(cse_bp_info, cse_cbfs_rw, size, target_rdev);
 
 error_exit:
@@ -774,7 +904,7 @@ static uint8_t cse_fw_update(const struct cse_bp_info *cse_bp_info)
 	struct region_device target_rdev;
 	enum cse_update_status status;
 
-	if (!cse_get_target_rdev(cse_bp_info, &target_rdev)) {
+	if (cse_get_target_rdev(cse_bp_info, &target_rdev) != CB_SUCCESS) {
 		printk(BIOS_ERR, "cse_lite: Failed to get CSE RW Partition\n");
 		return CSE_LITE_SKU_RW_ACCESS_ERROR;
 	}
@@ -801,7 +931,7 @@ static const char *cse_sub_part_str(enum bpdt_entry_type type)
 	}
 }
 
-static bool cse_locate_area_as_rdev_rw(const struct cse_bp_info *cse_bp_info,
+static enum cb_err cse_locate_area_as_rdev_rw(const struct cse_bp_info *cse_bp_info,
 		size_t bp, struct region_device  *cse_rdev)
 {
 	struct region_device cse_region_rdev;
@@ -809,8 +939,8 @@ static bool cse_locate_area_as_rdev_rw(const struct cse_bp_info *cse_bp_info,
 	uint32_t start_offset;
 	uint32_t end_offset;
 
-	if (!cse_get_rw_rdev(&cse_region_rdev))
-		return false;
+	if (cse_get_rw_rdev(&cse_region_rdev) != CB_SUCCESS)
+		return CB_ERR;
 
 	if (!strcmp(cse_regions[bp], "RO"))
 		cse_get_bp_entry_range(cse_bp_info, RO, &start_offset, &end_offset);
@@ -820,14 +950,14 @@ static bool cse_locate_area_as_rdev_rw(const struct cse_bp_info *cse_bp_info,
 	size = end_offset + 1 - start_offset;
 
 	if (rdev_chain(cse_rdev, &cse_region_rdev, start_offset, size))
-		return false;
+		return CB_ERR;
 
 	printk(BIOS_DEBUG, "cse_lite: CSE %s  partition: offset = 0x%x, size = 0x%x\n",
 			cse_regions[bp], start_offset, size);
-	return true;
+	return CB_SUCCESS;
 }
 
-static bool cse_sub_part_get_target_rdev(const struct cse_bp_info *cse_bp_info,
+static enum cb_err cse_sub_part_get_target_rdev(const struct cse_bp_info *cse_bp_info,
 	struct region_device *target_rdev, size_t bp, enum bpdt_entry_type type)
 {
 	struct bpdt_header bpdt_hdr;
@@ -835,22 +965,22 @@ static bool cse_sub_part_get_target_rdev(const struct cse_bp_info *cse_bp_info,
 	struct bpdt_entry bpdt_entries[MAX_SUBPARTS];
 	uint8_t i;
 
-	if (!cse_locate_area_as_rdev_rw(cse_bp_info, bp, &cse_rdev)) {
+	if (cse_locate_area_as_rdev_rw(cse_bp_info, bp, &cse_rdev) != CB_SUCCESS) {
 		printk(BIOS_ERR, "cse_lite: Failed to locate %s in the CSE Region\n",
 				cse_regions[bp]);
-		return false;
+		return CB_ERR;
 	}
 
 	if ((rdev_readat(&cse_rdev, &bpdt_hdr, 0, BPDT_HEADER_SZ)) != BPDT_HEADER_SZ) {
 		printk(BIOS_ERR, "cse_lite: Failed to read BPDT header from CSE region\n");
-		return false;
+		return CB_ERR;
 	}
 
 	if ((rdev_readat(&cse_rdev, bpdt_entries, BPDT_HEADER_SZ,
 		(bpdt_hdr.descriptor_count * BPDT_ENTRY_SZ))) !=
 		(bpdt_hdr.descriptor_count * BPDT_ENTRY_SZ)) {
 		printk(BIOS_ERR, "cse_lite: Failed to read BPDT entries from CSE region\n");
-		return false;
+		return CB_ERR;
 	}
 
 	/* walk through BPDT entries to identify sub-partition's payload offset and size */
@@ -862,17 +992,17 @@ static bool cse_sub_part_get_target_rdev(const struct cse_bp_info *cse_bp_info,
 
 			if (rdev_chain(target_rdev, &cse_rdev, bpdt_entries[i].offset,
 				bpdt_entries[i].size))
-				return false;
+				return CB_ERR;
 			else
-				return true;
+				return CB_SUCCESS;
 		}
 	}
 
 	printk(BIOS_ERR, "cse_lite: Sub-partition %s is not found\n", cse_sub_part_str(type));
-	return false;
+	return CB_ERR;
 }
 
-static bool cse_get_sub_part_fw_version(enum bpdt_entry_type type,
+static enum cb_err cse_get_sub_part_fw_version(enum bpdt_entry_type type,
 					const struct region_device *rdev,
 					struct fw_version *fw_ver)
 {
@@ -883,14 +1013,14 @@ static bool cse_get_sub_part_fw_version(enum bpdt_entry_type type,
 			!= SUBPART_ENTRY_SZ) {
 		printk(BIOS_ERR, "cse_lite: Failed to read %s sub partition entry\n",
 				cse_sub_part_str(type));
-		return false;
+		return CB_ERR;
 	}
 
 	if ((rdev_readat(rdev, &man_hdr, subpart_entry.offset_bytes, SUBPART_MANIFEST_HDR_SZ))
 			!= SUBPART_MANIFEST_HDR_SZ) {
 		printk(BIOS_ERR, "cse_lite: Failed to read %s Sub part entry #0 manifest\n",
 				cse_sub_part_str(type));
-		return false;
+		return CB_ERR;
 	}
 
 	fw_ver->major = man_hdr.binary_version.major;
@@ -898,7 +1028,7 @@ static bool cse_get_sub_part_fw_version(enum bpdt_entry_type type,
 	fw_ver->hotfix = man_hdr.binary_version.hotfix;
 	fw_ver->build = man_hdr.binary_version.build;
 
-	return true;
+	return CB_SUCCESS;
 }
 
 static void cse_sub_part_get_source_fw_version(void *subpart_cbfs_rw, struct fw_version *fw_ver)
@@ -907,8 +1037,8 @@ static void cse_sub_part_get_source_fw_version(void *subpart_cbfs_rw, struct fw_
 	struct subpart_entry *subpart_entry;
 	struct subpart_entry_manifest_header *man_hdr;
 
-	subpart_entry = (struct subpart_entry *) (ptr + SUBPART_HEADER_SZ);
-	man_hdr = (struct subpart_entry_manifest_header *) (ptr + subpart_entry->offset_bytes);
+	subpart_entry = (struct subpart_entry *)(ptr + SUBPART_HEADER_SZ);
+	man_hdr = (struct subpart_entry_manifest_header *)(ptr + subpart_entry->offset_bytes);
 
 	fw_ver->major = man_hdr->binary_version.major;
 	fw_ver->minor = man_hdr->binary_version.minor;
@@ -916,21 +1046,21 @@ static void cse_sub_part_get_source_fw_version(void *subpart_cbfs_rw, struct fw_
 	fw_ver->build = man_hdr->binary_version.build;
 }
 
-static bool cse_prep_for_component_update(const struct cse_bp_info *cse_bp_info)
+static enum cb_err cse_prep_for_component_update(const struct cse_bp_info *cse_bp_info)
 {
 	/*
 	 * To set CSE's operation mode to HMRFPO mode:
 	 * 1. Ensure CSE to boot from RO(BP1)
 	 * 2. Send HMRFPO_ENABLE command to CSE
 	 */
-	if (!cse_boot_to_ro(cse_bp_info))
-		return false;
+	if (cse_boot_to_ro(cse_bp_info) != CB_SUCCESS)
+		return CB_ERR;
 
 	return cse_hmrfpo_enable();
 }
 
-static uint8_t cse_sub_part_trigger_update(enum bpdt_entry_type type, uint8_t bp,
-		const void *subpart_cbfs_rw, const size_t blob_sz,
+static enum csme_failure_reason cse_sub_part_trigger_update(enum bpdt_entry_type type,
+		uint8_t bp, const void *subpart_cbfs_rw, const size_t blob_sz,
 		struct region_device *target_rdev)
 {
 	if (region_device_sz(target_rdev) < blob_sz) {
@@ -941,11 +1071,11 @@ static uint8_t cse_sub_part_trigger_update(enum bpdt_entry_type type, uint8_t bp
 	}
 
 	/* Erase CSE Lite sub-partition */
-	if (!cse_erase_rw_region(target_rdev))
+	if (cse_erase_rw_region(target_rdev) != CB_SUCCESS)
 		return CSE_LITE_SKU_SUB_PART_UPDATE_FAIL;
 
 	/* Update CSE Lite sub-partition */
-	if (!cse_copy_rw(target_rdev, (void *)subpart_cbfs_rw, 0, blob_sz))
+	if (cse_copy_rw(target_rdev, (void *)subpart_cbfs_rw, 0, blob_sz) != CB_SUCCESS)
 		return CSE_LITE_SKU_SUB_PART_UPDATE_FAIL;
 
 	printk(BIOS_INFO, "cse_lite: CSE %s %s Update successful\n", GET_BP_STR(bp),
@@ -954,7 +1084,7 @@ static uint8_t cse_sub_part_trigger_update(enum bpdt_entry_type type, uint8_t bp
 	return CSE_LITE_SKU_PART_UPDATE_SUCCESS;
 }
 
-static uint8_t handle_cse_sub_part_fw_update_rv(uint8_t rv)
+static enum csme_failure_reason handle_cse_sub_part_fw_update_rv(enum csme_failure_reason rv)
 {
 	switch (rv) {
 	case CSE_LITE_SKU_PART_UPDATE_SUCCESS:
@@ -989,12 +1119,12 @@ static enum csme_failure_reason cse_sub_part_fw_component_update(enum bpdt_entry
 
 	/* Trigger sub-partition update in CSE RO and CSE RW */
 	for (size_t bp = 0; bp < ARRAY_SIZE(cse_regions); bp++) {
-		if (!cse_sub_part_get_target_rdev(cse_bp_info, &target_rdev, bp, type)) {
+		if (cse_sub_part_get_target_rdev(cse_bp_info, &target_rdev, bp, type) != CB_SUCCESS) {
 			rv = CSE_LITE_SKU_SUB_PART_ACCESS_ERR;
 			goto error_exit;
 		}
 
-		if (!cse_get_sub_part_fw_version(type, &target_rdev, &target_fw_ver)) {
+		if (cse_get_sub_part_fw_version(type, &target_rdev, &target_fw_ver) != CB_SUCCESS) {
 			rv = CSE_LITE_SKU_SUB_PART_ACCESS_ERR;
 			goto error_exit;
 		}
@@ -1013,7 +1143,7 @@ static enum csme_failure_reason cse_sub_part_fw_component_update(enum bpdt_entry
 		printk(BIOS_INFO, "CSE %s %s Update initiated\n", GET_BP_STR(bp),
 				cse_sub_part_str(type));
 
-		if (!cse_prep_for_component_update(cse_bp_info)) {
+		if (cse_prep_for_component_update(cse_bp_info) != CB_SUCCESS) {
 			rv = CSE_LITE_SKU_SUB_PART_ACCESS_ERR;
 			goto error_exit;
 		}
@@ -1029,14 +1159,14 @@ error_exit:
 	return rv;
 }
 
-static uint8_t cse_sub_part_fw_update(const struct cse_bp_info *cse_bp_info)
+static enum csme_failure_reason cse_sub_part_fw_update(const struct cse_bp_info *cse_bp_info)
 {
 	if (skip_cse_sub_part_update()) {
 		printk(BIOS_INFO, "CSE Sub-partition update not required\n");
 		return CSE_LITE_SKU_SUB_PART_UPDATE_NOT_REQ;
 	}
 
-	int rv;
+	enum csme_failure_reason rv;
 	rv = cse_sub_part_fw_component_update(IOM_FW, cse_bp_info,
 			CONFIG_SOC_INTEL_CSE_IOM_CBFS_NAME);
 
@@ -1048,7 +1178,7 @@ static uint8_t cse_sub_part_fw_update(const struct cse_bp_info *cse_bp_info)
 	return handle_cse_sub_part_fw_update_rv(rv);
 }
 
-void cse_fw_sync(void)
+static void do_cse_fw_sync(void)
 {
 	static struct get_bp_info_rsp cse_bp_info;
 
@@ -1067,7 +1197,7 @@ void cse_fw_sync(void)
 		return;
 	}
 
-	if (!cse_get_bp_info(&cse_bp_info)) {
+	if (cse_get_bp_info(&cse_bp_info) != CB_SUCCESS) {
 		printk(BIOS_ERR, "cse_lite: Failed to get CSE boot partition info\n");
 
 		 /* If system is in recovery mode, don't trigger recovery again */
@@ -1082,7 +1212,7 @@ void cse_fw_sync(void)
 
 	/*
 	 * If system is in recovery mode, CSE Lite update has to be skipped but CSE
-	 * sub-partitions like NPHY and IOM have to to be updated. If CSE sub-parition update
+	 * sub-partitions like NPHY and IOM have to be updated. If CSE sub-parition update
 	 * fails during recovery, just continue to boot.
 	 */
 	if (CONFIG(SOC_INTEL_CSE_SUB_PART_UPDATE) && vboot_recovery_mode_enabled()) {
@@ -1096,7 +1226,7 @@ void cse_fw_sync(void)
 		return;
 	}
 
-	if (!cse_fix_data_failure_err(&cse_bp_info.bp_info))
+	if (cse_fix_data_failure_err(&cse_bp_info.bp_info) != CB_SUCCESS)
 		cse_trigger_vboot_recovery(CSE_LITE_SKU_DATA_WIPE_ERROR);
 
 	/*
@@ -1117,22 +1247,141 @@ void cse_fw_sync(void)
 	if (!cse_is_rw_bp_status_valid(&cse_bp_info.bp_info))
 		cse_trigger_vboot_recovery(CSE_LITE_SKU_RW_JUMP_ERROR);
 
-	if (!cse_boot_to_rw(&cse_bp_info.bp_info)) {
+	if (cse_boot_to_rw(&cse_bp_info.bp_info) != CB_SUCCESS) {
 		printk(BIOS_ERR, "cse_lite: Failed to switch to RW\n");
 		cse_trigger_vboot_recovery(CSE_LITE_SKU_RW_SWITCH_ERROR);
 	}
 }
 
-static void ramstage_cse_fw_sync(void *unused)
+void cse_fw_sync(void)
 {
-	bool s3wake;
-	s3wake = acpi_get_sleep_type() == ACPI_S3;
+	timestamp_add_now(TS_CSE_FW_SYNC_START);
+	do_cse_fw_sync();
+	timestamp_add_now(TS_CSE_FW_SYNC_END);
+}
 
-	if (CONFIG(SOC_INTEL_CSE_LITE_SYNC_IN_RAMSTAGE) && !s3wake) {
-		timestamp_add_now(TS_CSE_FW_SYNC_START);
-		cse_fw_sync();
-		timestamp_add_now(TS_CSE_FW_SYNC_END);
+static enum cb_err send_get_fpt_partition_info_cmd(enum fpt_partition_id id,
+	struct fw_version_resp *resp)
+{
+	enum cse_tx_rx_status ret;
+	struct fw_version_msg {
+		struct mkhi_hdr hdr;
+		enum fpt_partition_id partition_id;
+	} __packed msg = {
+		.hdr = {
+			.group_id = MKHI_GROUP_ID_GEN,
+			.command = GEN_GET_IMAGE_FW_VERSION,
+		},
+		.partition_id = id,
+	};
+
+	/*
+	 * Prerequisites:
+	 * 1) HFSTS1 CWS is Normal
+	 * 2) HFSTS1 COM is Normal
+	 * 3) Only sent after DID (accomplished by compiling this into ramstage)
+	 */
+
+	if (cse_is_hfs1_com_soft_temp_disable() || !cse_is_hfs1_cws_normal() ||
+		!cse_is_hfs1_com_normal()) {
+		printk(BIOS_ERR,
+			"HECI: Prerequisites not met for Get Image Firmware Version command\n");
+		return CB_ERR;
+	}
+
+	size_t resp_size = sizeof(struct fw_version_resp);
+	ret = heci_send_receive(&msg, sizeof(msg), resp, &resp_size, HECI_MKHI_ADDR);
+
+	if (ret || resp->hdr.result) {
+		printk(BIOS_ERR, "CSE: Failed to get partition information for %d: 0x%x\n",
+			id, resp->hdr.result);
+		return CB_ERR;
+	}
+
+	return CB_SUCCESS;
+}
+
+static enum cb_err cse_get_fpt_partition_info(enum fpt_partition_id id,
+		 struct fw_version_resp *resp)
+{
+	if (vboot_recovery_mode_enabled()) {
+		printk(BIOS_WARNING,
+			"CSE: Skip sending Get Image Info command during recovery mode!\n");
+		return CB_ERR;
+	}
+
+	if (id == FPT_PARTITION_NAME_ISHC && !CONFIG(DRIVERS_INTEL_ISH)) {
+		printk(BIOS_WARNING, "CSE: Info request denied, no ISH partition\n");
+		return CB_ERR;
+	}
+
+	return send_get_fpt_partition_info_cmd(id, resp);
+}
+
+/*
+ * Helper function to read ISH version from CSE FPT using HECI command.
+ *
+ * The HECI command only be executed after memory has been initialized.
+ * This is because the command relies on resources that are not available
+ * until DRAM initialization command has been sent.
+ */
+static void store_ish_version(void)
+{
+	if (!ENV_RAMSTAGE)
+		return;
+
+	if (vboot_recovery_mode_enabled())
+		return;
+
+	struct cse_specific_info *info = cbmem_find(CBMEM_ID_CSE_INFO);
+	if (info == NULL)
+		return;
+
+	struct cse_fw_partition_info *version = &(info->cse_fwp_version);
+	size_t size = sizeof(struct fw_version);
+
+	/*
+	 * Compare if stored cse version (from the previous boot) is same as current
+	 * running cse version.
+	 */
+	if (memcmp(&version->ish_partition_info.prev_cse_fw_version,
+		&version->cur_cse_fw_version, sizeof(struct fw_version))) {
+		/*
+		 * Current running CSE version is different than previous stored CSE version
+		 * which could be due to CSE update or rollback, hence, need to send ISHC
+		 * partition info cmd to know the currently running ISH version.
+		 */
+
+		struct fw_version_resp resp;
+		if (cse_get_fpt_partition_info(FPT_PARTITION_NAME_ISHC, &resp) == CB_SUCCESS) {
+			/* Update stored cse version with current version */
+			memcpy(&(version->ish_partition_info.prev_cse_fw_version),
+				&(version->cur_cse_fw_version), size);
+
+			/* Since cse version has been updated, ish version needs to be updated. */
+			memcpy(&(version->ish_partition_info.cur_ish_fw_version),
+				&(resp.manifest_data.version), size);
+
+			/* Update the CRC */
+			cbmem_store_cse_info_crc(info);
+		}
 	}
 }
 
-BOOT_STATE_INIT_ENTRY(BS_PRE_DEVICE, BS_ON_EXIT, ramstage_cse_fw_sync, NULL);
+static void ramstage_cse_misc_ops(void *unused)
+{
+	if (acpi_get_sleep_type() == ACPI_S3)
+		return;
+
+	if (CONFIG(SOC_INTEL_CSE_LITE_SYNC_IN_RAMSTAGE))
+		cse_fw_sync();
+
+	/*
+	 * Store the ISH RW Firmware Version into CBMEM if ISH partition
+	 * is available
+	 */
+	if (soc_is_ish_partition_enabled())
+		store_ish_version();
+}
+
+BOOT_STATE_INIT_ENTRY(BS_PRE_DEVICE, BS_ON_EXIT, ramstage_cse_misc_ops, NULL);
