@@ -57,7 +57,7 @@ static void gma_init(struct device *const dev)
 	 * In case of non-FSP solution, SoC need to select another
 	 * Kconfig to perform GFX initialization.
 	 */
-	if (CONFIG(RUN_FSP_GOP)) {
+	if (CONFIG(RUN_FSP_GOP) && display_init_required()) {
 		const struct soc_intel_common_config *config = chip_get_common_soc_structure();
 		fsp_report_framebuffer_info(graphics_get_framebuffer_address(),
 					    config->panel_orientation);
@@ -117,7 +117,7 @@ uintptr_t graphics_get_framebuffer_address(void)
 
 	memory_base = graphics_get_bar(dev, PCI_BASE_ADDRESS_2);
 	if (!memory_base)
-		die_with_post_code(POST_HW_INIT_FAILURE,
+		die_with_post_code(POSTCODE_HW_INIT_FAILURE,
 				   "Graphic memory bar2 is not programmed!");
 
 	memory_base += CONFIG_SOC_INTEL_GFX_FRAMEBUFFER_OFFSET;
@@ -140,7 +140,7 @@ static uintptr_t graphics_get_gtt_base(void)
 	if (!gtt_base) {
 		gtt_base = graphics_get_bar(dev, PCI_BASE_ADDRESS_0);
 		if (!gtt_base)
-			die_with_post_code(POST_HW_INIT_FAILURE,
+			die_with_post_code(POSTCODE_HW_INIT_FAILURE,
 					   "GTTMMADR is not programmed!");
 	}
 	return gtt_base;
@@ -148,12 +148,12 @@ static uintptr_t graphics_get_gtt_base(void)
 
 uint32_t graphics_gtt_read(unsigned long reg)
 {
-	return read32((void *)(graphics_get_gtt_base() + reg));
+	return read32p(graphics_get_gtt_base() + reg);
 }
 
 void graphics_gtt_write(unsigned long reg, uint32_t data)
 {
-	write32((void *)(graphics_get_gtt_base() + reg), data);
+	write32p(graphics_get_gtt_base() + reg, data);
 }
 
 void graphics_gtt_rmw(unsigned long reg, uint32_t andmask, uint32_t ormask)
@@ -173,6 +173,18 @@ static void graphics_dev_read_resources(struct device *dev)
 		if (res_bar0->flags & IORESOURCE_PREFETCH)
 			res_bar0->flags &= ~IORESOURCE_PREFETCH;
 	}
+
+	/*
+	 * If libhwbase static MMIO driver is used, IGD BAR 0 has to be set to
+	 * CONFIG_GFX_GMA_DEFAULT_MMIO for the libgfxinit to operate properly.
+	 */
+	if (CONFIG(MAINBOARD_USE_LIBGFXINIT) && CONFIG(HWBASE_STATIC_MMIO)) {
+		struct resource *res_bar0 = find_resource(dev, PCI_BASE_ADDRESS_0);
+		res_bar0->base = CONFIG_GFX_GMA_DEFAULT_MMIO;
+		res_bar0->flags |= IORESOURCE_ASSIGNED;
+		pci_dev_set_resources(dev);
+		res_bar0->flags |= IORESOURCE_FIXED;
+	}
 }
 
 const struct device_operations graphics_ops = {
@@ -199,6 +211,7 @@ static const unsigned short pci_device_ids[] = {
 	PCI_DID_INTEL_MTL_P_GT2_1,
 	PCI_DID_INTEL_MTL_P_GT2_2,
 	PCI_DID_INTEL_MTL_P_GT2_3,
+	PCI_DID_INTEL_MTL_P_GT2_4,
 	PCI_DID_INTEL_APL_IGD_HD_505,
 	PCI_DID_INTEL_APL_IGD_HD_500,
 	PCI_DID_INTEL_CNL_GT2_ULX_1,
@@ -223,22 +236,6 @@ static const unsigned short pci_device_ids[] = {
 	PCI_DID_INTEL_CFL_S_GT2_3,
 	PCI_DID_INTEL_CFL_S_GT2_4,
 	PCI_DID_INTEL_CFL_S_GT2_5,
-	PCI_DID_INTEL_ICL_GT0_ULT,
-	PCI_DID_INTEL_ICL_GT0_5_ULT,
-	PCI_DID_INTEL_ICL_GT1_ULT,
-	PCI_DID_INTEL_ICL_GT2_ULX_0,
-	PCI_DID_INTEL_ICL_GT2_ULX_1,
-	PCI_DID_INTEL_ICL_GT2_ULT_1,
-	PCI_DID_INTEL_ICL_GT2_ULX_2,
-	PCI_DID_INTEL_ICL_GT2_ULT_2,
-	PCI_DID_INTEL_ICL_GT2_ULX_3,
-	PCI_DID_INTEL_ICL_GT2_ULT_3,
-	PCI_DID_INTEL_ICL_GT2_ULX_4,
-	PCI_DID_INTEL_ICL_GT2_ULT_4,
-	PCI_DID_INTEL_ICL_GT2_ULX_5,
-	PCI_DID_INTEL_ICL_GT2_ULT_5,
-	PCI_DID_INTEL_ICL_GT2_ULX_6,
-	PCI_DID_INTEL_ICL_GT3_ULT,
 	PCI_DID_INTEL_CML_GT1_ULT_1,
 	PCI_DID_INTEL_CML_GT1_ULT_2,
 	PCI_DID_INTEL_CML_GT2_ULT_1,
@@ -305,12 +302,24 @@ static const unsigned short pci_device_ids[] = {
 	PCI_DID_INTEL_ADL_P_GT2_8,
 	PCI_DID_INTEL_ADL_P_GT2_9,
 	PCI_DID_INTEL_ADL_S_GT1,
+	PCI_DID_INTEL_ADL_S_GT1_1,
+	PCI_DID_INTEL_ADL_S_GT2,
+	PCI_DID_INTEL_ADL_S_GT2_1,
+	PCI_DID_INTEL_ADL_S_GT2_2,
 	PCI_DID_INTEL_ADL_M_GT1,
 	PCI_DID_INTEL_ADL_M_GT2,
 	PCI_DID_INTEL_ADL_M_GT3,
 	PCI_DID_INTEL_ADL_N_GT1,
 	PCI_DID_INTEL_ADL_N_GT2,
 	PCI_DID_INTEL_ADL_N_GT3,
+	PCI_DID_INTEL_RPL_S_GT0,
+	PCI_DID_INTEL_RPL_S_GT1_1,
+	PCI_DID_INTEL_RPL_S_GT1_2,
+	PCI_DID_INTEL_RPL_S_GT1_3,
+	PCI_DID_INTEL_RPL_HX_GT1,
+	PCI_DID_INTEL_RPL_HX_GT2,
+	PCI_DID_INTEL_RPL_HX_GT3,
+	PCI_DID_INTEL_RPL_HX_GT4,
 	0,
 };
 
