@@ -183,10 +183,6 @@ int get_valid_prmrr_size(void);
  */
 void enable_pm_timer_emulation(void);
 
-/* Derive core, package and thread information from lapic ID */
-void get_cpu_topology_from_apicid(uint32_t apicid, uint8_t *package,
-		uint8_t *core, uint8_t *thread);
-
 /*
  * Initialize core PRMRR
  *
@@ -211,4 +207,22 @@ bool is_tme_supported(void);
  */
 void set_tme_core_activate(void);
 
+/*
+ * This function checks if the CPU supports SGX feature.
+ * Returns true if SGX feature is supported otherwise false.
+ */
+bool is_sgx_supported(void);
+
+/*
+ * This function checks if the CPU supports Key Locker feature.
+ * Returns true if Key Locker feature is supported otherwise false.
+ */
+bool is_keylocker_supported(void);
+
+/*
+ * This function prevents the Three Strike Counter from incrementing.
+ * It helps to collect more useful CPU traces for debugging.
+ */
+void disable_three_strike_error(void);
+
 #endif	/* SOC_INTEL_COMMON_BLOCK_CPULIB_H */
