@@ -2,9 +2,11 @@
 
 #include <arch/romstage.h>
 #include <cbmem.h>
+#include <commonlib/bsd/helpers.h>
 #include <console/console.h>
 #include <cpu/x86/mtrr.h>
 #include <cpu/x86/smm.h>
+#include <intelbasecode/ramtop.h>
 #include <intelblocks/fast_spi.h>
 #include <intelblocks/systemagent.h>
 #include <types.h>
@@ -68,6 +70,13 @@ void fill_postcar_frame(struct postcar_frame *pcf)
 	 */
 	printk(BIOS_DEBUG, "top_of_ram = 0x%lx\n", top_of_ram);
 
+	/*
+	 * Store the top_of_ram (ramtop) into the CMOS if SOC_INTEL_COMMON_BASECODE_RAMTOP
+	 * config is enabled.
+	 */
+	if (ENV_ROMSTAGE && CONFIG(SOC_INTEL_COMMON_BASECODE_RAMTOP))
+		update_ramtop(top_of_ram);
+
 	postcar_frame_add_mtrr(pcf, top_of_ram - 16 * MiB, 16 * MiB, MTRR_TYPE_WRBACK);
 
 	/* Cache the TSEG region */
