@@ -1,12 +1,11 @@
-/* SPDX-License-Identifier: GPL-2.0-only */
+/* SPDX-License-Identifier: GPL-2.0-only OR MIT */
 
 #ifndef SOC_MEDIATEK_MT8195_DEVAPC_H
 #define SOC_MEDIATEK_MT8195_DEVAPC_H
 
 #include <device/mmio.h>
 #include <soc/addressmap.h>
-
-void dapc_init(void);
+#include <soc/devapc_common.h>
 
 enum devapc_ao_offset {
 	SYS0_D0_APC_0 = 0x0,
