@@ -21,14 +21,15 @@
 #define QCLIB_VERSION 0
 
 /* store QcLib return data until CBMEM_CREATION_HOOK runs */
-static void *mem_chip_addr;
+static struct mem_chip_info *mem_chip_info;
 
 static void write_mem_chip_information(struct qclib_cb_if_table_entry *te)
 {
+	struct mem_chip_info *info = (void *)te->blob_address;
 	if (te->size > sizeof(struct mem_chip_info) &&
-	    te->size == mem_chip_info_size((void *)te->blob_address)) {
-		/* Save mem_chip_addr in global variable ahead of hook running */
-		mem_chip_addr = (void *)te->blob_address;
+	    te->size == mem_chip_info_size(info->num_entries)) {
+		/* Save mem_chip_info in global variable ahead of hook running */
+		mem_chip_info = info;
 	}
 }
 
@@ -37,19 +38,20 @@ static void add_mem_chip_info(int unused)
 	void *mem_region_base = NULL;
 	size_t size;
 
-	if (!mem_chip_addr) {
+	if (!mem_chip_info || !mem_chip_info->num_entries ||
+	    mem_chip_info->struct_version != MEM_CHIP_STRUCT_VERSION) {
 		printk(BIOS_ERR, "Did not receive valid mem_chip_info from QcLib!");
 		return;
 	}
 
-	size = mem_chip_info_size(mem_chip_addr);
+	size = mem_chip_info_size(mem_chip_info->num_entries);
 
 	/* Add cbmem table */
 	mem_region_base = cbmem_add(CBMEM_ID_MEM_CHIP_INFO, size);
 	ASSERT(mem_region_base != NULL);
 
 	/* Migrate the data into CBMEM */
-	memcpy(mem_region_base, mem_chip_addr, size);
+	memcpy(mem_region_base, mem_chip_info, size);
 }
 
 CBMEM_CREATION_HOOK(add_mem_chip_info);
@@ -167,7 +169,7 @@ static void dump_te_table(void)
 	}
 }
 
-__weak int qclib_soc_blob_load(void) { return 0; }
+__weak int qclib_soc_override(struct qclib_cb_if_table *table) { return 0; }
 
 void qclib_load_and_run(void)
 {
@@ -218,12 +220,6 @@ void qclib_load_and_run(void)
 	}
 	qclib_add_if_table_entry(QCLIB_TE_DCB_SETTINGS, _dcb, data_size, 0);
 
-	/* hook for SoC specific binary blob loads */
-	if (qclib_soc_blob_load()) {
-		printk(BIOS_ERR, "qclib_soc_blob_load failed\n");
-		goto fail;
-	}
-
 	/* Enable QCLib serial output, based on Kconfig */
 	if (CONFIG(CONSOLE_SERIAL))
 		qclib_cb_if_table.global_attributes =
@@ -244,6 +240,12 @@ void qclib_load_and_run(void)
 		printk(BIOS_INFO, "qcsdi.entry[%p]\n", qcsdi.entry);
 	}
 
+	/* hook for SoC specific binary blob loads */
+	if (qclib_soc_override(&qclib_cb_if_table)) {
+		printk(BIOS_ERR, "qclib_soc_override failed\n");
+		goto fail;
+	}
+
 	dump_te_table();
 
 	/* Attempt to load QCLib elf */
@@ -256,7 +258,7 @@ void qclib_load_and_run(void)
 	prog_set_entry(&qclib, prog_entry(&qclib), &qclib_cb_if_table);
 
 	printk(BIOS_DEBUG, "\n\n\nQCLib is about to Initialize DDR\n");
-	printk(BIOS_DEBUG, "Global Attributes[%x]..Table Entries Count[%d]\n",
+	printk(BIOS_DEBUG, "Global Attributes[%#x]..Table Entries Count[%d]\n",
 		qclib_cb_if_table.global_attributes,
 		qclib_cb_if_table.num_entries);
 	printk(BIOS_DEBUG, "Jumping to QCLib code at %p(%p)\n",
