@@ -1,79 +1,35 @@
-/*
- * Copyright (c) 2011 The Chromium OS Authors.
- * Copyright (C) 2009, 2010 Carl-Daniel Hailfinger
- * Copyright (C) 2011 Stefan Tauner
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but without any warranty; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
+
+#define __SIMPLE_DEVICE__
 
 /* This file is derived from the flashrom project. */
-#include <stdint.h>
-#include <stdlib.h>
+
 #include <string.h>
-#include <compiler.h>
 #include <bootstate.h>
 #include <commonlib/helpers.h>
 #include <delay.h>
-#include <arch/io.h>
+#include <device/mmio.h>
+#include <device/pci_ops.h>
 #include <console/console.h>
-#include <device/pci_ids.h>
+#include <device/device.h>
 #include <device/pci.h>
 #include <spi_flash.h>
-
 #include <spi-generic.h>
+#include <timer.h>
+#include <types.h>
+
+#include "spi.h"
 
 #define HSFC_FCYCLE_OFF		1	/* 1-2: FLASH Cycle */
 #define HSFC_FCYCLE		(0x3 << HSFC_FCYCLE_OFF)
 #define HSFC_FDBC_OFF		8	/* 8-13: Flash Data Byte Count */
 #define HSFC_FDBC		(0x3f << HSFC_FDBC_OFF)
 
-
-#ifdef __SMM__
-#include <arch/io.h>
-#define pci_read_config_byte(dev, reg, targ)\
-	*(targ) = pci_read_config8(dev, reg)
-#define pci_read_config_word(dev, reg, targ)\
-	*(targ) = pci_read_config16(dev, reg)
-#define pci_read_config_dword(dev, reg, targ)\
-	*(targ) = pci_read_config32(dev, reg)
-#define pci_write_config_byte(dev, reg, val)\
-	pci_write_config8(dev, reg, val)
-#define pci_write_config_word(dev, reg, val)\
-	pci_write_config16(dev, reg, val)
-#define pci_write_config_dword(dev, reg, val)\
-	pci_write_config32(dev, reg, val)
-#else /* !__SMM__ */
-#include <device/device.h>
-#include <device/pci.h>
-#define pci_read_config_byte(dev, reg, targ)\
-	*(targ) = pci_read_config8(dev, reg)
-#define pci_read_config_word(dev, reg, targ)\
-	*(targ) = pci_read_config16(dev, reg)
-#define pci_read_config_dword(dev, reg, targ)\
-	*(targ) = pci_read_config32(dev, reg)
-#define pci_write_config_byte(dev, reg, val)\
-	pci_write_config8(dev, reg, val)
-#define pci_write_config_word(dev, reg, val)\
-	pci_write_config16(dev, reg, val)
-#define pci_write_config_dword(dev, reg, val)\
-	pci_write_config32(dev, reg, val)
-#endif /* !__SMM__ */
-
 static int spi_is_multichip(void);
 
-typedef struct spi_slave ich_spi_slave;
+static void spi_set_smm_only_flashing(bool enable);
 
-static int ichspi_lock = 0;
-
-typedef struct ich7_spi_regs {
+struct ich7_spi_regs {
 	uint16_t spis;
 	uint16_t spic;
 	uint32_t spia;
@@ -83,9 +39,10 @@ typedef struct ich7_spi_regs {
 	uint16_t preop;
 	uint16_t optype;
 	uint8_t opmenu[8];
-} __packed ich7_spi_regs;
+	uint32_t pbr[3];
+} __packed;
 
-typedef struct ich9_spi_regs {
+struct ich9_spi_regs {
 	uint32_t bfpr;
 	uint16_t hsfs;
 	uint16_t hsfc;
@@ -116,27 +73,33 @@ typedef struct ich9_spi_regs {
 	uint32_t srdl;
 	uint32_t srdc;
 	uint32_t srd;
-} __packed ich9_spi_regs;
+} __packed;
 
-typedef struct ich_spi_controller {
+struct ich_spi_controller {
 	int locked;
 	uint32_t flmap0;
+	uint32_t flcomp;
 	uint32_t hsfs;
 
-	ich9_spi_regs *ich9_spi;
+	union {
+		struct ich9_spi_regs *ich9_spi;
+		struct ich7_spi_regs *ich7_spi;
+	};
 	uint8_t *opmenu;
 	int menubytes;
 	uint16_t *preop;
 	uint16_t *optype;
 	uint32_t *addr;
 	uint8_t *data;
-	unsigned databytes;
+	unsigned int databytes;
 	uint8_t *status;
 	uint16_t *control;
 	uint32_t *bbar;
-} ich_spi_controller;
+	uint32_t *fpr;
+	uint8_t fpr_max;
+};
 
-static ich_spi_controller cntlr;
+static struct ich_spi_controller cntlr;
 
 enum {
 	SPIS_SCIP =		0x0001,
@@ -188,51 +151,54 @@ enum {
 	SPI_OPCODE_TYPE_WRITE_WITH_ADDRESS =	3
 };
 
-#if IS_ENABLED(CONFIG_DEBUG_SPI_FLASH)
+#if CONFIG(DEBUG_SPI_FLASH)
 
 static u8 readb_(const void *addr)
 {
 	u8 v = read8(addr);
-	printk(BIOS_DEBUG, "read %2.2x from %4.4x\n",
-	       v, ((unsigned) addr & 0xffff) - 0xf020);
+
+	printk(BIOS_DEBUG, "read %2.2x from %4.4lx\n",
+	       v, ((uintptr_t)addr & 0xffff) - 0xf020);
 	return v;
 }
 
 static u16 readw_(const void *addr)
 {
 	u16 v = read16(addr);
-	printk(BIOS_DEBUG, "read %4.4x from %4.4x\n",
-	       v, ((unsigned) addr & 0xffff) - 0xf020);
+
+	printk(BIOS_DEBUG, "read %4.4x from %4.4lx\n",
+	       v, ((uintptr_t)addr & 0xffff) - 0xf020);
 	return v;
 }
 
 static u32 readl_(const void *addr)
 {
 	u32 v = read32(addr);
-	printk(BIOS_DEBUG, "read %8.8x from %4.4x\n",
-	       v, ((unsigned) addr & 0xffff) - 0xf020);
+
+	printk(BIOS_DEBUG, "read %8.8x from %4.4lx\n",
+	       v, ((uintptr_t)addr & 0xffff) - 0xf020);
 	return v;
 }
 
 static void writeb_(u8 b, void *addr)
 {
 	write8(addr, b);
-	printk(BIOS_DEBUG, "wrote %2.2x to %4.4x\n",
-	       b, ((unsigned) addr & 0xffff) - 0xf020);
+	printk(BIOS_DEBUG, "wrote %2.2x to %4.4lx\n",
+	       b, ((uintptr_t)addr & 0xffff) - 0xf020);
 }
 
 static void writew_(u16 b, void *addr)
 {
 	write16(addr, b);
-	printk(BIOS_DEBUG, "wrote %4.4x to %4.4x\n",
-	       b, ((unsigned) addr & 0xffff) - 0xf020);
+	printk(BIOS_DEBUG, "wrote %4.4x to %4.4lx\n",
+	       b, ((uintptr_t)addr & 0xffff) - 0xf020);
 }
 
 static void writel_(u32 b, void *addr)
 {
 	write32(addr, b);
-	printk(BIOS_DEBUG, "wrote %8.8x to %4.4x\n",
-	       b, ((unsigned) addr & 0xffff) - 0xf020);
+	printk(BIOS_DEBUG, "wrote %8.8x to %4.4lx\n",
+	       b, ((uintptr_t)addr & 0xffff) - 0xf020);
 }
 
 #else /* CONFIG_DEBUG_SPI_FLASH ^^^ enabled  vvv NOT enabled */
@@ -287,27 +253,46 @@ static void ich_set_bbar(uint32_t minaddr)
 	writel_(ichspi_bbar, cntlr.bbar);
 }
 
-void spi_init(void)
-{
-	uint8_t *rcrb; /* Root Complex Register Block */
-	uint32_t rcba; /* Root Complex Base Address */
-	uint8_t bios_cntl;
-	device_t dev;
-	ich9_spi_regs *ich9_spi;
-	ich7_spi_regs *ich7_spi;
-	uint16_t hsfs;
-
-#ifdef __SMM__
-	dev = PCI_DEV(0, 31, 0);
+#if CONFIG(SOUTHBRIDGE_INTEL_I82801GX)
+#define MENU_BYTES member_size(struct ich7_spi_regs, opmenu)
 #else
-	dev = dev_find_slot(0, PCI_DEVFN(31, 0));
+#define MENU_BYTES member_size(struct ich9_spi_regs, opmenu)
 #endif
 
-	pci_read_config_dword(dev, 0xf0, &rcba);
-	/* Bits 31-14 are the base address, 13-1 are reserved, 0 is enable. */
-	rcrb = (uint8_t *)(rcba & 0xffffc000);
-	if (IS_ENABLED(CONFIG_SOUTHBRIDGE_INTEL_I82801GX)) {
-		ich7_spi = (ich7_spi_regs *)(rcrb + 0x3020);
+#define RCBA 0xf0
+#define SBASE 0x54
+
+static void *get_spi_bar(pci_devfn_t dev)
+{
+	uintptr_t rcba; /* Root Complex Register Block */
+	uintptr_t sbase;
+
+	if (CONFIG(SOUTHBRIDGE_INTEL_I82801GX)) {
+		rcba = pci_read_config32(dev, RCBA);
+		return (void *)((rcba & 0xffffc000) + 0x3020);
+	}
+	if (CONFIG(SOUTHBRIDGE_INTEL_COMMON_SPI_SILVERMONT)) {
+		sbase = pci_read_config32(dev, SBASE);
+		sbase &= ~0x1ff;
+		return (void *)sbase;
+	}
+	if (CONFIG(SOUTHBRIDGE_INTEL_COMMON_SPI_ICH9)) {
+		rcba = pci_read_config32(dev, RCBA);
+		return (void *)((rcba & 0xffffc000) + 0x3800);
+	}
+}
+
+void spi_init(void)
+{
+	struct ich9_spi_regs *ich9_spi;
+	struct ich7_spi_regs *ich7_spi;
+	uint16_t hsfs;
+
+	pci_devfn_t dev = PCI_DEV(0, 31, 0);
+
+	if (CONFIG(SOUTHBRIDGE_INTEL_I82801GX)) {
+		ich7_spi = get_spi_bar(dev);
+		cntlr.ich7_spi = ich7_spi;
 		cntlr.opmenu = ich7_spi->opmenu;
 		cntlr.menubytes = sizeof(ich7_spi->opmenu);
 		cntlr.optype = &ich7_spi->optype;
@@ -315,15 +300,15 @@ void spi_init(void)
 		cntlr.data = (uint8_t *)ich7_spi->spid;
 		cntlr.databytes = sizeof(ich7_spi->spid);
 		cntlr.status = (uint8_t *)&ich7_spi->spis;
-		ichspi_lock = readw_(&ich7_spi->spis) & HSFS_FLOCKDN;
 		cntlr.control = &ich7_spi->spic;
 		cntlr.bbar = &ich7_spi->bbar;
 		cntlr.preop = &ich7_spi->preop;
+		cntlr.fpr = &ich7_spi->pbr[0];
+		cntlr.fpr_max = 3;
 	} else {
-		ich9_spi = (ich9_spi_regs *)(rcrb + 0x3800);
+		ich9_spi = get_spi_bar(dev);
 		cntlr.ich9_spi = ich9_spi;
 		hsfs = readw_(&ich9_spi->hsfs);
-		ichspi_lock = hsfs & HSFS_FLOCKDN;
 		cntlr.hsfs = hsfs;
 		cntlr.opmenu = ich9_spi->opmenu;
 		cntlr.menubytes = sizeof(ich9_spi->opmenu);
@@ -335,20 +320,30 @@ void spi_init(void)
 		cntlr.control = (uint16_t *)ich9_spi->ssfc;
 		cntlr.bbar = &ich9_spi->bbar;
 		cntlr.preop = &ich9_spi->preop;
+		cntlr.fpr = &ich9_spi->pr[0];
+		cntlr.fpr_max = 5;
 
 		if (cntlr.hsfs & HSFS_FDV) {
-			writel_ (4, &ich9_spi->fdoc);
+			writel_(4, &ich9_spi->fdoc);
 			cntlr.flmap0 = readl_(&ich9_spi->fdod);
+			writel_(0x1000, &ich9_spi->fdoc);
+			cntlr.flcomp = readl_(&ich9_spi->fdod);
 		}
 	}
 
 	ich_set_bbar(0);
 
 	/* Disable the BIOS write protect so write commands are allowed. */
-	pci_read_config_byte(dev, 0xdc, &bios_cntl);
-	/* Deassert SMM BIOS Write Protect Disable. */
-	bios_cntl &= ~(1 << 5);
-	pci_write_config_byte(dev, 0xdc, bios_cntl | 0x1);
+	spi_set_smm_only_flashing(false);
+}
+
+static int spi_locked(void)
+{
+	if (CONFIG(SOUTHBRIDGE_INTEL_I82801GX)) {
+		return !!(readw_(&cntlr.ich7_spi->spis) & HSFS_FLOCKDN);
+	} else {
+		return !!(readw_(&cntlr.ich9_spi->hsfs) & HSFS_FLOCKDN);
+	}
 }
 
 static void spi_init_cb(void *unused)
@@ -368,13 +363,13 @@ typedef struct spi_transaction {
 	uint32_t offset;
 } spi_transaction;
 
-static inline void spi_use_out(spi_transaction *trans, unsigned bytes)
+static inline void spi_use_out(spi_transaction *trans, unsigned int bytes)
 {
 	trans->out += bytes;
 	trans->bytesout -= bytes;
 }
 
-static inline void spi_use_in(spi_transaction *trans, unsigned bytes)
+static inline void spi_use_in(spi_transaction *trans, unsigned int bytes)
 {
 	trans->in += bytes;
 	trans->bytesin -= bytes;
@@ -417,54 +412,53 @@ static void spi_setup_type(spi_transaction *trans)
 static int spi_setup_opcode(spi_transaction *trans)
 {
 	uint16_t optypes;
-	uint8_t opmenu[cntlr.menubytes];
+	uint8_t opmenu[MENU_BYTES];
 
 	trans->opcode = trans->out[0];
 	spi_use_out(trans, 1);
-	if (!ichspi_lock) {
+	if (!spi_locked()) {
 		/* The lock is off, so just use index 0. */
 		writeb_(trans->opcode, cntlr.opmenu);
 		optypes = readw_(cntlr.optype);
 		optypes = (optypes & 0xfffc) | (trans->type & 0x3);
 		writew_(optypes, cntlr.optype);
 		return 0;
-	} else {
-		/* The lock is on. See if what we need is on the menu. */
-		uint8_t optype;
-		uint16_t opcode_index;
-
-		/* Write Enable is handled as atomic prefix */
-		if (trans->opcode == SPI_OPCODE_WREN)
-			return 0;
-
-		read_reg(cntlr.opmenu, opmenu, sizeof(opmenu));
-		for (opcode_index = 0; opcode_index < cntlr.menubytes;
-				opcode_index++) {
-			if (opmenu[opcode_index] == trans->opcode)
-				break;
-		}
-
-		if (opcode_index == cntlr.menubytes) {
-			printk(BIOS_DEBUG, "ICH SPI: Opcode %x not found\n",
-				trans->opcode);
-			return -1;
-		}
-
-		optypes = readw_(cntlr.optype);
-		optype = (optypes >> (opcode_index * 2)) & 0x3;
-		if (trans->type == SPI_OPCODE_TYPE_WRITE_NO_ADDRESS &&
-			optype == SPI_OPCODE_TYPE_WRITE_WITH_ADDRESS &&
-			trans->bytesout >= 3) {
-			/* We guessed wrong earlier. Fix it up. */
-			trans->type = optype;
-		}
-		if (optype != trans->type) {
-			printk(BIOS_DEBUG, "ICH SPI: Transaction doesn't fit type %d\n",
-				optype);
-			return -1;
-		}
-		return opcode_index;
 	}
+
+	/* The lock is on. See if what we need is on the menu. */
+	uint8_t optype;
+	uint16_t opcode_index;
+
+	/* Write Enable is handled as atomic prefix */
+	if (trans->opcode == SPI_OPCODE_WREN)
+		return 0;
+
+	read_reg(cntlr.opmenu, opmenu, sizeof(opmenu));
+	for (opcode_index = 0; opcode_index < ARRAY_SIZE(opmenu); opcode_index++) {
+		if (opmenu[opcode_index] == trans->opcode)
+			break;
+	}
+
+	if (opcode_index == ARRAY_SIZE(opmenu)) {
+		printk(BIOS_DEBUG, "ICH SPI: Opcode %x not found\n",
+			trans->opcode);
+		return -1;
+	}
+
+	optypes = readw_(cntlr.optype);
+	optype = (optypes >> (opcode_index * 2)) & 0x3;
+	if (trans->type == SPI_OPCODE_TYPE_WRITE_NO_ADDRESS &&
+		optype == SPI_OPCODE_TYPE_WRITE_WITH_ADDRESS &&
+		trans->bytesout >= 3) {
+		/* We guessed wrong earlier. Fix it up. */
+		trans->type = optype;
+	}
+	if (optype != trans->type) {
+		printk(BIOS_DEBUG, "ICH SPI: Transaction doesn't fit type %d\n",
+			optype);
+		return -1;
+	}
+	return opcode_index;
 }
 
 static int spi_setup_offset(spi_transaction *trans)
@@ -514,7 +508,7 @@ static int ich_status_poll(u16 bitmask, int wait_til_set)
 	return -1;
 }
 
-static int spi_is_multichip (void)
+static int spi_is_multichip(void)
 {
 	if (!(cntlr.hsfs & HSFS_FDV))
 		return 0;
@@ -563,7 +557,7 @@ static int spi_ctrlr_xfer(const struct spi_slave *slave, const void *dout,
 		 * in order to prevent the Management Engine from
 		 * issuing a transaction between WREN and DATA.
 		 */
-		if (!ichspi_lock)
+		if (!spi_locked())
 			writew_(trans.opcode, cntlr.preop);
 		return 0;
 	}
@@ -598,7 +592,7 @@ static int spi_ctrlr_xfer(const struct spi_slave *slave, const void *dout,
 			return -1;
 		}
 
-		return 0;
+		goto spi_xfer_exit;
 	}
 
 	/*
@@ -625,9 +619,9 @@ static int spi_ctrlr_xfer(const struct spi_slave *slave, const void *dout,
 		writel_(trans.offset & 0x00FFFFFF, cntlr.addr);
 
 		if (trans.bytesout)
-			data_length = min(trans.bytesout, cntlr.databytes);
+			data_length = MIN(trans.bytesout, cntlr.databytes);
 		else
-			data_length = min(trans.bytesin, cntlr.databytes);
+			data_length = MIN(trans.bytesin, cntlr.databytes);
 
 		/* Program data into FDATA0 to N */
 		if (trans.bytesout) {
@@ -663,6 +657,7 @@ static int spi_ctrlr_xfer(const struct spi_slave *slave, const void *dout,
 		}
 	}
 
+spi_xfer_exit:
 	/* Clear atomic preop now that xfer is done */
 	writew_(0, cntlr.preop);
 
@@ -673,6 +668,7 @@ static int spi_ctrlr_xfer(const struct spi_slave *slave, const void *dout,
 static void ich_hwseq_set_addr(uint32_t addr)
 {
 	uint32_t addr_old = readl_(&cntlr.ich9_spi->faddr) & ~0x01FFFFFF;
+
 	writel_((addr & 0x01FFFFFF) | addr_old, &cntlr.ich9_spi->faddr);
 }
 
@@ -718,14 +714,13 @@ static int ich_hwseq_wait_for_cycle_complete(unsigned int timeout,
 	return 0;
 }
 
-
 static int ich_hwseq_erase(const struct spi_flash *flash, u32 offset,
 			size_t len)
 {
 	u32 start, end, erase_size;
 	int ret;
 	uint16_t hsfc;
-	uint16_t timeout = 1000 * 60;
+	unsigned int timeout = 1000 * USECS_PER_MSEC; /* 1 second timeout */
 
 	erase_size = flash->sector_size;
 	if (offset % erase_size || len % erase_size) {
@@ -755,8 +750,7 @@ static int ich_hwseq_erase(const struct spi_flash *flash, u32 offset,
 		hsfc |= (0x3 << HSFC_FCYCLE_OFF); /* set erase operation */
 		hsfc |= HSFC_FGO; /* start */
 		writew_(hsfc, &cntlr.ich9_spi->hsfc);
-		if (ich_hwseq_wait_for_cycle_complete(timeout, len))
-		{
+		if (ich_hwseq_wait_for_cycle_complete(timeout, len)) {
 			printk(BIOS_ERR, "SF: Erase failed at %x\n", offset - erase_size);
 			ret = -1;
 			goto out;
@@ -791,10 +785,10 @@ static int ich_hwseq_read(const struct spi_flash *flash, u32 addr, size_t len,
 	uint8_t block_len;
 
 	if (addr + len > flash->size) {
-		printk (BIOS_ERR,
+		printk(BIOS_ERR,
 			"Attempt to read %x-%x which is out of chip\n",
-			(unsigned) addr,
-			(unsigned) addr+(unsigned) len);
+			(unsigned int)addr,
+			(unsigned int)addr+(unsigned int) len);
 		return -1;
 	}
 
@@ -802,7 +796,7 @@ static int ich_hwseq_read(const struct spi_flash *flash, u32 addr, size_t len,
 	writew_(readw_(&cntlr.ich9_spi->hsfs), &cntlr.ich9_spi->hsfs);
 
 	while (len > 0) {
-		block_len = min(len, cntlr.databytes);
+		block_len = MIN(len, cntlr.databytes);
 		if (block_len > (~addr & 0xff))
 			block_len = (~addr & 0xff) + 1;
 		ich_hwseq_set_addr(addr);
@@ -841,7 +835,7 @@ static void ich_fill_data(const uint8_t *data, int len)
 		if ((i % 4) == 0)
 			temp32 = 0;
 
-		temp32 |= ((uint32_t) data[i]) << ((i % 4) * 8);
+		temp32 |= ((uint32_t)data[i]) << ((i % 4) * 8);
 
 		if ((i % 4) == 3) /* 32 bits are full, write them to regs. */
 			writel_(temp32, cntlr.data + (i - (i % 4)));
@@ -860,9 +854,9 @@ static int ich_hwseq_write(const struct spi_flash *flash, u32 addr, size_t len,
 	uint32_t start = addr;
 
 	if (addr + len > flash->size) {
-		printk (BIOS_ERR,
+		printk(BIOS_ERR,
 			"Attempt to write 0x%x-0x%x which is out of chip\n",
-			(unsigned)addr, (unsigned) (addr+len));
+			(unsigned int)addr, (unsigned int)(addr+len));
 		return -1;
 	}
 
@@ -870,7 +864,7 @@ static int ich_hwseq_write(const struct spi_flash *flash, u32 addr, size_t len,
 	writew_(readw_(&cntlr.ich9_spi->hsfs), &cntlr.ich9_spi->hsfs);
 
 	while (len > 0) {
-		block_len = min(len, cntlr.databytes);
+		block_len = MIN(len, cntlr.databytes);
 		if (block_len > (~addr & 0xff))
 			block_len = (~addr & 0xff) + 1;
 
@@ -886,9 +880,8 @@ static int ich_hwseq_write(const struct spi_flash *flash, u32 addr, size_t len,
 		hsfc |= HSFC_FGO; /* start */
 		writew_(hsfc, &cntlr.ich9_spi->hsfc);
 
-		if (ich_hwseq_wait_for_cycle_complete(timeout, block_len))
-		{
-			printk (BIOS_ERR, "SF: write failure at %x\n",
+		if (ich_hwseq_wait_for_cycle_complete(timeout, block_len)) {
+			printk(BIOS_ERR, "SF: write failure at %x\n",
 				addr);
 			return -1;
 		}
@@ -897,7 +890,7 @@ static int ich_hwseq_write(const struct spi_flash *flash, u32 addr, size_t len,
 		len -= block_len;
 	}
 	printk(BIOS_DEBUG, "SF: Successfully written %u bytes @ %#x\n",
-	       (unsigned) (addr - start), start);
+	       (unsigned int)(addr - start), start);
 	return 0;
 }
 
@@ -910,9 +903,8 @@ static const struct spi_flash_ops spi_flash_ops = {
 static int spi_flash_programmer_probe(const struct spi_slave *spi,
 					struct spi_flash *flash)
 {
-	uint32_t flcomp;
 
-	if (IS_ENABLED(CONFIG_SOUTHBRIDGE_INTEL_I82801GX))
+	if (CONFIG(SOUTHBRIDGE_INTEL_I82801GX))
 		return spi_flash_generic_probe(spi, flash);
 
 	/* Try generic probing first if spi_is_multichip returns 0. */
@@ -920,11 +912,9 @@ static int spi_flash_programmer_probe(const struct spi_slave *spi,
 		return 0;
 
 	memcpy(&flash->spi, spi, sizeof(*spi));
-	flash->name = "Opaque HW-sequencing";
 
-	ich_hwseq_set_addr (0);
-	switch ((cntlr.hsfs >> 3) & 3)
-	{
+	ich_hwseq_set_addr(0);
+	switch ((cntlr.hsfs >> 3) & 3) {
 	case 0:
 		flash->sector_size = 256;
 		break;
@@ -939,25 +929,207 @@ static int spi_flash_programmer_probe(const struct spi_slave *spi,
 		break;
 	}
 
-	writel_ (0x1000, &cntlr.ich9_spi->fdoc);
-	flcomp = readl_(&cntlr.ich9_spi->fdod);
-
-	flash->size = 1 << (19 + (flcomp & 7));
+	flash->size = 1 << (19 + (cntlr.flcomp & 7));
 
 	flash->ops = &spi_flash_ops;
 
 	if ((cntlr.hsfs & HSFS_FDV) && ((cntlr.flmap0 >> 8) & 3))
-		flash->size += 1 << (19 + ((flcomp >> 3) & 7));
-	printk (BIOS_DEBUG, "flash size 0x%x bytes\n", flash->size);
+		flash->size += 1 << (19 + ((cntlr.flcomp >> 3) & 7));
+	printk(BIOS_DEBUG, "flash size 0x%x bytes\n", flash->size);
 
 	return 0;
 }
 
+static int xfer_vectors(const struct spi_slave *slave,
+			struct spi_op vectors[], size_t count)
+{
+	return spi_flash_vector_helper(slave, vectors, count, spi_ctrlr_xfer);
+}
+
+#define SPI_FPR_SHIFT			12
+#define ICH7_SPI_FPR_MASK		0xfff
+#define ICH9_SPI_FPR_MASK		0x1fff
+#define SPI_FPR_BASE_SHIFT		0
+#define ICH7_SPI_FPR_LIMIT_SHIFT	12
+#define ICH9_SPI_FPR_LIMIT_SHIFT	16
+#define ICH9_SPI_FPR_RPE		(1 << 15) /* Read Protect */
+#define SPI_FPR_WPE			(1 << 31) /* Write Protect */
+
+static u32 spi_fpr(u32 base, u32 limit)
+{
+	u32 ret;
+	u32 mask, limit_shift;
+
+	if (CONFIG(SOUTHBRIDGE_INTEL_I82801GX)) {
+		mask = ICH7_SPI_FPR_MASK;
+		limit_shift = ICH7_SPI_FPR_LIMIT_SHIFT;
+	} else {
+		mask = ICH9_SPI_FPR_MASK;
+		limit_shift = ICH9_SPI_FPR_LIMIT_SHIFT;
+	}
+	ret = ((limit >> SPI_FPR_SHIFT) & mask) << limit_shift;
+	ret |= ((base >> SPI_FPR_SHIFT) & mask) << SPI_FPR_BASE_SHIFT;
+	return ret;
+}
+
+/*
+ * Protect range of SPI flash defined by [start, start+size-1] using Flash
+ * Protected Range (FPR) register if available.
+ * Returns 0 on success, -1 on failure of programming fpr registers.
+ */
+static int spi_flash_protect(const struct spi_flash *flash,
+			const struct region *region,
+			const enum ctrlr_prot_type type)
+{
+	u32 start = region_offset(region);
+	u32 end = start + region_sz(region) - 1;
+	u32 reg;
+	u32 protect_mask = 0;
+	int fpr;
+	uint32_t *fpr_base;
+
+	fpr_base = cntlr.fpr;
+
+	/* Find first empty FPR */
+	for (fpr = 0; fpr < cntlr.fpr_max; fpr++) {
+		reg = read32(&fpr_base[fpr]);
+		if (reg == 0)
+			break;
+	}
+
+	if (fpr == cntlr.fpr_max) {
+		printk(BIOS_ERR, "No SPI FPR free!\n");
+		return -1;
+	}
+
+	switch (type) {
+	case WRITE_PROTECT:
+		protect_mask |= SPI_FPR_WPE;
+		break;
+	case READ_PROTECT:
+		if (CONFIG(SOUTHBRIDGE_INTEL_I82801GX))
+			return -1;
+		protect_mask |= ICH9_SPI_FPR_RPE;
+		break;
+	case READ_WRITE_PROTECT:
+		if (CONFIG(SOUTHBRIDGE_INTEL_I82801GX))
+			return -1;
+		protect_mask |= (ICH9_SPI_FPR_RPE | SPI_FPR_WPE);
+		break;
+	default:
+		printk(BIOS_ERR, "Seeking invalid protection!\n");
+		return -1;
+	}
+
+	/* Set protected range base and limit */
+	reg = spi_fpr(start, end) | protect_mask;
+
+	/* Set the FPR register and verify it is protected */
+	write32(&fpr_base[fpr], reg);
+	if (reg != read32(&fpr_base[fpr])) {
+		printk(BIOS_ERR, "Unable to set SPI FPR %d\n", fpr);
+		return -1;
+	}
+
+	printk(BIOS_INFO, "%s: FPR %d is enabled for range 0x%08x-0x%08x\n",
+	       __func__, fpr, start, end);
+	return 0;
+}
+
+void spi_finalize_ops(void)
+{
+	u16 spi_opprefix;
+	u16 optype = 0;
+	struct intel_swseq_spi_config spi_config_default = {
+		{0x06, 0x50},  /* OPPREFIXES: EWSR and WREN */
+		{ /* OPCODE and OPTYPE */
+			{0x01, WRITE_NO_ADDR},		/* WRSR: Write Status Register */
+			{0x02, WRITE_WITH_ADDR},	/* BYPR: Byte Program */
+			{0x03, READ_WITH_ADDR},		/* READ: Read Data */
+			{0x05, READ_NO_ADDR},		/* RDSR: Read Status Register */
+			{0x20, WRITE_WITH_ADDR},	/* SE20: Sector Erase 0x20 */
+			{0x9f, READ_NO_ADDR},		/* RDID: Read ID */
+			{0xd8, WRITE_WITH_ADDR},	/* BED8: Block Erase 0xd8 */
+			{0x0b, READ_WITH_ADDR},		/* FAST: Fast Read */
+		}
+	};
+	struct intel_swseq_spi_config spi_config_aai_write = {
+		{0x06, 0x50}, /* OPPREFIXES: EWSR and WREN */
+		{ /* OPCODE and OPTYPE */
+			{0x01, WRITE_NO_ADDR},		/* WRSR: Write Status Register */
+			{0x02, WRITE_WITH_ADDR},	/* BYPR: Byte Program */
+			{0x03, READ_WITH_ADDR},		/* READ: Read Data */
+			{0x05, READ_NO_ADDR},		/* RDSR: Read Status Register */
+			{0x20, WRITE_WITH_ADDR},	/* SE20: Sector Erase 0x20 */
+			{0x9f, READ_NO_ADDR},		/* RDID: Read ID */
+			{0xad, WRITE_NO_ADDR},		/* Auto Address Increment Word Program */
+			{0x04, WRITE_NO_ADDR}		/* Write Disable */
+		}
+	};
+	const struct spi_flash *flash = boot_device_spi_flash();
+	struct intel_swseq_spi_config *spi_config = &spi_config_default;
+	int i;
+
+	/*
+	 * Some older SST SPI flashes support AAI write but use 0xaf opcde for
+	 * that. Flashrom uses the byte program opcode to write those flashes,
+	 * so this configuration is fine too. SST25VF064C (id = 0x4b) is an
+	 * exception.
+	 */
+	if (flash && flash->vendor == VENDOR_ID_SST && (flash->model & 0x00ff) != 0x4b)
+		spi_config = &spi_config_aai_write;
+
+	if (spi_locked())
+		return;
+
+	intel_southbridge_override_spi(spi_config);
+
+	spi_opprefix = spi_config->opprefixes[0]
+		| (spi_config->opprefixes[1] << 8);
+	writew_(spi_opprefix, cntlr.preop);
+	for (i = 0; i < ARRAY_SIZE(spi_config->ops); i++) {
+		optype |= (spi_config->ops[i].type & 3) << (i * 2);
+		writeb_(spi_config->ops[i].op, &cntlr.opmenu[i]);
+	}
+	writew_(optype, cntlr.optype);
+
+	spi_set_smm_only_flashing(CONFIG(BOOTMEDIA_SMM_BWP));
+}
+
+__weak void intel_southbridge_override_spi(struct intel_swseq_spi_config *spi_config)
+{
+}
+
+#define BIOS_CNTL		0xdc
+#define  BIOS_CNTL_BIOSWE	(1 << 0)
+#define  BIOS_CNTL_BLE		(1 << 1)
+#define  BIOS_CNTL_SMM_BWP	(1 << 5)
+
+static void spi_set_smm_only_flashing(bool enable)
+{
+	if (!(CONFIG(SOUTHBRIDGE_INTEL_I82801GX) || CONFIG(SOUTHBRIDGE_INTEL_COMMON_SPI_ICH9)))
+		return;
+
+	const pci_devfn_t dev = PCI_DEV(0, 31, 0);
+
+	uint8_t bios_cntl = pci_read_config8(dev, BIOS_CNTL);
+
+	if (enable) {
+		bios_cntl &= ~BIOS_CNTL_BIOSWE;
+		bios_cntl |= BIOS_CNTL_BLE | BIOS_CNTL_SMM_BWP;
+	} else {
+		bios_cntl &= ~(BIOS_CNTL_BLE | BIOS_CNTL_SMM_BWP);
+		bios_cntl |= BIOS_CNTL_BIOSWE;
+	}
+
+	pci_write_config8(dev, BIOS_CNTL, bios_cntl);
+}
+
 static const struct spi_ctrlr spi_ctrlr = {
-	.xfer = spi_ctrlr_xfer,
-	.xfer_vector = spi_xfer_two_vectors,
-	.max_xfer_size = member_size(ich9_spi_regs, fdata),
+	.xfer_vector = xfer_vectors,
+	.max_xfer_size = member_size(struct ich9_spi_regs, fdata),
 	.flash_probe = spi_flash_programmer_probe,
+	.flash_protect = spi_flash_protect,
 };
 
 const struct spi_ctrlr_buses spi_ctrlr_bus_map[] = {
