@@ -137,7 +137,8 @@ int mboot_hash_extend_log(uint64_t flags, uint8_t *hashData, uint32_t hashDataLe
 	printk(BIOS_DEBUG, "%s: SHA256 Hash Digest:\n", __func__);
 	mboot_print_buffer(digest->digest.sha256, VB2_SHA256_DIGEST_SIZE);
 
-	return (tlcl_extend(newEventHdr->pcrIndex, (uint8_t *)&(newEventHdr->digest), NULL));
+	return (tlcl_extend(newEventHdr->pcrIndex, (uint8_t *)&(newEventHdr->digest),
+			    VB2_HASH_SHA256));
 }
 
 /*
@@ -341,7 +342,7 @@ int __attribute__((weak))mb_measure(int wake_from_s3)
  *
  * mb_measure_log_start
  *
- * performs the measurements defined by the the board routines.
+ * performs the measurements defined by the board routines.
  *
  * The logging is defined by the mb_log_list structure
  *
