@@ -36,9 +36,9 @@ void cbmem_run_init_hooks(int is_recovery)
 	function_called();
 }
 
-void *cbmem_top_chipset(void)
+uintptr_t cbmem_top_chipset(void)
 {
-	return (void *)_cbmem_top_ptr;
+	return _cbmem_top_ptr;
 }
 
 static void *get_cbmem_ptr(void)
